# =========================================================================== #
# === CONVENTIONS FOR OUR COOKBOOK FILES
#
# This file is free software i.e. public domain. You can do whatever you  
# want with it (at your own "risk" of course. It is just data anyway.)
#
# I have tried to make the information herein correct, but many things  
# may become outdated eventually, or could be missing - keep this
# in mind.
# =========================================================================== #
# Program Version:
#
#     We might include information about Program Version. I modify source 
#   files to stick to my convention, which is the convention used in most 
#   source package names. I also store in .tar.bz2, and dont really make 
#   exceptions to this rule.
#
# Program Names:
#
#     Please store them in lowercased names only, no Uppercased stuff.
# 
#   orbit is fine, ORBit is not. I rather want to make no "guessing games", 
#   and follow a simple standard, than having weird Up AnD dOwNcAsEd 
#   variants all over the place.
#
#   You can freely use "-" but this may be changed in the future, I am
#   not yet sure. See below for a reasoning, but I think i will stick 
#   to allowing - in program names. Just note as reference name.
#   Right now you can use both - and _
#
#   However i STRONGLY discourage naming schemas such as:
#
#     "mjpegtools-1.6.3-rc3"
#
#   IF you want to make a release candidate, then you should do so
#   without appending a '-'. What's with this sub-versioning anyway?
#
#   For my case, I will simply rename this to something which
#   is easier to match with a simple regex.
#   ProgramName-1.0.0 sounds really much easier for me, as does
#   ProgramName-1.0.0rc3.
#
#   A few UPCASED variables have a special meaning here, for example
#   USE_URL means that we use the URL as defined in url1 as name.
#   This way you no longer have to specify the name explicitely - the
#   url1 is enough information.
#
# Multline Comments in Yaml files:
#
#   If you want to put multiline comments, especially when adding information 
#   to the description entry, then you might want to close the first line 
#   with a \, and in the new line you use "\n    " without the 
#   quotes.
#
# Also, a small advice: if an option is called "configure_options", 
# then do NOT write "program_configure_option" in here. I sometimes 
# miswrite this. This is bad, makes you lose time due to debugging ;-)
# Last but not least, similar to "t" or "true", you can also use "yes" or 
# "y" or no and n instead of true/false values.
# =========================================================================== #

# =========================================================================== #
# COMPLETE REFERENCE WITH DESCRIPTIONS OF THE AVAILABLE OPTIONS IN A YAML FILE
# =========================================================================== #
#
#foobar:                         # The name of program, very important due to 
#                                # the various shortcuts. This is more accurately 
#                                # called the "reference name", because if we 
#                                # want to compile a program, we use this name 
#                                # given here as our reference.
#                                #
#                                # '-' characters are permitted, as are '_'.
#                                #
#                                # As of 27.06.2006, I decided to seriously think 
#                                # about phasing out support for names such as:
#                                #   foo-bar-yamma: 
#                                # I think that
#                                #   foobaryamma:
#                                # is a better choice. I'll spend some more days
#                                # thinking about it. As of now, 01.10.2006, I think
#                                # I will sooner or later switch to disallow 
#                                # - in names. Yes, did so.
#                                # Since June 2015 you can also omit the main
#                                # reference name. In this case, we will infer
#                                # the name from the .yml file itself, e. g.
#                                # "konsole.yml" will become "konsole".
#                                #
# active_project:                # This boolean entry (yes/no) designates whether the
#                                # project is still actively maintained by any upstream
#                                # developer. By default we will assume that all projects
#                                # are actively maintained, so this setting will usually
#                                # only be found for projects that are no longer maintained,
#                                # aka projects that have been abandoned already.
# apply_patch: foo patch         # triggers a patch by invoking system(this_info)
#                                # Normally, you should put a line like "patch foo"
#                                # here but you can also do sed stuff etc.. like this one:
#                                #   sed -i 's@Sending processes@& configured via /etc/inittab@g' src/init.c
# archive_type: TB               # OPTIONAL entry that stores the type of an archive, like 
#                                # ZIP.
#                                # Default is TB, so you can omit this entry 
#                                # altogether for TB packages.
#                                # I mostly only store TB stuff. EXE is for .exe
#                                # and used for windows .exe files typically.
#                                # If you have a tar.gz archive, use TZ here. If you 
#                                # dont care or dont remember, putting DEFAULT here
#                                # will default to whatever format you chose to 
#                                # have... I prefer to start in .tar.bz2
#                                # Or just omit this at all :)
# autosymlink_lib64:             # If this is true then we will symlink lib64 to lib
#                                # as part of the post-installation step of the program
#                                # at hand, if it is compiled in an AppDir manner.
# blfs:                          # This entry is a helpful URL towards the BLFS homepage
#                                # of a given program. This is usually a String, but it
#                                # may also be an Array if you wish to pass in more than
#                                # one URL. Note that this entry can only exist if the
#                                # BLFS project actually has an entry for this.
# base_dir: ./TCLTK              # Stores the name of the base directory. No need to
#                                # append a trailing "/" character.
#                                # If this setting is omitted then we will use a 
#                                # relative naming convention to $SRC and append 
#                                # the UPCASED program name here.
#                                # If you want to build from another base dir, use
#                                # configure_base_dir
# build_static: true             # default is false. Set it to true if you want to compile 
#                                # something statically. The idea behind this is that, if a 
#                                # user enables it, he wants to compile the application 
#                                # statically, without caring at all what he needs to do
#                                # SPECIFICALLY FOR the given program - "build_static: true" 
#                                # needs to be enough as instruction set. 
#                                # "Computer, compile php5 statically." - "Yes, sir!"
# build_deps_on:                 # An array of package names that must be installed
#                                # to build the software but unnecessary for using
#                                # the software after installation.
# configure_base_dir: unix/x     # names base dir for ./configure-like installs
# configure_options: -prune      # contains extra config options like --enabled-shared etc.
#                                # Please note: If a user gives values here such as 
#                                # --prefix=/tmp/foobar/1.0 but the traditional value says sth
#                                # like true (so obviously, we have a conflict), we regard 
#                                # the value in configure_options as more important.
#                                # Why? Because configure_options is more fine-grained
#                                # than a general remark to dump something into a place.
# copy_source: true              # If copy_source is true, we copy the archive to 
#                                # to the location you specified in the config file.
#                                # Typically this would be /Depot/Temp. The default for
#                                # this setting is false for this value. Normally, you will
#                                # not change this setting, which means we wont copy the source
#                                # (It is not really needed, the source is already extracted.)
#                                #
# description: |                 # This variable can be used to describe a program in some
#                                # detail. It should not contain help-information that is
#                                # meant for the user; for this another entry called
#                                # extra_information: could be used. Some special macro-words
#                                # such as DISTRIBUTION_NAME, all upcased, will be assumed to
#                                # be special words, and will become the name of your
#                                # distribution. SELF_NAME will become the name of the package. 
#                                # I recommend to put install-tips not in this entry, but
#                                # instead into "extra_information" 
#
# do_not_compile: false          # if true then we will not compile the program in question.
#                                #
# do_not_symlink: true           # default value is false. If this is set to true, we will not
#                                # symlink after we finished compiling. This i.e. could 
#                                # be useful if you install a new glibc for example.
#                                #
# echo_yes: false                # If true, we echo "yes" before running configure script.
#                                # This option will rarely be needed, but i.e. with qt4 it
#                                # can be useful so that we dont have to ask any question.
#                                # Note that this setting will be rarely used. Default is: false
#                                #
# enable_static: false           # If true then we will enable static compilations.
#                                #
# extra_information:             # This usually contains extra help for installing, or just 
#                                # extra information that is somewhat useful for a user.
#                                #
# extract_to: some/location      # If this is set to a path, we extract our 
#                                # package to this directory.
#                                # Useful to extract packages into other directories, or 
#                                # to extract fluxbox style packages.
#                                #
# github:                        # This optional entry should either be a String, if it was
#                                # defined, or nil - the latter is the default value. If it
#                                # is a String then it will denote the URL to be used for
#                                # github - if that project can be found at github that is.
#                                #
# has_to_be_compiled:            # This can be true or false, or yes or no, respectively. If
#                                # true then we indicate that the given program at hand has
#                                # to be compiled.
#                                #
# installation_steps:            # installation_steps is an Array containing the specific
#                                # installation instructions in order to install a given
#                                # program. If omitted then we will default to "make".
#                                #
# installs_these_binaries:       # lists, separated binaries, which this program installs (will install normally).
#                                #
# installs_these_libraries:      # as above, but for libraries. Directories are currently omitted.
#                                # Also, it is sufficient to add something like "libshadow" instead of
#                                # "libshadow.so", "libshadow.so.0" etc.. because we will already 
#                                # assume that all these ought to exist.
# keep_extracted: false          # default false, thus this setting.05.be omitted. If false, we 
#                                # delete the extracted archive after we are finished with 
#                                # compilation. If true, we keep our extracted archives.
# last_update: 09.05.2010        # This keeps track when you last updated that entry. Personally
#                                # I use it only for some entries. It is not that useful ...
# localstatedir: f               # On default we try to use a localstatedir. We however WONT
#                                # do so if the user specified "false" here. I am not sure
#                                # how useful this is, so maybe I will remove this again one day.
# license: GPLv2                 # This keeps the license of a program. Note that GPLv1, GPLv2
#                                # and GPLv3 since those are the offical abbreviations.
# make_options: make,make docs   # If this is given, these commands separated by a "," will
#                                # be used as make commands INSTEAD of the standard make way.
#                                # I opted to have no spaces between the , since it makes things 
#                                # a little bit more compact. Additionally, there exist
#                                # shortcuts, such as RINSTALL1 to install a ruby
#                                # script or CMAKE for an installation via cmake.
#                                # Look at the file predefined_installation_instructions.yml
#                                # to find out their meaning please. bl $RBT_YAML/*predef*
#                                #
#                                # Since as of February 2018, you can also use a regular
#                                # Array definition in the yaml file.
#                                # Note that make_options is DEPRECATED since as of April 2019.
#                                # The reason for this decision was that I constantly mixed
#                                # it up with options to "make" itself, which instead became
#                                # parameters_to_make (see the entry in this file for that).
#                                # The new way to specify installation options is via
#                                # installation_steps.
#
# optional_start_message: > Starting qt compilation: # optional msg, not really that important
#                                                    # but it may be helpful to some users.
# override: true                 # Default value for override is: false. If 'override' is true, then 
#                                # the yaml setting for this program overrides the behaviour defined 
#                                # elsewhere. Note that this setting is not really that useful 
#                                # anymore IMHO.
#                                #
# parameters_to_make             # This setting passes the given options towards "make". The
#                                # typical use case is "make -j4" or a similar setting, so the
#                                # parameters_to_make would be " -j4" here.
#                                #
# program_name: tcl-8.5a3        # program_name stores the name of an archive. You can 
#                                # overrule the "assumed" name by using - use_this_program_name:
#                                # See below for more info about this variable. Also,
#                                # you can use some macro-names here, such as KERNEL_V
#                                # This will simply fetch the ENV variable KERNEL_V
#                                # which is the linux kernel version name.
#                                # The macro "NAME" here will use the name supplied on the
#                                # url1: target, i.e. if it is www.foobar.com/bla-1.0.1.tar.gz
#                                # then the macro will be expanded to "bla-1.0.1".
#                                # - The macro "FIRST_DOT_AS_HYPHEN" will replace "sendmail.5.2"
#                                #   into "sendmail-5.2".
#                                # The RBT scripts will calculate the variable @short_name from this
#                                # variable.
# pkgconfig_files: xau.pc        # This denotes the .pc file which a package uses. Can be multiple
#                                # pc files too. Must be an Array, actually, so start on a new
#                                # line, with leading '- ' token.
# preinstall: echo hi            # all preinstall things you like. 
#                                # Not sure if its currently fully implemented, 
#                                # its best to check and find out. UNFINISHED?
# postinstall: cd foo,make       # if you want to do postinstall things 
#                                # after make install, add here with , separated
#                                # Postinstall can also use two MACROs. Their names are
#                                # UNIVERSAL_LIBRARIES_DIR and UNIVERSAL_BINARIES_DIR
#                                # On a traditional Linux System, UNIVERSAL_BINARIES_DIR points
#                                # to /usr/bin, and the other var points at /usr/lib. On a
#                                # Gobolinux system, it will be /Programs/Name/Version/{bin,lib}
#                                # You.05.look at ncurses program how this variable is used -
#                                # the reason it exists is because this way we can symlink 
#                                # files without caring whether we are on a traditional or 
#                                # on a gobolinux system. 
# pre_make_commands: cp          # Commands or things to do before starting "make". This
#                                # entry must be an Array.
# rating: bad ok enjoyable decent good very_good excellent perfect # rates ONLY games.
#                                # HOWEVER as of 01.05.2006 I do now rate applications 
#                                # as well. Most games are probably bad ;-)
#
# required_deps_on: ruby         # list your debs here, separate them with ",", without the quotes
#                                # I suggest to just sort them alphabetically. Since April 2008
#                                # this option is no longer mandatory (thus it is optional).
#                                #
# recommended_deps_on: sdl       # This lists recommended dependencies, which 
#                                #.05.be useful to install.
#                                #
# ruby_premake_commands: cp      # same as premake_commands, but we do it the ruby way. Not sure 
#                                # if this is still needed ... POSSIBLY DEPRECATED
#                                #
# run_ldconfig                   # This is a per package setting. Accepts true or false. If true
#                                # we run ldconfig after installing. If false, we never run ldconfig
#                                # automatically. This will be a rare setting, glibc.05.need to
#                                # have it set to false.
#                                #
# run_make_check: yes            # if true/yes, then we run a "make check" between make and make install
#                                #
# short_description              # This entry is optional. It must not have more than one
#                                # line. The reason for this being so short is that this entry
#                                # can be used to give a really brief overview of the target
#                                # program. This allows us to display this on the commandline,
#                                # such as via a slack-desc file for slackware packages.
#                                #
# sub_dir: xfce                  # If this is set then we will use an "inner sub dir". For example, 
#                                # the xfce package has been subpackaged, and thus we prepend 
#                                # xfce to them all.
#                                #
# svn_url: svn://ww.foobar/program/trunk # This is used for svn checkouts.
#                                #
# symlink_pkgconfig_files        # Boolean. If true then we will symlink into /usr/lib/pkgconfig
#                                # but only if we run in AppDir mode and if the directory exists.
#                                #
# tags: editor video             # With tags you can group a program. fowl_mouth had this idea.
#                                # I myself tend to group it into stuff like editors_cookbook.yml
#                                # Note that as of Oct 2014 we recommend to use , to separate
#                                # more than one entry. The reason is that if we use ' ' as the
#                                # character token to split at, we can no longer make use of
#                                # "hex editor". [In Ruby this will be treated as an Array]
#                                # Since as of the year 2019, it is recommended to keep this
#                                # entry as an Array at all times.
#                                #
# prefix: f                      # Default for "prefix" is true, so you can omit it if you 
#                                # want to keep a standard file structure into /usr. If its false,
#                                # we will use Gobolinux directory structure under /Programs
#                                # Since 11.08.2006 you can specify ANOTHER prefix-value
#                                # as well and we will use that as our value.
#                                # If you give "none" then we wont assume any default prefix.
#                                # Example for this:
#                                #   - traditional: /opt/%NAME%
#                                # %NAME% is a special var and will be substituted
#                                # with the name of the program.
#                                # - traditional: /opt/%NAME-VERSION%
#                                # will be substituted with for example /opt/lcms/0.2.4
#                                # The uppercase value D here is a shortcut for /Users/debug, 
#                                # which is my favourite account for doing debugging etc.
#                                # Also, uppercased shorcuts, as in ULOC or UBIN will work too.
#                                # Last but not least, a 'f' will become a 'false'
#                                # and a 't' will become a 'true' here (because f  
#                                # is shorter to type than false).
#                                # url1 should denote the first download_location of a package, 
#                                # the other urls are normally help.
#                                # If you pass "true" to it we use the prefix /usr
#                                # and if it is set to false we will use the scheme
#                                #   $PROGRAMS/Applicationname/Applicationversion
#                                # Example for the latter:
#                                #   /Programs/Python/3.1
#                                # PROGRAM_NAME and PROGRAM_VERSION also work here. In fact, as
#                                # pseudo-macros, they will work everywhere.
# sed:                           # The stream editor. Use it to modify text.
# url1: http://sourceforge.net/project/showfiles.php?group_id=10894
# url2: some/other/url
# url3:
# url4:
# url5:                                 # Maximum number of url entries for now.
#                                       #
# use_autogen:  true                    # default is false. if true, we will run system 'autogen.sh'
# use_autoconf: true                    # default is false. if true, we will run system 'autoconf'
#
# configure_command: configure          # default is "configure" here . But you can overrule it via
#                                       # this yaml option. Another entry that.05.be useful here
#                                       # would be AppRun or ./0 used by Sakura.
# 
# use_this_make_command: make           # Default is make, but it could be cmake or qmake instead.
#                                       # Please do take note that make_options will overrule the
#                                       # setting here.
#                                       #
# use_glib_schema: true                 # If true then we will symlink into the glib schema
#                                       # directory.
#                 
# use_this_make_install_command:        # Use this setting for the "make install" step.
#                                       #
# use_this_program_name: libOGGx        # as you can see, you can use this variable to 
#                                       # overrule any other "guessed" names of a program.
#                                       # If you use the special word DEF here then we will assume
#                                       # you dont want any change (i sometimes add in a new name, 
#                                       # but later want to revert - its easier to just exchange
#                                       # it with DEF
#                                       #
# use_build_directory: true             # By default false. If true, we use a new dir called
#                                       # BUILD. This dir will be the default build-dir.
#                                       #
# use_this_build_directory: foobar      # At default, we use Dir.pwd to configure
#                                       # but the user.05.give us another 
#                                       # build directory. I no longer use this setting,
#                                       # but I will keep it, maybe others need it.
# =========================================================================== #
# === MANDATORY ENTRIES
#
# These are mandatory entries which must always be part of the 
# proper yaml entry. The other values usually default to a sane
# setting.
#
# There is only one mandatory entry so far, and this is:
#   program_name
# =========================================================================== #
# These are NOT or NO LONGER mandatory entries:
#
#   base_dir, traditional, required_deps_on
#
# And actually, ALL other settings which were not mentioned in the mandatory
# section. :)
#
# Nota bene:
#   required_deps_on is not necessary, because there exist programs which
# dont require anything else to _install_ it, for example .h header files.
#
# It would thus be sub-optimal to make this a mandatory entry. Also, since
# most packages depend on glibc anyway, these libraries will rarely be
# added to this entry.
# =========================================================================== #
# Note that you can remove lines with empty options.
# Do this, it saves on disk space. :)
# =========================================================================== #
# To peek at the guys from LFS, look at this entry (lfs tag):
#   http://www.linuxfromscratch.org/blfs/view/svn/
# =========================================================================== #
# I have a new proposal, an entry called:
#   do_not_compile: true
# If this is true, we wont compile.
# =========================================================================== #