# =========================================================================== #
# === Ruby Build Tools configuration settings
# =========================================================================== #
#
# This file here documents the various Configuration settings for use
# in the RBT Project.
#
# The individual .yml files describe the settings for the configuration
# values.
#
# Shortcuts exist for some variables, i.e. "t" can be used rather than true,
# and "f" can be used rather than false. This exists here purely to save some
# typing work. If you consider the longer variants to be more explicit and
# easier to understand, use them.
#
# The main idea - and key purpose of the individual yaml files - are that a
# user only needs to modify the respective .yml file in question, in order
# to adjust the behaviour of the various build scripts in a general way.
#
# This way, there is no actual need to modify the .rb source of any RBT script
# at all - a user need only change data in one place (i.e. in the given yaml
# ile) - et voila.
#
# Things will work from that point. Commandline-driven ways to change these
# settings are also available.
#
# To test the configuration via a .rb script, do:
#
#   rbtconfig
#
# =========================================================================== #
# To find out how many options are registered, do:
#   x = Dir[ENV['RSRC']+'/rbt/lib/rbt/yaml/configuration/*.yml'].size # 57
# =========================================================================== #

# =========================================================================== #
# === shall_we_log
#
# If shall_we_log is set to true, then we will log our actions - at the
# least the more important actions.
# =========================================================================== #

# =========================================================================== #
# === use_mrxvt
#
# If you use MRXVT, you can set this to true.
# =========================================================================== #

# =========================================================================== #
# === sys_info_directory
#
# System Information Directory.
# =========================================================================== #

# =========================================================================== #
# === email
#
# This email is used to report bugs or similar. If you dont need this, you
# can unset it.
# =========================================================================== #

# =========================================================================== #
# === capitalize_program_name                                (capitalize tag)
#
# If the setting `capitalize_program_name` is true then we will turn
# something such as "htop" into "Htop" if we create it into the
# /Programs hierarchy (or any other appdir for that matter).
#
# If it is set to false on the other hand, then we will downcase the whole
# program name. "LibFoo" would then become "libfoo".
# =========================================================================== #

# =========================================================================== #
# === run_simulation
#
# If run_simulation is set to a value of true, then we will just collect
# version information but we wont install/change/do anything. It may be
# interesting to find out what a specific action will do to the system -
# that is why this option exists, in principle.
# =========================================================================== #

# =========================================================================== #
# === cookbook_directory
#
# With the variable called `cookbook_directory` we specify where to find 
# the base directory of all of our individual cookbooks. If you use a
# leading $ for the value, then we assume that the users wants to use an
# ENV variable here.
#
# For example, $FOO would become ENV['FOO'].
#
# You can also specify the absolute location to your base cookbook
# directory of course.
# 
# If you have multiple .yml files in different places, you can put them
# into an - Array format. In this case we will use the first entry as
# the most important entry.
#
# You can also use a directory as identifier, in which case we will load all
# .yml files from that directory. In the past I used to have one cookbook
# file for games, one for video apps, one for audio, one for xorg related
# programs.
#
# Note that as of the year 2012, another approach is used - every program
# gets its own .yml file and its own directory where it resides.
#
# It is completely up to you how you wish to layout any "sub-cookbook"
# files.
#
# Specific Example for absolute paths:
#
#     /Users/x/DATA/PC/COOKBOOKS/YAML/
#     $COOKBOOK_DIRECTORY
#
# =========================================================================== #

# =========================================================================== #
# === enable_gobolinux_config
#
# If this setting is set to true then we use default variables fitting
# to a Gobolinux system. This should be done if you use GoboLinux. A
# commandline switch exists to toggle this permanently too:
#
#   rbt --permanently-enable-gobolinux-config
#
# =========================================================================== #

# =========================================================================== #
# === use_stow                                                     (stow tag)
#
# If true then we will use stow. Of course you must have stow
# available for this option to work.
#
# See for this URL in order to obtain stow:
#   http://ftp.gnu.org/gnu/stow/?C=M;O=D
# =========================================================================== #

# =========================================================================== #
# === sysbin_directory
#
# For system binaries.
#
# On Gobolinux Systems, this would become:
#
#  sysbin_dir: /System/Links/Executables
#
# =========================================================================== #

# =========================================================================== #
# === always_show_dependencies
#
# If 'always_show_dependencies' is true, then we will always feedback to
# the user which dependencies are known for a given program. I recommend
# to set this to true ONLY if you like to get extra information, or if
# you want to debug something. Otherwise, you should probably keep it
# to false, as the output can become a bit noisy.
# =========================================================================== # 

# =========================================================================== #
# === run_ldconfig
#
# This setting determines whether we shall run ldconfing or not.
# I recommend to keep this set to a value of true. It can be overruled via
# a per-program setting, by the way, in the respective .yml file.
# =========================================================================== #

# =========================================================================== #
# === play_song                                                  (audio tag)
#
# If play_song is true we will play a song once compilation has finished 
# its job.
# =========================================================================== #

# =========================================================================== #
# === use_ccache
#
# Whether we use ccache or not. I suppose most users want to turn this 
# option off.
# =========================================================================== #

# =========================================================================== #
# === make_command
#
# On Linux this is typically called "make". On Freebsd, you may have
# to change it from "make" to "gmake". That is the main reason for
# this configuration option. Note that individual cookbook files
# may overrule this setting on a per-program basis.
# =========================================================================== #

# =========================================================================== #
# === exit_on_unregistered_cookbook_entries
#
# If this setting is true, then we will exit when we find a cookbook entry
# that was not registered.
# =========================================================================== #

# =========================================================================== #
# === use_tags
#
# If this setting is set to true then we will symlink into the /System/Tags
# directory. We will use "tags" - some programs can be described via tags.
# For instance, mplayer carries the video tag.
# =========================================================================== #

# =========================================================================== #
# === shebang_to_use
#
# Use this setting as the shebang file.
#
# Other options i.e.:
#   /usr/bin/env ruby
#
# You can (and should) omit the leading #! part. If you want to use another
# shebang, just replace the following. Normally this should not be needed
# though (and on a Gobolinux system it will work anyway).
# =========================================================================== #

# =========================================================================== #
# === store_into_this_directory
#
# This variable specifies the base directory where we store all things.
# Please keep a trailing / here for this variable.
# =========================================================================== #

# =========================================================================== #
# === stop_on_error
#
# If true then we will stop every time an error occurs. The method for this
# will be called stop_on_error().
# =========================================================================== #

# =========================================================================== #
# === sysshare_directory
#
# Where to keep System shared files. On FHS system this would be
# /usr/share. On Gobolinux it would be /System/Links/Shared
# =========================================================================== #

# =========================================================================== #
# === audio_directory
#
# Mostly only useful on my home system.
# =========================================================================== #

# =========================================================================== #
# === pkgconfig_directory
#
# The pkgconfig dir is the directory where the .pc files are kept. On FHS
# systems, this is usually /usr/lib/pkgconfig. On Gobolinux, this would be
# /System/Links/Libraries/pkgconfig
# =========================================================================== #

# =========================================================================== #
# === system_logs
#
# The entry system_logs keeps the location for our system log directory. A
# popular location for FHS-based system would be /var/log. Personally, I
# use /System/Logs as specifed by my $SYSLOGS ENV variable.
# =========================================================================== #

# =========================================================================== #
# === temp_directory
#
# The temp_directory variable keeps a reference to the TEMPORARY Directory
# used by the RBT Scripts. We need this directory to extract packages into
# that directory. It is usually the directory /tmp on FHS systems and the 
# directory /Depot/Temp/ on Gobolinux systems.
#
# On IRC, 2011, t_s_o said Gobolinux probably uses another value, the
# following:
#
#   temp_directory: /System/Variable/tmp
#
# So if you use Gobolinux, you probably want to use the above setting.
# =========================================================================== #

# =========================================================================== #
# === sysinclude_directory
#
# On FHS this is probably /usr/include whereas on Gobolinux this is 
# /System/Links/Headers
# =========================================================================== #

# =========================================================================== #
# === system_settings_directory
#
# This is the traditional /etc directory in *nix.
# =========================================================================== #

# =========================================================================== #
# === system_directory
#
# With this setting you should point to your /System directory, if you have
# one. On Gobolinux this is /System. Not sure if FHS has anything like
# that - it probably does not. 
# =========================================================================== #

# =========================================================================== #
# === syslib_directory
#
# On FHS this setting probably defaults to /usr/lib whereas on Gobolinux
# this is /System/Links/Libraries. On my System, I have settled for 
# /System/Libraries.
# =========================================================================== #

# =========================================================================== #
# === depot_directory
#
# On Gobolinux this is /Depot. On FHS systems, this does not exist in such a
# way, but you can probably use /tmp. Or just create a new /Depot directory.
# =========================================================================== #

# =========================================================================== #
# === enable_use_flags
#
# If enable_use_flags is set to true, then we will use USE flags.
# They are very similar to Gentoo USE flags and allow you to set global 
# settings - for example, you can enable "+kde" in the use flags and every
# compilation will be affected, trying to include kde-specific stuff.
# Or you can use "-arts" and whenever possible we will try to compile
# with "--without-arts".
# =========================================================================== #

# =========================================================================== #
# === report_how_many_programs_are_registered
#
# If true then we will report how many programs are registered. A command
# line switch exists to overrule this. Personally, I keep it to false
# these days.
# =========================================================================== #

# =========================================================================== #
# === enter_extracted_directories
#
# If this is true then we will automatically enter extracted tarballs. Not
# sure why this is needed ... if I can't find out until end of 2013 then
# it will be removed - or the explanation will be written here.
# =========================================================================== #

# =========================================================================== #
# === compile_modular_xorg_into_this_prefix
#
# This variable is used to compile all modular xorg packages into a certain 
# prefix. To disable this variable set it to false. This variable will _ONLY_ 
# be honoured if you ALSO set modular_xorg_own_prefix to true. 
# You can also use environment variables such as $PROGRAMS here.
# =========================================================================== #

# =========================================================================== #
# === modular_xorg_own_prefix
#
# True means "honour another variable", false means "ignore this setting".
# =========================================================================== #

# =========================================================================== #
# === use_sqlite3_database
#
# If use_sqlite3_database is true we will use a sqlite3 database.
# =========================================================================== #

# =========================================================================== #
# === check_for_library_names
#
# Same as the option check_for_binary_names, but valid only for library
# names.
# =========================================================================== #

# =========================================================================== #
# === create_package
#
# If we want to create a package or not.
# =========================================================================== #

# =========================================================================== #
# === file_last_compiled_program
#
# Specify where to log last compiled program info. Should be a .html file.
# =========================================================================== #

# =========================================================================== #
# === packages_directory
#
# Here you can define where to find packages. My default package directory
# resides at /Depot/Packages. I use an environment variable to point to
# it, which is called $PACKAGES_DIR. e $PACKAGES_DIR
# =========================================================================== #

# =========================================================================== #
# === report_size_of_chained_programs
#
# If this variable is true, then we will report the size of all programs in
# our compile chain. (Currently I have set this to false because it is 
# buggy... too slow and a bit silly.)
# =========================================================================== #

# =========================================================================== #
# === use_another_prefix
#
# The configuration setting `use_another_prefix` __GLOBALLY__ overrides most
# other available settings and thus naturally has a VERY high importance.
# Example values can be seen to its right side, I recommend to keep it at
# false in most scenarios. If it is set to another value, this value will
# be used for __EVERY__ --prefix setting, i.e. you could compile to /usr 
# dir with this setting.
# =========================================================================== #

# =========================================================================== #
# === compile_everything_into_standalone_directory
#
# This setting is different to the setting "use_another_prefix", in that if 
# it is set to true, we will compile every program into a standalone 
# directory. It is as if the setting "prefix: " was set to false for
# every available program, if this setting is true.
# =========================================================================== #

# =========================================================================== #
# === use_svn
#
# This setting is a bit tricky and in general you can well set it to false
# and not much will happen. I set it to true because I needed this 
# functionality, which I will explain now. If use_svn is true, then
# we will always use the svn_url when we want to download the latest 
# software, IF this variable was set. Mplayer is an example where I
# have set it. For this to work we will need "svn_url" setting in our
# cookbook.
# =========================================================================== #

# =========================================================================== #
# === show_extra_information
#
# If show_extra_information is true then we show extra information of any 
# program which we compile (provided that this information also exists.)
#
# This information is retrieved from the "extra_information:" setting in
# the yaml file. I recommend to keep this setting here to true if you are
# an user who loves to get as much information as possible, or if you need 
# help compiling something. The extra information may be helpful/useful.
# Personally I keep it set to true, but if you want less verbosity, you 
# probably want to set it to false.
# =========================================================================== #

# =========================================================================== #
# === report_specific_program_name
#
# If report_specific_program_name is true, then we output the program
# name when reporting to the user (in most cases).
# =========================================================================== #

# =========================================================================== #
# === run_as_user
#
# With run_as_user you can specify another user, i.e root or superuser.
# "default" is a special variable which will be replaced with the real 
# name of the user in question.
# =========================================================================== #

# =========================================================================== #
# === try_to_download_if_not_found
#
# If the configuration option `try_to_download_if_not_found` is set to true,
# then RBT will try to download the program in question, then continue.
# Otherwise, if this option was set to false, we will just exit instead
# and display an error message to the user in the event that the program
# can not be found locally already.
# =========================================================================== #

# =========================================================================== #
# === check_for_binary_names
#
# This setting applies ONLY when you try to find a binary, but you do
# not remember to which package it belongs to. In this case we will
# automatically search for the program which has that binary name,
# and compile it. Please be aware that this setting right now can
# only be updated or created manually. To do this, do
#   ry foo update_binary_names
# =========================================================================== #

# =========================================================================== #
# === debug                                                 (debug section)
#
# Set the debug variable to true to enable debugging. Most users wont need
# it though. Also, this setting will not be honoured if a file called
# 'debug.yml' exists. The reason for this is simple - debug.yml can be
# generated automatically from the commandline. This way, a user can
# quickly change the debug setting - but as said, normally only a developer
# needs this setting anyway. You can probably keep this set to false,
# won't need it.
# =========================================================================== #

# =========================================================================== #
# === be_verbose
#
# If true then we feedback lots of extra information to the user.
# =========================================================================== #

# =========================================================================== #
# === use_absolute_symlinking
#
# This setting can be either true or false. If it is false then we simulate 
# Gobolinux-like behaviour and symlink with a ../../../ so that it works in
# a chrooted environment as well. I recommend to keep this set to false. 
# =========================================================================== #

# =========================================================================== #
# === recipes_directory
#
# Our recipes dir. We will keep our recipes in this directory.
# Should contain a trailing '/' character.
# =========================================================================== #

# =========================================================================== #
# === source_directory
#
# This setting governs where we will keep the source directories for our
# software. Due to mostly historical reasons, I myself keep all my Sources
# under the directory /Users/x/SRC
#
# The environment variable $MY_SRC points at this location on my system.
#
# On Gobolinux, this is instead typically the value of:
#
#   /Files/Compile/Archives/
#
# To use it in the RBT scripts, do something like this:
#
#   CONFIG.source_directory
#
# However, this is no longer needed to be a part of RBT, as the Cookbooks
# project takes care of this these days.
#
# If we wish to overrule it, we will provide a commandline-option for this
# anyway. Otherwise we can default to the Cookbooks project.
#
# I have decided to keep this option for now though (Aug 2014) - perhaps
# in the future we will completely eliminated it. Let's see.
# =========================================================================== #

# =========================================================================== #
# === check_if_compiling_is_in_progress
#
# The following config option is a bit more difficult to understand.
#
# Say you are compiling a program called `foobar-1.0` and it takes a long
# time before it is finished. Meanwhile, you start another compilation process, 
# this time one that compiles several programs one after the other (chained 
# compiling). Then it may happen that inside that chain you will start 
# another round of compiling the program foobar-1.0 again, and we may end 
# up extracting the content of foobar-1.0 again or removing the old dir, 
# which will cause the old compile to fail (and can be annoying if it took a 
# long time to compile this. I recommend you to keep this set to true, but
# if you dont need it, just set it to false. (Lately, I am setting it to 
# false as well)
# =========================================================================== #

# =========================================================================== #
# === use_systemu
#
# If use_systemu is true, we will try to use systemu instead of standard
# output to console.
# =========================================================================== #

# =========================================================================== #
# === programs_directory                                       (programs tag)
#
# The setting `programs_directory` specifies where to install/compile a
# given program.
#
# On Gobolinux this will default to /Programs.
#
# On FHS systems you could use /Programs too or something else like /pkg
# if you feel that this could look nicer. Simply set the value of this
# directory to the right side.
#
# programs_directory: USER_DIR/Programs # This can be used for $HOME
#                                       # directories too.
# $PROGRAMS # Change this to /Programs if you don't have defined 
#           # the variable $PROGRAMS.
#
# You can also toggle this from the commandline since Sep 2014:
#
#   rbt htop --extract_to_this_directory=/home/kumar/Robert/Temp
#
# =========================================================================== #

# =========================================================================== #
# === colourized_output
#
# If the setting `coloured_output` is set to true, then we will use colours.
#
# (This setting can be overruled on the commandline via the option
# --nocolor.)
#
# In other words, this allows you to globally disable the colour output.
#
# If you wish to toggle this setting from the commandline, use the file
# rbt_config. See its --help output for further information.
# =========================================================================== #

# =========================================================================== #
# === use_strip
#
# If the `use_strip` setting is existing and set to true, then we will
# apply `strip` on the compiled binaries. This presently only works if
# we also use the ntrad (AppDir) target, such as on GoboLinux systems.
#
# If this value is set to true then we will run this command:
#
#   strip --strip-debug
#
# on all compiled binaries.
# =========================================================================== #

# =========================================================================== #
# === use_autofixing
#
# If autofixing is enabled, then RBT will attempt to automatically resolve
# problems, whenever possible.
# =========================================================================== #