This document describes some of the theory behind the prefix variable
- and how RBT makes use of it.

Every program that is to be installed, will have a certain prefix, that
is, a location under where it may put its associated files.

On a typical Linux distribution the main prefix in use will typically
be at the /usr/ hierarchy, sometimes /usr/local/ and rarely /opt/.

In GNU configure based systems, this is usually to be done through
invoking configure with this prefix target:

    ../configure --prefix=/usr

The default, when --prefix is omitted, normally is /usr/local/.

On GoboLinux-like systems, the prefix will instead be:

    /Programs/PROGRAM_NAME/PROGRAM_VERSION

Here, PROGRAM_NAME stands for the name of the given program at
hand, such as "evince"; and PROGRAM_VERSION stands for the 
program version at hand, such as "1.5.0". So in other words,
you can expect this version of evince to be found at:

    /Programs/Evince/1.5.0/

Under that directory you may have bin/, lib/ and similar
subdirectories.

(1) On initial startup of the RBT scripts, the RBT project will
simply use the prefix that is defined in any particular cookbook
file at hand, if this file is registered.

We will refer to this prefix as the "cookbook-prefix".

Note that the "cookbook-prefix" can be omitted in a cookbook file,
in which case the default prefix for it will be towards /usr/. This
is mostly to remain compatible to most linux distributions out
there.

The letter "t" in a cookbook file is synonymous to /usr/. (It is
"t" because t is a shortcut for "traditional prefix", and
traditional prefix in this context means "oldschool" aka a
/usr/ setup.)

In the event that the user has not specified anything for the prefix
variable, it will be automatically inferred as mentioned above -
hence defaulting to /usr/.

(2) The second prefix variant will be the one that the user
decides to actually use, and will overrule any other prefix in use.

This one is an optional step and this "user-prefix" will default
to nil, meaning that it has not been set yet. The nil value there
thus indicates that the user has NOT wanted to change anything.

In this case, again, the "cookbook-prefix" will be used.

The user-changable prefix can only be modified via a method call
towards:

    set_user_prefix()

You can also use environment variables to be used as a--prefix,
such as by issuing this long command:

    --use-this-environment-variable=XFCE_PREFIX
    --use-this-environment-variable=$XFCE_PREFIX

In this latter case, we will use this environment variable as
prefix value. This is done through the method
use_this_environment_variable().

The current prefix in use can be queried via the method
"prefix?". This will usually return the "cookbook_prefix?"
in use, unless the user-prefix has been designated, in which
case the user-prefix will instead be returned.

The above may sound a bit complicated, but the reason why this
was necessary is so that we have sufficient flexibility when
working with prefixes.