# =========================================================================== #
# === Conventions
#
# Because the RBT project is fairly large and possibly complex, it is
# important to note down some of the important conventions that are
# used in the project itself - otherwise confusion might arise.
#
# We need a clear vision on what to use, and what we wish to avoid
# for in this project.
# =========================================================================== #

- To exit from one of the scripts, consistently use the following method
  rather than exit() directly:

    exit_program()

  The reason is that this entry point gives us a simple way to internally
  change that behaviour if we have to - perhaps we want to do some cleanup
  actions before we exit, or we may refuse to exit after all, too. This
  is why exit() should not be used directly - use the wrapper method
  exit_program() instead. It gives us more control over as to what we
  want to do.

- To exit AND include a message to the user, use either one of the following
  methods:

    warn_and_exit(message)
    warn_and_exit(message)

- To warn the user of something, this could be used:

    warn 'Something good has happened.'
    warn 'Something bad has happened.'

- It is preferred to use ' instead of " in strings IF these strings
  do not contain a variable inside of them.

  If, however had, these string objects have a variable, then one
  should use "", since string interpolation is faster than using
  the + operator.

  Example:

    "Foo#{var}"

  vs

    'Foo'

  or
    'Foo'+var

- Avoid using method_missing() in the RBT scripts or plugins in general,
  whenever possible.

  While method_missing() is a useful and great feature, it can lead to
  a more complicated design in many cases. Maintaining simplicity via
  method_missing() seems to encourage a more brittle design rather
  than well-thought through design.

  The RBT project should strive for maximal consistency while maintaining
  simplicity in design decisions, whenever that is possible.

- "lib" shortcuts are allowed, as aliases to an existing .yml file.

  For instance, if we have a project called "foo", then "libfoo" is an
  implied alias to it. (That yaml file aka libfoo.yml must exist, of
  course.)
