#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# This is the webpage for the RBT project.
#
# It has a table of contents since at least Feb 2014.
#
#   rf rbtcgi
# =========================================================================== #
require 'cyberweb/autoinclude'
require 'rbt/without_gui_bindings'

w {
t 'The Ruby Build Tools'
css_style '
b.EVENT { color: coral; }
pre {font-size: 1.10em}
p {
  font-weight:    400;
  margin:      0.25em;
  margin-top:   0.5em;
  padding:        4px;
  text-indent:    0em;
}
div.default {
  padding:         1em;
  padding-left: 1.25em,
  padding-top:  0.75em;
  margin-top:   0.5em;
}
span.directories { font-weight: bold; color: orange; }
span.header {font-weight:bold; color: yellow; }'
favicon 'PC/OS/LINUX/DISTRIBUTIONS/GOBOLINUX/POWERED_BY_GOBOLINUX.jpg'
body_css_class 'pad1px GOTHIC BG_Black white'
body_css_style 'font-family: Tahoma,Times,sans-serif;'
font_size '1.35em'
use_jquery
}

COLOUR_BROWN = '#630'

parameters = Cyberweb.params?
# =========================================================================== #
# Standard border for <p> tags and similar.
# =========================================================================== #
STANDARD_BORDER='border-left:5px solid green; border-top:5px solid green'
RED_SQUARE = sg(:red_square, 'bblack1 marr4px')

CMD1 = 'cyan marl2em padl1em mart0_5em marb0_5em BLOCK wid70'
CMD2 = 'cyan marl3em padl1em mart0_5em marb0_5em bordorange1px BLOCK wid70'

CURRENTLY_NOT_IMPLEMENTED = '(Warning: This is currently not implemented.)'
ARRAY_ALL_IDS = []

# =========================================================================== #
# === command
#
# This does a command.
# =========================================================================== #
def command(this_cmd)
  s2 this_cmd, CMD1,'','border-left:1px solid orange;'
end; alias cmd command # === cmd

# =========================================================================== #
# === header
# =========================================================================== #
def header(text, optional_css = '')
  id = text.dup.gsub(/ /,'_').downcase
  image_with_link = '<a href="#'+id+'">'+sg(:dot7,
    'marr8px')+'</a>'
  h2 image_with_link+text, 
     optional_css, id,'color: #00ee55;margin-top:1em;'
  ARRAY_ALL_IDS << id
end

# =========================================================================== #
# === header_stylish_brown
# =========================================================================== #
def header_stylish_brown(
    i, optional_css = '', optional_id = '', optional_css_style = ''
  )
  h2 i, :header_stylish_brown,
     'header_stylish_brown  '+optional_css, optional_id, optional_css_style
end

# =========================================================================== #
# === result
# =========================================================================== #
def result(i)
  e i,'color: RANDOM_COLOUR'
end

# =========================================================================== #
# Begin the main content
# =========================================================================== #
doc('wid88 pad1px mart1px') {

w.use_this_javascript "

function toggle_visibility(id) {

  var e = document.getElementById(id);

  if (e.style.display == 'block')
    e.style.display = 'none';
  else
    e.style.display = 'block';
}"
  bagger = sg(:bagger,'mars5px', :drag)
  h1 sg('PROGRAMMING_LANGUAGES/RUBY/RUBY_BUILD_TOOLS_LOGO.png')+bagger+
     ' RubyBuildTools: Webinterface', :cyan,'','margin-bottom:4px;'
  # ========================================================================= #
  # === Tutorial Tag TUT tag
  # ========================================================================= #
  div('mart0_3em'){
    header_stylish_brown 'The RubyBuildTools Tutorial', 'gold'
    header_stylish_brown 'Project :: Ruby Build Tools'
    div('mart1em'){
      e 'Welcome to this Tutorial (last update: <b>'+
         Time.now.strftime("%d.%m.%Y")+'</b>).'
      br
      e 'This tutorial has a fairly simple goal, which is to try to teach other human
      beings how to use the <b class="orange">RubyBuildTools (RBT)</b>.'
      br
      e 'It also has a secondary goal, which is to provide some reasoning
      and motivation for having created this project.'
      br
      e 'Reworded differently, this tutorial should provide you with 
      an understanding how the RBT project may be useful for you you.'
      br
      e 'This document here is essentially an introduction to the possibilities
      of what can be done with the <b>RBT-scripts</b>. It will also show a
      lot of specific examples, usually for use in the commandline. Ideally
      though, we can also make use of RBT through a GUI interface or the
      WWW.'
      br
      e 'You should be able to see which instructions are for the commandline,
      by simply looking for lines that are padded to the right and
      having a thin, colourized border.'
      br
      e 'Here is one simple example for this:'
      br
      cmd '  rbt --version?'
      br
      e 'The above would, on the commandline, tell you which version of
      the RBT scripts you are currently using. At the time of writing this
      tutorial, the current version was <b>'+RBT.version?.to_s+'</b>.'
      br
      e 'In the event that you do happen to find a bug, it would be 
      appreciated if said bug could be reported on the
      github-issue tracker at'
      br
      e '<a href="https://github.com/shevegen/rbt" class="slateblue marl2em">
        https://github.com/shevegen/rbt/issues
      </a>'
      br
      e 'or via email to me at 
      <b class="yel">shevegen@gmail.com</b> directly.'
      br
      e 'All bug reports will additionally be stored in a local file, a
      "todo" file if you so will. This makes it easier for me to work
      through different issues one at a time - and sometimes postpone
      if things may be too hard to solve or take too long to implement.'
      br
      e 'One important aspect of this tutorial is, regardless over
      whether you are a "newbie" or a "knowbie", to understand how
      things work, the reasoning behind them, how they can be used
      and how they can be applied to help you in your endeavours.'
      br
      e 'The RBT Project appreciates having knowledgable users - there
      should not be any artificial separation between "developers" and
      "users", because in the end, knowledge should be made available
      for everyone, at their own discretion, how they wish to apply
      it.'
      br
      anm 'In the event that you think that the tutorial is missing 
      out some important information, report this as well please,
      so that this part of the Tutorial can be improved.',
      'marl1em BLOCK'
      br
      e "Now, without further ado, let's get started."
      br
      header_stylish_brown sg(:gitarre,'marr5px')+
      '  Installing the RBT gem', '','margin-left:1em;margin-top:1px;'
      p {
        e 'You may have to first download and install one gem, the
        configuration gem.'
        br
        cmd '  wget http://shevegen.square7.ch/configuration-1.0.18.gem'
        cmd '  gem install ./configuration-1.0.18.gem'
        br
        e 'Next, try to install the rbt gem:'
        br
        cmd '  gem install rbt'
        br
      }
      header_stylish_brown sg(:gitarre,'marr5px')+
      '  Introduction', '','margin-left:1em;margin-top:1px;'
      br
      p {
        e 'One of the main design decisions behind the 
        <span class="crimson">Ruby Build Tools (RBT)</span> 
        Project is to remain <b>extremely flexible</b>.'
        br
        e 'That means that there will be no "one way to use them".'
        br
        e 'There will be multiple ways and you can pick whatever way
        suits you the most. Different paradigms may coexist with
        one another. For instance, there is a class that allows
        one to compile from a single file, with just the remote
        URLs listed in that file.'
        br
        e 'The RBT scripts will strive to remain logical, and
        consistent.'
        br
        e 'Complexity is ideally kept to a minimum, giving explanations
        whenever possible, so that the user is in full control over
        every decision in the end.'
        br
        e 'One example for this can be noticed for commandline
        options:'
        br
        l2em DOT9+'If a long option is called 
        <b class="green">"--skip-extracting"</b> or
        <b class="green">"skip_extracting"</b>, then a shortcut like 
        <b class="green">"se"</b> will - and should - work exactly 
        as the longer version. This is valid for every commandline 
        option - there are many aliases to them. So, if you see
        a long commandline option, you can also try a shorter
        variant of that - and it should work. You can also omit
        all "_" - they are just used to make it easier to read
        the specific command at hand.'
        e '<b class="FI">skipextracting</b> would work just the same.'
      } 
      p {
        e 'The whole RBT project consists of various scripts, scripts 
        which are the culmination of a very interesting endeavour 
        into the '+ string_link('http://www.gobolinux.org/', 
        sg(:external,'marr4px')+'Gobolinux world',0,
        'marr0px slateblue BO')+'.'
        br
        e 'This endeavour is explained in the following box 
        <i>"History of the RBT Tools"</i>, which you can skip
        if you are not interested in it.'
      }
      # ===================================================================== #
      # The History of the RBT Project.
      # ===================================================================== #
      h5 'Click here to toggle the visibility of the next paragraph.',
         'pad0_5em padlem','',
         'border: 1px dotted lightblue',
         'onclick="toggle_visibility(\'history_of_the_rbt_project\');"'
      div(id: 'history_of_the_rbt_project') {
        e sg(:dokument,'marr8px')+'The History of the Ruby Build 
        Tools [RBT] Project', 'header marl4em'
        p('ind1em pad1em marl2em','',STANDARD_BORDER) {
          e 'When I started to write these scripts, possibly
          back in 2005 or 2006, I wanted to have them all written
          in '+RUBY+' <b class="crimson">ruby</b> (my favourite
          programming language at that time, and still as of today),
          and write them on my own - so I actually would understand
          what they were doing in detail.'; br
          le 'Back then - and I still am of this opinion even today
          - I thought that shell scripts ultimately tend to grow in
          complexity way faster and steeper than other programming
          languages would do. They also lean towards becoming really
          ugly to further extend with more code, at which point sooner
          or later maintaining them is no more real fun anymore. And
          when it is no fun to maintain something, you start to lose
          interest and eventually cease maintaining it or adding
          anything new to it.'
          br
          le 'In Ruby the situation is a <b>lot</b> better, as you 
          can (re)arrange functionality freely into classes and modules, 
          extend these lateron easily, and use one of the various GUI 
          bindings to present the user with an easy-to-use graphical 
          solution.'
          br
          le 'In my opinion, Ruby is a much better language than shell 
          scripts and the logic found in such scripts. The GNU Sourcemage 
          project is trying to use only bash scripts, and they dont really 
          evolve quite much from my point of view. Users prefer
          distributions like Archlinux for a reason.'
          le 'Big software projects will hardly ever be restricted to 
          shell scripts - it is only the Linux world which still embraces 
          this archaic concept of using ugly shell scripts.'
          br
          le 'Also, well written Ruby code tends to be much nicer to read 
          than the equivalent code in a shell script. Python code may be 
          just as useful when compared to Shell Syntax, but since Ruby is
          my main language, I chose to use Ruby rather than Python.'
          br
          le 'In summary - I think <b class="crimson">Ruby</b> provides
          you with more options than shell scripts will ever be able to
          give to you in the long run.'
        }
      }
      p('mart1em') {
        e 'Let us reflect an important difference between a 
        "traditional", and a "Gobolinux-like OS", because it is important 
        to know about this difference with regards to using the 
        <span class="crimson">RBT scripts</span>.'
      }
      p {
        e sg(:dot7)+' A traditional system conforms (more or 
        less) <b>completely</b> to the FHS standard, and thus uses 
        /usr, /usr/lib and all the directories the FHS mandates one
        to have.'
        br
        e 'This also means that the files of any given program will be 
        spread around multiple locations and directories (/usr/bin, 
        /usr/lib, /etc and so on). Most distributions conform to this 
        setup, although sometimes they break the FHS standard (as did 
        Sabayon on May 2009 when their LiveCD used /usr/qt/3)',
        'marl2em'
        br
        e 'In my opinion this approach is slightly problematic and
        not very elegant.'
        br
        le 'You can not easily uninstall programs again unless you
        use a package manager to help you (or somehow hope that
        you can do "make uninstall").'
        br
        e sg(:dot7)+' A GoboLinux like system uses a 
        slightly different approach - the AppDir approach.
        Every program is (or should be) self-contained in a single
        directory and will then get symlinked into other locations,
        into directories like /System, /Programs and so on. 
        In a way this is similar to AppDirs, but it is also rare to
        see in the UNIX world in general; I think only PC-BSD follows
        a similar scheme, Whitix also follows a somewhat related model, 
        and Mac OS-X also has a very similar scheme but focuses on 
        a graphical approach with their AppDirs.','marl2em'
      }
      p('mart1em') {    
        e 'The <b class="ud">biggest advantage</b> is that this scheme 
        is <span class="underline">a LOT</span> simpler to use, while 
        remaining easy, pure and very clean.'
        br
        e 'If you want to remove a program, you just remove the 
        <b>AppDir</b> directory.'
        e "That's it basically."
        br 
        e 'No need for a package manager to "sanitize" your system 
        again. (Additionally, GoboLinux uses "legacy" symlinks to stay 
        completely compatible with the FHS. These symlinks are by 
        default hidden from the user and can be unhidden using 
        "gobohide --unhide target".'
        br
        e 'It is often amusing to read posts of people claiming that
        GoboLinux is not following the FHS - but every FHS-mapped
        dir will in fact work just as well on GoboLinux, 
        "cd /usr/bin", "cd /opt" and so forth - so in fact the 
        statement that GoboLinux is fully FHS compliant is 100% true.'
        e 'The FHS does not forbid the use of symlinks, it merely
        mandates that these directories need to be available at that
        position, which they are.)','marl2em'
        br
        e 'More explanations will follow in detail lateron, 
        but for now, just remember the <b class="yel ud">basic distinction</b> 
        between a traditional, and a Gobolinux-like System (Gobolinux was
        a pioneer in this area) lies in the <b>Filesystem layout</b>.'
      }
      header_stylish_brown 'Scope of the RubyBuildTools and Platforms'
      p {
        e 'Ultimately, the <span class="crimson">RBT tools</span> will not
        be restricted to only run on Linux, they will work on different
        platforms. However, the first concern will be to have them work
        under Linux. Wherever Ruby works, the RBT scripts should work on
        that platform as well.'
        br
        e 'For now please, use a terminal/shell and lets start.'
        br
        e 'Type or copy/paste the following (do not forget to
        press the <b>enter key</b>):'
        br
        cmd '  rbt htop'
        br 
        e 'If everything works, then you should be able to see some
        information printed on to the screen. The RBT scripts are
        compiling the application <b>htop</b> right now!'
        br
        e 'Once the scripts have finished compilation, you will be
        notified as to where you can find the compiled files.'
        br
        anm 'Note that since as of November 2013, you can also pass in
        a full URL as argument.'
        br
        anm 'That way, you can download from a remote site and immediately
        continue from there.'
        br
        e 'Examples for the latter way:'
        br
        cmd '  rbt http://ftp.gnome.org/pub/GNOME/sources/jhbuild/3.10/jhbuild-3.10.0.tar.xz'
        cmd '  rbt https://downloads.sourceforge.net/project/open-cobol/open-cobol/1.1/open-cobol-1.1.tar.gz'
        br
        e 'This will generate the necessary .yml files, download and 
        repackage (repackaging is optional). Then you can decide whether
        you wish to continue or not.'
      }
    p {
      e 'On a default, out-of-the-box system, htop can now be found 
      under <b>/Programs/Htop/Current</b>, or rather $PROGRAMS
      variable rather than hardcoded /Programs. Note that you can
      change this in the configuration settings too.'
      br
      e 'Lets go there to have a look:'
      command '  cd $PROGRAMS/Htop/Current; ll'
      e 'You may notice the directory structure. It is how Gobolinux 
      has layout their directory structure of directories within /Program 
      as well, but there are some differences, which we will cover
      later.'
      br
      e 'Typically, you will find a 
      <span class="directories">bin/</span> directory and a
      <span class="directories">share/</span> directory
      there. Inside bin/ lies the single binay called "htop".'
      br
      e 'You may also see a file called <b>CompileTimeInformation</b>
      which contains information about the compilation process.'
      br
      e 'Anyway, the most important point to note here is this:'
      br
      e sg(:pfeil_rechts,'marr4px')+'All of htop is self-contained 
      in this directory.','marl3em yel FI'
      br
      e 'This in fact is the biggest difference to FHS-based 
      systems. Others call this approach (more or less) 
      AppDirs or similar.'
      br
      e 'Personally I prefer the word "self-contained" since it is 
      more direct to the point that everything that belongs to the 
      package directly, is aggregated inside one directory, and we 
      wont confuse it with Mac OS X Appdirs for example.'
    }
    p('ind0px') {
      e 'If you want to, for instance, uninstalle htop again,
      simply remove the directory:'
      br
      cmd '  rm -rf /Programs/Htop/'
      br
      e 'or just the specific version:'
      br
      cmd '  rm -rf /Programs/Htop/0.8.1'
      br
      e sg(:ausrufungszeichen,'marr4px')+'Note that 
      I recommend to set the variable $PROGRAMS so instead of 
      hardcoding the above, rewrite it like this:'
      cmd '  rm $PROGRAMS/Htop/7.0'
      e 'To get the value of $PROGRAMS, simply do:'
      cmd '  echo $PROGRAMS'
      e 'The removing of a dir should trigger an event, 
      called <b class="EVENT">on_remove_directory</b>, and 
      this will automatically invoke a few cleanup
      scripts.'
      br
      e 'This however, only works on Distributions specifically set up
      to deal with the RBT Scripts or if you use something like the
      Diamond Shell.'
    }
    p {
      e 'On most Linux Distributions you would find these things 
      under <b class="palevioletred">/usr/local</b> normally (if 
      you compiled on your own <b>without</b> a --prefix option).'
      br
      e 'Other people like to compile it into their home directory,
      like 
      <b class="palevioletred">$HOME/directory_name/program_version</b>.'
    }
    p {
      e 'Now, the problem with <b class="palevioletred">/usr/local</b> 
      is that it is an additional layer of complexity, with all 
      the disadvantages the FHS already has. Additionally,
      <b class="palevioletred">/usr/local</b> is typically exempt
      from the package manager of the Distribution in question -
      which means you are, once again, on your own.'
      br
      e 'The /usr hierarchy *must* be managed with a 
      package manager/system manager, because the files
      were already spread around.'
      br
      e 'The FHS is a rather messy solution which causes more problems
      than it solves.'
      br
      e 'Only with the help of package and system manager can users
      manage their FHS-based systems - without that, it would quickly
      become an unmaintainable chaos.'
      br
      e 'Personally, I do not encourage you to do use the FHS,
      but there is one clear advantage:'
      br
      marl2em '- If you use the FHS approach, you will have less 
      problems this way because it is the "standard" way to do
      things on Linux systems. The Gobolinux approach is a lot
      cleaner though.'
    }
    p('smaller marl2em'){
      e sg(:angry_smiley,'marr6px')+'It is rather tedious to persuade 
      other people in the Linux world that they should not rely on 
      inflexible hardcoded paths. Most individuals with an opinion are
      so <b>very</b> stubborn, dead-set on the "established" way to
      do something, unable to get their brains to think differently
      and consider other options as well.'
      br
      e 'It is especially annoying when you are confronted "upstream".
      A waste of time to argue.'
    }
    p('smaller marl5em'){
      e sg(:explanation)+'Some people are so infected by this FHS
      doctrine that they hate other ideas, such as self-contained
      application directories.'
      e 'In doing so they fail to see that this solution 
      is <span class="underline">a LOT</span> better and more 
      elegant in the long run than the ugliness of the FHS-imposed, 
      arbitrary rule-set.'
      br
      marl2em '(What is even worse, the LSB builds and <b>extends</b> 
      upon the FHS.)'
    }
    p('ind0px mart2em'){
      e 'Back on track though - lets say you like the FHS approach 
      (after all, Linux is or should be all about choice) and want 
      to compile htop into a directory like /usr.'
      e 'No problem!'; br
      e 'There are several ways to do this - one that I 
      like is to modify the specific .yaml file in question.'; br
      e 'Simply open up your local <yel>cookbook.yml</> version -
      or the individual .yml file in question, in case you
      follow the "one program per file" approach - like so:'
      br
      cmd '  bluefish /System/Settings/cookbook.yml &'
      cmd '  $EDITOR $COOKBOOK_DIRECTORY/ALL_COOKBOOKS/cookbook.yml &'
      br
      e '(Of course, use another editor like nano or vim 
      if you like. For simplicty, the rest of the instruction set 
      will refer to bluefish as the main graphical editor, just use
      the editor you prefer.)'
      br 
      e 'This file, <span class="yel">cookbook.yml</span>, is 
      quite large. In November 2011 I had about 2600 registered 
      programs there in all my cookbook files. Dont worry about 
      reading everything in those files though ;-) just jump straight 
      to the entry called "htop:" (by pressing CTRL+F then typing 
      htop:) and press enter.'
      br
      e 'You should now be at the entry for htop. Have a 
      brief glance to see what is there.'
      e '(At the time of writing this tutorial, htop could be 
      found in the file <yel>cookbook.yml</>.)','marl3em'
      br 
      e 'Look at the important option " prefix:"'
      marl1em 'You will see a f next to it, which simply means "false".'
      e 'As you may have guessed, a "t" would indicate a value of 
      "true". RBT always tries to allow shortcuts when it is possible,
      but you can be explicited as well if you wouldd like to.'
      br
      e 'For now dont think too much about the meaning of 
      traditional itself, just change the "f" letter to a "t".'
      e 'This has changed the "traditional" option to be true, 
      and true in this context roughly means "yes, we want to compile 
      in a FSH way", thus the prefix target would become /usr.'
      br
      e 'If this convention confuses you, you could use an explicit
      setting such as this too:'
      br
      cmd ' prefix: /usr'
      br
      e 'Though it is a bit longer to type.'
      br
      anm 'Please note that you can overrule this option on
      the commandline, via --prefix=, like so:'
      br
      cmd '  rbt htop --prefix=$J'
      cmd '  rbt htop --prefix=/usr/local'
      cmd '  rbt htop --prefix=$HOME'
      br
      anm 'Note that the last option there, the --prefix=$HOME,
      also is available via a short-cut flag:'
      br
      cmd '  rbt htop --into-home-dir'
      cmd '  rbt htop --home-dir'
      br
      anm 'And you can also compile into your home dir, in
      an AppDir variant, by using this:'
      br
      cmd '  rbt htop --appdir-into-home-dir'
      br
      e 'The above is equivalent to:'
      br
      cmd '  --prefix=$HOME/Programs/Htop/1.0.1'
      cmd '  --prefix=/root/Programs/Htop/1.0.1'
      br
      e 'You can also omit the <b>prefix:</b> option entirely, it will 
      <b>default</b> to prefix=/usr/ in that case. But you could also 
      put in /opt here or even /opt/PROGRAM_NAME - the last bit is 
      a "macro" which will simply be expanded to the real program
      name.'
      br
      e 'I myself tend to use this here a lot in the yaml files:'
      br
      cmd ' prefix: f'
      br
      e 'Which translates to "compile in a Gobolinux like fashion".
      In other words, a <b>standalone AppDir structure</b>.'
      br
      e 'Once you have done this change, save the file, and go to 
      your shell again.'
      br
      e 'Then type this in your shell (and press the enter key):'
      br
      cmd '  rbt htop'
      br
      e 'You will see that htop will be compiled, but this 
      time it will use that other prefix - the /usr prefix.'
      br
      e 'Which means, after (a successful) compilation, you will 
      find the htop binary at <b>/usr/bin/htop</b>.'
      br
      e '(Htop was created by Hisham who also started Gobolinux 
      together with AndrÃ©, by the way. Htop is a really cute small 
      program.)','marl3em smaller FI'
      e 'By changing a single letter in the <yel>cookbook.yml</> 
      we have instructed our compile tools to build to another 
      prefix. You can set other prefixes to this "traditional: value"
      setting, for example /pkg if you want to compile into the 
      directory /pkg and so on.'
      br
      anm 'You can also pass arbitrary -- configure options.'
      br
      e 'For instance, you can do this:'
      br
      cmd '  rbt utillinux ntrad --without-ncurses'
      br
      e 'This would attempt to compile the program called
      utillinux, use an AppDir prefix, and also pass
      in --without-ncurses as commandline option.'
    }
    p {
      e "Let's review the concepts learned so far, in a nutshell:"
      br
      lem sg(:red_square,'marr5px')+"Basically, all you had to do was
      to change one letter in the 'recipes collection', in order
      to compile it to another prefix."
      br
      e 'This is one of the <b class="ud orange">principles of the 
      RubyBuild Tools</b>. Things should be simple, clean and 
      as logical as possible - for the <b>end user</b>.'
      br
      e 'IF we need complexity for some reason, then we need to be able
      to manage it with ease and explain why we have done so. It would
      be not much fun to be buried under a load of complexity.'
      br
      e 'If you have a proposal for a simplification, voice it.'
      br
      e 'Make sure you thought about it for a bit before issuing 
      though, it makes our life easier if you are completely convinced 
      that this can be useful to have.'; br
      br
      e 'One final word about this "traditional:" setting for now:'
      br
      l2em 'You can of course overrule this setting on commandline 
      just as well.'
      br
      e 'So ultimately, you do not have to edit any of the various
      <yel>cookbook.yml</> files. The commandline must allow
      the user to overrule any setting, at runtime.'
      br
      e 'Or even try to ignore it completely and just provide a URL 
      to a tarball of some sorts, to download and compile something.'
      br
      e 'To show you how to overrule the setting in the cookbook files, 
      look at the following examples:'
      cmd '  rbt htop traditional'
      cmd '  rbt htop non_traditional'; br
      e 'Of course, "compile htop ntrad" works too as does 
      "compile htop t".'
      br
      anm 'If you try to pass both options (<i>non_traditional</i>
      and <i>traditional</i>) then we will display a warning, and
      favour only one of those options given, which right now is
      the last option given. In general, <ud>options that come last
      are allowed to overrule earlier options given</ud>. We
      however display a warning in this case because it does
      not seem to make a lot of sense.'
      br
      e 'Please note - if you do not have a cookbook.yml file yet or 
      if you want to generate a new one, you can do so by issuing:'
      br
      cmd '  rbt generate_new'
      br
      e 'This will try to generate a new cookbookfile into
      /Depot/Temp and if this does not work, it defaults to the
      current working directory.'
    }
    p {
      e 'Lets look back at <yel>cookbook.yml</> now and contemplate 
      on the size of it. It is indeed quite large... '
      e 'There may be some people who will say that having 
      <b>one big file</b> like <yel>cookbook.yml</> is a bad thing. '
      br
      e 'I agree partially, and this is the reason why you can use 
      as many .yml files as you want.'
      br
      e 'The RBT Scripts should allow you to freely build and install
      applications, and thus they should not restrict you.'
      br
      e 'You can use multiplie .yaml files or you could create and
      use standalone recipes: Enter The Recipes World!'
      br
      e 'More on Recipes later - stay tuned.'
    }
    # ========================================================================= #
    # === Validation
    # ========================================================================= #
    header_stylish_brown 'Validating the cookbook entries'
    p {
      e 'If you wish to validate the cookbook entries, you can do so via:'
      br
      cmd '  cookbook --validate-entries'
    }
    # ========================================================================= #
    # === Batch-update all registered gems
    # ========================================================================= #
    header_stylish_brown 'Batch-update all registered gems'
    p {
      e 'You can batch-update all registered gems via:'
      br
      cmd '  rbt --update-all-gems'
      br
      e 'Note that this will only work if you use the Cookbooks project.'
    }
    # ========================================================================= #
    # === Disabling the use of cookbook aliases
    # ========================================================================= #
    header_stylish_brown 'Disabling the use of cookbook aliases'
    p {
      e 'The cookbooks submodule, part of RBT, makes use of several aliases
      to program names. This is mostly a convenience feature.'
      br
      e 'Sometimes this may lead to unexpected or unwanted results,
      such as when you want to compile program **a**, but the
      cookbooks project renames the program to **b** instead.'
      br
      e 'In order to avoid this, a commandline-option exists for
      the RBT project.'
      br
      e 'Invoke it like so:'
      br
      cmd '  rbt sdlttf --do-not-use-cookbook-aliases'
      cmd '  rbt sdlttf --do-not-use-cookbook-alias'  
      br
      e 'This will specifically disable making use of cookbook
      aliases.'
    }
    # ========================================================================= #
    # === Registering errors during compilation/installation
    # ========================================================================= #
    header_stylish_brown 'Registering errors during compilation/installation'
    p {
      e 'Some problems and errors may happen during compilation or
      installation of programs.'
      br
      e 'These are handled internally by RBT::Compile, but they will
      also be registered in a module-method onto the RBT Namespace.'
      br
      e 'It is available via the following code:'
      br
      cmd '  RBT.error_message?'
      br
      e 'Note that this will be reset to nil, its deault value, whenever a
      new program is compiled.'
    }
    # ========================================================================= #
    # === Program Versions
    #
    # Program Version tag, versions tag. rf rbtcgi program_versions
    # Here we can set specific versions.
    # ========================================================================= #
    header_stylish_brown 'Program Versions'
    p {
      e 'You can overrule the version specified in the cookbook yaml 
      file from the commandline. To do this, look at the following two 
      examples:'
      br
      cmd '  rbt nano --version=2.1.9'
      cmd '  rbt nano --version=2.2.2'
      cmd '  rbt nano --version=2.4.3'
      cmd '  rbt gtk+ --version=2.24.29'
      br
      anm 'Note though that this has a chance to fail right now,
      i.e. if the remote mirror does not have this specific version,
      or if you do not have it available locally.'
      br
      e 'You can also use a slight shortcut:'
      br
      cmd '  rbt nano --v=2.4.3'
      br
      e 'And an even shorter variant is to just give the leading
      name, and have the RBT script just pick one that fits:'
      br
      cmd '  ry python --2'
      br
      e 'You can also use a more complete copy-paste version
      setting (like you copy, then paste the full name), 
      like this:'
      br
      cmd '  rbt nano --version=pango-1.24.2.tar.bz2'
      br
      e 'The .tar.bz2 part, and the part before that, will 
      simply be ignored.'; br
      anm 'Also note that you can retrieve a listing of all locally
      available versions.'
      br
      e 'Use the following to get at this information:'
      br
      cmd '  rbt gcc available_versions?'
      cmd '  rbt gcc ?'
      br
      e 'The result would be something like:'
      br
      result '================================================================================'
      result ' - (1) gcc-3.4.6'
      result ' - (2) gcc-4.0.4'
      result ' - (3) gcc-4.1.2'
      result ' - (4) gcc-4.2.4'
      result ' - (5) gcc-4.3.3'
      result ' - (6) gcc-4.4.2'
      result ' - (7) gcc-4.4.3'
      result ' - (8) gcc-4.5.0'
      result '================================================================================'
      br
      e 'You can then compile one of these version, by doing i.e. this here:'
      br
      cmd '  compile gcc v4'
      cmd '  compile gcc 4'
      e 'You can omit the v too. The v is just a mnemonic shortcut for 
      "version", but the position number alone suffices anyway:'
      br
      cmd '  rbt gcc 3'
      br
      e 'This also works with negative numbers, like:'
      br
      cmd '  rbt perl -3'
      cmd '  rbt perl v-3'
      br
      e 'The above would compile the Perl version that is the -3 most
      recent. In other words, we start to count from the current version,
      and step back 3 times from that. (This will only work if you
      have the perl versions available locally).'
      br
      e 'If you give a bogus entry like v-66126363 and that version
      could not be found, we will instead treat it as input that you
      meant to refer to the very last version anyway.'
      br
      e 'A positional index with a word works for the first and last 
      entry too. The convenient keywords for this are called "first" 
      and "last". So rather than type 1 or 8, you can do this:'
      br
      cmd '  rbt v first'
      cmd '  rbt v last'
      br
      e 'The last version can be useful.'
      br
      e 'Aliases to these do exist:'
      br
      cmd '  rbt pcre vlatest'
      cmd '  rbt pcre vnewest'
      cmd '  rbt pcre latest'
      cmd '  rbt pcre newest'
      br
      e 'You can also query for the current RBT Versions.'
      br
      e 'To do this, do either of the following:'
      br
      cmd '  rbt htop --version'
      cmd '  rbt VERSION?'
      cmd '  rbt --version?'
      br
      e 'Since as of December 2015, you can also compile a random
      version.'
      br
      e 'This requires of you to have the other versions available
      locally though.'
      br
      e 'Usage example:'
      br
      cmd '  rbt gcc --use-random-version'
    }
    p(id: 'to_bash'){
      e 'The scripts are written in ruby, but for several reasons 
      one might want to prefer using shell scripts instead.'
      br
      e 'You can generate the small shell scripts by issuing 
      something like:'
      br
      cmd '  rbt --to_bash'
      br
      e 'Or shorter:'
      br
      cmd ' ry --tobash'
      cmd ' ry tobash'
      br
      e 'A single file will be created as a result, it can normally
      be found under' 
      br
      bold '/System/Settings/Scripts/bash_programs_rc','yel marl1em'
    }
    p {
      e 'Now, this is pretty handy, but what if you do <b>NOT</b> 
      want to compile at all?'
      br
      e 'No problem. Use a package.'
      br
      e 'Example:'
      br
      cmd '  rbt python install_package'
      cmd '  rbt fox package'
      br
      e 'This will install python from a binary package (and 
      download it from a remote site if needed, i.e. if it can 
      not be found locally.)'
    }
    p {
      e 'There are many more options available to the scripts that handle 
      these things but for a starter, this should be it.'
      br
      e 'But what if you <b>dont</b> have this package on your computer?'; br
      e 'By default the RBT Scripts will assume that you 
      already downloaded this package and put it in some location, 
      like <b>/Depot/Packages</b>.'; br
      e 'But you can download '+sg(:pfeil9,'marr4px')+' 
      a package with the scripts too:'
      cmd '  rbt python download'
      e 'This would (try to) fetch the latest (source package) of python 
      and download it to your machine.'
      anm 'Note that this command may fail due to several 
      reason, such as that the remote server is not available or 
      that the location has moved to another website and so on.','marl2em'
      br
      e 'As of September 2014 we also added a configuration option to
      yaml files called:'
      br
      bold 'do_not_download'
    }
    # ======================================================================= #
    # === Currently compiling programs
    # ======================================================================= #
    p {
      e 'Currently compiling programs'
      br
      e 'You can query which program(s) are currently compiled by issuing:'
      br
      cmd '  rbt currently_compiled?'
    }
    # ======================================================================= #
    # === Compile hooks
    # ======================================================================= #
    p {
      e 'Compile hooks'
      br
      e 'Compile hooks are defined in the cookbooks gem, in the file
      <b>compile_hooks.yml</b>.'
      br
      e 'In that file you can hook the compilation of some programs
      to the compilation of other programs.'
      br
      e 'This is somewhat similar to the chained programs.'
      br
      e 'For example, if you compile ffmpeg, and have mpv installed,
      then a new compilation of ffmpeg will trigger the re-compilation
      of mpv. This will only be done if you have mpv installed
      at <b>$PROGRAMS/Mpv</b> though.'
    }
    # ======================================================================= #
    # === Batch-download all source archives
    # ======================================================================= #
    p {
      e 'You can batch-download all source archives, via the RBT project.'
      br
      e 'To do so, do issue either of the following commands:'
      br
      cmd '  rbt --batch-download-all-source-archives'
      br
      e 'You can also download only programs that have a specific tag:'
      br
      cmd '  download_source_archives --tags=plasma'
      br
      e 'This would download only source archives that have the
      plasma tag.'
    }
    # ======================================================================= #
    # === clang compiler (clang tag)
    # ======================================================================= #
    p {
      e 'You can use clang rather than gcc to compile something.'
      br
      e 'To enable this, do use any of the following variants:'
      br
      cmd '  rbt htop --use-clang'
      cmd '  rbt htop --clang'
      cmd '  rbt htop --do-use-clang'
    }
    # ======================================================================= #
    # === Porg tag
    # ======================================================================= #
    p {
      e 'Porg'
      br
      e 'You can use porg to install a program.'
      br
      e 'To query whether porg is in use, you can issue this command:'
      br
      cmd '  rbt --use_porg?'
      br
      e 'Porg will keep track of program files that are installed.'
      br
      e 'You can also disable porg on the commandline via:'
      br
      cmd '  rbt --disable-porg'
      cmd '  rbt --disable_porg'
      cmd '  rbt --disableporg'
      br
      e 'Or just directly edit the respective .yml file.'
    }
    # ======================================================================= #
    # === Strip tag
    # ======================================================================= #
    p {
      e 'Strip'
      br
      e 'You can use <b>strip</b> to reduce the size of the binaries.'
      br
      e 'This will be done via the configuration option called
      use_strip.'
      br
      e 'If this configuration option is set to true, then we will
      run "strip --strip-unneeded" by default.'
    }
    # ======================================================================= #
    # === Installed files
    # ======================================================================= #
    header 'Installed files'
    p {
      e 'You can ask RBT which files belong to a specific program.'
      br
      e 'Do this like so:'
      br
      cmd '  rbt htop --installed-files?'
      cmd '  rbt htop --installed-files'
    }
    # ======================================================================= #
    # === meson
    # ======================================================================= #
    header 'meson'
    p {
      e 'You can specifically enable the meson-build by issuing this:'
      br
      cmd '  ry glib --use-meson'
    }
    # ======================================================================= #
    # === M4 Macros
    # ======================================================================= #
    header 'M4 Macros'
    p {
      e 'We can symlink .m4 Macros into the directory
      <b>/usr/share/aclocal</b>. This obviously only makes sense if the
      program at hand was installed into an AppDir-like prefix, rather
      than straight into the /usr prefix hierarchy.'
    }
    # ======================================================================= #
    # === Information tag.
    # ======================================================================= #
    p {
      e 'The RBT project allows you to also generate some HTML pages,
      similar to how the LFS (<b>Linux from scratch</b>) project.'
      e 'The central ruby script that does that is called
      <b>central_information_agency.rb</b>.'
      br
      e 'In order to generate a bunch of HTML pages, do this:'
      br
      cmd '  cia lfs'
      br
      e 'or'
      br
      cmd '  cia html_pages'
    }
    # ======================================================================= #
    # === Binary and Library Checking.
    # ======================================================================= #
    p {
      e 'You can specify another source directory (base_dir) by doing
      something like this:'
      br
      cmd '  rbt mantis --source_dir=/Depot/j'
    }
    # ======================================================================= #
    # === GoboLinux SymlinkProgram
    # ======================================================================= #
    p {
      e 'On GoboLinux, you can use SymlinkProgram. On non-GoboLinux systems,
      SymlinkProgram is not available by default; the RBT suite contains 
      class RBT::SymlinkProgram, which behaves somewhat similar, but not
      quite.'
      br
      e 'It may be better to use SymlinkProgram on GoboLinux, and the
      following entry point allows you to use the GoboLinux SymlinkProgram
      variant rather than class RBT::SymlinkProgram:'
      br
      cmd '  rbt htop --use-gobolinux-symlinkprogram'
    }
    # ======================================================================= #
    # === Symlink all upcased programs to downcased variants
    # ======================================================================= #
    p {
      e 'If you ever have to, or want to, symlink all upcased program
      entries to downcased program variants (excluding the first character),
      then you can use this invocation line:'
      br
      cmd '  rbt --symlink-all-upcased-programs-to-downcased-variants'
      br
      e 'This will create a symlink, for example, at
      /Programs/Zlib for /Programs/ZLib.'
      br
      e 'The reason it was added was because GoboLinux uses semi-upcased
      directory names; and this makes things a bit more complicated 
      sometimes.'
    }
    # ======================================================================= #
    # === Batch Compiling / Chain compiling.
    # ======================================================================= #
    p {
      e 'You can batch-compile several programs in one go.'
      br
      e 'The syntax is:'
      br
      cmd '  rbt --batch-compile=mate'
      cmd '  rbt --batch-compile=ruby_addons'
      cmd '  rbt --batch-compile=xorg_protos'
      br
      e 'The first example will compile all of mate; the last example
      will compile all xorg-protos.'
      br
      e 'Note that for this to work, you first must have defined
      a compile-chain in the file cookbook_chained_programs.yml,
      which is part of the RBT project.'
      br
      e 'Alternatively, you can just specify the programs that you
      wish to compile, separated by a "," character, such as:'
      br
      cmd '  rbt --batch-compile=python,php,perl,ruby'
    }
    # ======================================================================= #
    # === UpdateEntry
    # ======================================================================= #
    header 'UpdateEntry'
    p {
      e 'You can use class RBT::UpdateEntry to update an entry in one of
      the available <b>cookbooks</b>.'
      br
      e 'This class accepts, as <b>parameter</b>, a remote URL - the URL to
      the program in question.'
      br
      e 'So for example, if you wish to change to another glib version,
      you could issue this command:'
      br
      cmd '  update_entry http://ftp.gnome.org/pub/gnome/sources/glib/2.53/glib-2.53.4.tar.xz'
      cmd '  ue http://ftp.gnome.org/pub/gnome/sources/glib/2.53/glib-2.53.4.tar.xz'
      cmd '  rbt --update-entry http://ftp.gnome.org/pub/gnome/sources/glib/2.53/glib-2.53.4.tar.xz'
      br
      e 'UpdateEntry will store the last downloaded program in a specific
      file. You can then make use of this from RBT, by doing something
      like this here:'
      br
      cmd '  rbt lastdownload'
    }
    # ======================================================================= #
    # === Binary and Library Checking.
    # ======================================================================= #
    header 'Binary and Library Checking'
    p {
      e 'You can also compile a program if you only know the name of a
      binary. For example, to compile the package "xine-ui", you could
      supply the name of one of its <span class="yel">binaries</span>, 
      like so:'
      br
      cmd '  rbt xine-check'
      br
      e 'xine-check is a binary of xine-ui. The above will actually 
      compile <b>xine-ui</b>.'
      br
      e 'Another example:'
      br
      cmd '  rbt basename'
      br
      e '<b class="yel">basename</b> belongs to the package 
      <b>coreutils</b>, and thus we will attempt to compile 
      coreutils.'
      br
      anm 'Note that you can change this in the config file, look
      for the option `check_for_binary_names`.'
      br
      e 'One last example - say, you know the binary name which is
      called `startx`. You attempt to compile it, and we will set
      it to the proper package name:'
      br
      cmd '  rbt startx'
      br
      e 'startx belongs to xinit and the above is the same as:'
      br
      cmd '  rbt xinit'
      br
      e 'And the same mechanism works for library names just as well
      (if they were registered in RegisteredLibaries.yml that is).'
      br
      e 'So for instance:'
      br
      cmd '  rbt libqui.so'
      br
      e 'The above would actually try to compile QT4 and would thus be 
      the same as if you would have entered:'
      br
      cmd '  rbt qt4'
      br
      e 'In the end, this is just a slight convenience to make your 
      life easier.'; br
      e 'The default behaviour for the scripts is to treat the given
      input as alias to a real program name, and if this alias is
      not found we check for binary names, then for library names.'; br
      e sg(:red_dot,'marr4px')+'Sometimes, this is not desired. 
      Consider that you input "ld". Now let us also assume that you 
      already have an entry called "ldmud" in your scripts. But "ld" 
      is also the name of the binary "ld" in the binutils package.'
      br
      e 'To force that we seek for the binary "ld" and not for 
      "ldmud" you could do this:'
      br
      cmd '  rbt ld treat_as_binary_name'
      br
      e 'Same works for library names as well:'
      br
      cmd '  rbt ld treat_as_library_name'
    }
    # ======================================================================= #
    # === Libtool tag.
    # ======================================================================= #
    header 'Libtool'
    p {
      e 'Libtool is a shell script. It makes use of .la files.'
      br
      e 'Since most programs come via .pc (pkgconfig) files these
      days, .la files are rarely needed.'
      br
      e 'If you wish to automatically get rid of .la files after
      compilation, then you can use this switch:'
      br
      cmd '  --remove-la-files'
    }
    # ======================================================================= #
    # === ldconfig tag.
    # ======================================================================= #
    header 'ldconfig'
    p {
      e 'Note that the current behaviour is to (silently) run ldconfig 
      after compilation has finished. This can be modified in the
      config file, see the option there called `run_ldconfig`.'
      br
      e 'You can also disable the running of ldconfig from the commandline
      for the program you wish to install.'
      br
      e 'In order to do this, do:'
      br
      cmd '  rbt htop no_ldconfig # Main command'
      cmd '  rbt htop noldconfig # Alias to the above ^^^'
      cmd '  rbt htop nold'
    }
    # ======================================================================= #
    # === Convert to FHS tag
    # ======================================================================= #
    header 'Convert to FHS'
    p {
      e 'If you ever want to convert a given package to FHS style, use this:'
      br
      cmd '  rbt php5 convert'
      cmd '  rbt php5 cpackage'
      cmd '  rbt php5 convert_package'
      cmd '  rbt nimrod cpa'
      br
      e 'This is, right now, using the files under /Programs - or more 
      accurately, it uses the setting specified in your configuration yaml
      file, the value of "programs_dir" - , and
      will then copy these into the /usr prefix.'
      br
      e 'To get a listing of available programs, you can do:'
      br
      cmd '  rbt --available_programs?'
      cmd '  rbt available_programs?'
      cmd '  rbt available_cookbooks?'
      br
      e 'This will iterate over all programs specified at programs_dir.'
    }
    # ======================================================================= # 
    # === Installing older versions. rf rbt Installing_older_versions
    #
    # Version tag. rf rbtcgi installing_older_versions
    # ======================================================================= #
    header 'Installing older versions'    
    p {
      e 'What if you want to install an older version of python?'
      br
      e 'We briefly addressed this elsewhere on this page already, but we
      will now go into more detail.'
      br
      lem sg(:info,'marr4px','drag_INFO_IMAGE','','INFO')+'
      The <b class="yel">cookbook.yml</b> normally 
      "supports" only the version you specified in 
      <b>"<lightblue>archive_name:</>"</b>'
      br
      e 'For example, if you jump to the "python:" entry in the cookbook.yml 
      file, you will see that its version is 2.5.1.'
      br
      e sg(:anmerkung)+'In later revisions python apps are gathered
      inside python_cookbook.yml instead of the general cookbook.yml 
      file.'; br
      e 'You can change this permanently by writing another name/version 
      there.'; br
      e 'Because editing of various cookbook.yml files can be a little annoying, 
      and because flexibility is a good thing, there exists another way to 
      specify a different version manually on the commandline:'
      br
      cmd '  compile python --version=2.4.1'
      br
      e 'This works with every version, even "newer" ones.'; br
      e 'The name of the version should however be part of the filename.'
      br
      e 'For more information about this, and an even <b>easier way</b>, 
      jump <a href="#program_versions" class="slateblue">here</a> to.'
    }
    # ======================================================================= # 
    # === Viewing LFS and BLFS URLs.
    # ======================================================================= #
    header 'Viewing LFS and BLFS URLs'
    p {
      e 'The <b>Linux From Scratch</b> (LFS) Project has instructions on how
      to compile programs from source.'
      br
      e 'These are often so useful that support for this was added into the
      Cookbooks project. Many cookbook yaml files can point towards the
      LFS/BLFS page, which will often contain additional useful information.'
      br
      e 'You can try to show this URL by doing this:'
      br
      cmd ' blfs curl'
      br
      e 'This would show the BLFS URL of curl.'
      br
      e 'Since as of July 2017, you can also directly paste the content
      of the remote BLFS webpage onto the terminal, if there is a remote
      BLFS page associated with a given program.'
      br
      e 'The way to do this is:'
      br
      cmd '  ry gcc --paste-blfs-page'
      br
      e 'If you want to show all programs that have a blfs entry, use the
      following command:'
      br
      cmd '  ry --show-programs-with-a-blfs-entry'
    }
    # ======================================================================= # 
    # === Viewing all packages
    # ======================================================================= #
    header 'Viewing all packages'
    p {
      e 'To '+sg(:lens)+'view a list of all packages on your system,
      do this:'
      br
      cmd '  rbt php packages?'
      cmd '  rbt python packages?'
      br
      e 'The result would be something like this:'
      br
      cmd '["/Depot/Packages/Python-2.5.tar.bz2"]'
    }
    # ======================================================================= # 
    # === Package up the RBT Scripts
    # ======================================================================= #
    header 'Package up the RBT Scripts'
    p {
      e 'You can package all the latest RBT Scripts like so:'
      br
      cmd '  rbt package_cookbooks'
      cmd '  rbt --package_cookbooks'
      br
      e ' This will create a directory in /Depot/Archives and package the
      RBT Scripts into that directory.'
    }
    # ======================================================================= # 
    # === Dump all .rb files.
    # ======================================================================= #
    header 'Dump all Recipe files'
    p {
      e 'To dump all recipe files (Gobolinux files, ebuild files, 
      .rb recipe files) do this:'
      br
      cmd '  rbt x dumpall'
    }
    # ======================================================================= # 
    # === Suppress Commandline
    # ======================================================================= #
    header 'Suppress Commandline Output'
    p {
      e 'To suppress configure spamming useless information to you, and to
      also suppress other things, do this:'
      br
      cmd '  rbt htop v2 suppress-output'
      cmd '  rbt htop v2 soutput'
      cmd '  rbt htop v2 sout'
      cmd '  rbt htop sout'
      br
      e 'You can also make all output completely disappear:'
      br
      cmd '  rbt htop --be-completely-quiet'
      br
      e 'This will suppress all output.'
    }
    # ======================================================================= # 
    # === Generate completion (completion tag).
    # ======================================================================= #
    header 'Generate and use shell completion'
    p {
      e 'You can generate a bash shell completion file from within 
      RBT.'
      br
      e 'In order to do so, try this:'
      br
      cmd '  rbt x generate_shell_completion'
      br
      e 'This will write a simple shell script which you can source if
      you want to use ry <TAB> to compile programs.'
      anm 'This also works for Zsh by the way.'
      br
      e 'The file in question is called <b>generate_shell_completion.rb</b>.'
      br
      e 'If called standalone from the commandline directly, it will 
      generate .sh shell files for both Bash and Zsh.'
    }
    # ======================================================================= # 
    # === Find out local location of tarballs
    # ======================================================================= #
    header 'Find out local location of tarballs'
    p {
      e 'To find out the local location of a tarball use this:'
      br
      cmd '  rbt gcc local_url'
      cmd '  rbt gcc lurl'
      br
      e 'The result will be something like this:'
      br
      cmd '  => The local file for gcc can be found at:'
      cmd '  /Users/x/SRC/GCC/gcc-4.6.1.tar.bz2'
    }
    # ======================================================================= # 
    # === Build inside a subdirectory (build directory).
    # ======================================================================= #
    header 'Build inside a subdir'
    p {
      e 'Sometimes you dont want to build in the current directory, but
      may want to use a subdirectory instead. (For example, glibc requires
      you to build inside a subdirectory).'
      br
      e 'To do this from the commandline, simply do one of the following
      commands:'
      br
      cmd '  rbt php --subdir=PHP_BUILD_DIRECTORY'
      br
      e 'The above would build in the directory called "PHP_BUILD_DIRECTORY".
      If this directory does not exist (which is the case usually), then 
      the RBT scripts will create that directory.'
      br
      e 'The default name for the subdirectory is called
      <b class="red">BUILD/</b> and you can do default to
      this directory:'
      br
      cmd '  rbt pango -b'
      br
      e 'This will create a <b class="red">BUILD/</b> directory and change 
      the working directory into it, while compiling.'
      br
      e 'In practise, defaulting to the BUILD/ directory is the easiest
      solution if you want to compile in your own directory.'
      br
      anm 'Please also note that you can specify build directories
      in the respective cookbook file as well - look at the description
      of the variables called `<span class="yel">use_build_directory</span>`, 
      and `<span class="yel">use_this_build_directory</span>`.'
      br
      e 'Alternative syntax exists as well,of course. If you want to
      use the default build directory for a project you compile, you
      can pass the option <b>usebuilddir</b> like so.'
      br
      cmd '  rbt automake --use-build-dir'
      cmd '  rbt automake usebuilddir'
      cmd '  rbt htop usebuilddir'
      cmd '  rbt php use_build_dir'
      br
      e 'You can also specify a specific build directory to use.'
      br
      e 'One way that this works is via this:'
      br
      cmd '  rbt htop --use-this-build-dir=/Depot/j'
      br
      anm 'The RBT scripts will append a trailing / if it is a
      directory and if it does not yet have a trailing / character
      presently, so the above command is the very same as passing
      in <b>/Depot/j/</b>.'
      br
      e 'If you want to use a build directory, but do not care
      about the name, you can also use a random name, like 
      so:'
      br
      cmd '  rbt php --use-random-build-directory'
    }
    # ======================================================================= #
    # === Register all programs. Register tag.
    # ======================================================================= #
    header 'Register all programs'
    p {
      e 'It may be that you want to register all programs in the yaml
      Database. To do this, run this command:'
      br
      cmd '  rbt --register_all_programs'
      cmd '  rbt --rap'
    }
    # ======================================================================= #
    # === Makefiles
    # ======================================================================= #
    header 'Makefiles'
    p {
      e 'Most open source projects on Linux that require you to compile
      C or C++ code, come with a file called a Makefile.'
      br
      e 'The command "make" can be used to interpret the rules stored
      in said file.'
      br
      e 'Sometimes no such file is provided though and instead, some
      other variant is used. For instance, a generic file called
      <b>Makefile.linux</b>.'
      br
      e 'This file is a template file that can be used to generate a
      Makefile file.'
      br
      e 'The common syntax for this would be:'
      br
      cmd '  make -f Makefile.linux'
      br
      e 'However had, the RBT scripts will automatically check whether
      such a file exists, in the event that no Makefile exists. If
      this i the case, then the make command will be modified to
      first created this Makefile.'
    }
    # ======================================================================= #
    # === Find out how many programs are registered
    # ======================================================================= #
    header 'Find out how many programs are registered'
    p {
      e 'To find out how many programs are registered, do one of this:'
      br
      cmd '  rbt --n_programs?'
      cmd '  rbt n_registered?'
      cmd '  rbt registered?'
      br
      e 'It will then feedback something such as:'
      br
      anm 'RBT::ReportTheRegisteredPrograms: 2619 programs are registered
      in our cookbooks as of 30.03.2014.'
      anm 'RBT::ReportTheRegisteredPrograms: 2832 programs are registered
      in our cookbooks as of 18.02.2016.'
      anm 'RBT::ReportTheRegisteredPrograms: 2862 programs are registered
      in our cookbooks as of 18.08.2016.'
      anm 'RBT::ReportTheRegisteredPrograms: 3185 programs are registered
      in the cookbooks as of 06.10.2017.'
      br
      e 'As you can see, more and more programs are (slowly) added and
      registered over the various years.'
    }
    # ======================================================================= #
    # === Extract Archives
    # ======================================================================= #
    header 'Extracting Archives'
    p {
      e 'You can extract a tarball and finish the program. In order to 
      do this, try:'
      br
      cmd '  rbt php eonly'
      br
      e 'This would extract the latest PHP tarball and then exit the program.'
      br
      e 'Also note something else, related to <b>Extracting</b>. You can use
      the argument --extract_to (or its alias --homedir or --temp_dir) to
      specify another <b>extract target</b> to use. You can even use
      shell environment variables here.'
      br
      e 'Usage Examples:'
      br
      cmd '  rbt htop --extract_to=pwd'
      cmd '  rbt htop --extract-to=$J # This makes use of the shell
      environment $J'
      cmd '  rbt htop --temp_dir=/tmp'
      cmd '  rbt htop --homedir=pwd'
    }
    # ======================================================================= #
    # === Removing all directories in the temp directory
    # ======================================================================= #
    header 'Removing all directories in the temp directory'
    p {
      e 'If you ever need to or want to remove all directories in the
      temporary directory (by default: /Depot/Temp/), then the following
      option may be useful:'
      br
      cmd '  ry --remove-all-temp-directories'
    }
    # ======================================================================= #
    # === View urls of certain programs
    # ======================================================================= #
    header 'View urls of certain programs'
    p {
      e 'To view the URLs of certain programs, you can do this:'
      br
      cmd '  rinfo'
      cmd '  rinfo video'
      cmd '  rbt libxrandr url'
      cmd '  url libxrandr'
      br
      e 'Also note that you can set the program name by using 
      USE_URL - we will then use the URL from url1'
      br
      e 'The USE_URL is automatically assumed if you only provide a
      remote URL.'
    }
    # ======================================================================= #
    # Use first URL
    # ======================================================================= #
    header 'Use the first URL instead of program name:'
    p {
      e 'You can overrule the program name and specifically force 
      the usage of the first URL entry.'
      br
      e 'You do this like so:'
      br
      cmd '  rbt ffmpeg use_url'
      cmd '  rbt libgtkhtml use_url'
      br
      e 'Do note though that for this to work, it requires a proper 
      url1 entry in the cookbook file.'
    }
    # ======================================================================= #
    # === Output all URLs.
    # ======================================================================= #
    header 'Use the first URL instead of program name:'
    p {
      e 'You can output all URLs like so:'
      br
      cmd '  rbt --output_all_urls'
      cmd '  rbt all_urls?'
      br
      anm 'Note that this will also save this result in a file.'
    }
    # ======================================================================= #
    # === Use a specific compile version
    # ======================================================================= #
    header 'Use a specific compile version'
    p {
      e 'You can overrule the program_version? that will be used for
      app-dir like compilation like this:'
      br
      cmd '  rbt glib --compile-version=2.16.33_ROEBE'
      br
      e 'The above command will try to compile glib, but will use a
      specific version that you defined here. Typically it should
      be only numbers, but you can also use non-numbers.'
      br
      e 'The above would translate into this prefix:'
      br
      e '/Programs/Glib/2.16.33_ROEBE'
    }
    # ======================================================================= #
    # === md5sum tag
    # ======================================================================= #
    header 'md5sum'
    p {
      e 'Verify the md5sum:'
      cmd '  rbt gcc4 md5'
      cmd '  rbt libogg md5'
      br
      anm 'Note that you can disable the md5sum calculation, by
      using the switch --no-md5sum:'
      br
      cmd '  rbt htop --no-md5sum'
    }
    # ======================================================================= #
    # === check locally tag
    # ======================================================================= #
    header 'Check locally'
    p {
      e 'You can check for the latest packages locally via'
      br
      cmd '  rbt htop latest'
    }
    # ======================================================================= #
    # === Feedback tag
    # ======================================================================= #
    header 'Feedback about a program'
    p {
      e 'To get feedback about i.e. php, do:'
      br
      cmd '  rbt php feedback_all'
      cmd '  rbt ruby fall'
      br
      e 'You can also make use of SanitizeCookbookDataset.'
      br
      cmd '  rbt php --scookie'
    }    
    # ======================================================================= #
    # Using multiple yaml files
    # ======================================================================= #
    header 'Use multiple yaml files','multiple_yaml_files'
    p {
      e 'You can use more than one cookbook yaml file. Just specify this in
      the config file, at the entry <b>"cookbook_directory"</b>.'
      br
      e 'The first entry should always point to your most important
      cookbook file though, the one you use most of the time, or is 
      the largest cookbook file.'
      e 'You can of course just use one big cookbook file.'
      br
      e 'Additionally, you can use a "one-file-per-program" approach.'
      br
      e 'I myself transitioned into this approach several years ago. It
      simply was easier to update information stored in individual 
      directories.'
      br
      e 'You can use a special "macro", which will load all files of a
      given directory.'
      br
      e 'Example:'
      br
      cmd ' - /some/directory/YAML/'
      br
      e 'The above would load all files inside that directory.'
      br
      anm 'Yes, this was changed some time ago, I think in 2009. RBT will
      automatically try to load all .yml files from that directory.'
    }
    # ======================================================================= #
    # === Query whether we have a binary file.
    # ======================================================================= #
    header 'Querying whether we have a binary file','query_binary_file'
    p {
      e 'If you want to find out if we have a registered program,
      whose binary name is "file", do this:'
      br
      cmd '  rbt file binary?'
      br
      e 'In other words, you can find out if we have such a binary name.'
      br
      e 'Different programs will have different binary names, and sometimes
      it is important to find out which binary belongs to what package.'
    }
    # ======================================================================= #
    # === Query Inclusions
    # ======================================================================= #
    header 'Query Inclusions','inclusions'
    p {
      e 'To find out whether all your files are included in the cookbook
      file, do this:'
      br
      cmd '  rbt htop inclusions?'
      br
      e 'It will report which files were not included.'
      br
      e 'This is mostly helpful when you want to cleanup unneeded 
      packages.'
    }
    # ======================================================================= #
    # === Static Compilation.
    # ======================================================================= #
    header 'Static Compilation'
    p {
      e 'You can compile an application statically by passing <b>static</b>
      to <b>compile.rb</b>.'
      br
      e 'Example:'
      br
      cmd '  rbt make static'
      br
      anm 'Note that this will not work for every program.'
      br
      e 'In the event that you, on the commandline, try to enable static
      compilation, and the yaml file specifies otherwise, such as via a
      configure switch like --disable-static, we will ignore the latter
      option and allow the user to specifically overrule this setting.
      RBT always assumes that the user knows what he is doing.'
      br
      e 'Note that you can also zero these flags via a commandlie
      option:'
      br
      cmd '  rbt strace --no-compile-time-flags'
      cmd '  rbt strace --plain'
      br
      e 'This option means that all CFLAGS etc... will be ignored
      altogether; that is, they will set to an empty again.'
    }
    # ======================================================================= #
    # === Searching for something
    # ======================================================================= #
    header 'Searching for something','searching'
    p {
      e 'To search for something, do this:'
      br
      cmd '  rbt ruby search'
      br
      e 'To find a specific list of packages, do this:'
      br
      cmd '  rbt pyt*'
      br
      e 'The * will tell us that we intend to find all packages
      starting with "pyt".'
      br
      e 'This can work with single letters too, that is - if you wish
      to discover all programs that are registered in the Cookbooks
      project starting with a, you can do:'
      br
      cmd '  rbt e*'
      br
      e 'The "*" is the important token here that will trigger this
      functionality.'
      br
      anm 'Be careful though if you use a shell like Bash or 
      Zsh - the * can be expanded by the shell.'
    }
    # ======================================================================= #
    # === Prefix Tag
    # ======================================================================= #
    header 'Using another prefix','prefix'
    p {
      e 'You can specify a different prefix to use when compiling a 
      program.'
      br
      e 'To do this, do:'
      br
      cmd '  rbt gimp --prefix=$J'
      cmd '  rbt gimp --prefix=/opt'
      br
      e 'Some nifty shortcuts exist. For example, to compile gimp
      into a standalone directory, into /Programs/Gimp/LatestVersion,
      you can do:'
      br
      cmd '  rbt gimp ntrad'
      cmd '  rbt gimp non_traditional'
      br
      anm 'Please also note that you can use the Option <b>noprefix</b> to
      avoid using <b>any</b> specific prefix. (This is the same as passing
      --prefix=/usr/local though on GNU systems.)'
      br
      e 'If you have the need to find out the prefix that will be used,
      do this:'
      br
      cmd '  rbt htop prefix?'
      br
      e 'Do note that you can also use "compound-prefixes". These are
      prefixes such as /Programs/Xfce/4.12, where you may want to put
      all xfce-related programs into.'
      br
      e 'Syntax is like this:'
      br
      cmd '  ry xfce4panel --prefix=:xfce'
      br
      e 'In order for this to work, an environment variable called
      XFCE_PREFIX may have to exist. If such an environment variable
      does not exist, then the program_version is guessed. I recommned
      to set such a prefix variant.'
      br
      e 'The leading : is important here - do not forget it.'
    }
    # ======================================================================= #
    # === Specify another program to compile.
    # ======================================================================= #
    header 'Specify another program to compile','another_program_to_compile'
    p {
      e 'You can specify another program to compile.'
      br
      e 'For this there are currently two ways possible:'
      br
      cmd 'ry x --this_file=/path/to/mousetweaks-3.1.1.tar.bz2'
      cmd '$RBT/Compile.rb /path/to/mousetweaks-3.1.1.tar.bz2'
    }
    # ======================================================================= # 
    # === Dependencies
    # ======================================================================= #
    header 'Handling Dependencies'
    p {
      e 'To get with dependencies do:'
      cmd '  rbt php with-deps'
      cmd '  rbt gcc3 wdeps'
      cmd '  rbt feh wdeps'
      e 'Dependencies can be specified as <b>ranged</b>, which 
      means that they will be valid only under some conditions.'
      br
      e 'For example, consider the following entry which you can
      find in some entries of the cookbook:'
      br
      cmd '  glib2 >= 2.17.6'
      br
      e 'This entry means that the program requires glib2
      of at least version 2.17.6 or higher.'; br
      e 'The Ruby Build Tools will notify the user if
      the dependency does not match. The default compilation mode
      will be to interrupt compilation when a dependency is not 
      met.' 
    }
    # ======================================================================= #
    # === Merging Cookbook Files.
    # ======================================================================= #
    header 'Merge Cookbook Files'
    p {
      e 'If you do have multiple cookbookfiles (or if you want to merge
      your existing cookbooks into one new file),
      you can do this:'
      br
      cmd '  rbt php merge'
      br
      e 'By default this command will merge into the 
      file DEPOT_DIR+merged.yml'
    }
    # ======================================================================= #
    # === Debug tag
    # ======================================================================= #
    header 'Debug'
    p {
      e 'If you want to debug the RBT Scripts, you can set the 
      <b class="orange">@debug</b> value on the fly via:'
      cmd '  rbt x debug'
      e 'This will toggle the debug value simply. Set this to true 
      if you want to debug. If set to true then we will feedback extra
      information.'
      e 'You can also query the debug value by doing this:'
      cmd '  rbt x debug?'
    }
    # ======================================================================= #
    # === Common operations
    # ======================================================================= #
    header 'Applying a common operation'
    p {
      e 'You can apply a common operation to multiple programs at the same 
      time. First, let us assume that you want to compile all versions of GTK:'
      cmd '  rbt gtk all_versions'
      e 'This would compile all the versions of GTK you have downloaded.'
    }
    # ======================================================================= #
    # === Using Stow
    # ======================================================================= #
    header 'Stow'
    p {
      e 'If you want to use stow, simply set the configuration option
      for Stow to true (in the <b>configuration file</b>)'
      br
      e 'You can query whether you have stow enabled, by doing:'
      br
      cmd '  rbt --use-stow?'
    }
    # ======================================================================= #
    # === Included program
    # ======================================================================= #
    header 'Included program'
    p {
      e 'You can find out whether a specific program is included or not
      by doing this:'
      cmd '  rbt htop include?'
    }
    # ======================================================================= #
    # === whereis program
    # ======================================================================= #
    header 'Whereis'
    p {
      e 'If you forgot where you defined your program entry, you can
      use <b>SourceDefinition.rb</b> to locate it. This is aliased to 
      the action where_is.'
      br
      e 'So, try this:'
      br
      cmd '  wis gcrypt'
      br
      e 'This will tell you where gcrypt is installed.'
    }
    # ======================================================================= #
    # === Symlink headers
    # ======================================================================= #
    header 'Symlink headers'
    p {
      e 'If a cookbook file has this entry:'
      lem '  symlink_headers: t'
      e 'Then this means that the RBT scripts will attempt to symlink
      all entries under the respective include/ directory into the
      FHS /usr/include/ hierarchy.'
      br
      e 'Why is this useful? Sometimes you may wish to have a hybrid
      system, that is, with some AppDirs under /Programs, and some
      non-AppDirs installed at /usr.'
      br
      e 'You can also invoke this on the commandline by issuing 
      --symlink-headers, like this:'
      br
      cmd '  ry htop ntrad --symlink-headers'
    }
    # ======================================================================= #
    # === Symlinking all programs
    # ======================================================================= #
    header 'Symlinking all programs'
    p {
      e 'If you want to (re)symlink all programs in the main directory
      called <b>/Programs/</b>, just do this  any of these:'
      br
      cmd '  rbt x symlinkall'
      cmd '  rbt x traverse'
      cmd '  rbt --resymlink'
      cmd '  rbt --symlink_every_program'
      br
      e 'The last option may quite possibly be the easiest one to remember.'
      br
      anm 'A note about <b>symlinks</b>. The GoboLinux way is to keep the Version
      as a Symlink, under <b>/Programs/Name/Version</b>. RBT also uses this
      scheme when compiling in a gobolinux-like (== App)way.'
      br
      e 'Say that you have successfully compiled htop version 2.0.2. Then,
      a symlink will also be set, called <b>Current</b>. This one could
      then be found at:'
      br
      lem '/Programs/Htop/Current'
      br
      e 'Which would point at:'
      br
      lem '/Programs/Htop/2.0.2'
    }
    # ======================================================================= #
    # === Sleeping.
    # ======================================================================= #
    header 'Glib-schemata'
    p {
      e 'Some programs, such as evince, require a .xml file that is
      installed into a certain directory.'
      e
      e 'You can enable symlink-action of that .xml file by issuing the
      following command:'
      br
      cmd '  rbt ntrad --use-glib-scheme'
    }
    # ======================================================================= #
    # === Sleeping.
    # ======================================================================= #
    header 'Sleep before continuing'
    p {
      e 'You can use a <b>delay</b> before compiling (or doing another
      action).'
      br
      e 'This works like so:'
      br
      cmd '  rbt htop sleep5'
      cmd '  rbt htop sleep55'
      br
      e 'That is technically the same as doing'
      br
      cmd 'sleep 5; ry foo'
      e 'or'
      cmd 'sleep 55; ry foo'
      br
      e 'This would mean that you would want to wait for 5 or 55 seconds
      before starting your action.'
    }
    # ======================================================================= #
    # === Downloading source files
    # ======================================================================= #
    header 'Downloading Source Files'
    p {
      img :wings, 'FLR', 'drag_wings'
      e 'As explained elsewhere in this document, you can download the source
      code of different programs via RBT. So for example, say that you 
      wish to download the sourcecode of python from its official homepage,
      then you can do this:'
      br
      cmd '  rbt python download'
      br
      e 'This would attempt to download python, as specified per the
      <b>python.yml</b> file.'
      br
      e 'An alternative syntax (=<i>shortcuts</i>) exists as well, e. g.:'
      br
      cmd '  rbt python --downloads'
      cmd '  rbt python down'
      cmd '  rbt python wget'
      cmd '  rbt python w'
      br
      e 'It is up to your personal preference what you wish to use
      here.'; br
      e 'I prefer to use "wget", but "download" is probably a better, 
      more general description of the process in question.'
      br
      e 'Please also note that you can specify different versions here.'
      br
      e 'This can be done like in this way:'
      br
      cmd '  rbt kernel --version=2.6.30.1 download'
      br
      e 'This would attempt to download the kernel version 2.6.30.1'
      br
      e 'Note though that this has a chance for failure, and thus might
      fail, e.g. if the ftp site changed, so it is no foolproof
      mechanism.'
      br
      e 'Also note that there is a configuration option which allows
      you to automatically attempt to download a missing file.'
    }
    # ======================================================================= #
    # === svn tag.
    # ======================================================================= #
    header 'Subversion'
    p {
      e 'You can try to download SVN sources, by issuing this command:'
      br
      cmd 'ry dosbox svn'
      br
      anm 'Note that this depends on the setting <b class="yel">svn_url</b>
      in the respective cookbook file. The <b>svn_url</b> setting must be 
      specified, as otherwise RBT can not download the code via svn.'
    }
    # ======================================================================= # 
    # === Chain-Compiling
    # ======================================================================= #
    header 'Compiling several programs in one go (Chain-Compiling)'
    p {
      e 'Compiling several programs one-after-the-other is what the RBT
      tools will refer to as <i>chain-compiling</i>.'
      br
      e 'It is handled by the ruby script called chain_compile.rb,
      <b class="crimson">class RBT::ChainCompile</b>. The code
      can be found within the file called
      $RBT/utility_scripts/chain_compile.rb'
      br
      e 'This script allows you to compile several programs in one go, thus
      making your life easier if you need to compile several programs
      quickly.'
      br
      e 'There are basically <b>two ways</b> to trigger this.'
      br
      lem 'The first one uses Installer.rb, i.e. the file we aliased to
      `compile` earlier on.'
      br
      e 'The syntax for this is:'
      br
      cmd '  rbt sdl wrap # This is no longer working. I removed this feature 
      for now. If you need this feature, use "chained sdl" instead.'
      br
      cmd '  rbt sdl chained'
      br
      e 'Because the above idiom may be commonly used, you may omit the 
      last argument if you prepend a <b class="yel">@</b> to the name.'
      br
      e 'For example:'
      br
      cmd '  rbt @sdl'
      br
      e 'In this regard, @ acts as a shortcut for wrap/chained mode.'
      br
      e 'However, there were a few design problems, and for now this is
      disabled - use "chained sdl" instead. One day I may reenable
      this feature.'
      br
      e 'The other way uses chained directly. Example:'
      br
      cmd '  chained sdl'
      br
      e 'Personally I prefer the latter version <cyan>"chained sdl"</>, 
      mostly because I feel it is cleaner to use another command 
      for compiling multiple programs in one go.'
      anm 'In this example <b>chained</b> is aliased to 
      <b>ChainedCompiling.rb</b>.','marl2em'
      br
      e 'In later versions I switched to "chained" command instead
      of chained.'
      br
      e 'Examples with some modular xorg chains follow:'
      br
      cmd '  chained xorg_protos'
      cmd '  chained xorg_apps'
      br
      e 'The above commands would compile several xorg related
       programs.'
      br
      e 'This is also doable from within rbt itself, as of
      June 2017. Issue this command:'
      br
      cmd '  rbt protos'
      br
      e 'To find out which chain is used, simply look 
      at the <b>cookbook_chained_programs.yml</b> file.'
      br
      e 'If you only want to compile a single program from within
      that chain for some reason, you can use specific positions.'
      br
      e 'For example:'
      br
      cmd '  chained xorg_apps 3'
      br
      e 'would compile the third program of xorg_apps. 
      (At the time of this writing, this program would be 
      <b>beforelight</b>)'
      br
      e 'The default is also to report the size of the tarballs used in
      this chained compilation process - this can be altered via the
      config file. (At March 2009 this was not a very sophisticated
      solution - it needs to be heavily improved.)'
      br
      e 'The info for compiling several programs at once is written 
      (and can thus be changed) inside 
      <span class="lightblue marl2em">cookbook_chained_programs.yml</span>'
      cmd '  cat $COOKBOOK_DIRECTORY/cookbook_chained_programs.yml'
    }
    # ======================================================================= #
    # === Headers only. Headers tag.
    # ======================================================================= #
    header 'Copy only the headers'
    div {
      e 'If you want to copy only the headers of a program, do this:'
      cmd '  rbt gmp only_headers'
    }
    # ======================================================================= #
    # === Do not show program names.
    # ======================================================================= #
    header 'Do not show programs names'
    div {
      e 'By default, the RBT Scripts will show the name of the Ruby 
      script that is running an execution.'
      e 'You can disable this by doing:'
      cmd '  rbt htop dont_show_names'
      cmd '  rbt htop dontshownames'
    }
    # ======================================================================= #
    # === SCRIPTS AND THE STRUCTURE
    # ======================================================================= #
    header 'The Scripts and their (Infra)Structure'
    div {
      e 'The scripts are written in '+string_link(:remote_ruby,
      sg(:ruby))+'<b class="red">ruby</b>, and although I 
      have tried to keep them rather simple, at times they can be a 
      bit complex (and some code parts are quite dated, and thus 
      may not be easy to understand - not even for myself.)'
      e 'I do however often reconsider design decisions, and start 
      rewrites every now and then to clean up the existing 
      infrastructure.'; br
      e 'If you want to explore these scripts, you should first look at '
      e '  Query.rb','marl2em grey'
      e 'This file consists of one class, which has 
      only one basic job:'
      br
      marl2em '  Load the data from the cookbook.yml file by using
      DatabaseLoader.rb and answer queries the user gives us.'
      br
      e '<b>DatabaseLoader.rb</b> will load the yaml dataset, 
      and then you can actually use Query.rb to query whether 
      a package is included or not.'; br
      e 'That is basically all these two files do.'
      br
      e 'DatabaseLoader.rb is much larger because it ensures that 
      the dataset is also correct.'
      br
      e 'Now, with this information we come to the most important
      script of the Ruby Build Tools - the <b>Installer.rb</b>'
      e 'This is a bit complex. What you must keep in 
      mind is that this file tries to fullfil many different 
      user wishes at once:'
      e 'A user may want to install, uninstall, compile, copy, 
      get the URL of a program, create .deb packages, create 
      dependency graphs, create shell scripts, ebuilds, 
      Gobolinux recipes, md5chksum it and and and.'
      e 'Now add all these tasks together in one program, and 
      the result will be quite large (though the code should 
      be rather straight-forward). Maybe I will split it up
      in several new files, but for now this approach must 
      suffice.'; br
      e 'There are helper files available too.'; br
      e 'The main purpose of Installer.rb is to be the 
      <b>general wrapper</b> around these various tasks.'
      e 'Basically, whatever you want to do, you should be 
      able to do so via Installer.rb.'
      e 'It is a basic duty of this script to invoke other 
      scripts as needed as well, acting as THE general 
      wrapper script'; br
      e 'We now move to a more detailed section of how 
      Installer.rb actually works.'
      # ===================================================== #
      # compile.rb
      # ===================================================== #
      h4 'compile.rb'
      e 'If you pass arguments to class Compile, then all "-" will be removed.'
      e 'It makes it easier to label packages this way.'
      e 'Installer.rb provides some logging capabilities:'; br
      e 'The <b>general log file</b> will usually be 
      stored here:'
      link '/InstalledSourceFiles.html',
           '/InstalledSourceFiles.html',1,
           'marl1em slateblue'
      e 'Additionally, a file called '
      link '/System/Information/LastCompiledProgram.html',
      '/System/Information/LastCompiledProgram.html',1,
      'marl1em slateblue'
      e 'will be created.'; br
      e 'Installer.rb accepts the program name as command:'
      br
      e 'Typical Invocation examples are like so:'
      br
      cmd '  rbt ebml'
      cmd '  rbt libxml'
      cmd '  rbt libogg'
      br
      anm 'If you omit the name of a program, the scripts
      will try to automatically guess a proper name using 
      the script <b>GuessName</b>. It is however recommended 
      to set the name manually, rather than guessing.'
      e 'Also note, since December 2009 <b>GuessName</b> has been mildly
      deprecated. In Jul 2011 GuessName has been removed. The macro 
      USE_URL (or shorter, URL) will be auto-used in case you forgot 
      to set the program name.'
      br
      e 'To install in Gobolinux style (via the commandline, which will 
      overrule any settings in the respective cookbook at hand), you
      can do something like this:'
      br
      cmd '  rbt python gobo'
      cmd '  rbt gcc gobostyle'
      cmd '  rbt sed gstyle'
      cmd '  rbt gcc nt'
      br
      e 'I recommend you to try out gobo, it opens eyes. ;-)'
      br
      e 'In the above first example, Python would be compiled 
      into this directory by default:'
      cmd '  /Programs/Python/VersionNummer'
      e 'In effect, this will overrule any setting defined 
      in the YAML file. The reason for this behaviour is that the 
      user specifically wanted this behaviour.'; br
      e 'Whatever the user wants to use, is the way-to-go for 
      us.','ud marl2em yel bigger'
      br
      e 'If you want to skip extracting (i.e. because you 
      already have it extracted) then do this:'
      br
      cmd '  rbt php skip_extracting'
      br
      e 'or more formally:'
      br
      cmd '  rbt gimp --skipextract'
      e 'or for the lazy guys (includes me)'
      br
      cmd '  rbt php se'
      br
      e 'You can also skip running any configure scripts, by doing this:'
      br
      cmd '  rbt php skip_configure'
      cmd '  rbt php sc'
      br
      e 'And you can combine this too, like:'
      br
      cmd '  rbt php skip_configure skip_extract'
      br
      e 'If you want to extract the package, and also run the configure
      script, but do not want to start compiling via make and make install
      yet, do this:'
      br
      cmd '  rbt binutils dc'
      br
      e 'dc means "dont compile", and this works too:'
      br
      cmd '  rbt binutils dont_compile'
      br
      e 'Please note that the last compiled programm will be stored
      in a yaml file. This will then allow you to easier acces the
      last compiled program and a pointer to it will be available
      if you use :last or @last as commandline argument.'
      br
      e 'For example, you can do this:'
      br
      cmd '  rbt :last url'
      cmd '  rbt @last url'
      br
      e 'This allows you to recompile the program quickly.'
      br
      cmd '  rbt @l'
      br
      e 'Yes, @l is an alias to @last.'
      br
      e 'Also note that we will store the configure options that are 
      used to compile a program in a yaml file. Typically this is 
      <b>last_compiled_configure_options.yml</b>.'
      # ======================================================================= #
      # === Querying configure options
      # ======================================================================= #
      e 'You can also query configure options for a given program.'
      br
      br
      e 'To do this, try this:'
      br
      cmd '  rbt gcc3 configure?'
      cmd '  rbt htop configure_option?'
      br
      e 'Note that the script <b>ExclusiveLogic.rb</b> will check if a configure
      script makes sense. If you pass both --enable-fading and --disable-fading
      to the Installer.rb script, for instance, then ExclusiveLogic.rb will warn
      you of this mistake. This will allow you to fix the problem before you
      continue.'
      br
      e 'If you do not want to symlink after compilation has finished, add the
      option <b>dont_symlink</b> (or shorter, ds):'
      br
      cmd '  rbt gcc4 dont_symlink'
      cmd '  rbt gcc4 ds'
      br
      e 'Of course you can freely combine those instructions together:'
      br
      cmd '  rbt gcc4 dont_symlink skip_extracting'
      br
      e 'To create a new '+sg(:present, 'marr4px')+'package, you 
      use this command:'
      br
      cmd '  rbt fox cpackage'
      cmd '  rbt htop cpackage'
      cmd '  rbt fox cpackage archlinux'
      br
      e 'Default for this (if you omit i.e. archlinux) to create a
      .deb packages.'
      br
      e 'If you pass i.e. archlinux to it, we will create a package
      for Archlinux.'
      br
      e 'You can explicitely create a .deb Package as well by using
      something like:'
      br
      cmd '  rbt gcc3 cdeb'
      br
      e 'If you want to create a slackware package for the editor
      "tea" instead, do this:'
      br
      cmd '  rbt tea slackware'
      br
      e 'If you want to create a gentoo ebuild, you can do this:'
      br
      cmd '  rbt gcc3 ebuild'
      br
      anm 'A word about conventions ...'
      br
      e 'After the ./configure string, we will normally use --prefix first.'
      e 'After --prefix come the other options.'
      e 'You can also create dependency graphs, although this 
      right now is not top notch state of the art ... ;-P'
      e 'See Query for more information about this.'
      br
      e 'To search/search for certain packages or entries, 
      do something such as:'
      br
      cmd '  rbt ruby search'
      cmd '  rbt gcc s'
      br
      e 'If you instead have several chained programs such as e17, then
      you can do something like this:'
      br
      cmd '  rbt e17 wrap url'
      cmd '  chained e17 url'
      br
      e 'For the crazy guys, you can do this:'
      br
      cmd '  rbt foo all'
      br
      e 'This basically will compile EVERYTHING!'
      e 'It is probably not a good idea to do this, 
      but then again, whoever *wants* to do it can do so. 
      Perhaps for test purposes... go on.'
      br
      e '(This is no different than running each program entry 
      in the cookbook.yml manually anyway, no magic at all.)'
      br
      e 'If you however want to compile a defined set of programs, 
      there exists another way - chained programs in cookbook.yml'
      br
      e 'Via chained programs you simply specify a certain
      chain of compilation. I use this to compile xfce for 
      example, or kde.'
      br
      e 'Via <b class="yel">generate_custom_yaml</b> you can 
      dump specific programs into a new yaml file.'
      br
      e 'Usage example:'
      br
      cmd '  rbt gcc dump_yaml'
      cmd '  rbt gcc dyaml'
      cmd '  rbt x generate_custom_yaml'
      br
      e 'Installer.rb has different install procedures like "python
      setup.py install", which are stored in a macro such as PINSTALL1'
      e 'You can use this macro in <b>cookbook.yml</b>'; br
      # Supported Build Types in the RBT Scripts.
      e 'Currently these Types are supported in the Ruby Build Tools 
      scripts:'
      p('ind0px') {
        e '- .rb Files (ruby)','marl3em yel'
        e '- .py Files (python)','marl3em yel'
        e '- Cmake Files','marl3em yel'
        e '- GNU Autoconfigure Files','marl3em yel'
      }
      # ======================================================================= #
      # === Copy the whole yaml database.
      # ======================================================================= #
      e 'You can copy the whole yaml database by doing this:'
      br
      cmd '  rbt x copy_yaml'
      br
      # ======================================================================= #
      # === Copy the whole yaml database.
      # ======================================================================= #
      e 'You can copy the whole yaml database by doing this:'
      br
      cmd '  rbt x copy_yaml'
      br
      # ======================================================================= #
      # === Use configure options from an old version
      # ======================================================================= #
      e 'If you wish to re-use very long configure options, such as
      from gcc, then you can use this command:'
      br
      cmd '  ry gcc --use-this-version=7.1.0 --use-old-configure-options'
      br
      e 'This will use the configure options from the currently installed
      gcc compile.'
      # ======================================================================= #
      # === KDE Konsole tab title
      # ======================================================================= #
      e 'You can disable the renaming of the KDE Konsole tab, even if the
      configuration setting specifies otherwise, on the commandline, by
      issuing something like the following:'
      br
      cmd '  ry pango dontrenametab'
      cmd '  ry pango dont-rename-tab'
      cmd '  ry pango --dont-rename-tab'
      br
      # ======================================================================= #
      # === Disable colours
      #
      # Disable colours tag, disable colour tag.
      # ======================================================================= #
      e 'If you dont want to display colours, either disable them 
      globally in the config.yml file (the entry is called 
      `use_colours_output`) or pass --nocolor (or --nocol or --nocolour) 
      on the cmdline like this:'
      br
      cmd '  rbt libmemcached --nocolor'
      cmd '  rbt libmemcached --nocolor'
      cmd '  rbt libmemcached --nocolour'
      cmd '  rbt libmemcached --nocolours'
      cmd '  rbt libmemcached nocolours'
      br
      anm 'Note that you can also change the colours. For this,
      edit the file <b>colours.yml</b>' # colours tag
      br
      e 'The above will, however had, only disable the colours for the
      current run.'
      br
      e 'You can find out whether you use colours specifically
      or not, by issuing this command:'
      br
      cmd '  rbt --use-colours?'
      br
      e 'You can also permanently disable the colours by
      issuing this command:'
      br
      cmd '  rbt --permanently-disable-colours'
    }
    # ======================================================================= #
    # === Do not use any configure option
    # ======================================================================= #
    div('pad1_5em s2em marl3em mart1em','','border:1px dotted darkorchid') {
      h3' Do not use any configure option'
      e 'If you do not want to use any configure option, use the following
      command:'
      br
      cmd '  ry htop --dont-use-configure-options'
    }
    # ======================================================================= #
    # === Last configure option
    # ======================================================================= #
    div('pad1_5em s2em marl3em mart1em','','border:1px dotted darkorchid') {
      e 'You can show the last configure option used by issuing:'
      br
      cmd '  rbt last_options?'
      br
      anm 'Note that this requires you to have at least compiled one
      program, as otherwise we have not yet stored a configure option.'
    }
    # ======================================================================= #
    # === Last compiled options
    # ======================================================================= #
    div('pad1_5em s2em marl3em mart1em','','border:1px dotted darkorchid') {
      e 'You can display the last n compiled programs.'
      br
      e 'This is done like so:'
      br
      cmd '  rbt --last-compiled?=6'
      cmd '  rbt --last-4-compiled?'
    }
    # ======================================================================= #
    # === Log Directory
    # ======================================================================= #
    div('pad1_5em s2em marl3em mart1em','','border:1px dotted darkorchid') {
      e 'We will use a base directory for logging output, which will be
      used to store log files and similar files.'
      br
      e 'If you want to know where the log directory is on your system,
      then you can ask rbt, using this command:'
      br
      cmd '  rbt rbt_log_dir?'
    }
    # ======================================================================= #
    # === Program Settings
    # ======================================================================= #
    header 'Programs Settings'
    p {
      e 'On traditional Linux structures, system-wide settings will
      be stored under the directory <b>/etc</b>.'
      br
      e 'The RBT project will offer the same behaviour - or it will
      default to make use of its own Settings directory, as is
      done on GoboLinux as wel).'
      br
      e 'If you want to backup all these stuff, you can use '
      br
      l2em '  ProgramsBackup.rb','yel'      
    }
    # ======================================================================= #
    # === Install only the bin/ subdirectory
    # ======================================================================= #
    header 'Install only the bin/ subdirectory'
    p {
      e 'You can also install only the bin/ subdirectory. Every other directory
      that belongs to this program will not be installed.'
      br
      e 'Example:'
      br
      cmd '  rbt htop --only-binaries'
    }
    # ======================================================================= #
    # === Compile and upload the binaries
    # ======================================================================= #
    header 'Compile and upload the binaries'
    p {
      e 'You can compile and then upload all binaries. This currently
      only works for my own FTP host.'
      br
      e 'Usage Example:'
      br
      cmd '  rbt make ntrad upload-binaries'
      br
      e 'This would upload all binaries of make, from
      <b>/Programs/Current/Make/bin</b>.'
      br
      anm 'Do also note, for my own system (or if you enable this
      by setting an environment variable called IS_ROEBE to 1),
      we can upload the current ruby to a remote website via
      ftp. The following example shows how to invoke this:'
      br
      cmd '  ry --upload-ruby-binary'
    }
    # ======================================================================= #
    # === Environment Variables
    # ======================================================================= #
    header 'Environment Variables'
    p {
      e 'You can set certain environment variables, such as LDFLAG and
      similar.'
      br
      e 'If you need a generic way to do so, you can do so in the
      yaml file directly, via:'
      br
      cmd  '  set_env_variables:'
      cmd2 '  GLL_DRV: i915,r600,nouveau,radeonsi,svga,swrast'
      br
      e 'This would be equivalent to setting this in a shell, such as
      bash.' 
      br
      e 'Note that you can also disable the use of ANY environment
      variable, including CLFAGS, by issuing the following command:'
      br
      cmd '  --do-not-use-special-flags'
      br
      anm 'This will NOT remove the PATH variable though, as otherwise
      we may be unable to compile anything.'
      br
      e 'Sometimes you may not want to use ANY environment variable
      and clear ENV. This can be done by issuing the following
      command:'
      br
      cmd '  ry gtk --clear-env'
      br
      anm 'Note the $PATH setting will be kept though, as otherwise
      we could not compile anything if the compiler is not found
      to be within the $PATH variable.'
    }
    # ======================================================================= #
    # === Convert Program
    # ======================================================================= #
    header 'Convert Program'
    p {
      e 'You can convert a program too.'
      br
      e 'To convert a program such as htop to non-traditional, do this:'
      br
      cmd '   rbt htop to_programs'
    }
    # ======================================================================= #
    # === Gobolinux Recipes
    # ======================================================================= #
    header 'Gobolinux Recipes'
    p {
      e 'In order to create Gobolinux Recipes for i.e. python, do this:'
      br
      cmd '  rbt python goborecipe'
      br
      e 'Note that this will create the recipe in the current working
      directory.'
    }
    # ======================================================================= #
    # === Highest
    # ======================================================================= #
    header 'The Highest n programs'
    p {
      e 'You can show the largest local programs, a Top 50 for instance,
      by doing:'
      br
      cmd '  rbt x highest'
    }
    # ======================================================================= #
    # === Create a slackware slack-desc file
    # ======================================================================= #
    header 'Slackware slack-desc files'
    p {
      e 'You can create a slackware slack-desc file.'
      br
      e 'Do this:'
      br
      cmd '  rbt ruby --generate-slack-desc-file'
      cmd '  rbt ruby --slack-desc'
    }
    # ======================================================================= #
    # === Standalone Yaml Files
    # ======================================================================= #
    header 'Standalone Yaml Files'
    p {
      e 'To create standalone Yaml files, do this:'
      br
      cmd '  rbt ruby create_standalone_yml'
    }
    # ======================================================================= #
    # === Tags                                                     (Tags tag)
    #
    # This documents the /System/Tags hierarchy.
    #
    # /System/Tags
    # ======================================================================= #
    header '/System/Tags' # rf rbt tags
    p(id: 'tags') {
      e 'The /System/Tags directory is a special directory. If you set the
      configuration option called <b>use_tags</b> in the file
      <b>configuration.yml</b> to <b>true</b>, then we will honour
      the <b>tags: variable</b> in the individual cookbook yaml file.'
      br
      e "What does this actually mean? Let's look at a specific example."
      br
      e 'Say you have an <b>editor</b> like vim, defined in a yaml file.'
      br
      e 'The yaml file option for vim includes a line such as:'
      marl3em 'tags: editor','yel'
      br
      e 'Now we have tagged the program vim to be an editor. When vim
      is then compiled, we also symlink the vim binaries into:'
      br
      cmd '  /System/Tags/Editors/vim'
      br
      e 'This is somewhat useful in that you can group different
      programs according to what feature they give you.'
      anm 'You can also search for tags, via the class
      SearchForTags, or invoke it from the commandline:'
      cmd '  stag mate # Search for all tags with mate'
      cmd '  stag mate WITH_URL # ^^^ same as above but also
      output the URL.'
      br
      e 'If at any moment you wish to setup the tags again, do
      this:'
      br
      cmd '  rbt --setup-tags'
      br
      e 'Note that you can also combine searching via tags.'
      br
      e 'For instance, consider:'
      br
      cmd 'stag gnome python'
      br
      e 'This would yield all entries that combine both
      of these tags.'
      br
      e 'Let us look at another example.'
      br
      cmd '  tags kde ftp'
      br
      e 'This will yield the following results:'
      br
      cmd '  1) kasablanca'
      cmd '  2) kftpgrabber'
      br
      anm 'As of February 2017, we will exit when an invalid
      tag is in use. The <b>idea</b> behind this is to clean
      up all the various cookbook files properly, and specify
      the tags that are allowed.'
      br
      e 'Note that you can also display all valid tags, that
      is, the ones in /System/Tag, by issuing the following
      command:'
      br
      cmd '  rbt --valid-sys-tags?'
      br
    }
    # ======================================================================= #
    # === Commandline Options
    # ======================================================================= #
    header 'Commandline Options'
    p {
      e 'As was already written somewhere else, you can pass in several
      commandline options to the Installer.rb script. You can also use
      commandline options that include <b>--</b> and these options
      can overrule other configure options.'
      br
      e "For instance, let's assume that the configure options in the 
      yaml file include an option like <b>--enable-gtk</b>. You can
      then overrule this on the commandline by doing:"
      br
      cmd '  rbt weechat --disable-gtk'
      br
      e 'This will overrule the setting in the cookbook.yml file
      and compile weechat with a disabled gtk.'
    }
    # ======================================================================= #
    # === PACKAGE USER TAG
    # ======================================================================= #
    header 'Package Users'
    p {
      e 'This concept is easy to understand:'
      e 'Every package belongs to a certain user.'
      e 'This way you can immediately identify which file belongs whereto.'
      br
      e 'By default this setting is <b>turned off</b> though.'
    }
    # ======================================================================= #
    # === PDF Generation for Users
    # ======================================================================= #
    header 'PDF Generation Users'
    p {
      e 'If you want to generate PDF files which you can print or distribute
      anyway you see fit, have a look at the PdfGenerator.rb script (Note
      though, right now the output is a bit ugly... I will look into this
      later.)'
    }
    # ======================================================================= #
    # === Simulate
    # ======================================================================= #
    header 'Simulate running'
    p {
      e 'This option - also known as <b>no harm</b> Option, allows you to
      run the RBT Scripts in a simulation mode.'
      br
      e 'This works like so:'
      br
      cmd '  rbt php simulate'
    }    
    # ======================================================================= #
    # === View Available Licenses
    # ======================================================================= #
    header 'View Available Licenses'
    p {
      e 'To view available licenses, do this:'
      br
      cmd '  rbt --view-available-licenses'
      cmd '  rbt --licenses?'
      br
      e 'You can filter for specific licenses on a given program, such as
      by issuing this command:'
      br
      cmd '  rbt gcc license lgpl'
      br
      e 'This would filter for all <b>LGPL license</b> (case-sensitiveness
      is ignored for finding results).'
    }
    # ======================================================================= #
    # === Global flags
    # ======================================================================= #
    header 'Global flags'
    p {
      e 'Global flags are compile-time flags that you can use, which
      will be appended to GNU configure runs.'
      br
      e 'These flags are stored in the file global_flags.yml'
      br
      e 'You can either manually modify this file, or use commandline
      instructions to modify this file.'
      br
      e 'For example, if you wish to append <b>--disable-static</b>
      to all configure invocations via RBT, then you can issue
      the following command:'
      br
      cmd '  rbt --add-global-flag=--disable-static'
      cmd '  rbt --add-global-flag --disable-static'
      br
      e 'Likewise, to remove a global flag, you can do:'
      br
      cmd '  rbt --remove-global-flag=--disable-static'
      cmd '  rbt --remove-global-flag --disable-static'
      br
      e 'You can reset all global flags again, by issuing:'
      br
      cmd '  rbt --clear-global-flags'
      br
    }
    # ======================================================================= #
    # View Available Programs
    # ======================================================================= #
    header 'View Available Programs'
    p {
      e 'To view which programs are available, do this:'
      br
      cmd '  rbt x all_programs?'
      br
      anm 'As of JUL 2011, the above no longer works. compile is a direct
      alias to Compile.rb now. Try the following instead:'
      br
      cmd '  rbt x all_programs?'
    }
    # ======================================================================= #
    # === View Configure Options
    # ======================================================================= #
    header 'View Configure Options'
    p {
      e 'You can view the configure options that will be used for a
      program by doing this here:'
      br
      cmd '  config? glibc'
      cmd '  rbt glibc qco'
      cmd '  rbt glibc query_configure_options'
    }
    # ======================================================================= #
    # === Changing Versions
    # ======================================================================= #
    header 'Changing Versions'
    p {
      e 'You can always edit the .yml files in question using a simple
      text editor. But sometimes you may only want to update a specific
      entry without opening an editor, and you can do this by issuing
      the following command:'
      br
      cmd 'newversion ruby-1.9.2'
      br
      e 'This would find the entry (i.e. at ruby_cookbook.yml), and change
      the version of ruby to be 1.9.2'
      br
      e 'The advantage is that you dont have to open an editor to change it.'
    }
    # ======================================================================= #
    # === Edit tag, editor tag
    # ======================================================================= #
    header 'Editing the cookbooks yaml files'
    p {
      e 'You can edit individual cookbook entries from the commandline.'
      br
      e 'For example, to edit nano, do:'
      br
      cmd '  rbt --edit-file=nano'
      cmd '  rbt --edit=nano'
    }
    # ======================================================================= #
    # === Postinstall tag.
    # ======================================================================= #
    header 'Postinstall'
    p {
      e 'The <b>Postinstall-Phase</b> will begin after compilation has 
      finished. This also means that the postinstall step can only happen
      if the compilation procedure was successful.'
      br
      e 'Usually, via <b>postinstall actions</b>, you can do something like
      remove certain directories or files, create new files or new directories,
      handle symlinks and similar actions. It is quite rarely needed, but at
      least this functionality exists whenever you need it.'
      br
      e 'You can also disable this behaviour if you want to, 
      at run-time.'
      br
      e 'Use this:'
      br
      cmd '  rbt htop --no-postinstall'
    }
    # ======================================================================= #
    # === Autogen tag.
    # ======================================================================= #
    header 'Autogen'
    p {
      e 'You can specifically enable autogen. Usage examples:'
      br
      cmd '  rbt htop --use-autogen'
      cmd '  rbt htop --enable-autogen'
      br
      e 'Conversely, you can also specifically disable autogen. This
      is only necessary in rare cases when you do not wish to have
      autogen to be run. Only a few programs will require autogen
      normally.'
      br
      cmd '  rbt htop --disable-autogen'
    }
    # ======================================================================= #
    # === Other Scripts
    # ======================================================================= #
    header 'Other Scripts'
    p {
      e 'The script RBT::ScanSourceArchive will scan through your main
      archive repository and checks if a found tarball there is registered 
      in your cookbook files.'
    }
    # ======================================================================= #
    # === Shebang Fixing.
    # ======================================================================= #
    header 'Shebang Fixing'
    p {
      e 'You can fix all Shebangs of the RBT Scripts by issuing:'
      cmd '  rbt --shebang'
    }
    # ======================================================================= #
    # === Configuration Settings. Config tag.
    # ======================================================================= #
    header 'Configuration Settings'
    p {
      e 'You can quickly overrule a specific configuration setting by 
      doing something like this:'
      br
      cmd '  rbt rbt.use_ccache'
      br
      e 'The above will try to use ccache. If ccache was not found,
      the scripts will not try to make use of it.'
      br
      anm 'Also note that the configuration will be optional - if 
      you do not have a configuration file, we will assume some 
      default values.'
      br
      e 'If you need to find out the configuration directory
      of your running DiamondShell, do this:'
      br
      cmd '  config_dir?'
      br
      e 'This will give you the main directory for all the
      yaml configuration files.'
      br
      e 'If you wish to view the whole RBT Configuration, consider
      issuing the following command:'
      br
      cmd '  rbt config?'
    }
    # ======================================================================= #
    # === sed tag
    # ======================================================================= #
    header 'sed'
    p {
      e 'Some programs require sed. RBT also comes with a sed-wrapper, so
      sed is ultimately not required. Note that the sed-wrapper supports
      only a very small subset of sed-operations, so it may fail for
      some more advanced use cases.'
      br
      e 'If you do not want to use sed, you can disable it like so:'
      br
      cmd '  rbt libmad --disable-sed'
    }
    # ======================================================================= #
    # === CMAKE Tag.
    # ======================================================================= #
    header 'CMake'
    p {
      e 'RBT supports cmake files.'
      br
      e 'Here come some conventions.'
      br
      e 'Right now, if a CMake*.txt file was not found, then we will
      not run cmake. Instead, we will try to run GNU configure instead.'
      br
      anm 'Note that you can force the use of cmake from the commandline
      by passing <b>--cmake</b>, </b>-cmake</b> or <b>cmake</b> to
      <b>bin/rbt</b>. Using <b>--cmake</b> is the preferred variant; I
      recommend to default to this option if you are using it.'
      br
      e 'As of Oct 2017, some KDE-related programs insist on /usr/lib/cmake/
      so if the "is on roebe" setting is set, and we use an app-dir prefix,
      then the cmake/ subdirectory will be symlinked into /usr/lib/cmake/,
      but only directories, not files.' 
    }
    # ======================================================================= #
    # === LEGALESE TAG
    # ======================================================================= #
    header 'Legalese / Disclaimer'
    p {
      e 'Now to some boring '+sg(:dokument)+'legalese - otherwise evil 
      people may decide to sue me, right?)'; br
      e 'So ladies and gentlement, without further ado ... 
      here we go ...'; br
      e '- The source code is <b>provided as is</b>. No explicit 
      guarantees are given of any kind. We do however try to fix all 
      possible bugs, and we are committed to what we wrote. Chances are 
      rather high that it will run on your system, and even quicker 
      if you provide feedback.)'
      e '- Once you DOWNLOAD these scripts, they in fact help YOU 
      to automate tasks which YOU could do from a shell _alone_.'
      e 'It is NOT that WE do it for YOU, you are doing so. '
      e 'You are in the whole process (and control), it is thus 
      your own risk to use them. This is also the reason 
      why there are so many settings - we want people to change 
      the scripts to their likings as much as possible and 
      as much as it makes sense.'
      br
      e 'It is entirely in your responsibility to ensure that 
      your police state does allow what you do - we have no 
      control over your actions and we do not encourage you to 
      fight against your police state or do any other 
      activities that should not be done ... remember, 
      obedience to the law means freedom, cuz otherwise the 
      armed and armoured guys come in - and get you to jail. ;-)'
      br
      e "However we try to ensure our best that these scripts do 
      not teleport your cats to Texas, nor make polar bears appear 
      at your local friseur in Goteborg or bring Ninjas to your 
      aunt's place... Still, use at your own risk."; br
      e 'This section is more or less needed so noone goes and 
      claims that what these scripts are doing is harmful, or 
      is making any <b>derivative work</b> upon others, or that 
      they <b>endorse illegal activity</b>.'
      e 'We dont even care about this crap. Lawyers that 
      fight for their money purse are annoying.'
      e 'Once you use these scripts THEY ARE YOURS.'
      e ' <b>THEY ARE NOT A DERIVATIVE WORK</b>.' 
      e 'They are scripts which _YOU_ control and in essence _YOU_ 
      use. We only have in common that we helped write these scripts.'
      e 'They are _NOT_ under GPL. They are not even under a BSD 
      licence. If, you need a license, you may license them under 
      the freest possible ruby license.'; br
      e 'Once you have these scripts and use them, they BECOME YOUR 
      PROPERTY and you are FREE TO DO WHAT YOU WANT WITH THEM with 
      <b>one</b> exception - you MUST clearly and openly state that 
      you wrote these scripts IF you publish them.' 
      e 'This means you may not claim that we are the author of 
      your scripts if you decide to relicense them to the public 
      in any way. (If you dont release them to others, it doesnt 
      matter anyway. You can do whatever the hell you want to do 
      with them anyway.)'
      e 'This is the only conditional clause.'; br
      e 'The reason is simple. We dont care if you make money 
      with it, we dont actually care WHAT you do with them but 
      if you use them, <b>leave us totally alone</b>.'
      e 'It must be clear to you that the scripts are 
      YOUR scripts at the moment you download them. We do not own
      the datastream.'
      e 'In a way you immediately become a full sublicensee.'
      anm 'If the laws of your state do not allow this, 
      then YOU MAY NOT DOWNLOAD/USE THESE SCRIPTS!'
      br
      e 'If you dont want to do this, or are in doubt of your 
      state law, then do <b>NOT</b> download these scripts.'
    }
    # ======================================================================= #
    # === Conventions tag. rfrbt Conventions
    # ======================================================================= #
    header 'Conventions'
    p('FW4') {
      e 'This is the reference for important conventions used as
      part of the ruby build tools project.'; br
      e '- Conventions about naming entries (how to properly name them):'
      br
      e '/Users/x/SRC/PHP/phpwiki-1.3.12.tar.bz2','BO yel'
      marl5em '  <span class="green">@program_path</span> = <span class="lightblue">/Users/x/SRC/PHP/phpwiki-1.3.12.tar.bz2</span>'
      marl5em '  <span class="green">@archive_type</span> = <span class="lightblue">.tar.bz2</span>'
      marl5em '  <span class="green">@program_full_name</span> = <span class="lightblue">phpwiki-1.3.12.tar.bz2</span>'
      marl5em '  <span class="green">@base_dir</span> = <span class="lightblue">/Users/x/SRC/</span>+<span class="green">@base_dir</span> # specify the base directory of the program in question'
      marl5em '  <span class="green">@short_name</span> = <span class="lightblue">phpwiki</span>'
      marl5em '  <span class="green">@program_name</span> = <b class="green">@short_name</b>+<b class="green">@program_version</b> 
      Example: <span class="lightblue">phpwiki-1.3.12</span>'
      marl5em '  <span class="green">@program_version</span> = <span class="lightblue">1.3.12</span>'
      br
      e 'Since Feb 2012 we follow a new convention when it comes to options
      like --no-harm. All options that include -- will become an integrated
      part of the main case/menu data structure.'      
    }
  }
  # ========================================================================= #
  # === Interesting Links
  # ========================================================================= #
  div('pad1em','','border:1px dotted orange; min-width:85%;'){
    h3 RED_SQUARE+'Interesting / Useful Links','gold'
    p('ind0px marl3em') {
      link :rbt_todo,
        'A list of Things Todo specifically for the Scripts',1,'slateblue'
      link :local_gobopage,'GoboLinux',1,'slateblue'
      link :rbt_homepage,'RBT Project (remote)',1,'slateblue'
    }
    # ======================================================================= #
    # === Contributing to the RBT project (Contributing Tag)
    # ======================================================================= #
    h3 RED_SQUARE+'Contributing to this project','gold'
    p('ind0px marl2em'){
      e 'This project is appreciative about contributions of 
      all sorts, especially in regards to these topics:'
      e '- Ideas (especially about new/missing features)','marl2em'
      e '- Code contribution of ALL sorts. Please try to adhere to
      the coding standard, but it is not so important, as I will
      make your ruby code adhere to the standards anyway, so just
      send it to me.','marl2em'
      e '- Coverage (so that more people know about it)','marl2em'
      br
      e 'The code base is far from perfect, so anything that can 
      be done here to seriously improve it is important, however 
      right now adding more features is more important.'
    }
    # ======================================================================= #
    # === Pseudo-Macros
    # ======================================================================= #
    h3 RED_SQUARE+'Pseudo Macros','gold'
    p {
      e 'We use some Pseudo-Macros in the RBT Project.'
      br
      e 'This section here details them:'
      simple_table2(
        [
          'PROGRAM_NAME','Will be replaced with the name of
           the program, for instance: "htop"',
          'PROGRAM_VERSION','Will be replaced with the actual 
           version of the program, example: "1.0.0"'
        ], 'marl2em'
      )
    }
    # ======================================================================= #
    # === Compile random program.
    # ======================================================================= #
    p {
      anm 'Note that you also compile a random program.'
      br
      e 'This can be done by issuing this command:'
      br
      cmd '  compile RANDOM'
      cmd '  compile random'
    }
    # ======================================================================= #
    # === Future and Todo tag
    # ======================================================================= #
    h3 RED_SQUARE+'Future Outlook','gold'
    p {
      e 'The future is hard to predict, but I do have some ideas 
      on my mind. Not everything seems to be possible in pure 
      Ruby, but who knows - maybe we will include some C.'; br
      e 'I will also include other fancy ideas. Here goes:'; br
      e '- Ability to compile a project without being 
      dependent on hardcoded Makefiles, cmake, or whatever.','marl2em'
      e '- Ability to snapshot these compilations at any time,
      and being able to resume the compile process lateron.','marl2em'
      e '- Ability to generate Makefiles etc... for projects.','marl2em'
      e '- We should realize a generic plugin architecture,
      so that build types like cmake or scons work easily.','marl2em'
    }
  }
}

# ===================================================================== #
# Show Todo List. Doesn't work right now.
# ===================================================================== #
div('default','','border: 1px dashed crimson;') {
  h2 'Show Todo List'
  # read_display '../../../lib/rbt/TODO_AND_BUGS/RBT_TODO','mars1em bblack1'
}

div('default') {
  h3 'Release History and Version for the RBT Versions'
  table2('marl1em','','',
    '<b class="yel">Version</b>','When',
    '1.0.0','In the future, when we are 1:1 compatible with the
    GoboLinux Scripts.')
}

div('pad1em') {
  # ========================================================================= #
  # Die Intramap Tag.
  # ========================================================================= #
  h3 RED_SQUARE+'Intramap','gold','intramap'
  p('ind0px marl2em') {
    ARRAY_ALL_IDS.sort.each { |id|
      link '#'+id, 
        sg(:green_dot,'bblack1 marr5px')+id.gsub(/_/,' '),1,'slateblue BO marl1em'
    }
  }
}

# =========================================================================== #
# === Formular
#
# rf rbtcgi append_this_program
# =========================================================================== #
div('pad1em s1_5em marl2em mart0_5em','append_this_program',
    'border:1px dotted darkorchid'){
  header_stylish_brown sg(:dot7, 'marr6px')+'Append this 
  Program to <b class="yel">cookbook.yaml</b>'
  div('mars3em'){
    form 'self','get','add_program'
      table 'marl3em'
        make_row 'Name of Program to add:','name_of_program','BG_Black lightblue s1em'
        make_row 'Base Dir:','base_dir','BG_Black lightblue s1em'
        make_row 'Description:','description','BG_Black lightgreen s1em'
        make_row 'URL:','url','BG_Black lightblue s1em'
        array = RBT.available_programs
        input_with_datalist(array)
      ctable
      if parameters.keys.size > 2
        name = parameters['name_of_program'].to_s
        url = parameters['url'].to_s
        base_dir = parameters['base_dir'].to_s
        description = '    '+parameters['description'].to_s
        p {
          s2 'Adding program: ','yel'
          s2 name,'green'
          s2 'with url: ','yel'
          e url,'lightblue'
          query = Query.new('')
          query.add_program_to_cookbook( 
            name, url, base_dir, description
          )
          e 'Program added to database!'
        }
      else
        input :s,'Submit Data','marls2em','submit_page',
        'background-color: white; color:black;
        border-top:3px solid gold;
        border-left:1px solid gold;'
      end
    cform
  }
  div {
    # ======================================================================= #
    # cookbook.yml tag.
    # ======================================================================= #
    h4 'Cookbook.yml'
    e 'You can apply patches, like a linux kernel patch, by adding 
    an <b>apply_patch:</b> entry in the cookbook.yml file.'
    e 'Also note that you can use this entry in a conditinal. For
    instance:'
    br
    cmd '  apply_patch: http://connie.slackware.com/~alien/slackbuilds/id3lib/build/id3lib-3.8.3_gcc4.diff if PROGRAM_VERSION == 5.14.1'
    br
    e 'The above line would only be applied if the program version would
    match. This allows us to apply patches in a conditional way.'
    br
    e 'Since 13.09.2007 you can also manipulate the cookbook.yaml
    file via a .cgi Page.'
    br
    e 'You are currently viewing this page :)'
    br
    e 'Just look at the following formular.'
    # ======================================================================= #
    # === Remove Program
    # ======================================================================= #
    div('pad1_5em s2em marl3em mart1em','RemoveProgram','border:1px dotted darkorchid') {
      field_with_legend 'Remove Program','yel'
      p('white mar2px pad1px') {
        e 'Use this formular if you want to remove a program 
        from your system.'
        table 'marl1em'
          make_row 'Name of program which should be removed:',
          'remove_this_program','BG_Black lightblue'
        ctable
      }
      cfieldset
    }
  }
  p { link 'self','ReloadThisPage',1,'slateblue' }
}

div {
  h2 'Listing some RBT files now:'
  # read_file_with_these_lines('ShebangFixer',1,5)
  div('marl1em pad1em padt0_5em','','RAND_BORDER_1PX') {
    name = '../shebang_fixer.rb'
    h3 File.basename(name),'mart1px yel FI'
    if File.exist? name
      pre read_file_with_these_lines(name, 1, 12), 'bblack1 pad1em smaller'
    end
  }
}
  # ========================================================================= #
  # === SITEMAP TAG.
  #
  # We don't use the autogenerated sitemap for now.
  # ========================================================================= #
  div('posab smaller BG_Black pad0_4em','',
      'right:0.2em; top:1em;border:2px solid '+COLOUR_BROWN+';'){
    h2 'Sitemap','gold marb1px s0_5em'
    _ = sg(:square_violet, 'bblack1 marr4px')
    p('mars1em smaller mart1px ind0px') {
      link '#Installer.rb', _+'Installer.rb',1,'BO slateblue'
      link '/Depot/Information/URLs.html',_+'URLs.html',1,'BO slateblue'
      link 'self',_+'Generate PDF',1,'BO slateblue'
      link '#multiple_yaml_files',_+'Use multiple yaml files',1,'BO slateblue'
      link :gobolinuxdist,_+'Lokale GobolinuxPage',1,'BO slateblue'
      link :projects,_+'Lokale Projects Page',1,'BO slateblue'
      link 'https://rubyforge.org/projects/rubybuildtools/',
        _+'Homepage for Ruby Build Tools',1,'BO slateblue'
      link 'http://code.google.com/p/ruby-build-tools/',
        _+'Google Project page for the Ruby Build Tools',1,'BO slateblue'
      link 'http://tasktaste.com/projects/shevegen/rbt',
        _+'Task Taste for the Ruby Build Tools',1,'BO slateblue'
    }
  }
}