#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
require 'yaml'
require 'rbt/version/version.rb'
require 'rbt/toplevel_methods/n_programs_available.rb'
require 'rbt/toplevel_methods/install_which_binaries.rb'

Gem::Specification.new { |s| # Gemspec for the RBT-Project.

  s.name    = 'rbt'
  s.version = RBT.version?
  s.date    = Time.now.strftime('%Y-%m-%d')

  DESCRIPTION = <<-EOF

This gem is called RBT, which is an abbreviation for "Ruby Build Tools".

Essentially, this project is a collection of different ruby scripts/classes
that may aid in the process of compiling or installing programs from source,
but also allow installation from binary, from a remote or from a local
location (e. g. a file in a directory on the local harddisc/SSD).

There are presently #{RBT.n_registered_programs?} registered
programs available in the RBT project.

For more information about the project, have a look at the documentation,
which can be found at the bottom right side on the homepage of this
gem ( or simply visit the following link: http://www.rubydoc.info/gems/rbt/ ) 

EOF

  s.summary     = DESCRIPTION
  s.description = DESCRIPTION

  # =============================================================== #
  # Show this when a user installs the Project.
  # =============================================================== #
  s.post_install_message = <<-EOF

  With the RBT scripts, you should be able to install or compile
  a program.
    
  Essentially, the syntax for this should be:

      rbt htop
      rbt htop ntrad

  Or read the help options via:

      rbt --help

  The first thing you should do is to tweak the file configuration.yml
  to your liking. You should set the configuration option source_dir
  to the base directory where you wish to keep all your source
  archives.

  To get an overview over the available scripts that are part of
  this project, issue the following command on the commandline:

      rbt --show-available-utility-scripts

  It is recommended to run the following command after having
  installed the cookbooks gem, which will expand all the
  various .yml files into their expanded, sanitized version:

      rbt --expand 

  You can use this gem from the commandline by using "scookie".
    
  Examples:

      scookie htop
      scookie ruby
      scookie php
      scookie python

  This should show information about these programs. Note that you need
  the .yml files from the Cookbooks project for this - in order to
  install these files, do install the cookbooks project:

      gem install rbt # or
      gem install rbt --user-install

  You can also toggle this information to some limited extent. For instance,
  you can do:

      scookie htop --appdir

  Which will use htop in an AppDir approach, with a standalone prefix.

  To disable colours, try:

      scookie htop --nocolours

  For the documented help options, see:

      scookie --help

  If you wish to search for common tags, such as tag "ruby" or tag
  "php", then try this:

      search_for_tags ruby
      search_for_tags php

  This can be found in the subdirectory bin/search_for_tags of
  this gem.

  For more documentation, the official documentation, have a look at:

      #{RBT::URL_TO_THE_DOCUMENTATION}

  If you encounter any bug or oddities, feel free to drop an email
  to:

      shevegen@gmail.com

  (I do not check for mails daily, though, so this may take a 
  while to process - apologies for that.)

  Thank you and have fun!

  EOF

  s.extra_rdoc_files = %w()
  
  s.authors     = ['Robert A. Heiler']
  s.email       = 'shevegen@gmail.com'
  s.files       = Dir['**/*']
  
  s.executables = RBT.will_install_which_binaries?

  s.license     = 'GPL-2.0'
  s.homepage    = 'https://www.rubydoc.info/gems/rbt/'

  s.required_ruby_version     = '>= '+RUBY_VERSION
  s.required_rubygems_version = '>= '+Gem::VERSION
  s.rubygems_version          = '>= '+Gem::VERSION

  # bl $RUBY_SRC/rbt/doc/project_dependencies.md

  _ = '/Users/x/DATA/PROGRAMMING_LANGUAGES/RUBY/src/rbt/doc/project_dependencies.md'
  dataset = File.readlines(_).select {|entries| entries.strip.start_with? '-' }
  dataset.each {|entry|
    s.add_dependency(
      entry.delete('-').strip
    )
  } # bl $RUBY_SRC/rbt/doc/project_dependencies.md

}