[![logo](http://shevy.bplaced.net/RUBY_BUILD_TOOLS_LOGO.png)](https://rubygems.org/gems/rbt)
[![logo](http://shevy.bplaced.net/COOKBOOKS_LOGO.png)](https://rubygems.org/gems/rbt)
[![Gem Version](https://badge.fury.io/rb/rbt.svg)](https://badge.fury.io/rb/rbt)

[![forthebadge](http://forthebadge.com/images/badges/built-with-love.svg)](https://www.gobolinux.org/)
[![forthebadge](http://forthebadge.com/images/badges/made-with-ruby.svg)](https://www.ruby-lang.org/en/)

## RBT - The "Ruby Build Tools" Project

The main goal for the **RBT project** is to allow the user to install software, primarily
on **Linux systems**, and primarily from **source** - that is, to compile the program
at hand. While the focus is predominantly on the Linux platform, whenever possible 
support for **other platforms** will be added as well; Linux is the **main target OS**,
though.

While this is the main goal for the **RBT project**, the full scope of the **RBT project**
is a wider one: the **RBT project** also tries to help as **a build toolset**, that is,
as means to **help people manage software** on their respective computer systems in general.
This definition also includes the tasks done by package managers, although support for
this within RBT itself is presently (2019) fairly minimal.

Additionally, the **RBT** project may aid in the process of <b>autogenerating files
and code</b> related to installing and compiling, such as **generating standalone
shell scripts** or archive formats, such as .rpm or .deb. For example, one part
of **RBT** can be used to autogenerate completion files for bash and zsh. Another
script can modify **Makefile**s and change the prefix in that file.

In some ways **RBT** attempts to be a **swiss army knife for
compilation/installation**.

Even if you do not intend to install or compile anything through RBT, many different
programs are registered in this project, which may be of help in other ways, such as
if you wish to determine which particular binary on a given Linux system belongs to
which particular program, or if you intend to implement a package manager yourself.
(The dataset for the standalone programs is available under the namespace
**RBT::Cookbooks**. For a list of all registered programs, you can simply
query **RBT.available_programs?**).

The <b>primary core functionality</b> of the RBT project, however had, lies in
**compiling programs from source**, as this is what I use RBT for on an almost
daily basis.

## Installing the rbt gem itself

The canonical way to install the **rbt** gem is:

    gem install rbt

However had, I recommend the --user-install option; this makes it easier to
use the **bin/** scripts that are part of this gem:

    gem install rbt --user-install  

Then you can use them easily if you have the bin/ to your home directory
in your $PATH. While the /usr hierarchy as prefix is fine, some distributions,
in particular debian, have a habit to split up gems a lot. This is also
another reason why the --user-install option is better, in my opinion.

## The history and philosophy of the RBT Project - philosophic considerations

This subsection in this README-document contains some information as to how
the RBT project was initially started, and how it has changed over the course
of time. Additionally some of the **philosophy** behind the RBT project will
be explained.

The **RBT project** in itself started out initially as means to <b>re-create
the shell scripts' logic</b>, and functionality, used in <b>GoboLinux</b>,
several years ago.

I was already using **Ruby** at that time and I did not understand the
shell scripts that were used in **GoboLinux**. Shell scripts can be quite
confusing to read; parameters do not go in normally into a regular 
function via a function/method signature, but instead are accessed as
special semi-global variables such as **$1** or **$2** within the
function body. I find this extremely unintuitive, confusing and quite
ugly.

So the seemingly logical thing to do here was to re-create the functionality
in **Ruby**, as I find well-written ruby code **much** simpler and easier
to read and understand than shell scripts. Of course people can write ugly
and confusing code in ruby too; and there is a lot of somewhat decently
written shell code spread over the www as well. But by and large, I found
it to be true that well-written ruby code tends to be a lot better to
read/maintain/change than well-written shell code, for equivalent 
functionality.

At the least for me personally, I have a hard time trying to understand
what shell scripts do in general, even though the shell scripts used in
GoboLinux are one of the easiest and nicest to read, in my opinion.
The GNU Sorcery distribution also had some good shell scripts, but the
GoboLinux scripts were better written, in my opinion.

Still, shell scripts are difficult to understand, and not very elegant,
and elegance/beauty, which is a subjective metric, was one important
reason as to why I did want to use ruby rather than shell scripts. I
prefer using elegant systems. Both GoboLinux, from a conceptual point
of view, and Ruby, are elegant systems/technologies, so it made sense
to want to combine different ideas.

Elegance/beauty/symmetry in style helps ease the **flow of mind** in
some ways. Why make something complicated if you can keep it simple?

People who may have been using other languages, such as PHP or perl,
may agree with this e. g. when comparing these languages to, say, 
ruby or python.

Anyway. This was sort of **the initial impetus** for creating the
**RBT project**. Lots of years have gone by since then ... well over a
decade, really. I can not even pinpoint to a single year when the RBT
project was started, since it started out as a set of semi-random .rb
files/scripts, but I think the rbt project was created, through
the old cookbooks project, in the year **2005**, give or take. The
file called **COOKBOOK_STATISTICS.md** keeps some old entries. For
example, in late October in the year **2005**, the old Cookbooks project,
which was eventually changed into RBT, already had **274 programs registered**,
so it had to have been started prior to October 2005. For comparison -
in the ~middle of May 2019, **3593 programs** were already a part of
the **RBT project**.

Naturally if you have a project that is that old, a lot of things
change; partially the scope of the project too, which has expanded.
The primary focus is still on compile-related activities. I use the
RBT project almost daily in order to get something to compile/install.

In some ways, this also means that one secondary goal of the RBT project
is to be able to build a LFS ("Linux from Scratch") system from scratch,
but not every functionality within the RBT project is equally well
polished or battle-tested.

Without buzzword speak, RBT itself is in many ways just a large, happy
hack project, rather than something that has a perfect design. While
a good design is immensely useful, it is not quite how I approached 
the problem domain; I more try to solve a given problem at hand, and
then move on doing other things. Of course I do try to keep the code
clean and well documented, but you know how it goes for any hobby
project. Sometimes you have more time (and motivation) and sometimes
you lack both.

As **a rule of thumb**, though - the simpler the task that you may want
to achieve through RBT, in an automatic, scripted way, the easier it
should be for RBT to be of help here.

As it was already mentioned, in the past there used to be a separate
project called **Cookbooks**. That project specifically handled the
recipes for the programs, that is, **how to install or compile something
specifically**. In the year **2018** the Cookbooks project has been
fully merged into the RBT project, primarily to make syncing the code
easier. The **RBT::Cookbooks** namespace handles the source files'
logic, whereas RBT itself handles how to interprete this information,
in particular through the main class to use for compiling something,
which is the class **RBT::Compile**.

When you have a look at the RBT project, including the documentation
on the homepage of the RBT gem, you may notice that the documentation
is quite detailed and will explain quite a bit of the **internals**,
that is, of the internal code used in RBT. This code base is
predominantly written in ruby. As such, the whole RBT project caters
**heavily** towards people who already use ruby as their main language
of choice.

Why did I decide to also document and explain some internals? After all
this will lead to a lengthier documentation. Now, there are two main
reasons for this:

The first reason is that it actually helps me remember when I made a
specific decision. When looking back at it, perhaps even years lateron,
I need to understand why I did choose to go to a particular route.

The second reason, and more important one, is that I want to treat
users as intelligent people, so I will try to explain to them how 
some of the RBT internals work. Naturally not all of RBT will be
explained that way, but more important parts, such as e. g. RBT::Compile
in particular, will have several entries that will explain what is
done, and why.

I am aware that not everyone using RBT may use ruby. For example, you
may use python as your primary language rather than ruby, but would 
still like to make use of the **yaml files** in the RBT project.

I will try to keep the yaml files and format well-documented and
well-written; this is an ongoing effort, though. Keep in mind that some
entries are literally very very old by now. (You can also compare this
to the old GoboLinux recipes, which you can find on Github. Several
of these projects no longer exist.)

You can generate a sanitized variant of these yaml files from ruby, and
then just keep on using these .yml files in other projects - after
all that is **one advantage of yaml**, that we are freed from any 
single programming language (such as e. g. hardcoding information
into a **.rb file** or a **.py file**). If you wish to generate these
**expanded yaml files**, you can use **rbt --expand-cookbooks**, but
you may wish to **set to another temporary directory** before doing
this - see other parts of this document for how to do so.

That still leaves some room to explain some of **the philosophy of
the RBT project**.

There are different philosophies, but to name a few key ones:

- Try best to solve the use cases that the user at hand has, without
getting into the way or preventing specific actions if a user really
wants to do so.

- Try to be "apolitical", "agnostic" and as flexible as possible. For
example, while I  personally do not use systemd, the RBT project in
itself has to be as flexible to allow those who use/depend on systemd
the use of RBT.

- Try to provide the most common functionality through RBT, as much
as that is useful, including acting as a (surrogate) package manager
or as a set of tools to auto-generate files. For instance, RBT 
includes one class that can modify the PREFIX variable in a Makefile.
That way people can easily change the target prefix of a Makefile
without having to use any other commandline options.

- Focus on Linux as a first-class citizen but do not exclude any
other operating system, if possible. Primary testing will happen
on Linux but other operating systems should be supported whenever
we can (if ruby works on these systems that is; but I will also
keep in mind resource-constrained systems, e. g. small/embedded
systems; for example, the RBT project can also autogenerate
shell scripts, so this could in theory be extended to work on
systems that only have shell scripts rather than ruby installed.
Do note, though, that for small systems, it is often better
to compile on a larger desktop machine than on such a small
constrained device; compiling on a raspberry may take even
days.)

## Why a new project rather than extending existing code?

This is a valid question. There are a few problems with the existing
projects out there, though.

The scope of these projects is often limited. For example, mac **homebre**w
is a package manager, but other than that it isn't doing that much; 
and it does not work that well on linux or windows - I tried it in 2019
on linux and it did not work for me. Since I felt that Linux is a
second-class citizen, I did not invest more time into the issues I have
had.

I also did want to have a project that works as well as ruby does on
any given operating system, as long as ruby is supported.

But there are also **API-differences** and the way how these systems are
used. For example, the package managers do not allow you to easily
use another prefix. Most of them, at the least on linux, assume that
**/usr/** is the only prefix that is to be used (or /usr/cellar or
something like this). I do not like this restriction - I want more
flexibility, and complete control over the prefix in use at all
times.

This list can be extended, but ultimately it was less of an issue
to create a project that has these goals in mind, rather than try
to retrofit existing code bases out there towards my use cases.

## @source_base_directory

The very important instance variable **@source_base_directory**, defined
on the toplevel **module RBT**, can be used to denote the directory
where archives of different programs are to be kept (or are
already kept).

For **example**, on my home system I store the source code to many
different programs in the directory **/Users/x/SRC/**. That name is
mostly due to historic reasons, as the user "x" was just a generic
name I would use for various different tasks, including making
regular backups.

As other users may wish to use another directory rather than the
one listed above, such as **/Files/Archives/** or **/Depot/Archive/**
or something like that, there are essentially three different ways
how to define your own target directory for keeping such
**source archives**:

(**1**) Simply set a new target altogether, via the **method** called
**RBT.set_source_base_directory()**. A shorter API exists
as well, an alias to this method, called **RBT.set_source_directory()**.

You can also use **RBT.source_directory = path**, if you would like
to.

As **argument** to this method, simply pass in the path to the directory
that you wish to use, such as **/opt/** or **/home/src/** or any other
location that you may wish to use.

Explicit examples for the API that was mentioned above, will be shown
next:

    RBT.set_source_directory('/opt')
    RBT.set_source_base_directory('/opt')
    RBT.source_directory = '/opt'

Note that this method will **ensure** that a trailing '/' token will be
the last character here. (Directories should end with a trailing **/** -
it makes the subsequent code easier when we can assume that this is
always the case. I am not entirely sure why Linux/UNIX does not seem
to share the same notion.)

The method call to **RBT.set_source_directory()** will not make the above
directory persistent, mind you, so you may want to read on for
option (**2**).

(2) Another option is to use a setting in the main **configuration file**,
typically a yaml file called **source_base_directory.yml**, the path to
the source directory can be set as well. This is the method that I use
primarily.

In the directory of the RBT project where the configuration files are
stored, you can simply use the target directory, by putting the path 
to the source archives into that file.

You can also use **environment variables** in this file - simply 
use ALL_CAPS to make use of such environment variables. (This
won't work in all settings, e. g. for .cgi files it may not
work since .cgi files may use fewer environment variables. But
for regular commandline-work, this works fine. I may use an
ALL CAPS constant such as MY_SRC_DIR there in this file.)

(3) Not everyone will be able to modify the directory where the
configuration is kept, so a third way exists, using the 
**environment variable** called **RBT_SOURCE_DIRECTORY**.

In general, RBT may use environment variables that have a leading
**RBT_** as prefix. Simply define this variable for your shell
pointing to the directory that you wish to use and it should work
fine:

    export RBT_SOURCE_DIRECTORY=/Files/Archives

The last option may work for you if **option (2)** is not available
or not possible.

## Return an Array of programs from a remote URL

In the **RBT project** there is code that allows us to query the
remote programs listed on an URL.

This was a necessary addition because there are some websites that
have made available several programs on the same URL - think of
the GNOME project or the KDE project or the LFS/BLFS project page.
These all list several programs there.

The basic API for obtaining an **array of programs** from a remote
URL is the following:

    RBT.return_programs_from_this_url
    RBT.return_program_from_this_url('https://download.kde.org/stable/plasma/5.10.5/')

It will return an Array of programs at that site, if it can be
found.

## psych or syck

You can determine **which yaml-engine to use**.

In the past, the RBT project would try to use the syck engine,
but since psych has become the default, RBT will try to make
use of psych by default. This can be changed via the configuration
file **use_psych_or_syck.yml** that resides at
**rbt/yaml/configuration/use_psych_or_syck.yml**.

A top-level API also exists to query which yaml-engine is
currently used:

    RBT.yaml_engine?

If, for some reason, you have to or want to modify this,
you can use the following API to toggle any other
yaml engine:

    RBT.yaml_engine=
    RBT.set_yaml_engine

## Configuration of the RBT project

Most of the configuration is done through yaml files (**.yml**). These
individual yaml files are normally kept under the subdirectory
**rbt/yaml/configuration/**.
         
For example, there is a file called **be_verbose.yml**. This determines
as to whether the scripts are verbose by default or as to whether they
are not. You can use shortcuts here, such as "t" which stands for true,
or "f" which stands for false. Of course you can also use true and false,
as content of said file. Additionally you can use "yes" or "no".

So if you put the string **no**, without quotes, into the file called
**be_verbose.yml** then the RBT scripts will not be verbose by default.

Similar possibilities exist for the other files that accept a Boolean
value (true/false aka yes/no). Obviously this would not work when you
are expected to input a **file path**, such as the path to a certain
directory. In this case you simply have to provide a string to the
path in question - but you can also use environment variables if you
have defined them. For example, I keep an environment variable called
$MY_PROGRAMS, so I can input that String (**$MY_PROGRAMS**) into the
file called **programs_directory.yml** and all RBT scripts will refer
to that as the main /Programs/ directory (it is indeed usually 
**/Programs/** but it could also point to my home directory or
any other target).

## How does the format of a cookbook recipe, the respective .yml file, look like?

Each **cookbook entry** is just an **individual yaml file**, with the extension
name being **.yml**. For example, information about the programming language
**ruby** will be stored in the file called **ruby.yml**; information about the
programming language **python** will be stored in the file **python.yml**;
information about the **mate-desktop** will be stored in the file
**matedesktop.yml**. If there are any "-" tokens as part of the name, then
these will be removed when rbt tries to determine the filename. 

This **yaml file** should contain all the required information to get that
particular program to install on a given system.

Additional information can be added as well, such as **maintainer**(s),
**extra information** of the program at hand, tips and hints in how to install
or use the program, sed-modifications, pkgconfig files (aka .pc files) that are
installed alongside with, patchsets that are to be applied, binaries, headers
and libraries and so on and so forth.

It can be a quite detailed file, mostly due to being able to allow some flexibility
during installation.

Many programs "out in the wild" are **well-behaving** and will honour a 
**--prefix** variable given (or **-DCMAKE_INSTALL_PREFIX=**, such as in
use by programs that depend on **cmake**, for compilation). Such programs
usually require **little to no modification**.

Unfortunately, some other programs come with **flawed installation scripts**,
hardcoded paths or workarounds that have to be employed to get these to work -
very often sed-operations, such as done in the LFS project. Some programs
may also have already become obsoleted, due to changes e. g. in glibc/gcc
and other programs. If there is no maintainer of the downstream package
then this normally means that other people have to either maintain that
package; or provide patches to get this projects to work again.

As **a reference project**, the **LFS project** provides a **lot** of useful
information in how to get many programs to work. I consider the **LFS
project** to be hugely important for the whole Linux ecosystem, much more
important than the various different distributions.

At any rate, let's now have a look at the **fish.yml** recipe, to give
a specific example and explain that.

This **fish.yml** file will look like this, as of **August 2018**:

    fish:
     binaries:
     - fish
     - fish_indent
     - fish_key_reader
     short_description: |
      FISH is an interactive (commandline) shell.
     description: |
      FISH is the Friendly Interactive Shell, a smart and user-friendly
      command line shell for macOS, Linux, and the rest of the family.
     url1: https://github.com/fish-shell/fish-shell/releases/download/2.7.1/fish-2.7.1.tar.gz
     url2: http://fishshell.com/files/2.1.0
     url3: http://sourceforge.net/projects/fish/
     homepage: http://fishshell.com/
     tags:
     - shell
     prefix: f
     keep_extracted: t
     last_update: 08 Jan 2018

The very first entry, on top, is the name of the program itself, in this
case, **fish**.

While this part could in theory be omitted (since the name of the program
is already stored as the first part of the very filename), I found that it
is useful to have this entry within the file, primarily as a visual identifier
if you work with it in an editor. It helps me a lot, for instance.

Note that no **-** or **_**characters are allowed there in this entry - it is
really a stripped down name of the program at hand only.

If the name would be e. g. "gtk-doc", with an URL such as
**http://ftp.gnome.org/pub/gnome/sources/gtk-doc/1.26/gtk-doc-1.26.tar.xz**,
then the name there for the gtkdoc.yml file would have
to be **gtkdoc:** - thus, all '-' were removed. The name that appears there
is also the name of the .yml file at hand. The .yml files may also not 
include any '-' or '_' characters. Do also note that while I am following
this convention since many years, it may one day be changed to allow for
'-' and '_'. But for the time being, the strict requirement is that we may
not have '-' or '_' characters there.

Next, let's look at the entry **binaries:**. This entry tells us which
binaries/executables this particular program will install. This also
allows us to remove binaries in the /usr/bin/ hierarchy, if you compiled
into an AppDir prefix - for more information here, see class
**RBT::PurgeBinariesOfThisProgram**.

The next entry is **short_description**, which is e. g. equivalent
to slackware's pkg-format, as introduction to the specific program
at hand, in a short notation.

Then we have the entry **main description**, followed by several **url**
entries. The first url entry, aptly named **url1**, is the most important
entry. It tells us where to find the archive to the program at hand.
This is ideal the URL to an existing archive, but it could also be
a github repository, in principle.

Then we also have the **homepage** entry, which denotes the homepage of
the program (or any other webpage that has relevant information, in the
event that there no homepage entry exists for that particular program).

Then we have **tags**: which just groups the program into popular
**tag-classifiers**, such as "shell" in this case. Then the prefix in
use, which is f for false, meaning "non-traditional" aka "appdir" prefix
such as GoboLinux makes use of. Then there is the keep_extracted 
setting, which tells us whether to keep the archive extracted or not
after extracting/compiling it. Last but not least, the last_update
tag tells us when this program was last updated. The default format
here is dd.mm.yyy, but shortcuts are allowed. For example, Oct, for
October, can be used rather than 10, and so on. 

The next entry is <b>prefix</b>. This one is mostly synonymous for
the --prefix= option given to GNU autoconfigure based projects.
A prefix value of t means true, which stands for /usr/. A prefix
value of f means false, which stands for the particular AppDir-like
prefix. In the case here, for fish, this would expand to
<b>/Programs/Fish/2.7.1</b>. GoboLinux uses a very similar AppDir
layout by default, although with slightly different program names
that can also be upcased, e. g. "KDE-Libs" and so forth. (RBT will
offer both variants here; the GoboLinux naming scheme, and the
simpler scheme where only the first character is upcased, and
the other characters are all downcased.)

There are many more entries, most of which should be documented
under the doc/ subdirectory of this gem. What follows next are
some more mentions of important (and less important) entries.

The entry **configure_base_dir: unix** determines which base
directory is to be used. In the above example, RBT would
cd into the directory called unix/. This is important for some
programs such as **tk** or **xvid** - have a look at their
directory structure to understand this setting.

Since as of December 2018 it is possible to use github URL that
are truncated. Take the program **conky**. It has an url
like this:

    https://github.com/brndnmtthws/conky/archive/v1.11.0.tar.gz

This will now be automatically expanded into the correct 
naming scheme, aka **conky-1.11.0**. No idea why GitHub is not
doing so on their own automatically; the tarball is even 
wrongly built, since it extracts to precisely that name 
after download. But anyway, ask GitHub about this, not me.

The content of these individual .yml files is essentially just
a Hash that describes the various attributes of the given
project at hand (and some additional information when it is
required for installing/compiling this program).
  
## Cookbooks - Recipes to install programs from source

The **Cookbooks module**, part of **module RBT** and thus residing
under the **RBT::Cookbooks** namespace, attempts to collect recipes
and instructions that allow you to **install programs** from source
(primarily).

This project has well over 3350 programs registered so far - and that
number continues to grow slowly.

If you are interested in seeing the specific programs and the associated
programs versions and URLs, have a look at the side pane on the right
hand side, at __**Available program versions**__.

**What is the use case for this project? How could this project help you?**

The use case for the Cookbooks component, part of the RBT project, is
to **provide the necessary instructions for compilation/installation of
programs**. These instructions will be primarily stored in <b>yaml</b>
files.

For example, the text-editor called <b>nano</b> will have some information
that details how it should be compiled, in the file called <b>nano.yml</b>.

This is valid for all programs that are registered, by the way - for each
program, there will be a single .yml file that describes that program
in question.

Now, how could this be of help to you or anyone else?

This of course depends on your use case at hand, but say that you may want
to compile a <b>linux from scratch</b> system and then a <b>beyond linux
from scratch</b> system on top of it. You can - and should - follow the
official instructions of course, but once you understand the basic concepts,
perhaps you may want to have more control over the programs at hand or
wish to compile/install more programs, including their dependencies.
That's exactly one use case for the **RBT project**, through its
Cookbooks subcomponent.

There are more use cases of course, such as updating your programs
from source quickly. Keep in mind that a lot of information is stored
in all the yaml files together - describing the program at hand, where
to find the source, how to install it, and so forth. This information
may always be helpful to some people. Many yaml files include the
homepage link, so rather than having to use google, one can quickly
jump to the registered homepage at hand (the entry in the .yml file
will be called <b>homepage:</b>).

The **Cookbooks project** is a component of the **RBT** (Ruby Build Tools)
project. The latter project, aka <b>rbt</b>, is the one that formally
integrates the dataset provided by the module Cookbooks part, into
general compile/install related activities. I use rbt to compile
programs from source or otherwise install them. 

If the cookbooks project and the RBT project are used in <b>combination</b>,
then they are in **some** ways similar to the **MacOSX homebrew project**
or the **fpm project** - sort of.

There may be some philosophical differences between these projects, but
the cookbooks project and the RBT project will attempt very hard to provide
similar functionality as fpm and mac homebrew do, too, in the long run.
Please do keep in mind that this is a hobby project here, and as a consequence,
it may not move as fast as <b>homebrew</b> does.

The similarity towards **fpm** is that the cookbooks project will also attempt
to allow you to **interconvert** the registered dataset into .deb or .rpm files
or gobolinux files or snapcraft files - or even homebrew formulas. But that
functionality is very limited (as of in 2018). It will definitely become better
over time. The slackware support is a bit better since I myself mostly use
slackware and GoboLinux, but the slackware support in cookbooks/rbt still
requires quite some more polish and clean-up before it can be easily used.

The **RBT project** is in some ways similar to the LFS/BLFS project
in that it attempts to put together information that would be useful for
when a user wishes to compile some program from source, in particular
when these programs are requiring modifications to their build system,
such as when "sed" is used - but the RBT project could also be used
when installing a binary program. In doing so, the RBT project can
also be used as a basic **building block** for creating package managers
or other compile-tools in Ruby. If you have a project like this and may
want to make use RBT, and require some modification, let me know - I will
try to adapt the project to make them useful for these cases as well.

Now - let us have a look as to **how information is stored** in the
**RBT project**.

The information for the respective programs is stored in yaml files by
default, largely due to convenience and "historic reasons". Taken together,
we could call the collection of yaml files a "yaml database" - even though
it is not a real database, per se.

These individual .yml files, by default, reside in the subdirectory called
**yaml/individual_cookbooks/** of this gem. The .yml files usually also
make use of shortcuts and abbreviations, which will be sanitized when
the specific yaml file is loaded by the Cookbooks project.

For example, a common entry point may be like so:

    "keep_extracted: t"

The "t" at the end is a shortcut for "true". The entry **keep_extracted** 
means that, after an archive is extracted, it will be kept rather than
removed, after compilation has finished.

The Cookbooks project operates under the assumption that all archives that
the user will use, are put into a certain base directory, which, for the
purpose of this file here, will be called the **archive directory**.

This is the directory where your archives should reside. You can ultimately
decide which directory that shall be.

On my home system, this archive directory is at **/Users/x/SRC/**, mostly
due to historic reasons how I kept making backups. On GoboLinux, you may
want to use perhaps /Files/Archives/ or /Depot/Backup/ or something 
similar. You can query the current archive directory in use by issuing
this command:

    cookbooks --archive-directory?

There, source files can be stored, such as **htop-2.0.2.tar.xz**
or **ruby-2.4.1.tar.gz** and so forth.

Since many other users may wish to use another base directory for
their archives, you can define another archive directory on the
commandline like so:

    cookbooks --permanently-set-base-directory-to=/opt

The above would assume that all source archives will be stored
within the **/opt/** hierarchy.

Or use the more prevalent rbt name:

    rbt --permanently-set-base-directory-to=/Mount/USB1/last_backup/SRC/
    rbt --permanently-set-source-archive=/Mount/USB1/last_backup/SRC/
    rbt --set-source-dir=/Mount/USB1/last_backup/SRC/

The above would assume that all source archives will be stored
within the **/Mount/USB1/last_backup/SRC/** hierarchy.

Since as of April 2019, the following shorter variant can also be used
instead:

    rbt --source-dir=/SRC

I added this primarily because typing the longer variants above, on a
freshly set-up computer, is somewhat cumbersome. So the shorter variant
should do fine in these cases.

Within that archive directory, subdirectories have to be created
for the respective program at hand.

Actually, ruby may already do this for you, via the **rbt project**;
but I recommend that you first start slowly and manually create
directories for now.

So, pick just some program, anyone will do fine as long as it is
registered in the RBT project. Create that archive directory if
it does not yet exist, cd into it, create the directory of the
program which is the program name (downcased, and without any
'-' or '_' tokens).

Let's break this down and use specific examples, to illustrate
what is meant - we wil pick the program "htop" as example.

First, create a subdirectory called "htop/", within the source
archive directory that you picked.

On any linux/unix system, "mkdir" should be able to allow you to do
that - but you could also use Ruby's fileutils class to do so, via
**FileUtils.mkdir_p()**.

Next, download the remote source for this program locally (you can
also use the rbt project to do so for you: **"rbt htop --download-source"**
but for the first steps, again, I recommend to do these steps manually
instead).

    wget http://hisham.hm/htop/releases/2.0.2/htop-2.0.2.tar.gz

More specifically, on my home system, I'd do the following things manually
(but keep in mind that ruby does this for me automatically):

    mkdir /Users/x/SRC/htop
    cd /Users/x/SRC/htop
    wget http://hisham.hm/htop/releases/2.0.2/htop-2.0.2.tar.gz

If you, for instance, want to store multiple versions of GLib, then a 
downcased directory called glib/ has to exist within that 
source archive directory - just as the example above shows.

Different versions of **GLib** can then be stored in that directory.

The path would, in my case, be:

    /Users/x/SRC/glib/

That's it, essentially!

Past this point, the cookbooks project can feedback some information
about htop via "scookie".

For example, to use scookie (<b>bin/scookie</b>) and thus display information
about a program in a colourized manner, on the commandline, do:

    scookie glib
    scookie htop

Take note that you can enable this through **bin/rbt** (aka **rbt** on
the commandline) as well, such as in the **following examples**:

    rbt --scookie=glib
    rbt --scookie=htop

The associated .yml file is always available anyway, with or without the
archive being available locally as well. So you can always query the
information stored in these .yml files the moment you install the
**rbt project**; it can thus function as a weak "database". (I will
at some point add **SQL instructions** in how to create a SQL database
and use it with the RBT project. But note that the .yml way will
most likely remain the default way; primary reason being out of
convenience, since it is so trivial to edit a single text file such
as .yml files.)

If you are wondering why the name "scookie" was picked - it originated
from the **sanitize_cookbook_dataset** functionality, which ensures
that the dataset is correct. I was using the abbreviation "scook" to
refer to sanitize_cookbook_dataset, but I found that "scookie" is
easier to remember. Everyone likes cookies, right?

The **cookbooks project** should now be usable to display information
about this project - provided that it has been registered of
course. If it is not yet registered, then someone has to register
it; there are some helper scripts that can aid you there, but as
a rule of thumb, I'll add new program entries when needed. They
are essentially just a single .yml file.

Currently, there are more than 3000 programs that are registered
in the Cookbooks project; almost 3200 already and this number is
very slowly continuing to grow.

Cookbooks may also keep track of information such as whether
a given progam at hand will install a .pc file (pkgconfig file).
This may help if you are on a non-GoboLinux system and wish to
find out which particular program installed which specific
.pc file, too. (On GoboLinux this is trivial to find out since
it keeps AppDir-related and versioned programs, whereas on
the traditional FHS preferring /usr prefix, the individual 
.pc files would usually reside under /usr/lib/pkgconfig/, 
without always giving you a simple way to ind out as to where
they came from.)

You can query this by using "scookie", which will show this, and
more, in a colourized manner.

An example follows:

    scookie libvorbis

If you wish to support this project, for now please just test it and
report any bugs or report inadequate or confusing documentation.

The project is still in a beta stage and it will take quite a long
while before it can leave that beta stage.

## Obtaining the number of registered programs

You can obtain the **number of registered programs** from the
commandline by issuing any of the following:

    cookbooks --registered_programs?
    cookbooks --n_programs?
    cookbooks registered_programs?

It also works like so:

    rbt --n_programs?

Although the output may be a little bit different.

This would then output something such as this:

    -> 3330 programs registered as of 30 January 2018.
    -> 3414 programs registered as of 18 September 2018.

## Brief commandline introduction

If you wish to see a **brief introduction on the commandline**,
issue the following command:

    cookbooks --tutorial

This tutorial is really just very short - the more verbose,
long documentation here may be sufficiently more useful. :)

## Optional dependencies for the RBT gem

The **wget gem** is recommended, but you can also use the RBT
project without the wget gem. In that case, if the wget gem is not 
available, the RBT project will attempt to use the wget 
binary via a **system()**-call, hence an "external" program,
to ruby.

This requires the installation of wget e. g. from this URL:

    http://ftp.gnu.org/pub/gnu/wget/?C=M;O=D

Or any other way that your operating system supports.

Of course, most Linux systems will have "wget" available by
default, so the above more applies towards operating systems
such as windows (although more recent windows versions
also include a linux-subsystem including bash, the WSL).

## Increment Program Version

class **RBT::Cookbooks::IncrementProgramVersion**, stored in the
file **rbt/utility_scripts/increment_program_version.rb**, can be
used to **automatically update a local program version** to a
newer version, provided that the new version can be "guessed"
by the script.

Let's use a specific example, because it wil be easier to explain
this when we can be specific.

Take the program called **wget**. Let's say that we wish to
increment the program version of wget to a more recent one.

An **older version**, as a tarball release, may exist at
a remote location such as this one here:

    ftp://ftp.gnu.org/gnu/wget/wget-1.19.1.tar.gz

The (more recent) version in use may be for version
<b>1.19.2</b>.

**class IncrementProgramVersion** will first test whether a
remote URL such as:

    ftp://ftp.gnu.org/gnu/wget/wget-1.19.2.tar.gz

**exists**. If it does exist, it will be downloaded and the
new program version will be changed towards **wget 1.19.2**.

This can be done for all programs, and it allows a simple
update-mechanism for individual programs.

If you have to or want to batch-update several programs,
this is also possible for some of the *nix stack. For example,
there are some classes that can download the whole KDE5 stack;
have a look at the KDE entry or the mate-desktop entry, in
this (autogenerated) README.md file here.

## Commandline ways to manipulate the dataset

There are some classes that can be used to manipulate the dataset
stored in the individual cookbooks files (the corresponding .yml
file). This is presently quite limited, though.

The only commandline-manipulation that is currently available,
is done by **class RBT::Cookbooks::ToggleKeepExtractedValue**.

This acts as a **toggle-switch**. If you invoke the class, and pass
the name of the program at hand, then the keep_extracted: value
will be toggled, from false to true and from true to false.
    
## Additional information

Consider having a look at the **doc/ subdirectory** of this project's
gem, for some more information. Another useful subdirectory may be the
**lib/yaml/** subdirectory, which also contains some specifications
such as **specification_of_registered_cookbook_entries.yml**.

## Validate the cookbook recipes

You can invoke, from the commandline, the following:

    cookbook --validate-entries

This will validate the dataset stored in the various .yml files.

It will run the test-code in the validation/ subdirectory.

This is not so useful for casual users though, more interesting
for people who may want to contribute recipes and such.

## Stable software versus Unstable software

This subsection here explains how the RBT project currently (in
January 2019) views stable and unstable software releases
**upstream** - that is, by other developers, companies and so forth.

From my own experience, unstable releases may bring in new features,
but this often comes at a cost. Compilation errors may happen; bugs
may happen while running the software. What is even worse is that
other software may depend on functionality that may be changed 
during an unstable release. (While it may also be changed in a 
stable release eventually, stable releases in general often have
problems fixed whereas unstable releases often do not.)

In particular the latter is one reason why I focus on stable 
releases - there will be fewer issues in the long run, thus 
easening the maintenance burden. That also means that the Cookbook
projects tries to be **moderately conservative** - stable releases
are favoured, unstable releases are often skipped. For example, 
I skip odd-versioned releases of the GNOME project and also of
the KDE project. Waiting for stable releases just a little bit
longer is worth the time, IMO.

This is a loose philosophy, not a strict policy, so expect some
deviations - but in general, as a rule of thumb, there will be
stable releases preferentially in the RBT project. (In
the future we may have to change the cookbook entries to denote
whether a release is stable or unstable, but for now, in the
beginning of the year 2019, this is how the RBT project will
handle updates of any its registered programs.) 

## The difference between "build tools" and a "package manager"

A **package manager** allows you to handle packages in one way or 
another; installing, removing, tweaking, changing, activating,
selecting software. Things like that. As such, a package manager
is very useful in itself - it can help keep your system up-to-date
with new, or patched, **packages**.

**Build tools** in this context, though, are much more general. The
above description of a package manager can easily fit into a set of
build tools as well, but the scope of "build tools" is overall 
simply larger and more inclusive.

**Build tools** may also include helper code that can fix build-related
problems, problems in Makefiles, act as sed-wrapper, and so on and
so forth. A **package manager** on the other hand rarely needs to
tamper with sed instructions, since that should have already been
handled by the developers who maintain the packages for a particular
distribution.

So the scope of the build tools is larger and not as narrowly defined as
just a "mere package manager" alone. Another good comparison here is the
**ublock origin** extension or the **adblock plus** extension - the
latter wants to block only or primarily ads. The former allows you to act
as a "general content blocker" under your control, which CAN include 
ads but can also block generic content, which makes it a lot more powerful
and flexible in its approach.

I consider the approach taking up by **ublock origin** option to be far more
useful to users in general, so this is in some ways how one could look at
the difference between a "package manager" per se, and "build tools" on
the other hand, too.

**Build tools** could easily include a package manager but a package manager
will probably never include build-tools related helper scripts as its primary
means to go about handling anything. The RBT project is going the "build
tools" route - it attempts to collect code that can help with building,
installing, compiling packages in general, but also automatic patching
and automatic generation of files, and so forth.

If you wish to install the **rbt** project, via RubyGems, you can issue
the following command:

    gem install rbt

Do note that some functionality can work outside of the RBT::Cookbooks
namespace, such as **RBT::Libtool** related actions.

Now, when the above installation method has worked - and if it has not,
please do consider reporting this - you should be able to compile
something via the RBT scripts.

Let's start with "htop" as example, a useful program written by
one of the original creators of the **GoboLinux** project:

    rbt htop

This should, if everything works fine, download htop, extract
and compile it.

If you wish to use another prefix, such as:

    /Programs/Htop/2.0.1

you can do this:

    rbt htop --appdir
    rbt htop non-traditional
    rbt htop ntrad

ntrad stands for "non-traditional". Note that the entry is **NOT**
hardcoded towards **/Programs/** - you can define your own app-dir
prefix and even make use of $SHELL variables. For example, I point
to this directory via a custom variable called $MY_PROGRAMS, which
is stored in the file programs_dir.yml that is part of the RBT
project. More about this elsewhere though - for now just keep in
mind that this will install into a versioned AppDir directory, but
you can of course use any other variant such as "/pkg" or "/Programme"
or "/opt" - whatever. Note that you can omit the trailing '/'
there, so "/Programs/" is the very same as "/Programs" -
the RBT scripts will internally make sure that this will be
a directory anyway, and thus have a trailing **/** character.

**AppDirs**, for the purpose of this document, means that what belongs
to one program, goes into the same, versioned directory.

You can also use another prefix, in particular /usr. This
is called the "traditional" prefix. In the respective 
yaml file, the cookbook file, this entry is called "prefix: ".
If you see a "t" there then this is short for "traditional",
and it will be expanded towards "/usr" when used. Whereas
the character "f" will be expanded to mean "non-traditional",
thus AppDir prefix. Do note that you can also use shortcuts
there and absolute targets - the reason why single characters
are used is largely to save me, but also others, some typing
work. If you want to use the longer variants, you can do so
just fine, but I assume that after some typing work, you
may be bored of having to type long words. :)

Anyway, here is the example to use **/usr** as prefix:

    rbt htop --traditional
    rbt htop --trad
    rbt htop trad

Use any variant there. I prefer the last one, "trad", because
it is shorter to type.

If you wish to look at the available help options that are
available for rbt, do:

    rbt --help

Note that this will showcase mostly just the more common
ones. There are many more options that can be used.

There is a fairly large tutorial available as well. I 
have to publish it in .pdf format eventually.

## Colourizing output

In general, --disable-colours or --disable-colours or --nocol
should **disable colour output**. The rest of this subsection here
explains some of the colour-settings used by RBT.

By default, RBT will attempt to colourize output from some
external programs, in particular "make" and "make install".

The class **RBT::ColourizeParser** will handle most of this.
It can also be used in standalone scripts such as ColourMake
or ColourMakeInstall, which will run a coloured variant of
"make" and "make install".

The idea behind the colours is to mostly make it easier to detect
what is going on; and in particular where errors may occur. I
personally found the colours to be helpful, but it is indeed VERY
colourful, which is why a commandline option exists to disable
the colours.

The colours are presently hardcoded, but in the future we could use
different profiles and then let the user decide which (other) profile
to use. But I'll extend this part only when other people may want
to have this - for now, the default colour "profile" has to suffice.

If you wish to **permanently disable the colours** for the RBT
project, then you may use the following command:

    rbt --permanently-disable-colours

And to permanently enable the colours again, issue:

    rbt --permanently-enable-colours

Note that there are also a few classes that can handle colours
or colourizing. In particular, class **ColourizeParser** will
take any external output (e. g. output that has been taken
via system()-like calls by ruby) and colourizes it.

The general idea here is to make it visually easier to see
which problem may have been encountered while compiling
a program.

Users of RBT should be able to fully disable coloured output
at any moment in time.

## (Auto)Registering problems

The RBT suite attempts to identify errors, register them and
sometimes report these problems. The main idea behind this is
twofold:

    (a) To be able to auto-correct reported (and registered)
        problems.

    (b) To notify the user and optionally provide information
        as to why the error happens and how it could be fixed.

Ideally, autocorrecting would work to the extent that all
problems that occur, could be automatically solved. But
this is not trivial; while some problems can be quite easily
autocorrect, others can not. And sometimes there is ambiguity
so only the user can decide which variant is the correct one.

This is why the **notification part** is more important than the
**auto-correct part**. See the next subsection for some
autocorrection that can be done.

## Registering errors during compilation/installation

Some problems and errors may happen during compilation or
installation of programs.

These are handled internally by RBT::Compile, but they will
also be registered in a module-method onto the RBT Namespace.

It is available via the following code:

    RBT.error_message?

Note that this will be reset to nil, its deault value, whenever
a new program is compiled. 

## Autocorrecting some errors

The RBT scripts can attempt to auto-correct some errors. This
is presently in testing. The few errors that RBT can try to
auto-correct are mostly related to libtool.

If the file use_autofixing.yml contains a String called t or
true, meaning "enable autofixing", then auto-correcting will
be used. There is another option that has to be set to true
though, which is the "is_on_roebe?" settings, which indicates
my home directory use. Presently (Sep 2017) this option does
not work too reliably, so I am still testing it. You can 
test it too though if you enable "is_on_roebe".

## Some commandline options explained

If you do not want to pick up the configure options from a
specific .yml file, you can pass in the option
**--dont-use-configure-options** to avoid doing so.

Examples:

    rbt htop --dont-use-configure-options
    rbt glib --user-home-prefix --use-meson --no-configure-options
    rbt ruby --do-not-use-any-configure-options

This allows you to just use the "barebones" compilation mode;
may be helpful if you encounter some errors due to the 
configure options used.

If you want to find out which were the last 20 programs that were
compiled, then you can use this commandline option:

    rbt --last-compiled

## Finding all programs that start with a specific letter

If you wish to find all registered programs that start with
a specific letter, such as k, do this:

    rbt k*

This will output all programs that start with k and also
tell you how many programs with that letter exist.
  
## Status of the RBT project

The RBT project is currently (December 2017) in a testing stage,
somewhat "semi-complete". Or at the least, "sufficiently useful"
for my use cases.

**I do not recommend anyone to use this project as of yet, but
fearless testers could test it**.

Please be wary when using the RBT project, as there may be some
bugs, but also behaviour that may be surprising to you.

It will be "cleaned up" at a later stage.

The RBT project makes certain assumptions, a certain layout
of the filesystem, and so forth. While this can be tweaked,
for the time being, it is best to just follow these assumptions
for now. 

## Cmake

**Cmake support** in the RBT scripts exists, to some extent.

If you want to compile specifically via cmake, then you can
use the following instruction on the commandline:

    --use-cmake

Note that the target build system has to support cmake. Auto-generating
cmake files does not yet work for the **RBT gem** alone.

## Using meson

You can use the build-system meson but keep in mind that this
requires python version 3.x and the build tool called "ninja".

You can then add a dependency on meson in the individual cookbook
file, at **required_deps_on:**, or you can use meson via the
commandline switch **--use-meson**.

Usage example:

    rbt glib --use-meson

Note that presently (2018) this does not work perfectly well,
which may be understandable since meson/ninja is fairly new,
compared to GNU autoconfigure-based systems.

## Base subclass

The **RBT project** has a **base subclass**, residing at **rbt/base/base.rb**.

Most classes in the RBT namespace will inherit from this base class.

There is another class, called prototype.rb, which is pulled in by
base.rb. The reason why prototype.rb was needed is to avoid circular
dependency warnings - prototype.rb is much smaller than base.rb and
provides mostly just the core functionality that is sufficiently
useful for several classes.

## Aliases for program names

The cookbooks project makes use of several aliases to
program names. This is mostly a convenience feature.

Sometimes this may lead to unexpected or unwanted results,
such as when you want to compile program **a**, but the
**rbt project** renames the program to **b** instead.

In order to avoid this, a commandline-option exists for
the RBT project.

Invoke it like so:

    rbt sdlttf --do-not-use-cookbook-aliases
    rbt sdlttf --do-not-use-cookbook-alias  

This will specifically **disable** making use of <b>cookbook
aliases</b>. (That functionality only works if the Cookbooks
gem is used. It resides within the source of the Cookbooks
project rather than the RBT project.)

## Show the last configure option

Since as of January 2018, a way exists to show the last configure
option used for a given program. This depends on the file
<b>configure_command_database.yml</b>.

If this file exists, and if the sought program has been compiled
before (meaning that RBT has stored the configure command used
in that .yml ile) - say, the program <b>attica</b> was compiled,
then you can query this information from the commandline, via:

    rbt attica --show-last-configure-option

This could then be used to copy/paste, or modify if you experience
some problems related to compiling a program from source, for
instance.

Of course you can also always peek into that .yml file on your
own. Thankfully yaml is sufficiently readable, more so than
big, clunky XML files.

## Timing of the compilation (duration)

Since as of January 2018, code exists to tell us how long it
took to compile a given program at hand.

This requires the configuration option time_compilation set to
true it the corresponding yaml file (within the <b>configuration/</b>
subdirectory).

If this setting is set to <b>true</b>, then RBT::Compile will show, on the
commandline, how long it took to compile a program, in seconds (and also
in minutes). The default for this setting is <b>true</b> as of <b>January
2018</b>.

Note that the time it took to compile the given program at hand will
only be shown if there was no proble/error during the compilation
process. This may be fine-tuned at a later time, but for now, the
functionality at the least exists as part of <b>RBT::Compile</b>.

The time it took, in seconds, will also be stored in the file
called <b>database_storing_compile_times.yml</b>, in a simple key-value
hash. (This file will be generated into the <b>rbt_logs/</b>
subdirectory.)

If you compiled several programs via rbt, you can also show how 
long it took to compile these programs.

The commandline option for this is:

    rbt --show-compile-time-statistics

## Statistics

The RBT project will also keep track of some statistics.

For example, class <b>RBT::CompileStrategies</b> can be used to
query how many programs use meson, python, ruby or cmake, in
order to be compiled or installed.

A module-method allows you to query this, via:

    RBT.compile_strategies?

And from the commandline, you can invoke it via the following
invocation:

    rbt --compile_strategies?

If you would like to see a more comprehensive overview over
the whole RBT project, and associated statistics, you can
do so via either of the following variants:

    rbt --stats?
    rbt --stats?

This will essentially invoke class **RBT::ShowStatistics**.

The cookbook-submodule also has some statistics, which can be
seen here in the file **doc/COOKBOOK_STATISTICS.md**:


<pre>
# =========================================================================== #
# This file here (COOKBOOK_STATISTICS.md) was probably created in
# September 2005. It was later renamed from source_files.yml to 
# cookbook.yml (on the 04.05.2006), before it was then eventually
# renamed to COOKBOOK_STATISTICS.md years lateron.
# =========================================================================== #
# - How many programs do we have registered in the RBT project?
#
#   ->  274 programs registered as of 25.10.2005
#   ->  340 programs registered as of 31.12.2005
#
#   ->  851 programs registered as of 28.04.2006
#   -> 1114 programs registered as of 11.07.2006
#   -> 1168 programs registered as of 01.08.2006
#   -> 1195 programs registered as of 07.08.2006
#   -> 1230 programs registered as of 24.08.2006
#   -> 1245 programs registered as of 31.08.2006
#   -> 1270 programs registered as of 16.09.2006
#   -> 1315 programs registered as of 15.10.2006
#   -> 1352 programs registered as of 13.11.2006
#   -> 1402 programs registered as of 27.12.2006
#
#   -> 1432 programs registered as of 28.01.2007
#   -> 1452 programs registered as of 07.02.2007
#   -> 1506 programs registered as of 17.03.2007
#   -> 1547 programs registered as of 19.04.2007
#   -> 1584 programs registered as of 24.05.2007
#   -> 1598 programs registered as of 04.06.2007
#   -> 1649 programs registered as of 01.07.2007
#   -> 1704 programs registered as of 05.08.2007
#   -> 1736 programs registered as of 01.09.2007
#   -> 1822 programs registered as of 27.09.2007
#   -> 1835 programs registered as of 03.10.2007
#   -> 1887 programs registered as of 27.10.2007
#   -> 1931 programs registered as of 30.11.2007
#   -> 1994 programs registered as of 30.01.2007
#
#   -> 2107 programs registered as of 05.04.2008
#   -> 2172 programs registered as of 02.06.2008
#   -> 2202 programs registered as of 29.07.2008
#   -> 2256 programs registered as of 12.12.2008
#
#   -> 2269 programs registered as of 31.01.2009
#   -> 2288 programs registered as of 03.05.2009
#   -> 2318 programs registered as of 01.09.2009
#   -> 2325 programs registered as of 24.11.2009
#   -> 2328 programs registered as of 08.12.2009
#
#   -> 2347 programs registered as of 31.03.2010
#
#   -> 2373 programs registered as of 28.02.2011
#   -> 2380 programs registered as of 19.03.2011
#   -> 2393 programs registered as of 25.04.2011
#   -> 2482 programs registered as of 29.08.2011
#   -> 2491 programs registered as of 13.10.2011
#
#   -> 2516 programs registered as of 04.01.2012
#   -> 2270 programs registered as of 28.07.2012
#
#   -> 2518 programs registered as of 25.09.2013
#   -> 2524 programs registered as of 25.10.2013
#   -> 2540 programs registered as of 20.11.2013
#
#   -> 2600 programs registered as of 26.01.2014
#   -> 2609 programs registered as of 26.02.2014
#   -> 2628 programs registered as of 22.06.2014
#   -> 2642 programs registered as of 12.07.2014
#   -> 2644 programs registered as of 14.09.2014
#
#   -> 2652 programs registered as of 10.01.2015
#   -> 2742 programs registered as of 04.06.2015
#   -> 2780 programs registered as of 26.07.2015
#   -> 2781 programs registered as of 02.08.2015.
#   -> 2795 programs registered as of 06.09.2015. 
#   -> 2800 programs registered as of 24.09.2015.
#   -> 2803 programs registered as of 28.09.2015. 
#   -> 2817 programs registered as of 26.11.2015.
#
#   -> 2903 programs registered as of 01.04.2017.
#   -> 2976 programs registered as of 09.08.2017.
#   -> 3231 programs registered as of 23.10.2017.
#   -> 3264 programs registered as of 01.12.2017.
#
#   -> 3314 programs registered as of 04.01.2018.
#   -> 3405 programs registered as of 25.08.2018.
#   -> 3444 programs registered as of 14.10.2018.
#
#   -> 3582 programs registered as of 14.04.2019.
#   -> 3594 programs registered as of 12.05.2019.
#
# =========================================================================== #
</pre>



## systemd

I personally do not like systemd in the slightest nor can I align
with the, in my opinion, absolutely unnecessary and constantly
changing goals of the ever-expanding systemd project or the attitude
of the main systemd developers. It seems as if systemd's primary job
is not to solve any user-related problems, but instead to be of the
main benefit to those who wrote, created and maintain it, primarily.

This subsection here is, however had, not about projecting my personal
opinions about systemd onto others - this subsection is **how systemd
relates to RBT**, if it does so at all.

The RBT project is "apolitical", that is - it is for the people who
use it, the **users**. If users want to use systemd then this is
fine. And if users do not want to use systemd then this is fine as
well.

Note that the <b>LFS</b> (linux from scratch) project follows a similar
philosophy. It has a non-systemd and a systemd variant, so you can pick
either variant **at your own discretion**.

Compiling and installing programs should work at every moment
in time, as far as the **RBT project** itself is concerned. At
the least RBT will attempt to **make things work** and aid a
user in precisely that way.

However had, the **RBT scripts** are predominantly tested on
non-systemd linux systems, so it may be that parts of RBT
have to be changed/adjusted if you are using the RBT project
on a systemd-dependent setup.

I will accept bug fixes and behaviour changes in regards to
compliance with systemd systems **IF** non-systemd systems
can continue to work just fine as-is. That is, **changes may
not happen in an exclusive manner** - changes that do will be
rejected. They will of course be accepted if it does not 
break this assumption and if they are in the spirit of the
RBT project itself.

The primary reason for this here does not relate to my 
personal opinion either, but the simple question of the
amount of time that is available to me, which I invest
into various projects already. (RBT is only one project
among many more; and that's just computer-related
projects. I have many non-computer related projects,
hobbies and things that require time investment.)

## Creating a .yml yaml file for a specific program

If you modify a .yml file, from the rbt project,
and then wish to autogenerate a new expanded cookbook
dataset, then you can use this commandline switch from
within rbt to do so, too:

    rbt --create-yaml-file-for=subversion

This requires the cookbooks gem to be installed.

You can do so from the rbt project already as-is, so
why was this functionality added to rbt?

Mostly out of convenience. I feel that it may be simpler 
to make available functionality that is somewhat common
between the two projects (cookbooks and rbt) from rbt
itself too. After all, rbt is the project that sort of
builds upon - and thus extends - the cookbooks project.

## Issues and Problems with Compiling in general

This subsection is **not extensive**; it only provides some
information for somewhat advanced users. In other words, for
those people who already have compiled some programs more
or less successfully.

Some programs may fail due to various reasons, some of
which may be hard to debug.

For example, perhaps you may run into a "bad file descriptor"
problem one day. I did so in March 2018. The fix for me
here was to change the $PATH setting.

You can modify the $PATH setting for rbt too at runtime.

For example, to prioritize on <b>/usr/bin/</b>, you can use
this commandline switch:

    rbt --traditional-path

Of course you can also change the $PATH variable on your own
permanently - environment variables are in general available
in ENV, in ruby files.

## RBT_SOURCE_DIRECTORY

Since as of <b>April 2018</b>, there is a way to specify
another target for the <b>SRC_DIRECTORY</b>, that is - the
directory under which all other local source-archives
reside.

On my home system, the source directory resides under
<b>/Users/x/SRC/</b>.

If there is an environment variable called <b>RBT_SOURCE_DIRECTORY</b>
defined on your system, then this is the variable that we will
use across RBT for the source directory.

If there is no such variable then, by default, we will use
the setting specified by the <b>Cookbooks</b> project, if it
is available.

Do note that the variable <b>RBT_SOURCE_DIRECTORY</b> will
<b>overrule every other setting</b>, except for values
set from the commandline to RBT::Compile, such as via
<b>--use-this-source-directory=</b> - so use this variable
with care. If you do not need it, I recommend to simply not
set it at all; it really only exists in the event that
there are no better alternative ways, or because the 
alternative ways may be too cumbersome to use.

If you wish to find out the current source directory, you can
use either one of the following commandline flags:

    rbt --source-dir?
    rbt --source-archive?

## class RBT::SystemCompilePossibilities

class RBT::SystemCompilePossibilities is on my todo list. It 
essentially attempts to inform you of the compile-possibilities
of the given system at hand; and will also compile "missing"
or outdated programs. So it works as some sort of information
and auto-compile tool.

It is presently (middle of May 2018) in a very early beta stage.
Do not consider it to be complete as of yet - I will have to test
it extensively yet. The idea is to also use this as a basis for
an "automatic linux from scratch" system - or to complement it.

## Prefixes to use

The most common (default) prefix on most linux systems will be **/usr/**
or **/usr/local/**.

That is, if a program is compiled from source, and no explicit --prefix
is used, then most programs will default towards **/usr/local/** as
prefix. 

Sometimes **/opt/** is also used as prefix; and sometimes we can use
versioned AppDirs (Application Directories) or compile into the home
directory. The latter two ways were what the GoboLinux creators used
on university clusters.

Some convenient shortcuts exist here, in order to make working with
prefixes simpler:

    ry m4 ULOCAL

This will use the **--prefix=/usr/local/** setting, so it is
<b>exactly</b> the same as if you would have been issuing this
instruction:

    ry m4 --prefix=/usr/local/

So the only real benefit of using upcased ULOCAL is that you save
a few keystrokes - which is a real benefit for people who like
to type as little as possible. :-)

Let's look at another example:

    ry m4 --opt_prefix

This will use **--prefix=/opt/**.

If you want to, you can also compile into your $HOME directory
and build up an AppDir layout; this is how GoboLinux has been
created actually, a long time ago:

    ry htop --appdir-into-home-dir

If you only wish to compile into the home directory, without
using the AppDir approach, then you can use the following
invocation:

    ry htop --compile-into-home-dir

Do note that since as of the 26.01.2019 (26th January 2019) you
can also use **RBT::CompileIntoHomeDir.new** if you wish to
compile into your home directory. There is also a bin/ executable
available, called **home_dir**, so if you wish to compile htop
into your home dir, via proper prefix, you could use:

    home_dir htop

This is functionally equivalent to:

   ry htop --prefix=/root/

if you are the **superuser**; and if you are a regular user then of
course it will use your default **HOME_DIR** setting (usually
somewhere under **/home/**).

Note that **the last prefix** that was **used** for any given compilation,
is always stored on the module RBT "namespace", that is, the toplevel RBT.

You can query its value through the following ruby code:

    RBT.last_prefix_used?

This will either return nil, if no prefix was used yet (which
is the case on startup of the RBT project, for example), or
it will be a String that points to the last prefix that was
in use. That way, if other projects want to find out what
prefix was used in the RBT project, they can do so through
using the above method invocation.

## Determine how many core/processors the computer has

If you wish to find out how many processors/core a target
machine has, you can use any of the following two invocation
examples:

     rbt --n-processors?
     rbt --n-cores?

The code for this resides in the method
**try_to_output_how_many_processors_this_computer_has**, which
is part of **class RBT::Compile**. (This statement is correct at
the least for **October 2018**).

## Compile/Installation shell

Since as of <b>January 2018</b>, code exists that allows us to
**batch-compile programs**, via a shell-input. It also allows
us to query installation options and scan for what is
installed, where these programs are installed exactly as
well, and so forth.

This is <b>experimental</b> - do not expect for this to
work very well as of yet.

The future goal is that we can use this <b>special shell</b>
to easily install/remove any given software on any given
computer system at hand.

The <b>namespace</b> for this code currently resides
under <b>RBT::Compile::Shell</b>.

For the available options of the compile-shell, input
"help" for help (or just "h").

The general idea behind the compile-shell is that we can not
only batch-compile, or easily compile one program after the
other - but that we can also modify and control the compilation
process via a shell-like interface.

If you wish to change the prefix for compilation, this can
also be done, e. g. via:

    gmp trad

This would compile GMP with the traditional prefix
aka <b>/usr/</b> prefix.

If you want to find out which were the last n programs
that were updated, say n = 35, then you can do this in
the <b>compile-shell</b> interface:

    last_update? 35

The idea is to also have the compile-shell as some kind
of guide or tutor that can properly explain some cave-eats
or problems and make some "semi-seducated guesses", in
order to fix this. But this is a long-term goal - for
now, the primary purpose of the compile-shell is to aid
in batch-compiling programs.

Since as of March 2018, an autocompile-option exists in
the shell. This is currently unfinished but in the future
the idea is to allow us to automatically compile to the
most recent program versions, without necessiting user
input. This would then allow us to update a linux-system
to the most current program versions - stay tuned for
more improvements here in the future.

You can also edit the cookbooks, if you have the cookbooks
gem installed, from within the compile-shell, via:

    edit bluefish
    edit vim
    edit ruby

And so forth. This is only useful if you have the cookbooks
gem, since all modifications will take place on the <b>Cookbooks</b>
namespace there, rather than <b>RBT</b>.

You can query the main prefix in use via:

    prefix?

To set a new global prefix, do something like this:

    set_prefix=/usr/local

## Updating all registered gems

You can batch-update all <b>registered rubygems</b> via:

    rbt --update-all-gems

Note that this presently will only work if you use the Cookbooks
project. I am open to extend this functionality to allow other
projects to use another dataset, though.

The boolean constant **ALSO_AUTOMATICALLY_INSTALL_THE_UPDATED_GEM**,
defined in class RBT::Compile, controls whether the updated gems
will automatically be installed by the RBT project. If set to
true then RBT will not only download the respective gem, but
also install it. Since not everyone may want this, the constant
handles this case for the time being - I personally like this
constant set to true, since it saves me some keystrokes in the
long run.

If you do not want to, or can not, update all registered .gem 
entries, yet you still want to install the .gem files that are
registered, then you can use this commandline:

    rbt --install-all-rubygems

This will install all registered .gem files, in an alphabetical
manner. Dependencies will not be checked that way, but in
principle if all .gem have been properly registered in the
Cookbooks project, then this batch-gem installation should
work just fine.

## build_systems_priorities.yml

The file **build_systems_priorities.yml** simply determines which
build systems are to be prioritized over others. The entry on top
is the main one; then comes the entry at second position, then
at third and so forth.

Some programs may come with more than one way to be compiled, 
such as some gnome-related software (they switched or are 
switching from GNU autoconfigure to meson).

The reason why the file **build_systems_priorities.yml** had to
be created was so that we can favour one build tool over the 
other. Some configure-flags are not valid in every build tool
or may have to be changed, which is why this yaml file had to
be added. 

## ccache

There is a configuration setting, <b>use_ccache</b>, which, when
enabled, attempts to **make use of ccache for compilation**.

This may speed up repeated compilation cycles of the same program
quite efficiently.

If you do not need ccache or do not have it, you can disable the use
of ccache permanently, as far as RBT is concerned, via:

    rbt --permanently-disable-ccache

Alternatively you can also modify the file
<b>rbt/yaml/configuration/use_ccache.yml</b>, which is
where the configuration setting as to whether to use
ccache or not, resides. The allowed values are
<b>true</b> or <b>false</b>, <b>yes</b> and <b>no</b> (for
all configuration values, by the way).

If you only want to **disable ccache for the current run**, then you can
use any of the following variants (here we assume that the target program
you wish to compile is <b>fluxbox</b>):

    rbt fluxbox --disable-ccache 
    rbt fluxbox --do-not-use-ccache
    rbt fluxbox --no-ccache
    rbt fluxbox --ccache-

## Auto-resolve compile-related problems

Note that since as of September 2017, the **RBT scripts** can also
give some advice how to resolve some compile-related problems.

This is currently fairly minimal, but it will be extended 
subsequently in the future. It was also a reason why the namespace
Libtool has been integrated into RBT, as **RBT::Libtool**. This allows
us to auto-correct some libtool-related problems and keep all
libtool-related code within that namespace.

## Available programs and the corresponding program version

Every time the cookbooks gem is updated, a .html file is created
and uploaded at this location:

    http://shevegen.square7.ch/programs_version.html

This remote .html file will show which programs are registered, part of
the RBT project, and what the corresponding program versions are.

Note that you can also generate a .html page for some select programs,
via the commandline. For instance, if you wish to show ruby, python
and php, then you can use this commandline invocation:

    cookbook --create-html-page-for-these-programs=ruby,python,php

The input program names should be separated via **,** and not 
include any '-' or '_' character. There are also some ways to
display the full, e. g. **KDE stack**. More on this in another
section of this README file.

## Checking for binary duplicates

If you ever want to find out whether you have **binary duplicates**
at two different locations, such as at **/System/Index/bin/**
and at the more traditional **/usr/bin/** location, then you can
use the following method:

    RBT.check_for_binary_duplicates()

The code to this method resides in the file
**rbt/toplevel_methods/check_for_binary_duplicates.rb**.

You can also invoke this from the commandline, of course.

Usage examples for the commandline invocation:

    rbt --check-for-binary-duplicates
    rbt --binary-duplicates
    rbt --report-binary-duplicates
    rbt --report-duplicates

## Future Goals

The RBT Cookbooks project may not necessarily have truly grand goals 
as the scope is quite limited.

The primary project's goal is to gather information about source code,
and to present and make use of that data to other programs or suites,
be it on the commandline or via a GUI or via the WWW. Thus, the project
has to be a practical project rather than a visionnairy one.

But there are some **future goals** nonetheless.

- One goal is to translate the instructions stored in the various
yaml files, into **specific SQL instructions** which can be used to
populate a SQL table - this is currently (October 2017) not working,
but stay tuned; it will eventually work and be properly detailed
and documented. The idea behind this is to be able to store all
the data stored in the various .yml files, into a single database
instead.

- Another goal is to use the project in order to create a full
LFS/BLFS build, in combination with the RBT project.

- And yet another goal is, similar to the point above, to turn
an existing Linux-system into a Linux-system based on an **AppDir
approach**. Note that this is also covered by the RBT project
primarily, and it does not work yet (Jan 2018). That means that
rather than having files spread all over the place on a system,
they will be contained in the respective directory of the given
program at hand.

Stay tuned until these goals have been reached.

## Compile hooks

**Compile hooks** are defined in the **rbt gem**, in the file
**compile_hooks.yml**

In that file you can hook the compilation of some programs to the
compilation of other programs.

This is somewhat similar to the chained programs.

For example, if you compile **ffmpeg**, and have mpv installed,
then a new compilation of ffmpeg will trigger the re-compilation
of mpv. This will only be done if you have mpv installed
at $PROGRAMS/Mpv though.

<b>Be careful with this, though </b>- it can lead to problems,
in particular when compilation may fail. I do not know whether
I will keep this functionality (as of the year 2018); the future
will tell.

For now I recommend to use this feature only sparingly, if at
all. Manual control over the process may be simpler in the long
run.

## Advanced usage of the RBT project - helpful information

This **subsection** here will explain **how to use rbt on a linux
system** - at the least how I use it.

The subsection will also give some advice and it is intended largely
for somewhat more advanced users that may already know a thing or
two about ruby and linux systems.

The whole RBT project is somewhat inspired by the **Linux from scratch**
project.

That is, people who may want to maintain their own system, the way they
want to. This is also one focus that the RBT project has - to be able
to compile any program and <b>have things work</b> as-is. The fpm
project, for example, also follows that general philosophy to some
extent as well, by the way; you can use **fpm** to create packages
specific for a particular distribution.

So what can - or should - be done on a Linux system?

Well - this of course depends on what YOU want to have and need, and what
kind of base system you may want to use. Perhaps you have full superuser
access - but perhaps you may be in a more restricted cluster-environment
where you only have full access to your home directory; the latter may be
the case if you work on an university campus site. Either way, the RBT
project attempts to be useful in both cases.

Let's first start with a **linux desktop system**, since that is the one
that I am using most of the time.

I would recommend that you first compile your own version of a **core linux
software stack**. This will help in bootstrapping immensely.

The following command should suffice - but **review this list on your own
before you use it**; as a first step, it may be simpler to compile
step-by-step, rather than all in one giant step.

    rbt make sed awk coreutils utillinux gawk m4

(Note that the above may no longer work; you may have to be more
explicit like so:)

    rbt --compile-these-programs=make,sed,awk,coreutils,utillinux,gawk,m4

I recommend that, if possible, compile as many of these programs
as a static binary. That way, if you somehow break a dependent
shared library the tools will still continue to work.

You should also have a **working version of busybox** available,
as that helps in recovering if you made any mistake:

    rbt busybox

Make sure to compile busybox **statically**. That way you
become less dependent on .so files being available. (You may
have to compile busybox on your own; rbt currently does not
handle compiling busybox from source easily, due to the 
ncurses-based menu interface).

Once you have that set going, you should try to compile
your own version of GCC. This probably takes longer than
the steps above and may require a few more programs.
Make sure to enable C and C++ and perhaps a few more 
languages, should you need them.

    rbt mpfr gmp mpcr
    rbt gcc

Keep in mind that you can also use the LFS/BLFS instructions.
They are also available via the "blfs" entry in the 
respective cookbook file.

Glibc is obviously also a target candidate that could/should
be upgraded eventually but this is a delicate step - many
programs have to be recompiled if your glibc version changed.
My best advice here is to compile glibc into a versioned
AppDir (directory, such as /Programs/Glibc/2.28) but to 
NOT symlink it. An invocation example follows here:

    rbt glibc ntrad --do-not-symlink

Since different people may want to tell RBT to compile a 
subsystem for linux, a commandline option exists to allow
us to compile the core programs here.

Invoke it like so:

    rbt --compile-base-system
    rbt --compile-basic-system

The exact programs that will be compiled may change from time to
time. If you need a commandline result of the programs that will
be compiled that way, simply append a '?' character to the above
way to compile the base system, such as:

     rbt --compile-base-system?
     rbt --compile-basic-system?

If everything works here then you should have several 
"linux" programs under the **/Programs/** hierarchy installed.

Of course **/Programs/** can be changed to any other prefix
that you may want to use.

## Build directories

Some progams insist on using a build directory that is
separate from the directory where the source code is
kept.

For example, the KDE program called **kirigami2** requires
an "out-of-source" build directory.

There is a relevant entry in the cookbooks .yml files that
can be used, called **use_build_directory**. If this entry
is set to true then the RBT project will make use of a
build directory for compiling a particular program.

You can also get an overview over the programs that make
use of a build directory, by using any of the following
APIs on the commandline:

    ry --show-all-programs-that-make-use-of-a-build-directory
    ry --all-build-directory
    ry --all-build-directories
    ry --build-directories?
    ry --build-directories

## RBT project will install these executables

The RBT project will install the following executables in the **bin/**
directory:

    autoupdate_this_program
    binary_of
    blfs
    colour_make
    colour_make_install
    compile_these_programs
    compile_via_appdir_prefix_without_symlinking
    cookbooks
    create_new_cookbook
    create_program
    find_alternative_archive
    home_dir
    home_dir_without_symlinking
    homepage
    infobox
    ntrad
    parse_help
    rbt
    rbt_config
    rbt_download
    rbt_make
    rbt_test_alias
    remove_symlinks
    report_total_size_of_all_archives
    ruby_libtool
    scookie
    search_for_tags
    show_all_about
    show_configuration_options
    suggest_cookbook_for
    symlink_this_program
    to_current
    url_action

## Mate-desktop

You can try to update some of the **mate-desktop components**.

In order to do this, call either of the following variants.

    cookbooks --update-mate
    cookbooks --update-mate-desktop
    cookbooks --check-for-mate-updates
    cookbooks --check-for-mate-desktop-updates
    cookbooks --check-for-mate-desktop
    cookbooks --check-for-matedesktop

The first variant is the **shortest**, whereas the second one is
the main entry point in the **case/when** setup.

This instruction will check for new updates to the mate-desktop
releases and download them when found. Note that presently, any
more recent version is preferred, so unstable versions will often
be downloaded (since e. g. 1.19.1 is a higher version than e. g.
1.18.0).

Similar to the various KDE components (look at the KDE entry in this
file), if you wish to compile the **mate-desktop components** then
you can also use a simplified scheme, using "mate1", "mate2" and
so forth, in linear order. This will be replaced with the registered
name of the mate-desktop component.

Invocation example:

    rbt mate1
    rbt mate2
    rbt mate3
    rbt mate4

This is the very same as doing e. g.:

    rbt mate-desktop
    rbt libmatemixer

and so on. Using **numbers** may be simpler, though, as you do not
have to remember the names as-is, while still being able to use
the correct compilation order.

## GNOME

The gnome-project tarball archives can be found here:

    http://ftp.gnome.org/pub/gnome/sources/?C=M;O=D

The releases usually have even numbers for stable releases; and
odd numbers for unstable releases.

I myself usually stayed on the "bleeding edge", that is to use
unstable releases - but after some time, I now think that the stable
releases are better, since you will have less problems or errors in
the long run. Every time I encountered a problem related to an
unstable release, I ended up investing time trying to fix the
problem, which often was not possible. In the end I did waste too
much time for too little gain, so I decided to stop making use of
unstable gnome releases.

I added some code to the class that automatically downloads gnome-packages,
to only consider **even numbers**. But this behaviour can be toggled via a
**commandline switch** on the commandline, anyway - so the subsection here
just explains as to why I personally **favour stable gnome releases**.

You can also compile the gnome components in an ordered fashion,
by making use of **numbers**. (This also works for kde, xfce,
xorg, mate-desktop, games and the like.)

Examples:

    rbt gnome1
    rbt gnome2
    rbt gnome3
    rbt gnome4

And so forth.

To **batch compile** most of these components, try:

    rbt gnome1..200

## Module programs (the extended cookbook dataset)

**Module programs** refers to the ability to access all registered programs at
the **toplevel** "RBT." namespace. In other words, we can "copy" the program
names onto the RBT top-level module "namespace".

For instance, **htop** would be available at

    RBT.htop

if it is called as a "module program".

Note that this is **NOT** the default. By default the above way to call the
method .htop() on RBT would not work.

The program **binutils** would, as module program, be available at:

    RBT.binutils

And so on, and so forth. No "-" characters are allowed there, as these
are not allowed in methods written in ruby.

Note that this will return a sanitized Hash that holds all the information
required to compile a program from source - this dataset thus describes
these programs. That information may still have to be interpreted by a
program, of course.

Simply require the necessary file for this:

    require 'rbt/module_programs'

After that, the above methods will work. Let's see a few more examples
for this:

    RBT.wayland
    RBT.ruby
    RBT.php
    RBT.gnomemimedata
    RBT.evince
    
Note that **reader methods** via '?' character are available too,
although this could be toggled. The '?' will access the datastructure
as a **Hash** rather than a specialized object.

Invoke it like so:

    RBT.htop? # => {"archive_size"=>323560, "short_description"=>"ncurses-based interactive process viewer.", "github"=>nil, "set_env_variables"=>{"LDFLAGS"=>"-ltinfow"}, "homepage"=>"http://hisham.hm/htop/", "binaries"=>["htop"], "headers"=>[], "libraries"=>[], "postinstall"=>[], "pre_make_commands"=>[], "required_deps_on"=>["glibc => 2.3.2", "ncurses => 5.6"], "tags"=>[], "pkgconfig_files"=>[], "program_path"=>"/Users/x/SRC/htop/htop-2.0.2.tar.xz", "program_full_name"=>"htop-2.0.2.tar.xz" ... and so forth }

This functionality exists primarily because it may be easier for you
to use the above API calls. Internally though, the RBT project
will prefer API calls such as **RBT::Cookbooks::Cookbook.new :htop**
instead.

Note that the above works without any '-' as part of the name, so
there will not be **RBT.gnome-mime-data()** - it would be
**RBT.gnomemimedata()**.

When may you want to use this? Well, it could be used if you do want
to use a simpler API. It is very easy to remember, too: just use
"RBT." followed by the name of the program, without any '-' or
'_' characters there.

## Showing the last 750 updated programs

You can show the last updated programs, currently, by default, being
750 programs, like this:

    rbt --show-last-updated-programs
    rbt --slu

## Auto-removing .la files

If you do not like **.la files** (libtool files), you can set the configure
option called **delete_libtool_files** to **true**. Then, if a program
has been successfully compiled into an **AppDir prefix**, such as 
**/Programs/Libcanberra/0.30/** and at the least one .la file has been
found under the **lib/** subdirectory, then these .la files will be removed.
That is, **deleted**.

That way .la files do not taint the given host system any longer (or at the
least will be removed after having been created).

If, for some reason, you rather want to **NOT** remove .la files for the
current compilation run, you can always use the following commandline
flag: 

    --keep-la-files

You can also **purge all .la files** manually, via the commandline.

**class RBT::Libtool::RemoveLibtoolFiles** is able to do. Simply pass
to it the **path** of the directory that contains these .la files, and
that class will eliminate all these .la files. (Be sure to know 
exactly what you are doing before doing so on a directory such as
**/usr/lib/** - if in doubt, always keep full backups of directories 
like this.)

During normal compilation, class RBT::Libtool::RemoveLibtoolFiles
is invoked if you compile a program with an AppDir prefix into something
such as the **/Programs/** hierarchy, as value to **--prefix**.

Informally I myself call this class a **standalone libtool killer**.

If you want to use a top-level API instead, consider the following
variant:

    RBT.remove_libtool_files_from(this_directory = '')
    RBT.remove_libtool_files_from()

Again, pass in the path to the local directory that contains these
.la files.

Since as of September 2018, the last faulty .la file will also be
registered in a local .md file. You can then let the RBT::Libtool
namespace fix the last faulty .la file via:

     rubylibtool --remove-from-file
     rubylibtool --remove-from-stored-file

The reason why this exists is because I sometimes found myself
without a running xorg-server. Copy/pasting then was difficult
with the mouse (and I dislike **gpm**). So I wanted a way how
to fix these faulty .la files without requiring the mouse -
thus the idea was to store the last faulty .la file into a local
file). The usual location for this would be at
**/Depot/Temp/rbt/libtool/** but this of course depends on
where your **temp-directory** is set at.

You can also use **rbt** directly to remove a faulty .la
file, by passing in the path, such as:

    rbt /usr/lib/libpango-1.0.la

## Speeding up the compilation

This subsection deals with <b>speeding up the compilation</b>.

<b>make</b> accepts a commandline option, through
-jN, where N is the number of parallel builds to
use.

For example:

    make -j4

Will make use of <b>four parallel builds</b>. This should
speed up compilation significantly. Do not put any
incorrect number there. The maximum number should be the
maximum amount of processors that you have, which you
can find out via this way, on <b>Linux</b>:

    grep -c '^processor' /proc/cpuinfo

You can also obtain this from RBT itself, on the commandline,
via:

    rbt --n-processors?

Passing the option -j4 to make means to "spawn off 4 parallel
compiling processes". There is actually not a limit for the
number of job slots that can be used, but as a rule of thumb,
one should not exceed more than twice the number of CPU cores
that the local computer system has. So if we have 4 processors,
we should never use a number higher than 8 to "make -j".

Whether this is really true or not, I have absolutely no idea.
I think the main message here is to not use any arbitrary
high number.

If you want to append the default value to make -j, which
is the number of processors that the local computer has,
then do this, for example:

    ry flac ntrad --speed-up-the-compilation
    ry ruby --ntrad --speed-up-the-compilation

The important option here is <b>--speed-up-the-compilation</b>.

Note that you can also directly pass this -j option to rbt,
such as exemplified through the following example:

    rbt lame -j5

This is the same as doing "make -j5" by the way, during
the <b>make step</b>.

You may also define this in a cookbook file, by adding
a parameter called <b>parameters_to_make</b>. Be careful
though, since this may not always be what you may want,
in particular if you use different computers. An alternative
to this may be to use a yaml file, part of rbt, called
<b>use_maximum_speed_for_compilation.yml</b>.

This can also be disabled for the individual run, via:

     --do-not-use-maximum-speed
     --no-auto-speedup

## Working with (debian) .deb files

This subsection has been added mostly because one goal of the RBT project
is to handle .deb files in one way or another - so I wanted to collect
useful information in one place.

If you wish to create **.deb** files on your own, you may need the **dpkg-dev
package**. The following syntax should allow for this:

    apt-get install dpkg-dev

apt will handle the necessary dependencies.

## Sed operations

The binary called <b>sed</b> is well known - and often used -
on *nix operating systems. The primary reason for this is, most
likely, that <b>sed</b> is so useful and versatile. You can 
replace text very efficiently via sed.

If you look at the <b>LFS/BLFS homepage</b> (the "Linux from
Scratch" or the "Beyond Linux from Scratch" homepage), **sed**
is often used to change settings in various programs, before
compilation begins.

The **RBT project** also supports sed-related changes to programs.
The sed-operations have to be defined in the individual yaml file
that stores the instructions in how to compile a program from
source, which is part of the Cookbooks gem.

So for example, the <b>yaml file</b> for a given program simply
has a tag called <b>sed:</b>, which specifies an <b>Array</b>, and
also it specifies which sed operations are to be run, in a linear
order. The first entry is processed, then the second entry, and
so forth (if there are several entries that is).

<b>RBT</b> will then apply these sed-modifications by default.

If you do not need this for a given program then you can
<b>disable</b> this behaviour on the <b>commandline</b>, via:

    rbt htop --disable-sed
    rbt htop --disable-sed-modifications

Note that there are essentially **two** ways as to how to use 
sed-operations, as far as <b>RBT</b> is concerned. One is by
simply using the binary called "sed"; and the other is to use
<b>class SedWrapper</b>. That class is unfortunately a little
buggy as of February 2018 (and beyond) for some sed-operations.

So for the time being, RBT will <b>default</b> to the system
binary called "sed". If this is not available, then RBT may
default to <b>class SedWrapper</b>, which is a pure-ruby
variant.

Most Linux systems will have a <b>sed</b> binary, so this should
work fine on these systems; should also work fine on Windows
under the WSL subsystem these days. 

If you want to query whether the internal sed wrapper is used
in general, as-is, then use the following commandline option:

    rbt --use-the-internal-sed-wrapper?

The sed-operations are run by class RBT::ApplySedOperations.
This class can also be used standalone, that is, from the
commandline. Or you can invoke them from within rbt itself.

Example:

    rbt htop --only-sed # only apply the sed-operations, then exit.

## Compiling a random program

If, for whatever reason, want to compile a random program, then
you can use a commandline switch such as the following:

    rbt --random-program

## Upgrading programs through a file

You can read in input from a file, aka programs that have to be
compiled, stored in a file.

The default name for this file will be
**these_programs_can_be_upgraded.yml**, but you can also use
another name. The file has to exist, obviously.

Invocation examples:

    rbt --upgrade-from-this-file=these_programs_can_be_upgraded.yml
    rbt --upgrade-from-this-file=:default
    rbt --upgrade-from-this-file=:def

:default or :def will refer to the file **these_programs_can_be_upgraded.yml**.

The reason why a default filename exists is so that other projects
can auto-generate such a file, which we can then use for **RBT**.

## Compiling the dependencies of a program as well

You can compile the dependencies of a program if you want to,
via a commandline such as:

    ry kbreakout --compile-dependencies-as-well
    ry kbreakout --with-deps

Note that this will try to compile all the dependencies AND the program
as well, with the dependencies first, and the program that was given
as argument coming last.

This presently (October 2018) works in a brute-force manner, meaning
that it will compile the dependencies even if they are already installed.

In the future, this behaviour may change, but for the time being,
that has to suffice (the feature is thus somewhat experimental
right now; it has to be tested further).

## Internals of class RBT::Compile

**class RBT::Compile** is the main class of the RBT project, as it allows
us to compile or install a program from source. It is, more or less, the
central entry point to RBT - even though you can invoke the other .rb 
files directly, RBT::Compile often provides a simpler convenience access
to the disparate functions of the RBT project.

This subsection here will explain a few key decisions made, in the long
run; and will also explain a few internals.

Do not worry if this may be a bit confusing at first - it is meant
primarily for me, and for others who may want to extend the RBT
project one day.

The method set_configure_base_directory(), defined in the file **misc.rb**,
is used to determine the directory where the configure script ought to
reside. That way we can enter a separate build directory yet still be
able to invoke configure (or cmake) at the right directory target.

## Using a different set of configure-options

Sometimes you may want to overrule, on the commandline, the configure
options that are to be used by a program.

You can do so via the switch **--use-these-configure-options=**.

Usage examples:

    --use-these-configure-options="--enable-shared --enable-bootstrap --enable-languages=c,c++,lto,objc,fortran --enable-threads=posix --enable-checking=release --enable-objc-gc --with-system-zlib --enable-libstdcxx-dual-abi --with-default-libstdcxx-abi=gcc4-compatible --disable-libunwind-exceptions --enable-__cxa_atexit --enable-libssp --enable-lto --disable-install-libiberty --with-gnu-ld --verbose --with-arch-directory=amd64 --disable-gtktest --disable-multilib --target=x86_64-slackware-linux --build=x86_64-slackware-linux --host=x86_64-slackware-linux"
    rbt gcc --use-these-configure-options="--enable-shared --enable-bootstrap --enable-languages=c,c++,lto,objc,fortran --enable-threads=posix --enable-checking=release --enable-objc-gc --with-system-zlib --enable-libstdcxx-dual-abi --with-default-libstdcxx-abi=gcc4-compatible --disable-libunwind-exceptions --enable-__cxa_atexit --enable-libssp --enable-lto --disable-install-libiberty --with-gnu-ld --verbose --with-arch-directory=amd64 --disable-gtktest --disable-multilib --target=x86_64-slackware-linux --build=x86_64-slackware-linux --host=x86_64-slackware-linux"
    rbt poppler --use-these-configure-options="--enable-xpdf-headers"

# Keeping empty directories in the /Programs/ hierarchy

By default, class RBT::ToCurrent and class RBT::SymlinkProgram will remove
empty subdirectories of programs that are installed in an AppDir-like fashion.

This is not always wanted, though, so a commandline flag exists to disable
the auto-removal of empty directories:

    ry feh --ntrad --do-not-remove-empty-directories

## Games support

Similar as to how it has been done for KDE and mate-desktop, you
can use numbers to compile a particular game too.

For example:

    rbt game3
    rbt game5
    rbt game6

And so forth.

You can also compile all games, e. g. by doing this:

    rbt --compile-this-tag=games

Or, even simpler:

    rbt --compile-all-games

(And I also did set an alias at home, to invoke
this via **compile_all_games**).
    
## Compiling all programs that belong to a particular tag

Tags are registered in the individual .yml file of a program.

You can also batch-compile these programs, such as the instruction
of "compile all programs that belong to the tag called game":

    rbt --compile-this-tag=game
    rbt --compile-this-tag=gnome

The tag that you wish to compile has to exist of course, as otherwise
no program would be compiled.

The functionality exists mostly as a convenience feature, say if
you wish to compile all registered games.

## All binaries of every registered program

If you wish to determine the binaries of every registered program
then you can use the following **top-level API**:

    RBT.all_binaries?

This will return a **Hash**, where the keys constitute all binaries
that belong to a particular program at hand.

For example, the two binaries **xzcmp** and **xzcat** both point
to the main program **xz** (such as is available on an URL like
https://tukaani.org/xz/xz-5.2.4.tar.xz). The Hash will store
this association. Another example is the binary **xtrapproto*,
which will point towards the program called **xtra** (
"xtrapproto"=>"xtrap")

How may this functionality be useful?

This depends; if you ever want to find out to which particular
program a given binary belongs too, you could try to use this
API (or ask your package manager instead, of course).

You can also use the following **top-level API** to find out
whether a binary is registered within the RBT project or
whether it is not:

    RBT.has_this_binary? 'xzcmp' # => true

This will return a boolean value - **true** if it is registered
and **false** otherwise.

You can also gather some "statistical" information, such as
through how many binaries are registered in the RBT project,
via this method:

    RBT.report_how_many_binaries_are_registered

If you only want to see which binaries are registered in
all the registered programs of the RBT project, then you
can use this commandline:

    rbt --all-binaries?

## The mate-desktop

If you wish to compile the components of the mate-desktop,
you can try:

    rbt --compile-mate-desktop

Note that this presently (July 2018) does not handle the dependencies
correctly, so there is a very high chance of failure, unless you
already have the dependencies installed on your system.

You can also compile all of the mate-desktop into a standalone
directory, such as **/Programs/Mate/1.12/**.

Issue the following command for this:

    ry --mate-desktop-into-standalone-dir

## Show the last compiled programs

If you want to, for example, show the last 8 compiled programs,
then the following commandline may be useful:

    rbt --last-compiled=8

Note that you first have to have compiled some programs, as
otherwise nothing, or less than 8 programs, would be shown.

## Purging traditional FHS binaries and FHS libraries

The <b>RBT scripts</b> support hybrid systems, that is, systems that both
use the legacy/traditional prefix at <b>/usr</b>, and also GoboLinux-style
AppDirs such as /Programs/PROGRAM_NAME/PROGRAM_VERSION or similar layouts.
For example, php-7.2.1 would reside at <b>/Programs/PHP/7.2.1/</b> (or
<b>/Programs/Php/7.2.1/</b> - all depending on which naming scheme is
used precisely).

This may lead to having e. g. binaries duplicated, first at /usr/bin/,
and then also in the respective AppDir bin/ directory.

If you then wish to get rid of the old /usr/bin/ binary of a specific
program, you can issue the following commands:

     rbt --purge-traditional-binaries-from=xscreensaver
     rbt --purge-traditional-binaries-of=gnupg

The last entry is the program at hand, in these two examples
**xscreensaver** or **gnugp**.

Be sure to **really want to do so** before invoking this functionality.
It is mostly a convenience functionality, avoiding some manual 
removal of these binaries from the commandline.

A similar functionality exists for **libraries**. So if you want to
purge libaries at **/usr/lib/**, you can do this:

     rbt --purge-traditional-libraries-from=libva
     rbt --purge-traditional-libraries-of=libva

The relevant entries for binaries and libraries can be found in
the individual .yml cookbook - have a look at the **binaries:**
and **libraries:** entries of the particular .yml file at hand.

Since as of November 2018, an additional way exists to remove
binaries:

    rbt poppler --purge-its-binaries

This will currently (November 2018) only remove binaries found
under the **/usr/bin/** hierarchy. In the future **/usr/sbin/**
may be scanned as well, but for the time being only
**/usr/bin/** is targeted.

**Remember to do so only if you are sure that you want to do this.**

## Autoswitching python version

There is a configuration setting, stored in the file autoswitch_python.yml.

This can hold true or false (or yes or no). If set to true, then it
means that RBT will attempt to autoswitch the python version, e. g.
version 2 to version 3, if a program at compile-time depends on
the other python version.

This allows us to try to recompile the program at hand.

This will be tried only once, though, in order to avoid infinite
loops.

Do note that this functionality currently will only be used if
the RBT scripts run on "roebe", which is my home system. This may
also be enabled for all users one day in the future, but for now
it is mostly experimental on my home system. (You can of course
also enable "is on roebe"; I use an environment setting for
this.)

## Copying archives to the current working directory

If you ever need to copy some archives into the current working
directory (pwd), such as if you wish to copy them onto a USB
stick, then the following API may be of help to you:

    rbt --copy-these-archives=ruby,php,python
    rbt --copy-archives=kde5_plasma

The latter in particular will copy all KDE5-plasma components
onto a USB stick. This was the original use case as to why I
have added this functionality - I needed to copy the plasma
components to another computer, in order to compile them 
there.

Note that you could also, in principle, use abbreviations here:

    rbt --copy-these-archives=xorgser

This would copy the local directory containing the xorg-server
tarball. In order for this functionality to work, the directory
has to exist prior to invoking the above command(s).

## FILE_ABBREVIATIONS

**FILE_ABBREVIATIONS** is a constant pointing to a file that
keeps the abbreviations to the registered programs. Abbreviations
in this context are words such as "ph", which may stand for
"**php**". So in short, **an alias** to a longer name.

This may be useful if you wish to compile PHP. Rather than typing
"php", you can type "ph" and it should still work.

Do note that tab-completion support for bash and zsh also exists,
but I wanted to be able to find shorter aliases within the ruby
code itself too, so that an API such as this one may also work:

    RBT.compile 'php'
    RBT.compile 'ph'

Both should work. The longer variant, while it is more typing
work, is the recommended way, though.

The usual residency of the file pointed through the constant
FILE_ABBREVIATIONS is at:

    rbt/yaml/abbrevations_to_the_programs.yml

In short, the reason why this file exists is so that we can
use abbreviations, if we wish to compile programs from source.

## The directory /System/Tags/ and registered tags in the RBT project

The directory at **/System/Tags/** can be used to store **tag-related
binaries**. Other entries, such as in a **lib/** subdirectory, will
be ignored.

For example, take the game called **wesnoth**, which may, as a
binary, reside at a path such as **/Programs/Wesnoth/Current/bin/wesnoth**.

This binary may then be **symlinked** into a path such as
**/System/Tags/Games/wesnoth**. Note that this resides under the 
**/System/Tags/** hierarchy. This functionality thus effectively allows
you to quickly look at which application types you may have installed
on your computer system. All compiled/installed games that have been
registered with the tag "game" will then reside in the subdirectory
called **Games/**. It provides an overview over the system and I think
that this is of some use, since different tags are used to describe
the various programs at hands.

If you need an overview over all registered tags, from the commandline,
try to issue this command:          

    rbt --available-tags?

This **only** works for programs that are installed into the **/Programs/**
hierarchy for the time being. (Remember that /Programs/ is not hardcoded
as such but can be changed by the user; it just happens to be /Programs/
on my system, but you could take any other name too, such as /pkg/ or
/home/programs/ and so forth.)

**class RBT::CleanupSystemTags** can eliminate stray (non-existing)
symlinks, and it is invoked from class RBT::Compile, so the directory
at **/System/Tags/** will be guaranteed to **not** have any stray symlinks
whenever RBT::Compile is run. (This is checked in the **post-installation**
part of class RBT::Compile.)

The class **RegisteredTags** can be used to register all available
tags in a .yml "database".

Simply run this file to register every available tag into a yaml file,
which we can then load if we are to search for registered tags, rather
than relying on "grep" itself. Thus, searching for tags will now be
available as well without grep - you have to run it once to register 
the available programs before you can use it.

## The method try_to_find_an_alias_to_this_input_unless_the_program_exists()

The method **try_to_find_an_alias_to_this_input_unless_the_program_exists()**
can be used to find an alias name to a given input.

For example, the input **xorgser** will be converted to **xorgserver**. This
will help us compile programs, since abbreviations are available for all
the registered programs in the **RBT project**.

## Copying all headers to the current working directory

If you want to copy all .h files from, say, glibc, to the current
working directory, then you could do it via this commandline 
flag:

    rbt glibc --copy-headers-from=/usr/include

I needed this functionality so that I could copy a glibc version
installed into the **/usr** or **/** prefix, into
**/Programs/Glibc/VERSION_NUMBER/include/**.

There also exists a slightly simply API:

    ry glibc --copy-headers
    ry glibc --copy-headers-into-pwd
    ry glibc --copy-headers-to-pwd

The two variants are not completely the same, though. The second
variant will only copy .h files that reside in an AppDir-like
prefix, so it is more likely to fail. It will, however had,
never pick up any possibly incorrect .h files from **/usr/include/**.

## Guessing the build-type of a source package

You can try to guess the build type of a source directory.

First, extract the source tarball, such as for **zsh** (the
URL may be: http://www.zsh.org/pub/zsh-5.6.2.tar.xz).

Next, cd into that extracted directory and do:

  rbt --guess-build-type

This will return a String (or Symbol), such as "configure" or
"cmake" and so forth.

The functionality was needed so that RBT can **make a guess**
when information is otherwise unavailable about a given 
program at hand (e. g. not yet registered in a .yml file).

## Compile chain

Several programs require the <b>compilation of prior, other programs</b>.
For example, KDE or KDE Plasma or mate-desktop.

If you want to find out which programs are part of a particular
chain, you can try any of the following:

    ry --show-chain=plasma
    ry --show-compile-chain-of=mate

The first variant would show the proper compile chain that ought to be
usable for KDE plasma; the second variant would show the proper 
compile-chain for the **mate-desktop*.

Simply pass in the argument of the compile-chain that you seek. Note that
the main class that handles this on the commandline, class
**RBT::ShowCompileChain**, will do a little bit sanitizing in order to
find proper and plausible compile-chains.

Note that you can also batch-compile various components that are listed
that way, as they are registered in the file **chained_programs.yml**.

For example, to try to batch-compile the KDE5 applications framework,
you can try the following command:

    rbt --batch-compile=kde_apps
    rbt --batch-compile=kde-plasma

Or, simpler:

    rbt --kde-framework
    rbt --compile-kde-framework

Keep in mind that the **prior dependencies** ought to have been installed
already for this to work (that is, <b>to have compiled successfully</b>
before).

Since **as of July 2018**, a few simpler shortcuts exist as well.

For example, to compile all programs of the **KDE5 foundation**,
you can simply use the following command:

    rbt --compile-kde-foundation

You can also manually specify which programs you may wish to
**batch-compile** (that is, to compile them one after the
other). The following **example** shows how this can be done:

    rbt --compile-these-programs=make,sed,awk,coreutils,utillinux,gawk

This would specifically compile these programs in the given order,
e. g. starting with **make**, over to **sed**, **awk**,
**coreutils**, **utillinux** and finally **gawk**.

## Everything about symlinking

This subsection contains some information about how symlinking is
handled by the RBT project.

In general, the main functionality resides in the file
**rbt/toplevel_methods/symlink.rb**. When it comes to class
**RBT::Compile then the file is rbt/compile/symlink.rb**.

Certain settings in the cookbook .yml files can be used to do
some trivial symlinking jobs, such as symlinking **lib64/** to
**lib/**. The boolean setting **autosymlink_lib64** handles this.
If you need such a symlink-job during post-installation, then
set this entry to true in the respective .yml file, such as for
the program called **poppler** and the associated **poppler.yml**
file:

    autosymlink_lib64: t

Note that presently (November 2018) this is only honoured if
we compile in an **AppDir-like manner**.

If you wish to remove all symlinks under the **/Programs/** hierarchy,
or wherever you have your own particular equivalent of this main
**Application directory** set to, then you can use the following
**commandline switch**:

    rbt --remove-all-symlinks
    ry --remove-all-symlinks

(ry is an alias on my system towards rbt)

Why was this commandline flag added?

I needed to **test the re-creation of symlinks**, so I had to add
the functionality to also remove all symlinks prior to testing this
functionality.

## Determine to which program a particular .h file in /usr/include/ or an executable in /usr/bin/ may belong to

If you ever do want to find out to which program a particular .h
file (header file) belongs to, you can make use of the class
**RBT::QueryHeaderToPackage** for this task.

A specific usage example in ruby follows:

    RBT::QueryHeaderToPackage.new('/usr/include/')

You can also do so from the commandline, if your target is the
**/usr/include/** hierarchy.

Example:

    rbt --query-headers

You can designate another target directory to be checked for
.h files, via a commandline flag such as:

    rbt --query-headers=/usr/local/include/

This functionality also works for binaries found under **/usr/bin/**,
by issuing something like:

    rbt --query-programs
    rbt --query-programs?
    rbt --attribute?

This is equivalent to the following ruby code:

    RBT::QueryBinaryToPackage.new('/usr/bin/')

## Exiting at the "make" or "make install" stage

If you wish to instantly exit when reaching the "make install" stage,
you can use the following commandline for this:

    ry htop --stop-at-make-install-stage

Something similar exists for the "make" stage:

    ry htop --stop-at-make-stage

## class RBT::SymlinkIntoUsrLibDirectory

class **RBT::SymlinkIntoUsrLibDirectory** can be used to symlink content
into the /usr/lib/ directory. The name for this class is fairly unwieldy,
but I needed the functionality in a specific .rb file that I could
call from the commandline.

By default, RBT::Compile does **not** make use of this class, but you
can enable it via any of the following commandline flags:

    ry gtk+ --symlink-into-usr-lib-dir
    ry gtk+ --symlink-into-lib-dir
    ry gtk+ --symlink-lib-dir
    ry gtk+ --symlink-libs

Why may this functionality be useful to have?

For practical purposes it may be better to use a hybrid linux system,
e. g. one that has versioned AppDirs under /Programs/, but also keeps
symlinks at /usr/lib/. A similar approach is used by the program
**stow**, for example. Once you understand how stow works, it is 
quite easy to realize that this functionality is useful to be had,
so RBT also has the same or at the least a similar functionality.

Furthermore you can also call this class from the commandline, by
simply calling the .rb file at hand.

## Batch compiling all locally existing archives of a given program at hand

Since as of **23.11.2018** (23rd November 2018) it is possible to
batch-compile programs that reside locally, if they differ in their
versions, into an AppDir-like prefix.

This sounds difficult, so let's show an example to make this easier
to understand.

Say that you have a directory called **/Users/x/SRC/libsigc++/**. In
this directory you store source archives such as the following
ones:

    libsigc++-2.10.1.tar.xz
    libsigc++-2.6.2.tar.xz
    libsigc++-2.8.0.tar.xz
    libsigc++-2.9.3.tar.xz
    libsigc++-2.99.11.tar.xz

Now you may wish to compile all these, into their proper prefix,
such as that **libsigc++-2.10.1** will end up in **/Programs/Libsigc++/2.10.1/**
and so forth. In fact, this is precisely why I added this functionality.
I needed to be able to compile different versions, including old versions,
into the corresponding hierarchy under **/Programs/**.

The commandline for this is something like:

    ry libsigc++ --compile-all-available-local-versions
    ry libsigc++ --all-available-local-versions

This is a convenience feature.

## Check for duplicate binaries

class RBT::CheckForDuplicateBinaries can check for binaries that
are a duplicate. By default, this class will scan for the /usr/bin/
hierarchy, but you can specify other target directories. If a 
duplicate was found, the user is notified.

That way you, as the end user, can decide how to proceed - for
example, I needed this class so I can find out which files
I can remove on a **hybrid system**.

If you have aliased this class to <b>check_for_duplicate_binaries</b>
then you can invoke it like so, with another target directory:

    check_for_duplicate_binaries --dir=/usr/local/bin/
    check_for_duplicate_binaries --pwd
    check_for_duplicate_binaries PWD # Is the same as the above ^^^ variant.

If you only want to check for the current working directory, you
can use the bin/rbt executable as well, via:

    rbt --check-for-binary-duplicates
    rbt --check-for-duplicates
    rbt --duplicates-in-pwd?

## Determine to which particular program a specific binary name belongs to

If you wish to find out to which program, e. g., the binary called
"shar" belongs to, then you can query this from the commandline
via:

    ry --this-binary?=shar

Alternatively you can use this:
    
    ry --binary=env

The output of these would be:

    RBT::BinaryNameBelongsToWhichProgram: The binary called `shar` belongs to the program `sharutils`.
    RBT::BinaryNameBelongsToWhichProgram: The binary called `env` belongs to the program `coreutils`.

That is the main purpose of **class RBT::BinaryNameBelongsToWhichProgram**.

Since as of June 2019, an executable exists under **bin/** that can be used for
this task, the name being **binary_of** - mostly for convenience.

Example equivalent to the two above commands:

    binary_of shar
    binary_of env

## Binaries installed by a given program

Several programs (cookbooks) that are registered in this gem
will install binaries/executables under a bin/ subdirectory.
This will usually be at the prefix /usr/bin/ or at the prefix
/usr/local/bin/, for most programs on a Linux system. How does
the RBT project handle these executables?

In the **rbt project**, in the respective .yml files, there
will be an (optional) entry called "binaries:", followed by an
Array of binaries that this program will install.

So, for example, take the file called **binutils.yml**.

Binutils will, by default, install the following **binaries/executables**
when you compile it from source:

    addr2line
    ar
    as
    c++filt
    gprof
    ld
    nm
    objcopy
    objdump
    ranlib
    readelf
    size
    strings
    strip
    libiberty
    libbfd
    libopcodes

If you want to find out how many binaries are registered in the
cookbooks project so far, in total, then you can **invoke the cookbooks
executable** like in the following manner:

    cookbooks --n-binaries?

The result will then be something like this:

    RBT: There are 4102 registered binaries in all cookbook files.

## Environment settings

Sometimes environment settings, stored in ENV as far as ruby itself
is concerned, may interfere with compilation. For example, assigning
to the env variable called TZ may cause a build to fail (TZ stands
for TIMEZONE).

I consider this pretty awful for shells such as bash or zsh, but
I did not design any of them.

What this means as far as the RBT project is concerned is that we
may need a way to easily **AVOID** using any ENV variable that may
be faulty.

The simplest way how to do this may be by not using any ENV variable,
save for PATH (which has to work, as otherwise the compiler can not
find anything).

Usage examples:

    ry nss  --clear-envs ntrad
    ry nspr --clear-envs ntrad

This would attempt to compile both nss and nspr without any environment
variables (save for PATH).

Since as of **December 2018**, a new class called 
**RBT::CompileViaEnvironmentVariableAsPrefix** can be used to 
specifically compile into a prefix that is specified via an
**environment variable**.

This allows me to then use something like this on the commandline:

    env_prefix xfconf

And it will compile the XFCE xfconf component into the
prefix specified by the environment variable called XFCE_PREFIX,
which I happen to have set to a value such as
**/Programs/Xfce/4.12.0/**.

## Configure options and passing additional configure options

The content of the various .yml files may have an entry called
**configure_options**. This is usually the entry that keeps
track of the arguments that you may normally pass towards
**./configure**, such as **--enable-static** or **--disable-shared**.

Different build systems, such as meson, use a slightly different
syntax, such as **-Dlibpsl=false**. Additionally, I sometimes
want to preserve the old configure options, so that I can compile
different versions of a given program where the build system may
have changed. That is why I added, in December 2018, the ability
to use specific configure options for **meson**. These are called
**meson_configure_options**. An example for this may be the
file **networkmanager.yml**.

Some configure options can be passed to an instance of **RBT::Compile**
as well.

For example, if you wish to use --enable-shared then you can simply
pass it, like so:

    rbt htop --enable-shared

This will simply pass the configure option --enable-shared.

In the future, more options will probably be added. Some options
may be translated into the equivalent cmake-option, if cmake is
used, but for the time being, this is presently (November 2018)
not well supported. Stay tuned for later changes in this regard.

## How do I use the RBT project personally

How do I use the **rbt project** primarily, on my **Linux** systems?

When I compile a (new) program via **RBT**, such as by issuing "compile
htop" or "rbt htop" or "ry htop" (I make use of aliases a lot), then
the **RBT-scripts** will compile the program **htop** for me, as
specified by the corresponding "htop.yml" file, which is part of
the RBT project - within the **Cookbooks** namespace.

If I need to make a change to the compilation process, such as by
using a different default **prefix**, then I simply edit the
**htop.yml** file, change, add or remove the corresponding entry,
and things will (should) work past this point - but I can also
override the settings from the commandline, so I have all the
flexibility required available.

In many ways I thus control most aspects via these individual
.yml files. Other package managers do something similar if you
look at them.

The "default settings" that I may use go into a specific .yml file,
whereas the **commandline switches** allow me to specifically
override or cherry-pick other settings, based on the machine
that I am working with. For example, on **GoboLinux** the
AppDir prefix makes a lot of sense. On **slackware** on the other
hand I may decide to just use **/usr** as option given
to **--prefix=**.

On **traditional** linux distributions, **/usr** as prefix is
not the only possibility of course. Another popular way is to
compile into the **user's home directory**. A commandline switch
exists for the latter, called **--home-directory**. This is 
equivalent to whatever the user's home directory ought to
be, such as **/home/debug/**.

This is useful in the sense that, ideally, I can get away with only
passing the name of the program that is to be compiled, without
needing to change anything. It allows me to be lazy - and
just get things to "work", when they are compiled (if the compilation
process itself will work).

## Suggest the content of a cookbook .yml file

If you compile a program from source successfully, say **glib**,
into an AppDir prefix such as **/Programs/Glib/2.58.1/**, then
you may use class **RBT::SuggestCookbookFor** to display the
content of a .yml file for glib. This is obviously not perfectly
accurate, but may help to get things started when you wish to
create your own .yml files.

Invocation examples from the commandline:

    rbt --suggest-cookbook-for=glib
    rbt --suggest-cookbook-for=htop
    rbt --suggest-cookbook-for=pango

**Note** that this depends on these programs being installed
in the **programs_directory?**, which on my system is equal
to **/Programs/**. See elsewhere in this document how to set
this value to some other target location.

Do note that class **RBT::SuggestCookbookFor** is also used
to indicate (to me) if a specific .yml file needs to be 
updated, in regards to libraries. As of 28th December 2018
this is experimental; but I may need to extend this eventually,
and also include headers and binaries in this. The win-win
situation here would be that the .yml files that are part
of the RBT project, may become better and more complete
in the long run.

## Pre-install actions

Pre-install actions are actions taken before RBT::Compile
will invoke the configure script, or cmake script.

Typically the pre-install action involve calling a shell
script such as **bootstrap**.

You can also pass in which particular pre-install action to
do, via:

    rbt htop --preinstall="sh bootstrap"
    rbt htop --prerun="sh bootstrap"

    rbt htop --preinstall="sh bootstrap, clear"
    rbt htop --prerun="sh bootstrap; clear"

As can be seen above, you can separate multiple arguments via
either ',' or ';'.

## The clang compiler and LLVM support

LLVM offers the clang compiler. Compiling most programs should be
possible via **clang**. The content of the configuration file
**use_this_compiler.yml** tells RBT::Compile which compiler to
favour. If clang is available, and the content of the file
use_this_compiler.yml is clang, then clang will be used - otherwise
RBT::Compile will default to **gcc**.

The environment variable RBT_USE_THIS_COMPILER can overrule 
the content of this yaml file **if it is set**, so you will
always have a way to toggle the behaviour. (From within ruby
code, you can modify it via
**ENV['RBT_USE_THIS_COMPILER'] = compiler_to_use_goes_in_here**.)

To find out which compiler will be used you can use the following
from the commandline:

  rbt --compiler?

## Colourizing configure --help related output

If you wish to use colours for whenever you do:

    ./configure --help

then this is available via the executable at **bin/parse_help**.

The class that is doing this is called RBT::ConfigureHelp.

Simply invoke it in a directory that can respond to a
"./configure --help" output and you'll get some colourization.

This was added in January 2019, so expect this to not be quite
perfect as of yet.

You can also give, as input, an archive, such as foobar.tar.xz;
the class will extract this archive before doing a
**./configure --help** run.

## Querying the host system

The host system can be queried from within RBT via:

    rbt --host-system

## Inferring the build system that a particular program uses

If you look at different programs, they may use GNU configure,
meson, python setup.rb files, scons, cmake, waf and so forth -
in short, a vast array of different build systems.

The toplevel method called **RBT.infer_build_system()** can be
used to determine which build system is assumed, by RBT, to
be used. It will return a one-word **String** such as **meson**
or **configure**. Do note that this is NOT perfect - you should
not absolutely rely on this method, but for most programs this
should work fine.

The code for this resides in the file
**rbt/toplevel_methods/infer_build_system.rb**.

Some projects may use more than one build system, e. g. harfbuzz
would allow you to use either **configure** or **meson**. In
this case, what should the method **RBT.infer_build_system()**
return? It could return an Array of both build systems, or that
method could just "pick" one variant and return it. The latter
is presently (January 2019) the current default. A yaml file
keeps track of which build system is prioritized by default.

## The RBT project supports colours on the commandline.

By default, the scripts available within **module RBT** support
colours on the commandline. Colours may help put proper focus
on what is important and what is not.

If, for any reason, you do not want to use colours, you can disable
them either via the **configuration** (in the .yml file), or use
this toplevel API to disable colours:

    RBT.disable_colours
    RBT.disable_colors # <- This works as well, of course.

The above method-call will completely disable colours for the whole
RBT namespace.

The **configuration file** that is used to determine whether to use
colours is called <b>use_colours.yml</b>. Since not everyone may
be able to modify the .yml file, depending on the host setup (such
as in a restricted environment like at a university campus site), a
**commandline flag** is also available:

    rbt htop --disable-colours

## Auto-updating libraries installed under a versioned AppDir

You can auto-update the "libraries:" entry in a given .yml file,
but this is usually only useful when I do so. I update the
libraries entry; and eventually I will upload a new version of
RBT.

If you, for any reason, wish to try to do so too, you can do
so via any of the following invokation ways:

    rbt --update-libraries-of=poppler
    rbt --update-libraries-for=poppler

This would then update the file called **poppler.yml**, 
after making a backup of the old **poppler.yml** file first.
(Make sure to have set the environment variable called
**IS_ROEBE** to a value of 1).

## RBT::Cookbooks::CheckForInvalidEntriesInThisCookbook

**class RBT::Cookbooks::CheckForInvalidEntriesInThisCookbook** can be
used to check that only valid entries are registered in a cookbook
file.

It can be found under the subdirectory **rbt/validation/**.

## The log directory

The **log directory** for RBT is the directory into which RBT can
store various files. This directory will also be used for storing
extract archives, so the name **log directory** is a bit of a
misnomer - it simply is the main directory that RBT will use to
keep stuff stored that may be useful for compilation, or created
during a compilation/installation process.

On my home system this defaults to the directory
**/Depot/Temp/rbt/**.

You can also generate this log directory from the commandline via
either of the following:

    rbt --create-log-dir
    rbt --create-log-directory

## Usefulness of this project

If you may wonder how this project may be useful to you, well -
I think the **main target audience** will be fairly small, mostly
only more experienced users, in particular linux users. These
people often already know a lot about, not only linux, but also
programming.

While I will continually **improve the documentation** so that novice
users can understand what the **RBT project** is doing as well,
to the point of being easily able to use the project, I think that
there will not be many novice users who are even interested in
such a project in the first place.

So the project should perhaps be seen as a "complementary" project
to the LFS/BLFS project (Linux from Scratch etc..) - or for smaller
linux distributions as an additional set of scripts that aid in
managing a distribution.

At the very least, the programs found in this project have a
description, so this alone can be of help to
**distribution-creators**. And there are some classes that 
focus on validation the dataset, so that people who use the
project can be reasonable sure that everything is in proper
order.

## KDE

Many <b>KDE components</b> (== <b>programs</b>) are registered in the
rbt project.

These programs usually have the registered tag **kde** or **kde5**, in
the respective .yml file. You can also search for these via
**class Cookbooks::SearchForTags**; just pass in kde or kde5 as
argument to that file/class. (I have the file aliased to "stag",
so I can just do "stag kde5".) 

You can also **batch-download** KDE components - that is, to download
them all to the local filesystem (aka to your local **harddisc**).

There are several ways how to do this. Either call the respective
class directly. These .rb files will reside in the subdirectory
**check_for_updates/**.

Alternatively, you can invoke them directly from the commandline
like so:

    cookbooks --update-kde-plasma

This will download all of the KDE5 plasma components (unless they
are already available locally).

If you wish to download the **KDE5 applications**, then you can
try this command:

    cookbooks --update-kde-applications
    rbt --update-kde-applications

**Aliases** exist to the above:

    cookbooks --kde-apps
    rbt --kde-apps
    rbt --compile-kde-apps

In general, the **rbt project** attempts to allow aliases, for
sake of convenience. The "official" entry points are usually the
longer ones though, as they are less likely to conflict with
other instructions given on the commandline. ("update kde 
applications" is more explicit than "kde apps", but the latter
is shorter, and thus may be more convenient when used on the
commandline).

To update the **KDE5 Frameworks**, do:

    cookbooks --update-kde-framework

To update the **KDE5 porting aids**, do:

    cookbooks --update-kde-porting-aids

Note that you could also use '_' tokens in the above invocation examples,
or omit these, so the following would work just as well:

    cookbooks --update_kde_porting_aids
    cookbooks --updatekdeportingaids

You can also query the KDE "status", that is, which variant is
currently the most recent one. Use something like this:

    cookbooks --kde-status

The output of the above command, on the commandline, would be something
like this:

    KDE Frameworks version:      5.50
    KDE Applications version: 18.08.1
    KDE Plasma version:        5.13.5

This information may be useful if you go to the official KDE homepage
and want to find out whether there have been any updates recently. Then
you can compare your local version to the remote one and see whether
you may want to update the KDE components that you are presently
using.

Since as of the year **2018**, I am able to compile just about all of
the KDE components, through the rbt project. My main target is usually
the <b>KDE konsole</b> ( e. g. at 
https://download.kde.org/stable/applications/18.08.1/src/konsole-18.08.1.tar.xz ).

Iff I am able to compile it then I can usually compile the rest of KDE as well.
It is a fine konsole; give it a try if you haven't yet.

You can also use a **number** to compile a particular KDE component.

For example:

    ry --kde1
    ry --kde2
    ry --kde3

Will compile these registered kde components. The reason as to why
this functionality exists is that it makes batch-compiling the KDE
components easier. I simply have to increment a number and compile
the KDE suite en suite, without having to remember the name (my memory
is quite bad, so using numbers is so much easier).

You can drop the **--** too, of course:

    ry kde1
    ry kde2
    ry kde3

You can also chain these via a range-like syntax:

    ry kde1..100

This would compile the first 100 KDE components (if all goes well).

You can also be more verbose, if you'd like to. The following
demonstrates this:

    ry --kde1..kde250

The linear order of these numbers can be seen in the file
**rbt/yaml/chained_programs.yml**, under the respective KDE component
(kde-plasma, kde-applications and so forth).

If you wish to compile all of the KDE5 plasma components, you can
use either one of the following variants:

    rbt --compile-all-kde5-plasma-components
    rbt --compile-plasma

This is a convenience-shortcut. Of course the option with --batch=
also works.

## Obtaining the main URL for a given program 

The method **RBT.return_url1_of_this_program()** can be used
to return, as String, the remote URL to a program, also called
"url1". This entry usually denotes the path to a tarball/archive
that can be downloaded from the commandline.

If you need to determine the URL from the commandline, you can
try this:

    rbt --show-url=readline
    rbt --show-url=php
    rbt --show-url=ruby
    rbt --show-url=python

## Report the status of the project

You can report the status of the project via:

    rbt --report-status

## ColourMake and ColourMakeInstall

If you wish to use some colours when doing "make" or "make install"
then you can use **bin/colour_make** and **bin/colour_make_install**,
like in this way:

    colour_make
    colour_make_install

You can also tap into this code from the commandline, through the
**bin/rbt** interface, such as in this way:

    rbt --colour_make
    rbt --colour_make_install

Of course the slightly shorter variants work too:

    rbt --colourmake
    rbt --colourmakeinstall

And this as well:

    rbt --colour-make
    rbt --colour-make-install

## RBT::ShowAllAbout

You can quickly show all about a given program at hand, via the
class **RBT::ShowAllAbout**. I have aliased this class to
**show_all_about**, so then I can do:

    show_all_about ruby
    show_all_about make

This will show the content of the .yml file at hand. In order for
this to work, the Cookbooks project has to be available - and that
particular program has to exist, too. Thus meaning, that a .yml file
must exist, such as ruby.yml and so forth.

You can also invoke the above from the commandline, by issuing:

    rbt --show-all-about=ruby
    rbt --show-all-about=php

In these two cases, we would query the content of the two yaml files
called <b>ruby.yml</b> and <b>php.yml</b>.

You can also use the current working directory as "input", via:

    rbt --show-all-about-this-dir

For example, if the directory name is **/Depot/Temp/xcb-util-xrm-1.0/**,
then calling rbt via the **--show-all-about-this-dir** flag will
assume to be the same as if you would have done:

    rbt --show-all-about=xcb-util-xrm

This allows you to type a little less (and not have to think about
the name of the program at hand, since you are just using the working
directory as the "input" / argument).

## Extract to and the Temp directory

Archives such as **foobar-1.0.tar.xz** or **barfoo-2.0.zip** will first have
to be extracted before they can be compiled. The **RBT** project thus
needs to have this functionality; and you, as a prospective user, also
need to have (some) control over this behaviour, in particular if you
wish to **extract to a specific directory**.

The file <b>temp_directory.yml</b>, which is part of RBT's configuration,
designates the main directory to which RBT will extract an archive.

For example, if the content of this yaml-file is **/tmp/** then RBT
would extract archives into that directory. (Actually, RBT will
extract into the **rbt/** subdirectory, in order to not make a mess
of the target temp-directory; so a value of **/tmp/** would really
mean **/tmp/rbt/** instead.)

It is permissible and possible to use $FOO variables inside of that
yaml-file, to designate and make use of shell variables. For example,
I may use the variable called $TEMP as the content of the file
<b>temp_directory.yml</b>, which in turn points towards
<b>/Depot/Temp/</b> on my home system. This is the very same as if
you would have used **/Depot/Temp/** as the content of this yaml-file,
so the only net-gain is that you may not have to hardcode the same
path several times; instead, you can re-use shell variables that
way.

If you want to extract into the **home directory** instead, then
you can use any of the following commands to do so permanently:

    ry --permanently-extract-to-home
    ry --permanently-extract-to-home-dir
    ry --extract-to-home

You can also specify a specific target directory that is used
for the extraction step of a compilation that you are about
to start.

Consider the following example:

    ry htop ntrad --use-this-temp-dir=/opt/foobar_create_it 

This would use the directory **/opt/foobar_create_it** for
extracting the source archive.

I needed this functionality in particular because I sometimes
have to do two compilation runs at the same time, e. g. for
different target prefixes - so it made sense to also use a
**different* **temp dir** for extraction.

If you want to, or have to, you can also designate another
temp-directory, permanently, via the following **toplevel
API**:

    RBT.permanently_set_temp_dir
    RBT.permanently_set_temp_dir '/tmp/rbt/'

You can also quickly extract any local archive via this
**toplevel method**:

    RBT.extract()

The latter method can also be used on the commandline, through
**bin/rbt** via:

    rbt --extract htop

This will first **download htop** and then extract it, in the
**current working directory** (**cwd**).

You can also quickly extract a locally existing archive,
such as **bison-3.2.4.tar.xz**.

    rbt --extract-this=bison-3.2.4.tar.xz

Another functionality is that you can change the directory as to
where source archives will be extracted to, permanently, from
the commandline. Normally a good directory for this may be
**/tmp/** but you can choose another directory, too. I tend
to use /Depot/Temp/ as my temp-directory - to me it feels more
clean than /tmp/ but your mileage may vary.

Examples:

    rbt --permanently-set-temp-dir-to=/user/bioinf4/Temp/
    rbt --permanently-set-temp-dir=/user/bioinf4/Temp/

If you wish to extract to, for example, the target location
**/opt/test** then the following command should work:

    ry htop --extract-to=/opt/test

## Repackaging archives

You can quickly repackage archives, such as **foobar-1.0.tar.gz**
into **foobar-1.0.tar.xz**, by issuing this command:

    rbt --repackage=foobar-1.0.tar.gz

## Running strip on binaries

You can invoke the command called "strip" on binaries (executables),
from within **rbt** too.

A simple way to do so is to first navigate to a bin/ directory,
and then invoke the following command:

    rbt --strip-binaries

But be careful **before** running the above - this will really
run "strip" on every file found in the current working directory.

Only do so when you are sure you want to invoke **strip** there.

You can also specifically strip only the binaries that belong
to a certain program, such as through:

    rbt --strip-binaries-of=htop

## Batch-downloading all source archives

You can download all registered **source archives** by
calling **class RBT Cookbooks::DownloadAllSourceArchives**.

I use an alias to call that **.rb** file. Then I can do,
for example:

    download_all_source_archives

Alternatively, you can also **invoke the file from the commandline 
directly, by issuing either one of the following**:

    rbt --download-all-source-archives
    rbt --downloadallsourcearchives
    rbt --grab-source-archives
    rbt --grabsourcearchives

From within Ruby code, you can use the following **toplevel-API**
to download all source archives:

    RBT.download_all_source_archives

You can also download only those files that have a certain **tag**
registered. For example, if you want to download all programs that
have the tag "plasma" attached, you can issue the following 
command (and argument) this:

    download_all_source_archives --tags=plasma

In order for this to work, the tag has had to be **registered**
(and thus, logically, exist).

If you need an overview over the existing tags, issue this command:

    cookbooks --available-tags

## Available utility scripts of the RBT project

There is a large collection of small scripts available in the 
subdirectory called **rbt/utility_scripts/** - you can either
go into that directory to have a look, or show a summary by
calling:
  
    RBT.show_utility_scripts

Each .rb file there should have a small header that explains
what the respective .rb file will do. You may want to check
these out.

## Commandline examples for rbt

This subsection here has a few examples how I use rbt every now
and then. I will try to expand this subsection with useful
snippets in the long run, but they will not be explained in
this subsection - see for other parts of the README.me file
here for explanations, or use "rbt --help".

    rbt gdkpixbuf --home-dir --use-meson

## Show everything about a given program (show all)

There is a class called ShowAllAbout, which will simply show
the whole .yml entry of a given program.

I aliased this on my system to "show_all", so then I can type:

    show_all htop

And the content of the .yml file is shown. This is convenient
because you can quickly view the content of the yaml file
at hand.

Note that there exists another way as well, via scookie, but
scookie does not necessarily show all information, whereas
class ShowAllAbout will do so.

A commandline variant also exists via **cookbooks** itself:

    cookbooks --show-all-about=htop
    cookbooks --show-all=htop
    cookbooks --show-file=htop

Of course this works through **rbt** as well:

    rbt --show-all-about=htop
    rbt --show-all=htop
    rbt --show-file=htop

## Postinstallation steps

If you **compile a program from source**, sometimes there may be post-installation
steps that may have to occur after the program has been compiled. For example,
if you wish to **symlink** **cc** to point to **gcc**, then we need to have
this specified in the corresponding file called **gcc.yml**. The class that
handles these **postinstallation tasks** is called **class RBT::PostInstall**.

You can invoke this class manually too, if you want to - I aliased the name
"<b>post_install</b>" to point to the file in question. This I needed because
I sometimes need to perform post-installation steps through the commandline,
so it was simpler to have this a standalone, separate class.

Note that **postinstallation steps** can become quite complex, which is why
RBT tries to simplify some of the steps by using shortcuts.

For example, say that you wish to **copy several files** to another location
automatically after compilation has finished.

You can do so via the **copy_files** instruction. Let's show a specific
example for doing so:

    - copy_files { VERSION,assembly,common,eclipse,epub,epub3,extensions,fo,highlighting,html,htmlhelp,images,javahelp,lib,manpages,params,profiling,roundtrip,slides,template,tests,tools,webhelp,website,xhtml,xhtml-1_1,xhtml5 } /usr/share/xml/docbook/xsl-stylesheets-PROGRAM_VERSION

The leading ' - ' in the line above is for **YAML** and it denotes an entry that
is to be kept as an Array. The rest of this Array is quite easy to understand.
The instruction <b>copy_files</b> will simply copy several files; and these files
are denoted within the **square brackets { }**. These entries can b a file or a
directory and **RBT::Compile** will correctly handle these.

The very **LAST entry** is the target location that you wish to copy these
files to, which in the above case is at
**/usr/share/xml/docbook/xsl-stylesheets-PROGRAM_VERSION**.

Note that the upcased word PROGRAM_VERSION is considered a MACRO and will
automatically be expanded to the current program version at hand, e. g.
the one that is inferred or specified within the specific cookbook .yml
file.

You can have as many postinstallation steps as you want to, in the
specific .yml file. The entry that is important here is called
**postinstall**.

Of course you can use more commands than "just" copy_files; another
example would be the command **create_pkgconfig_file**, such as
in this way.

    postinstall:
    - create_pkgconfig_file

This would automatically try to create a pkgconfig .pc file under
the **pkgconfig/** directory. The file **libmad.yml** needs this.

Some programs may require a post-install symlink step, such as
**vte**. **vte** will install a file called **bin/vte-2.91**.
It is easier to approach this file as **bin/vte** so a symlink
option is used in the file **vte.yml**.

The corresponding entry in the postinstall section of that .yml
file is any of the following two:

    symlink_binary
    symlink_this_binary

Have a look at the file called **vte.yml** for a complete example.

## The $PATH variable

You can put <b>/usr/bin/</b> at the beginning of $PATH, via:

    rbt --traditional-path

This may be useful if you ever run into some PATH-related
problems, such as when you mix different paths. I have had
an issue with bash once, which is why I added the
above flag to resolve this issue.

Of course other ways may be to simply change <b>PATH</b> in
your shell, such as in <b>bash</b>:

    export PATH="/usr/bin:$PATH"

This is probably the best, as in most reliable way. Remember
that within **ruby**, PATH is available under:

    ENV['PATH']

## Autogen

Some programs require the program called "autogen". In the respective
cookbook file, this is the corresponding entry called
<b>use_autogen</b>.

If this entry is set to **true**, then **RBT::Compile** will invoke
autogen for that particular program.

If you want to find out whether a program, such as "pacman",
requires the use of autogen, you can use this query to find
out on the commandline:

    ry pacman -use-autogen?

The result may be output like this:

    RBT::Compile: No, autogen will NOT be used for the program pacman.

## Picking a specific program version

You can e. g. select a local version via:

    ry python --2

This will only work if you have a python version available locally
in the right directory, starting with the version number **2**. 

You can also use specific versions, such as in this way:

    ry caja 1.21.4 --trad

This latter example would require a tarball archive, a file, such
as **caja-1.21.4.tar.xz**, to exist in the main directory
for **caja/**. I tend to have multiple versions for many programs
stored locally, for added flexibility.

## Snapcraft files

Since as of **June 2018**, the **RBT project** can also autogenerate
the necessary information for an (_ubuntu_) **snapcraft file**.

The class that handles this file-creation is called
<b>RBT::Cookbooks::CreateSnapcraftFile</b>. It can be found
at the path **rbt/utility_scripts/create_snapcraft_file.rb**
within the RBT gem.

Note that the functionality is currently **very limited**; not every
aspect of a _snapcraft_ file is presently supported. But if you only
need a very simple snapcraft file, or want to auto-generate one and
then improve on it manually, then this may be useful for you to
make use of.

The **argument** to class <b>RBT::Cookbooks::CreateSnapcraftFile</b>
should be **the name of the program** that you wish to generate a
**snapcraft.yml** file for, such as "m4" or "php" or "ruby" and so
forth.

There are essentially **two different ways** how to go about creating
this snapcraft file:

(**a**) Either from within ruby code, you can do this:

    require 'rbt/utility_scripts/create_snapcraft_file.rb'

    RBT::Cookbooks::CreateSnapcraftFile.new('m4')

or, via a **toplevel API**:

    RBT.create_snapcraft_file('m4')
    RBT.snapcraft_file('m4')

or

(**b**)

from the <b>commandline</b>, do:

    cookbooks --create-snapcraft-for=NAME_OF_THE_PROGRAM_HERE
    cookbooks --create-snapcraft-for=ruby
    cookbooks --create-snapcraft-for=m4
    cookbooks --create-snapcraft-for=php
    cookbooks --create_snapcraft_for=php,ruby,m4

The last variant allows you to use a ',' separated list for
several programs.

The toplevel API exists mostly **due to convenience**; the commandline
variant may be the simplest to do so, though.

If you wish to look at the official documentation for a
**snapcraft.yaml** file, look here:

    https://docs.snapcraft.io/snapcraft-yaml-reference/4276

## Colours used by the RBT project

By default, the **rbt project** will use lots of different colours
on the commandline.

These colours are usually used on a **KDE Konsole** or a mate-terminal,
on my home system - and they work best with a **black background** in
these terminals.

If the default colours are to your dislike or do not work with your
given setup, you can always disable them via an option such as
"--disable-colours" or "--disable-colors", if you prefer the US
spelling.

Examples:

    rbt htop --disable-colours
    rbt htop --disable-colors

This would try to compile htop while disabling colour support for
the current compilation-run.

More shortcuts exist as well, such as "nocolours" and so forth.

If you want to **permanently disable the colours**, use a commandline
variant such as the following:

    rbt --permanently-disable-colours

This will modify the file '<b>use_colours.yml</b>' in the 
**yaml/** subdirectory of the RBT project, which is the file
that determines whether RBT will use any colours or not. By
default, colours will be used.

In the future I may provide colour support for users who prefer
a white or light background, but I would need a specific suggestion
and a name for this colour "scheme". Then I can add more colours
and even colour schemes for use here.

## CreateProgram (RBT::CreateProgram)

The class <b>RBT::CreateProgram</b>, residing at 
<b>rbt/utility_scripts/create_program.rb</b>, prepares a program
for installation under the <b>/Programs/</b> hierarchy. Keep in
mind that the target at <b>/Programs/</b> can be changed by the
user - so the rest of this paragraph should be kept in this
context. The main programs directory could be **/opt/** or 
**/pkg/** or any other target too.

The behaviour by class <b>RBT::CreateProgram</b> is loosely
modelled after the GoboLinux <b>PrepareProgram</b> script.
Differences exist mostly in regards to the usage pattern and
scope between these two programs; and what is reported to the
user.

Here is a link to the **GoboLinux program** called PrepareProgram:

    https://github.com/gobolinux/Documentation/wiki/PrepareProgram

In order to remain somewhat compatible to the GoboLinux PrepareProgram
script, options such as the following will work just fine:

    create_program --tree Foo 1.0

(Here we assume that <b>create_program</b> is an alias to the file 
<b>rbt/utility_scripts/create_program.rb</b>).

The much simpler variant would be to do either one of the following
instead, as far as the <b>RBT project</b> is concerned:

    create_program Foo 1.0
    create_program Foo-1.0
    create_program foo-1.0

I personally prefer the <b>last variant</b>, since it is the simplest
one for me to remember - but use whichever variant you prefer. The
last one is simple for me because I can sort of copy/paste the 
directory structure through just one argument, to that class.

You can also use this from the commandline, if you would like to
do so, via <b>rbt</b> like in the following examples:

    rbt --create-program=foo-1.0
    rbt --create-program=php-7.4.3
    rbt --create-program=python-3.3.3

Do take note that CreateProgram may be called from some RBT scripts,
in particular **RBT::Compile**. Some directories will not see that
their default symlink (called **Current**) is removed, such as gcc or
binutils, as otherwise subsequent compilation runs would not work
to the **Current** not being set properly.

You can also use the current working directory as input.

For example, say that you are in a directory called **yelp-3.30.2/**.

If you wish to make use of that directory, as the input, as if you
would have passed the above as String into RBT::CreateProgram.new(),
you can use either of the following two variants:

    rbt --create-program
    rbt --rcp
      
## Specifying which build system to use

By default, most **.yml** files that are part of the RBT project will
make use of the **GNU configure** build system - e. g. **configure**,
**make** and **make install**. You can, on the commandline, determine
which build system is to be used; and you can also instruct
**RBT::Compile** to **use another build system**.

For example, if you wish to specifically use **configure**, then
you can do this:

    ry htop --use-this-build-system=configure

Or, if you want to use an even shorter variant, the following example
is equivalent to the one issued above:

    ry htop --use-configure

Similarly, for **cmake** or **meson**, you could use any of the
following variants:

    ry htop --use-this-build-system=cmake
    ry htop --use-this-build-system=meson

## The XFCE desktop environment

If you wish to compile all of XFCE into a single directory, such
as <b>/Programs/Xfce/4.12.0/</b>, then you can issue the following
command:

    rbt --xfce-into-standalone-dir

Some aliases exist to the above, such as:

    rbt --compile-xfce-components-into-one-standalone-directory
    rbt --compile-xfce-components-into-a-standalone-directory
    rbt --compile-xfce-components-into-standalone-directory
    rbt --use-xfce-prefix

This functionality is experimental as of October 2018, and relies
on a hardcoded path; but in the future, this functionality may
be extended to more components than just XFCE, and not depend on
a hardcoded path. When it is changed, this section will also be
changed.

If you wish to compile all of XFCE into a standalone directory,
but wish to do so in a step-like fashion e. g. in order to
reduce errors, have a look at the subsection here in this
page called **Environment settings**.

You can also batch-compile the XFCE components via:

    ry --compile-xfce

## Configure options saved

The **configure options** that were used in order to compile a particular
program at hand, will be stored in the file called
<b>configure_command_database.yml</b>, which can typically be found
at a location such as:

    /Depot/Temp/rbt/configure_command_database.yml

The entries in this yaml file may look like this:

  minuet: cmake -DCMAKE_INSTALL_PREFIX=/usr/ /Depot/Temp/rbt/minuet-18.12.3/ 2>&1

So the format is - first comes **the name of the program**; followed by the
particular command that was invoked.

Note that this will **only** be stored in that .yml file if no problem was
encountered during compilation - thus, assumingly, would mean that the
compilation was successful or at the least continued without major
problems/errors encountered.

The primary reason as to why this functionality was added was so that the
user can quickly look at that .yml file and see which configure option was
used to compile a program. This may be useful at some later time when you
wish to find out what went wrong (if something went wrong) or perhaps
if you can not access the xorg-server but need to re-compile something.

## LXQt

**LXQt** is a lightweight desktop environment based on **Qt**. It may
be an alternative if **KDE5 plasma** is too heavy for a given
computer system.

You can compile individual **LXQt components**, such as in this 
convenient way, through **rbt**:

    rbt --lxqt1 # <- This refers to "libqtxdg"
    rbt --lxqt2 # <- This refers to "lxqtbuildtools"
    rbt --lxqt3 # <- This refers to "libsysstat"
    rbt --lxqt4 # <- This refers to "liblxqt"
    rbt --lxqt5 # <- This refers to "lxqtabout"
    rbt --lxqt6 # <- This refers to "lxqtadmin"
    rbt --lxqt7 # <- This refers to "lxqtconfig"
    rbt --lxqt8 # <- This refers to "lxqtglobalkeys"
    rbt --lxqt9 # <- This refers to "lxqtl10n"

and so forth.

Of course passing in the **name**, rather than these numbers,
will also work just fine. The number-based input exists
purely due to convenience.

Examples for the long names corresponding to **--lxqt1**, **--lxqt2**,
**--lxqt3**, **--lxqt4**, would be:

    rbt libqtxdg        # lxqt1
    rbt lxqtbuildtools  # lxqt2
    rbt libsysstat      # lxqt3
    rbt liblxqt         # lxqt4

(See the content of the file called **chained_programs.yml** for
the complete listing of all LXQt components.)

If you wish to compile all LXQt components, in the order specified
by the file **chained_programs.yml**, then you can use this
commandline instruction:

    rbt --compile-lxqt

You can also **manually update lxqt** via the following **top-level API**
from within ruby:

    RBT.update_lxqt

(But normally this should not be needed, as long as I maintain the
**cookbooks-dataset**, as I will already do so when a new lxqt
release is published.)

## Tab-Completion support for various shells

**Tab completion** support exists for some of the commands, for **bash**,
**zsh** and **fish**. These are completions that are invoked when the
user hits the <b>tab-key</b> on the keyboard.

For **RBT**, this can simplify compilation. Simply type "rbt", followed by
a space, and then the first character of the program that you wish
to install, followed by hitting the **TAB** key.

Let us next look at how **generate** these (**tab**) **completions**:

    rbt --generate-completions
    ry --generate-completions
    rbt --completion
    ry --completion

The code for this functionality resides in these two files:

    rbt/compile/shell.rb
    rbt/utility_scripts/generate_shell_completion.rb

Now - after you have sourced this file into your shell, e. g.
via **source /path/to/the/file.sh**, you should be able to
use the tab-completions, like:

    rbt gnomede**TAB**

**TAB** in the context above means **to press the tab key** on
the keyboard.

The tab-completion shell script can be generated anew via the
above .rb file, but since as of 27th March 2019 the shell code
for bash and zsh is also distributed with rbt itself. This
may simplify making use of it. I source that .sh file in my
.bashrc file since that same day.

If you want to find out where that .sh file exists on your system,
try this commandline invocation:

    rbt --shell-completion?

## Installing the registered python addons

The RBT project has some python add-ons registered. These can
be installed via either:

    rbt --enable-python-addons
    rbt --python-addons

You will most likely not need this, since this is catered to my
own use case (since I have to batch-install things on a new
computer, for example). But the option exists nonetheless, since
as of March 2019.

## Generate a .html file for several programs

A **.html file** can be generated via the
method **RBT.create_html_page_for_these_programs()**.

The input to this method should be an Array listing the
programs that you want to view in that .html file. Alternatively,
a String that contains these programs separated via a ',' token
can also be used.

Example:

    "php,ruby,elixir,enchant,m4"

The code that is part of the above method will generate a .html file
that shows some information about these programs, in the given
input-output order.

Commandline invocation example:

    cookbooks --create_html_page_for_these_programs="php,ruby,elixir,enchant,m4"

Why was this functionality added?

Mostly because I needed a way to autogenerate a .html file that contains
information about several programs, such as the whole kde5-set of archives.
This then generates a .html file with about ~170 kde5-related programs, and
can then be uploaded somewhere for other people to have a look at, not
unlike the LFS/BLFS project is doing already so.

In fact, if you also want to generate a .html file for the **whole
KDE toolchain**, try this:

    cookbooks --create_html_page_for_these_programs=:kde5_toolchain

You can also autogenerate a set of "homepages" for all programs,
via class <b>Cookbooks::GenerateHomepage</b>.

Either invoke the file directly (cookbooks/utility_scripts/generate_homepage/generate_homepage.rb)
or simply use this commandline instruction:

    cookbooks --generate-homepage

This will generate .html files that are somewhat similar to the LFS/BLFS
files.

## Downloading in general and downloading a remote URL into a specific local directory

The **rbt project** has some support for downloading a remote archive into the
local filesystem/directory. This functionality was added primarily to aid in
the setup stage of the RBT project, in particular when you are before a freshly
setup computer system.

If you would like to download an archive from a **remote URL** into a specific
(local) directory, you can use an **API** such as this here, from the
commandline:

    ry htop --download-into-this-directory=/opt/bla/

The **first argument** is the registered program that ought to be used; in
this case the program called **htop**. The second argument specifies the local
directory to be used, in this case **/opt/bla/**. Obviously you could also
download first, and then **mv** the file anyway, but sometimes you may want
to do it in one go.

If you wish to quickly download e. g. htop or another program, you can
also use the following simpler toplevel API (from within ruby):

    RBT.download :htop

This is treated as the very same way as if you would have used
the following API with the full URL instead:

    RBT.download 'http://hisham.hm/htop/releases/2.2.0/htop-2.2.0.tar.gz'

It thus is simply **a convenience feature**. Obviously
"wget http://hisham.hm/htop/releases/2.2.0/htop-2.2.0.tar.gz" is
simpler, but since RBT makes use of ruby, functionality such as 
this has to be included.

You can also use the current directory as input, e. g. if you
would first create the directory htop, then cd into it, and
then invoke that method:

    RBT.download :pwd

This may save a little bit of typing.

The functionality can also be used from the **commandline**, through
the **bin/rbt** script, such as in the following examples:

    rbt --download htop
    rbt --download ruby
    rbt --download python

This would download **htop**, or **ruby**, or **python**, respectively,
into the **current working directory**.

Note that the following syntax works as well:

    rbt --download=htop
    rbt --download=ruby

Use whatever you prefer here, ' ' or '=' work just fine.

This commandline option thus allows you to quickly **download remote
source archives**.

If you do not want to type "rbt --download" then you could
also use **bin/rbt_download**, aka:

    rbt_download
    rbt_download htop
    rbt_download python

Or put an alias to this to make this even shorter.

If you wish to find out which programs can be downloaded that
way, you could try to issue this command:

    rbt --show-downloadable-programs

This will display all the names of the registered programs,
more than 3500 (so it is quite a long output; you may want
to pipe the output into **more** or **less** on Linux).

You can also download remote programs into the **proper local
directory**. The "proper" directory is the one where RBT
expects source archives to reside at.

Examples:

    rbt --download-into-proper-directory htop
    rbt --download-proper ruby # <- This variant is significantly shorter

On my home system, this would download
http://hisham.hm/htop/releases/2.2.0/htop-2.2.0.tar.gz into
the local directory **/Users/x/SRC/htop/**. The exact local
position depends on your SRC_DIR (source directory). See
elsewhere in this documentation how the source directory
can be set for the RBT project.

Do note that since as of April 2019 we can also attempt to
download different versions of a given program. As an example, let's
say that we wish to download **glibc version 2.28**. You could
use this commandline way in order to achieve this:

    rbt --download glibc-2.28
    rbt --download glibc-2.27
    rbt --download glibc-2.26
    # and so forth

This has a few constraints, though. Firstly, the remote file
must exist at the assumed position. Second, not all file
extensions may be checked. The functionality can be improved
in the long run but for now, it is like that. (The above
should work reliably well, though).

Certain github-based URLs may also not work.

## Feedback wanted

Please consider providing feedback about rbt, whether you found anything
useful in it, whether there are some bugs or other problems, and suggestions
to improve it.

Presently oldschool email should work best ( **shevegen@gmail.com** ) - I may
not reply to every email, but I will most assuredly read every email and
consider making changes to RBT in the long run.

It is a hobby project, thus the amount of time that I invest into it is
limited, but I also use RBT on my own home (linux) systems and it works
fairly well on these systems. So I am interested in the use case of
other people here.

## RBT::Make

RBT::Make is the main class that may be invoked for make-related actions.

It also supports some toplevel methods such as:

    RBT::Make.run

Additionally it can also be used to run "make install" or even "ninja",
the alternative build system.

This class exists primarily to make it easier to deal with the "make"
and "make install" step. Some instructions can be given to make,
such as the -j option, and it is simpler to abstract this away through
RBT directly, so that we can use it when compiling any program from
source.

## The IS_ROEBE environment variable

The **IS_ROEBE environment variable** is set to **1**, aka true, on my home
systems. When this variable exists and is set, then RBT will behave
slightly different from the default. For example, on my home system
I tend to use a hybrid system between the **/usr/** hierarchy and
AppDirs under **/Programs/**. Sometimes it is useful to symlink from
/Programs/Foobar/1.0.0/lib/* into **/usr/lib/**. If the IS_ROEBE
environment variable is set to 1, then the RBT scripts will, after
compilation has completed successfully, symlink into the
**/usr/lib/** hierarchy.

**IS_ROEBE** to a value of 1).

## Infobox

**class RBT::Infobox** can show some information about the program
that is to be compiled/installed next - or a program that you may
wish to install.

For this to work, the program has to be **registered** prior, as part
of the RBT project.

You can specifically tell RBT to only show the Infobox,
by making use of the commandline flag called **--show-infobox-only**.

Usage examples for this:

    rbt ruby --show-infobox-only
    rbt python --show-infobox-only
    rbt php --show-infobox-only

If you do not want to see this infobox during compilation,
you can also **disable** it for the current compilation/installation
run, via a commandline flag such as **--no-infobox**:

    rbt ruby --no-infobox
    rbt python --no-infobox
    rbt php --no-infobox
    rbt sed --no-infobox

You can also use the infobox directly, through **bin/infobox**,
such as in:

    infobox ruby
    infobox php
    infobox sed

Note that this may show slightly different results, as the variant
that makes use of **rbt** will also honour specific configuration
settings, whereas the **infobox** variant will only try to make
use of the dataset stored in the respective .yml file. So if you
would like the information displayed to be more accurate, you should
use the longer variant with **rbt**; if you just need some general
information then **infobox <name>** should do just fine. 

Since as of **May 2019** the Infobox will also show which compiler
will be used. Normally this will be GCC but clang may also be
an option; this information serves to be indicative of that.

Since as of **June 2019** the Infobox will try to use Unicode symbols
on the commandline, for the outside box. This should look better than
using === and | (ASCII characters). Do note that this is work-in-progress;
it is not working perfectly well as of yet.

## Perl

In order to compile perl, you could simply issue the following command:

    ry perl

If you wish to install the various registered perl-addons, you can
try the following command:

    ry --compile-perl-addons

(Unfortunately there is some bug, where installation stops after the
first installed perl program. I will have to investigate this at a
later time; this notice has been written in 20.04.2019).

## porg

Porg is a small utility that allows a user to track which files are
installed. RBT supports the use of porg through a configuration
file, called **use_porg.yml**.

To query from the commandline whether porg is available on the
given computer system, try:

    ry --is-porg-available?

## Some options for the individual cookbook files explained

This section will explain some entries in a .yml file.

**run_configure**:

The option option **run_configure** is set to true by default.
If it is set to **false**, then RBT will **NOT** attempt to
invoke GNU ./configure.

This may be useful for programs that you are certain to not
have and use any **configure script**. The **man-pages** archive
at **https://www.kernel.org/pub/linux/docs/man-pages/man-pages-4.16.tar.xz**
is one such an example. It does not use any configure file, so
a configure step would fail. (Note that there may be other
ways to do the same, but for the time being, <b>run_configure</b>
seems simple and effective.)

## Changing the prefix in any given Makefile

Since as of 03.05.2019 (3rd May of 2019) it is now possible to change
the prefix in a given **Makefile**, via class **RBT::ChangePrefix**.

This class can modify the PREFIX variable in that Makefile. It was added
so that we can change the PREFIX target of any Makefile on the fly.

Currently it does not have a lot of configurable options but this may
change in the future. For the time being, the first argument should
be **the path to the Makefile** that should be modified.

## Xorg

The Xorg-Server is the main implementation for the graphical Linux
system. It is a display server for the X Window System. 

Some helper-code exists in the **RBT** project to make compiling
xorg-related components somewhat easier. One aspect here is that 
the RBT project has code that will easily allow you to compile
all of the **xorg-proto** programs.

To invoke that functionality, do:

    rbt --compile-all-protos

Do note that since some time, at about the year 2018 or so, the
individual protos have been bundled into the project called
**xorg-protos**.

If you want to display all <b>xorg-related entries on the
commandline</b>, do:

    rbt --tags=xorg

You can also **compile** the xorg components in an **ordered fashion**,
by making use of numbers.

Examples:

    rbt xorg1
    rbt xorg2
    rbt xorg3
    rbt xorg4

And so forth.

To batch compile most of these components, try:

    rbt xorg1..200

If you wish to compile post-xorg-server components, that is
components that ought to be installed after **xorg-server*
has been compiled/installed, then the following commandline
may be used:

    rbt --compile-xorg-server-post
    rbt --post-xorg-server

If you wish to only compile some of the addons, in particular
input-addons related to mouse, keyboard, and a few more
entries, then you can issue the following command:

    ry --compile-mouse-and-keyboard

## Gentoo and ebuilds

Gentoo's portage software makes use of **ebuilds**.

In theory, **RBT** can (auto)generate ebuild files but this has not been
tested sufficiently - mostly because I myself do not use Gentoo, so
someone else has to do the testing.

You can give the current implementation a try though, via:

    ry ruby --create_ebuild_recipe
    ry php --create_ebuild_recipe
    ry perl --create_ebuild_recipe
    ry python --create_ebuild_recipe

And see which parts aren't working well yet.

## YAML Engine

Ruby itself defaults to **psych** as its **main yaml engine**.

When the cookbooks project was started (before it was merged
into the RBT project), **syck** was the main yaml engine.

Since as of **January 2018**, the RBT project will default to
**psych**. If for some reason you prefer syck, then you can use a
commandline switch to denote which yaml engine is to be used.

To **enable syck**, do:

    cookbooks --use-syck

To **enable psych**, do:

    cookbooks --use-psych

This will modify the yaml file called **use_psych_or_syck.yml**.

Note that in order to use **syck**, you may have to install
the **syck gem** first, via:

    gem install syck

If you want to find out which yaml engine is in use, you can do:

    RBT.yaml_engine?

Both engines should work fine. The reason why I kept on using syck for
such a long time was mostly because I used to have some german comments
in the various yaml files, which had umlauts in non-UTF encoding. I
lateron switched to pure english instead, since it is easier for other
people, and since that time, psych works just fine for the **RBT 
project**.

Otherwise I recommend to default to psych as well, as this may make other
things simpler to work with (such as working with **UTF-8 / Unicode**).

## Installing libstdc++

If you wish to install libstdc++, according to
http://www.linuxfromscratch.org/lfs/view/stable/chapter05/gcc-libstdc++.html,
then the method called **RBT.install_libstdc_plus_plus()** may be of
help.

Usage example in ruby:

    require 'rbt/toplevel_methods/install_libstdc_plus_plus.rb'

    RBT.install_libstdc_plus_plus

Note that you may have to download gcc prior to invoking this method.

## Expanding the cookbook dataset within the RBT project

By default, the individual yaml files are kept in a **minimal state**,
primarily because it is easier to modify the dataset then, via an
editor.

A minimal state also means that the format is not terribly useful by
default.

For example, the setting **keep_extracted: t** is an abbreviated
form of the setting **keep_extracted: true**. (Note that this entry
means that **the archive will remain extracted after compilation has
finished**.)

When a yaml file is loaded up, the cookbooks project normally has
to sanitize that yaml file, which takes a little time. For individual
files this is not a huge deal, but sanitizing all ~3800 something
.yml files can take quite a long time, so code had to be added to
expand this dataset into its "final" variant.

If you do not want to wait, you can speed up this process by running
the **expand** functionality, which **will expand all yaml files**
and store them in the **expanded** (correct) format. Then, when such
a yaml file is loaded up lateron, no change has to be made at all,
thus no further time has to be spent sanitizing the information since
it is already sanitized.

In order to do so, you can invoke the cookbooks project like in
this way:

    rbt --expand
    rbt --expand-dataset 

You normally only have to do this once, but it is a purely
optional step.

**If** the expanded dataset is not available then the rbt 
project may sanitize the dataset.

(Also note that the expanded dataset could be converted into
**SQL instructions**. I'll still have to add that step at
a later time, though.)

Since as of June 2019, on my home system, rbt will expand
individual entries when no expanded cookbook exists for
that given program at hand as of yet.

## BLFS - Beyond Linux From Scratch (and also LFS)

class RBT::Blfs has "support" for BLFS in that it will ... simply show
a URL to the remote BLFS page of a given package, if it exists.

If you only need the BLFS homepage, to embed it into a homepage or
display it onto the commandline, you can use the following API for
this:

    RBT.blfs(:openssl) # => "http://www.linuxfromscratch.org/blfs/view/8.1/postlfs/openssl.html"

This will return the remote BLFS page, as URL.

You can also use class **RBT::LFS** to build/create a LFS from scratch.
This is presently (2019) quite incomplete, though.

## Permanently set to another programs directory

The default programs directory is at **/Programs/**. Programs will be
installed into that directory.

If you wish to permanently set to another directory you can use this
commandline flag:

    rbt --permanently_set_programs_dir=/pkg
    rbt --use-this-as-programs-dir=/pkg

## Static compilation

Some programs can be compiled statically, but this may also lead
to problems sometimes - such as when the program can <b>NOT</b>
be compiled statically.

The optional entry **build_static** in a cookbook file allows
us to determine whether compilation should be statically or
whether it should not be static.

If you wish to overrule this on the commandline and specifically
**disable** static compilation, you can use a commandline 
instructions such as:

    ry htop --do-not-build-statically

Note that as of **December 2018** you can also combine
--enable-shared --disable-static in one variant via:

    ry libxmu --shared-no-static ntrad

To determine whether a program can be compiled statically, you can
issue a command such as the following:

    ry tar --can-be-compiled-statically?

Do note that this requires a manual registration in the corresponding
yaml file; by default this will return false (aka no).

## Checking the validity of the yaml files

Within the **rbt project** code exists to ensure that the yaml files are
(mostly) **valid** and **correct**.

Code for this functionality can in general be found under the **validation/**
subdirectory. Most users will not need to use the files in that directory,
though - these scripts are mostly for developers or maintainers of the
**rbt gem**.

The code in the **validation/** subdirectory can be thought as some kind of
test-code, with the primary focus on ensuring that the cookbook yaml files
adhere to the given specification at hand - and are, in general, correct.

If you do happen to want to check the validity of the cookbook files,
then the following command can be run:

    cookbooks --validate

Cookbook entries that are invalid, will eventually be fixed - please 
be patient (or report the erroneous entry to the lead maintainer(s)).

Class **Cookbooks::ScanForIncompleteLastUpdateEntries** can be used
to ensure that the <b>last_update:</b> entries have a valid date
entry. The currently used scheme is for "DD MM YYYY" where DD and
YYYY are numbers and MM is the three-letter abbreviation for the
month at hand. For example, "22 Mar 2018" means the 22nd of
March in the year 2018. In the future I may allow for other 
formats, but for the time being this is the variant that is
to be used (I am open to add automatic display-conversion to
any other date format).

## pre_configure_steps

The entry called **pre_configure_steps** can be used in a cookbook
.yml file. It should be an Array.

The entries listed there will be run one after the other.

A typical example for this may be like this:

    - autoreconf -vfi

A few GNU configure programs require this step, such as **hunspell**.
Have a look at the **hunspell.yml** file to find out how this is
used.

## Configure profiles / decorators

This subsection is about **configure profiles**, also called **decorators**.

These **profiles** (or decorators) allow you to use different configure
"profiles", depending on the context. For example, if you build a
LFS system then the configure option that you may want to use will
be different for regular compilation of **GCC**. Thus, a profile
exists called **gcc_lfs_pass1.md**. This file contains the configure
options that will be used, excluding --prefix (because you will
still be able to decide which prefix to use here on your own).

This functionality has been added in **June 2019** and is still
experimental - use it with care, if you decide to use it at all.

Specific examples follow next.

In order to use the profile for GCC first pass, as described in the
LFS project, use this:

    ry gcc --profile=gcc_lfs_pass1

If you want to compile a minimal GCC, into its versioned AppDir prefix,
then you can use the following call:

    ry gcc --profile=gcc-minimal --ntrad

You can also use a slightly more explicit variant:

    ry qt --use-profile=qt
    ry qt --use-profile=qt --trad
 
To get an overview of the available profiles, you can issue the following
command:

    ry --available-profiles

## Using the DESTDIR variable

This subsection is just an explanation of a feature of a **Makefile**.

If you wish to use a specific --prefix value for a Makefile as well
then you can use DESTDIR such as in this way:

    make DESTDIR=/Programs/Bzip2/1.0.7 install

## Available program versions

The following list, as html-pre tag, shows which versions are currently
tracked, including the URL to these programs.

The **four entries** are:

(**1**) **program name**

(**2**) **program version**

(**3**) **last update of this program** (in regards to the cookbooks project)

(**4**) **remote URL**

The most useful entry of this list for users of the RBT project is probably
the **remote URL** entry there - the **most right** entry.

Note that some old programs are no longer available, which explains
why they have not been updated in many years. Can't update what is
no longer there now, can you. :)

(Very outdated program entries will eventually be removed from the
Cookbooks listing, in particular if the program is no longer
available.)


<pre>
0install                    2.12.1                19 Jun 2017   https://sourceforge.net/projects/zero-install/files/0install/2.12.1/0install-2.12.1.tar.bz2
3ddesktop                   0.2.9                 01 Jun 2014   https://sourceforge.net/projects/desk3d/files/3ddesktop/0.2.9/3ddesktop-0.2.9.tar.gz
3dpong                      0.5                   01 May 2014   ftp://ftp.billsgames.com/unix/x/3dpong/src/3dpong-0.5.tar.gz
Mako                        1.0.13                09 Jul 2019   https://files.pythonhosted.org/packages/fa/29/8016763284d8fab844224f7cc5675cb4a388ebda0eb5a403260187e48435/Mako-1.0.13.tar.gz
Python                      3.7.4                 09 Jul 2019   https://www.python.org/ftp/python/3.7.4/Python-3.7.4.tgz
R-3.6.0                                                         
R                           3.6.1                 09 Jul 2019   https://cran.r-project.org/src/base/R-3/R-3.6.1.tar.gz
Thunar-1.8.7                                                    
a2png                       0.1.5                 01 Jun 2014   http://sourceforge.net/projects/a2png/files/a2png/0.1.5/a2png-0.1.5.tar.gz
a2ps                        4.14                  01 Jun 2014   http://ftp.gnu.org/gnu/a2ps/a2ps-4.14.tar.gz
a52dec                      0.7.4                 01 Oct 2013   http://liba52.sourceforge.net/files/a52dec-0.7.4.tar.gz
aalib                       1.4rc5                01 Jan 2013   http://sourceforge.net/projects/aa-project/files/aa-lib/1.4rc5/aalib-1.4rc5.tar.gz
aamath                      0.3                   01 May 2014   http://fuse.superglue.se/aamath/aamath-0.3.tar.gz
abcde                       2.8.1                 23 Oct 2017   https://abcde.einval.com/download/abcde-2.8.1.tar.gz
abe                         1.1                   01 May 2014   http://kent.dl.sourceforge.net/sourceforge/abe/abe-1.1.tar.gz
abiword                     3.0.2                 21 Oct 2016   http://www.abisource.com/downloads/abiword/3.0.2/source/abiword-3.0.2.tar.gz
abnfgen                     0.17                  16 Sep 2016   http://www.quut.com/abnfgen/abnfgen-0.17.tar.gz
abstk                       0.1                   07 Feb 2016   http://www.gobolinux.org/abstk/AbsTK-0.1.tar.gz
abuse                       0.8                   01 Dec 2012   http://abuse.zoy.org/raw-attachment/wiki/download/abuse-0.8.tar.gz
accerciser                  3.32.1                07 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/accerciser/3.32/accerciser-3.32.1.tar.xz
accountsservice             0.6.46                08 Apr 2018   https://www.freedesktop.org/software/accountsservice/accountsservice-0.6.46.tar.xz
acl                         2.2.53                01 Jan 2019   http://download.savannah.gnu.org/releases/acl/acl-2.2.53.tar.gz
acoc                        0.7.1                 01 May 2014   http://www.caliban.org/files/ruby/acoc-0.7.1.tar.gz
acpi                        0.09                  01 May 2014   http://grahame.angrygoats.net/source/acpi/acpi-0.09.tar.gz
acpica                      11.10.2017            11 Oct 2017   http://www.acpica.org/download/acpica-unix-20080701.tar.gz
acpid                       2.0.31                17 Nov 2018   http://downloads.sourceforge.net/acpid2/acpid-2.0.31.tar.xz
acpitool                    0.5.1                 12 Oct 2017   https://sourceforge.net/projects/acpitool/files/acpitool/0.5.1/acpitool-0.5.1.tar.gz
actioncable                 5.2.3                 17 Apr 2019   https://rubygems.org/downloads/actioncable-5.2.3.gem
actionmailer                5.2.3                 17 Apr 2019   https://rubygems.org/downloads/actionmailer-5.2.3.gem
actionpack                  5.2.3                 17 Apr 2019   https://rubygems.org/downloads/actionpack-5.2.3.gem
actionview                  5.2.3                 17 Apr 2019   https://rubygems.org/downloads/actionview-5.2.3.gem
actionwebservice            1.2.6                 21 Sep 2017   https://rubygems.org/downloads/actionwebservice-1.2.6.gem
activejob                   5.2.3                 17 Apr 2019   https://rubygems.org/downloads/activejob-5.2.3.gem
activemodel                 5.2.3                 17 Apr 2019   https://rubygems.org/downloads/activemodel-5.2.3.gem
activerecord                5.2.3                 17 Apr 2019   https://rubygems.org/downloads/activerecord-5.2.3.gem
activeresource              5.1.0                 06 Nov 2018   https://rubygems.org/downloads/activeresource-5.1.0.gem
activestorage               5.2.3                 17 Apr 2019   https://rubygems.org/downloads/activestorage-5.2.3.gem
activesupport               5.2.3                 17 Apr 2019   https://rubygems.org/downloads/activesupport-5.2.3.gem
addressable                 2.6.0                 18 Feb 2019   https://rubygems.org/downloads/addressable-2.6.0.gem
adwaitaicontheme            3.32.0                19 Mar 2019   http://ftp.acc.umu.se/pub/GNOME/sources/adwaita-icon-theme/3.32/adwaita-icon-theme-3.32.0.tar.xz
aegisub                     3.2.2                 24 Jan 2018   http://ftp.aegisub.org/pub/archives/releases/source/aegisub-3.2.2.tar.xz
aeolus                      0.9.0                 15 Mar 2016   http://kokkinizita.linuxaudio.org/linuxaudio/downloads/aeolus-0.9.0.tar.bz2
afew                        1.3.0                 25 Nov 2018   https://files.pythonhosted.org/packages/dd/a5/2b59d9ba0501de90e7791fda0a0de886d8aa8b52b49a24e49ba8a444a062/afew-1.3.0.tar.gz
afterstep                   2.2.9                 01 May 2014   ftp://ftp.afterstep.org/stable/AfterStep-2.2.9.tar.bz2
aircrackng                  1.3                   18 Sep 2018   http://download.aircrack-ng.org/aircrack-ng-1.3.tar.gz
airsnort                    0.2.7e                01 Oct 2013   http://sourceforge.net/projects/airsnort/files/airsnort/airsnort-0.2.7e/airsnort-0.2.7e.tar.gz
airstrike                   6a                    01 Dec 2012   http://icculus.org/airstrike/airstrike-pre6a-src.tar.gz
aisleriot                   3.22.7                23 Oct 2018   http://ftp.gnome.org/pub/gnome/sources/aisleriot/3.22/aisleriot-3.22.7.tar.xz
akonadi                     19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/akonadi-19.04.2.tar.xz
akonadicalendar             19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/akonadi-calendar-19.04.2.tar.xz
akonadicalendartools        19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/akonadi-calendar-tools-19.04.2.tar.xz
akonadiconsole              19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/akonadiconsole-19.04.2.tar.xz
akonadicontacts             19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/akonadi-contacts-19.04.2.tar.xz
akonadiimportwizard         19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/akonadi-import-wizard-19.04.2.tar.xz
akonadimime                 19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/akonadi-mime-19.04.2.tar.xz
akonadinotes                19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/akonadi-notes-19.04.2.tar.xz
akonadisearch               19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/akonadi-search-19.04.2.tar.xz
akregator                   19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/akregator-19.04.2.tar.xz
alacarte                    3.11.91               01 Mar 2014   http://ftp.gnome.org/pub/GNOME/sources/alacarte/3.11/alacarte-3.11.91.tar.xz
alien                       8.95                  06 Oct 2015   http://http.debian.net/debian/pool/main/a/alien/alien_8.95.tar.xz
allegro                     5.2.5.0               04 Mar 2019   https://github.com/liballeg/allegro5/releases/download/5.2.5.0/allegro-5.2.5.0.tar.gz
alltray                     0.70                  12 Nov 2017   https://launchpad.net/alltray/historic-releases/0.70/+download/alltray-0.70.tar.gz
almanah                     0.11.1                25 Sep 2014   http://ftp.gnome.org/pub/GNOME/sources/almanah/0.11/almanah-0.11.1.tar.xz
alock                       94                    01 Jul 2011   http://alock.googlecode.com/files/alock-94.tar.bz2
alsadriver                  1.0.25                20 Jun 2015   ftp://ftp.alsa-project.org/pub/driver/alsa-driver-1.0.25.tar.bz2
alsafirmware                1.0.29                26 Jan 2019   ftp://ftp.alsa-project.org/pub/firmware/alsa-firmware-1.0.29.tar.bz2
alsalib                     1.1.9                 10 May 2019   ftp://ftp.alsa-project.org/pub/lib/alsa-lib-1.1.9.tar.bz2
alsaoss                     1.1.8                 26 Jan 2019   ftp://ftp.alsa-project.org/pub/oss-lib/alsa-oss-1.1.8.tar.bz2
alsaplayer                  0.99.81               01 Jun 2013   http://www.alsaplayer.org/alsaplayer-0.99.81.tar.bz2
alsaplugins                 1.1.9                 12 May 2019   ftp://ftp.alsa-project.org/pub/plugins/alsa-plugins-1.1.9.tar.bz2
alsatools                   1.1.7                 26 Jan 2019   ftp://ftp.alsa-project.org/pub/tools/alsa-tools-1.1.7.tar.bz2
alsautils                   1.1.9                 12 May 2019   ftp://ftp.alsa-project.org/pub/utils/alsa-utils-1.1.9.tar.bz2
amarok                      2.9.0                 07 Mar 2018   https://download.kde.org/stable/amarok/2.9.0/src/amarok-2.9.0.tar.xz
amidiplug                   0.7                   01 Nov 2012   http://www.develia.org/files/amidi-plug-0.7.tar.bz2
amigadepacker               0.04                  01 Jun 2014   http://zakalwe.fi/~shd/foss/amigadepacker/amigadepacker-0.04.tar.bz2
amoebax                     0.2.1                 01 Jan 2013   http://www.emma-soft.com/games/amoebax/download/amoebax-0.2.1.tar.bz2
amphetamine                 0.8.10                01 Jun 2014   http://n.ethz.ch/student/loehrerl/amph/files/amphetamine-0.8.10.tar.bz2
amtk                        5.0.0                 16 Jul 2018   http://ftp.gnome.org/pub/gnome/sources/amtk/5.0/amtk-5.0.0.tar.xz
amule                       2.3.1                 01 Jun 2014   http://sourceforge.net/projects/amule/files/aMule/2.3.1/aMule-2.3.1.tar.xz
analitza                    19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/analitza-19.04.2.tar.xz
analogger                   1.1.0                 08 Jul 2018   https://rubygems.org/downloads/analogger-1.1.0.gem
anjuta                      3.28.0                02 Jun 2018   http://ftp.gnome.org/pub/GNOME/sources/anjuta/3.28/anjuta-3.28.0.tar.xz
anjutaextras                3.26.0                11 Sep 2017   http://ftp.gnome.org/pub/gnome/sources/anjuta-extras/3.26/anjuta-extras-3.26.0.tar.xz
ansi                        1.5.0                 09 Jan 2018   https://rubygems.org/downloads/ansi-1.5.0.gem
antiword                    0.37                  01 May 2014   http://www.winfield.demon.nl/linux/antiword-0.37.tar.gz
anyremote                   1.1                   01 Jun 2014   http://anyremote.sourceforge.net/dload.html
aoetools                    36                    22 Feb 2017   https://downloads.sourceforge.net/project/aoetools/aoetools/aoetools-36.tar.gz
apacheant                   1.10.2                19 Feb 2018   https://archive.apache.org/dist/ant/source/apache-ant-1.10.2-src.tar.xz
apachemaven                 3.5.0                 03 Oct 2017   http://www-us.apache.org/dist/maven/maven-3/3.5.0/source/apache-maven-3.5.0-src.tar.gz
apl                         1.0                   01 Sep 2013   http://ftp.gnu.org/gnu/apl/apl-1.0.tar.gz
apophenia                   0.99                  01 Jun 2014   http://osdn.dl.sourceforge.net/sourceforge/apophenia/apophenia-0.16-23april06.tgz
aporia                      27.12.2011            01 Jun 2014   aporia-27.12.2011
appdatatools                0.1.8                 01 Sep 2014   http://people.freedesktop.org/~hughsient/releases/appdata-tools-0.1.8.tar.xz
appfs                       1.10                  17 Sep 2018   http://www.rkeene.org/devel/appfs/appfs-1.10.tar.gz
applewmproto                1.4.2                 01 May 2014   http://xorg.freedesktop.org/releases/individual/proto/applewmproto-1.4.2.tar.bz2
appres                      1.0.4                 01 Feb 2013   http://xorg.freedesktop.org/releases/individual/app/appres-1.0.4.tar.bz2
appstream                   0.12.0                26 May 2018   https://www.freedesktop.org/software/appstream/releases/AppStream-0.12.0.tar.xz
appstreamglib               0.7.14                18 Dec 2018   https://people.freedesktop.org/~hughsient/appstream-glib/releases/appstream-glib-0.7.14.tar.xz
apr                         1.7.0                 08 Apr 2019   http://www-us.apache.org/dist//apr/apr-1.7.0.tar.bz2
apricots                    0.2.6                 01 May 2014   http://www.fishies.org.uk/apricots-0.2.6.tar.gz
aproc                       0.1                   22 Mar 2016   http://stinfwww.informatik.uni-leipzig.de/~mai00bgn/aproc/tb/aproc-0.1.tar.bz2
aprutil                     1.6.1                 26 Oct 2017   http://archive.apache.org/dist/apr/apr-util-1.6.1.tar.bz2
apt                         1.7.0                 07 Oct 2018   http://ftp.debian.org/debian/pool/main/a/apt/apt_1.7.0.tar.xz
apulse                      02.10.2017            02 Oct 2017   http://freedesktop.org/software/pulseaudio/releases/apulse-02.10.2017
arandr                      0.1.10                01 Aug 2014   http://christian.amsuess.com/tools/arandr/files/arandr-0.1.10.tar.gz
aravis                      0.6.3                 10 Jun 2019   http://ftp.gnome.org/pub/gnome/sources/aravis/0.6/aravis-0.6.3.tar.xz
archivezip                  1.64                  04 May 2019   http://search.cpan.org/CPAN/authors/id/P/PH/PHRED/Archive-Zip-1.64.tar.gz
ardour                      2.8.16                01 Jan 2014   http://releases.ardour.org/ardour-2.8.16.tar.bz2
arel                        9.0.0                 22 Jul 2018   https://rubygems.org/downloads/arel-9.0.0.gem
argtable2                   13                    09 Sep 2018   http://prdownloads.sourceforge.net/argtable/argtable2-13.tar.gz
aria2                       1.16.0                01 Dec 2012   http://downloads.sourceforge.net/project/aria2/stable/aria2-1.16.0/aria2-1.16.0.tar.bz2
aris                        2.2                   11 Dec 2017   http://ftp.gnu.org/gnu/aris/aris-2.2.tar.gz
ark                         19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/ark-19.04.2.tar.xz
armagetronad                0.2.8.3.4             12 Nov 2017   https://launchpad.net/armagetronad/0.2.8/0.2.8.3.4/+download/armagetronad-0.2.8.3.4.src.tar.gz
art                         2.7                   24 Dec 2018   https://files.pythonhosted.org/packages/f0/71/5c7edcab50506511fd0a2aacba4b817fdfd3c84f79581a9c1482523f4dda/art-2.7.tar.gz
artanis                     0.3.1                 31 Dec 2018   https://ftp.gnu.org/pub/gnu/artanis/artanis-0.3.1.tar.bz2
artemis                     v16.0.11              14 Feb 2016   ftp://ftp.sanger.ac.uk/pub4/resources/software/artemis/v16/v16.0.11/artemis_v16.0.11.jar
artikulate                  19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/artikulate-19.04.2.tar.xz
asc                         2.6.1.0               14 Feb 2016   https://sourceforge.net/projects/asc-hq/files/ASC%20Source/2.6.1/asc-2.6.1.0.tar.bz2
ascal                       0.1.1                 01 May 2014   http://sourceforge.net/projects/ascal/files/ascal/the%20last%20one%20this%20year/ascal-0.1.1.tar.bz2
asciidoc                    8.6.9                 25 Jul 2017   https://sourceforge.net/projects/asciidoc/files/asciidoc/8.6.9/asciidoc-8.6.9.tar.gz
asciidoctor                 2.0.10                01 Jun 2019   https://rubygems.org/downloads/asciidoctor-2.0.10.gem
aspell                      0.60.6.1              01 May 2012   ftp://ftp.gnu.org/gnu/aspell/aspell-0.60.6.1.tar.gz
ast                         2.4.0                 14 Oct 2018   https://rubygems.org/downloads/ast-2.4.0.gem
asunder                     2.1                   01 May 2014   http://littlesvr.ca/asunder/releases/asunder-2.1.tar.bz2
asymptote                   2.44                  10 Apr 2018   https://downloads.sourceforge.net/asymptote/asymptote-2.44.src.tgz
aterm                       1.0.1                 01 May 2014   ftp://ftp.afterstep.org/apps/aterm/aterm-1.0.1.tar.bz2
atk                         2.32.0                11 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/atk/2.32/atk-2.32.0.tar.xz
atkmm                       2.28.0                04 Nov 2018   http://ftp.gnome.org/pub/gnome/sources/atkmm/2.28/atkmm-2.28.0.tar.xz
atomix                      3.30.0.1              05 Sep 2018   http://ftp.gnome.org/pub/gnome/sources/atomix/3.30/atomix-3.30.0.1.tar.xz
atool                       0.39.0                01 May 2014   http://savannah.nongnu.org/download/atool/atool-0.39.0.tar.gz
atril                       1.22.1                24 Apr 2019   https://pub.mate-desktop.org/releases/1.22/atril-1.22.1.tar.xz
atspi                       1.32.0                01 Dec 2012   http://ftp.gnome.org/pub/gnome/sources/at-spi/1.32/at-spi-1.32.0.tar.bz2
atspi2atk                   2.32.0                13 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/at-spi2-atk/2.32/at-spi2-atk-2.32.0.tar.xz
atspi2core                  2.32.1                09 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/at-spi2-core/2.32/at-spi2-core-2.32.1.tar.xz
attal                       1.0rc1                01 May 2014   http://www.attal-thegame.org/base.php?langue=en&page=accueil
attica                      5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/attica-5.59.0.tar.xz
attr                        2.4.48                23 Feb 2019   http://download.savannah.nongnu.org/releases/attr/attr-2.4.48.tar.gz
attrs                       19.1.0                12 May 2019   https://files.pythonhosted.org/packages/cc/d9/931a24cc5394f19383fbbe3e1147a0291276afa43a0dc3ed0d6cd9fda813/attrs-19.1.0.tar.gz
aubio                       0.4.7                 06 Jun 2019   https://aubio.org/pub/aubio-0.4.7.tar.bz2
audacious                   3.10.1                28 Dec 2018   http://distfiles.audacious-media-player.org/audacious-3.10.1.tar.bz2
audaciousplugins            3.10.1                06 Jan 2019   http://distfiles.audacious-media-player.org/audacious-plugins-3.10.1.tar.bz2
audacityminsrc              2.3.2                 15 May 2019   https://www.fosshub.com/Audacity.html/audacity-minsrc-2.3.2.tar.xz
audiere                     1.9.4                 01 May 2014   http://prdownloads.sourceforge.net/audiere/audiere-1.9.4.tar.gz
audiocdkio                  19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/audiocd-kio-19.04.2.tar.xz
audiofile                   0.3.6                 01 May 2014   https://github.com/downloads/mpruett/audiofile/audiofile-0.3.6.tar.gz
aufs                        09.01.2009            01 May 2014   http://aufs.sourceforge.net/
augeas                      1.8.1                 02 Sep 2017   http://download.augeas.net/augeas-1.8.1.tar.gz
aumix                       2.9.1                 17 Nov 2015   http://sourceforge.net/projects/aumix/files/aumix/2.9.1/aumix-2.9.1.tar.bz2
autoconf                    2.69                  01 Jul 2013   ftp://ftp.gnu.org/gnu/autoconf/autoconf-2.69.tar.xz
autofs                      5.1.5                 01 Nov 2018   http://www.kernel.org/pub/linux/daemons/autofs/v5/autofs-5.1.5.tar.xz
autogen                     5.18.16               09 Sep 2018   https://ftp.gnu.org/pub/gnu/autogen/rel5.18.16/autogen-5.18.16.tar.xz
automake                    1.16.1                12 Mar 2018   ftp://ftp.gnu.org/gnu/automake/automake-1.16.1.tar.xz
automoc                     27.03.2011            01 Nov 2012   http://techbase.kde.org/Getting_Started/Build/KDE4/Gentoo
automoc4                    0.9.88                01 Jan 2014   http://download.kde.org/stable/automoc4/0.9.88/automoc4-0.9.88.tar.bz2
autotoolset                 0.11.4                01 May 2014   autotoolset-0.11.4
autovivification            0.17                  05 Aug 2017   http://search.cpan.org/CPAN/authors/id/V/VP/VPIT/autovivification-0.17.tar.gz
avahi                       0.7                   04 Dec 2018   http://avahi.org/download/avahi-0.7.tar.gz
avantwindownavigator        0.4.2                 01 Sep 2014   https://launchpad.net/awn/0.4/0.4.2/+download/avant-window-navigator-0.4.2.tar.gz
avc                         0.8.2                 01 May 2014   http://avc.inrim.it/dist/avc-0.8.2.tar.gz
avidemux                    2.7.3                 21 Mar 2019   https://sourceforge.net/projects/avidemux/files/avidemux/2.7.3/avidemux_2.7.3.tar.gz
avifile                     0.7.45                01 May 2014   http://avifile.sourceforge.net/
avinfo                      1.0a12                01 May 2014   http://shounen.ru/soft/avinfo/avinfo-1.0a16.zip
awesome                     4.3                   28 Jan 2019   https://github.com/awesomeWM/awesome/releases/download/v4.3/awesome-4.3.tar.xz
axel                        2.17.3                23 May 2019   https://github.com/axel-download-accelerator/axel/releases/download/v2.17.3/axel-2.17.3.tar.xz
axtls                       2.1.3                 02 Aug 2017   https://sourceforge.net/projects/axtls/files/2.1.3/axTLS-2.1.3.tar.gz
axv                         0.1                   01 May 2014   http://axv.sourceforge.net/
b43fwcutter                 015                   01 May 2014   http://bues.ch/b43/fwcutter/b43-fwcutter-015.tar.bz2
babl                        0.1.66                13 Jun 2019   https://download.gimp.org/pub/babl/0.1/babl-0.1.66.tar.bz2
backports                   3.15.0                01 Jun 2019   https://rubygems.org/downloads/backports-3.15.0.gem
bacula                      5.2.9                 01 May 2014   http://sourceforge.net/projects/bacula/files/bacula/5.2.9/bacula-5.2.9.tar.gz
bakery                      2.5.1                 01 May 2014   http://ftp.gnome.org/pub/GNOME/sources/bakery/2.5/bakery-2.5.1.tar.bz2
ballbuster                  1.1                   01 Jan 2013   http://www.patrickavella.com/ballbuster-1.1.tar.bz2
baloo                       5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/baloo-5.59.0.tar.xz
baloowidgets                19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/baloo-widgets-19.04.2.tar.xz
balsa                       2.5.7                 09 Jun 2019   http://pawsa.fedorapeople.org/balsa/balsa-2.5.7.tar.bz2
bandit                      1.5.1                 02 Feb 2019   https://files.pythonhosted.org/packages/c9/60/2c967faf70596fcef42a0737c63fe1321b8e51e15eec8f7883e333eba5a5/bandit-1.5.1.tar.gz
banshee                     2.9.1                 10 Sep 2014   http://ftp.gnome.org/pub/GNOME/sources/banshee/2.9/banshee-2.9.1.tar.xz
baobab                      3.32.0                30 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/baobab/3.32/baobab-3.32.0.tar.xz
barcode                     0.98                  01 May 2014   ftp://mirrors.usc.edu/pub/gnu/barcode/barcode-0.98.tar.gz
barrage                     1.0.4                 01 May 2014   http://prdownloads.sourceforge.net/lgames/barrage-1.0.4.tar.gz
bash                        5.0                   07 Jan 2019   https://ftp.gnu.org/pub/gnu/bash/bash-5.0.tar.gz
bashcompletion              2.1                   01 Dec 2018   https://github.com/scop/bash-completion/archive/2.1.tar.gz
bazar                       1.1                   01 May 2014   http://cvlab.epfl.ch/software/bazar/bazar-1.1.tar.gz
bc                          1.07.1                22 Jul 2017   https://ftp.gnu.org/gnu/bc/bc-1.07.1.tar.gz
bchunk                      1.2.0                 01 May 2014   http://he.fi/bchunk/
bdftopcf                    1.1                   09 Nov 2017   https://www.x.org/releases/individual/app/bdftopcf-1.1.tar.gz
beagle                      0.3.6                 01 May 2014   ftp://ftp.gnome.org/pub/GNOME/sources/beagle/0.3/beagle-0.3.6.tar.bz2
bedtools                    2.27.1                19 Jan 2019   https://github.com/arq5x/bedtools2/releases/download/v2.27.1/bedtools-2.27.1.tar.gz
beecrypt                    4.2.1                 01 May 2014   http://sourceforge.net/projects/beecrypt/files/beecrypt/4.2.1/beecrypt-4.2.1.tar.gz
beforelight                 1.0.5                 01 Jan 2013   http://xorg.freedesktop.org/releases/individual/app/beforelight-1.0.5.tar.bz2
benchmarkips                2.7.2                 12 Oct 2017   https://rubygems.org/downloads/benchmark-ips-2.7.2.gem
bettererrors                2.5.1                 13 Apr 2019   https://rubygems.org/downloads/better_errors-2.5.1.gem
bftpd                       5.0                   29 Oct 2018   https://sourceforge.net/projects/bftpd/files/bftpd/bftpd-5.0/bftpd-5.0.tar.gz
bigdecimal                  1.4.4                 01 Jun 2019   https://rubygems.org/downloads/bigdecimal-1.4.4.gem
bigreqsproto                1.1.2                 01 Jun 2014   http://xorg.freedesktop.org/releases/individual/proto/bigreqsproto-1.1.2.tar.bz2
bijiben                     3.32.1                10 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/bijiben/3.32/bijiben-3.32.1.tar.xz
biloba                      0.9.3                 12 Mar 2019   https://sourceforge.net/projects/biloba/files/Biloba/0.9.3/biloba-0.9.3.tar.gz
bind                        9.15.1                20 Jun 2019   ftp://ftp.isc.org/isc/bind9/9.15.1/bind-9.15.1.tar.gz
biniax                      1.2                   01 Jan 2013   http://mordred.dir.bg/biniax/download.htm
binutils                    2.32                  02 Feb 2019   http://ftp.gnu.org/pub/gnu/binutils/binutils-2.32.tar.xz
bio                         1.5.2                 26 Sep 2018   https://rubygems.org/downloads/bio-1.5.2.gem
bioperl                     1.007002              02 Jan 2018   https://cpan.metacpan.org/authors/id/C/CJ/CJFIELDS/BioPerl-1.007002.tar.gz
biopython                   1.73                  23 Dec 2018   http://biopython.org/DIST/biopython-1.73.tar.gz
bioruby                     1.5.1                 17 Jul 2018   http://bioruby.open-bio.org/archive/bioruby-1.5.1.tar.gz
biosdevname                 0.7.1                 25 Dec 2015   http://linux.dell.com/files/biosdevname/biosdevname-0.7.1/biosdevname-0.7.1.tar.gz
bird                        2.0.3                 09 Jan 2019   ftp://bird.network.cz/pub/bird/bird-2.0.3.tar.gz
bison                       3.4.1                 23 May 2019   https://ftp.gnu.org/gnu/bison/bison-3.4.1.tar.xz
bitlbee                     1.2.1                 01 Jun 2014   http://get.bitlbee.org/src/bitlbee-1.2.1.tar.gz
bitmap                      1.0.9                 11 Feb 2019   http://xorg.freedesktop.org/releases/individual/app/bitmap-1.0.9.tar.bz2
bittorrent                  4.4.0                 01 Jun 2014   http://www.bittorrent.com/dl/BitTorrent-4.4.0.tar.gz
bkchem                      0.13.0                01 Feb 2014   http://bkchem.zirael.org/download/bkchem-0.13.0.tar.gz
blackbox                    0.70.1                01 Jun 2014   http://kent.dl.sourceforge.net/sourceforge/blackboxwm/blackbox-0.70.1.tar.bz2
blam                        1.8.12                24 Sep 2014   http://ftp.gnome.org/pub/GNOME/sources/blam/1.8/blam-1.8.12.tar.xz
blender                     2.79                  12 Sep 2017   http://download.blender.org/source/blender-2.79.tar.gz
bless                       0.5.2                 01 Nov 2012   http://download.gna.org/bless/bless-0.5.2.tar.gz
blessings                   1.5                   01 May 2014   http://pypi.python.org/packages/source/b/blessings/blessings-1.5.tar.gz
blinken                     19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/blinken-19.04.2.tar.xz
blobwars                    1.04                  01 Jan 2013   http://www.parallelrealities.co.uk/download.php?file=blobwars-1.04-1.tar.gz&type=zip
blockattack                 1.3.0                 01 Jan 2013   http://belnet.dl.sourceforge.net/sourceforge/blockattack/BlockAttackLinux112.tar.gz
blogilo                     17.08.3               09 Nov 2017   https://download.kde.org/stable/applications/17.08.3/src/blogilo-17.08.3.tar.xz
bluedevil                   5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/bluedevil-5.16.3.tar.xz
bluefish                    1.0.7                 26 Jul 2017   http://www.bennewitz.com/bluefish/stable/source/bluefish-1.0.7.tar.bz2
blueman                     2.1                   28 Jun 2019   https://github.com/blueman-project/blueman/releases/download/2.1/blueman-2.1.tar.xz
bluez                       5.50                  05 Jun 2018   http://www.kernel.org/pub/linux/bluetooth/bluez-5.50.tar.xz
bluezgnome                  0.14                  01 Jun 2014   http://www.kernel.org/pub/linux/bluetooth/bluez-gnome-0.14.tar.gz
bluezqt                     5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/bluez-qt-5.59.0.tar.xz
bluezutils                  2.25                  28 Jul 2016   http://bluez.sf.net/download/bluez-utils-2.25.tar.gz
bodr                        6                     01 May 2014   http://www.blueobelisk.org/
bol                         0.8                   01 May 2014   http://surfnet.dl.sourceforge.net/sourceforge/fpcbol/bol.tar.bz
bomber                      19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/bomber-19.04.2.tar.xz
bomberclone                 0.11.9                10 Oct 2017   https://sourceforge.net/projects/bomberclone/files/bomberclone/bomberclone-0.11.9.tar.gz
bombic                      0.0.1                 01 Jan 2013   https://sourceforge.net/projects/bombic/files/bombic/Bombic%200.0.1/bombic-0.0.1-src.tar.gz
bomns                       0.99.1                01 Jun 2014   http://ovh.dl.sourceforge.net/sourceforge/greenridge/bomns-0.99.1.tar.bz2
bookify                     1.5.0                 08 Aug 2018   https://rubygems.org/downloads/bookify-1.5.0.gem
boost                       1.68.0                13 Apr 2019   https://dl.bintray.com/boostorg/release/1.70.0/source/boost_1_70_0.tar.bz2
boostbuild                  2014-10               11 Sep 2017   https://github.com/boostorg/build/releases/download/2014.10/boost-build-2014-10.tar.bz2
boswars                     2.7                   01 Aug 2013   https://www.boswars.org/dist/releases/boswars-2.7-src.tar.gz
botocore                    1.12.146              12 May 2019   https://files.pythonhosted.org/packages/5a/4b/85cd24763385a680df8de460be28d195f44880a9727fe34b71273a9cdd95/botocore-1.12.146.tar.gz
bovo                        19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/bovo-19.04.2.tar.xz
brasero                     3.12.2                01 Aug 2017   http://ftp.gnome.org/pub/gnome/sources/brasero/3.12/brasero-3.12.2.tar.xz
breeze                      5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/breeze-5.16.3.tar.xz
breezegrub                  5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/breeze-grub-5.16.3.tar.xz
breezegtk                   5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/breeze-gtk-5.16.3.tar.xz
breezeicons                 5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/breeze-icons-5.59.0.tar.xz
breezeplymouth              5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/breeze-plymouth-5.16.3.tar.xz
bridgeutils                 1.6                   26 Jul 2017   https://www.kernel.org/pub/linux/utils/net/bridge-utils/bridge-utils-1.6.tar.xz
briquolo                    0.5.7                 01 Jun 2014   http://briquolo.free.fr/download/briquolo-0.5.7.tar.bz2
brlcad                      7.26.0                22 Feb 2017   https://sourceforge.net/projects/brlcad/files/BRL-CAD%20Source/7.26.0/brlcad-7.26.0.tar.bz2
brltty                      6.0                   25 Feb 2019   http://mielke.cc/brltty/archive/brltty-6.0.tar.xz
brotli                      1.0.2                 01 Dec 2017   https://github.com/google/brotli/archive/v1.0.2.tar.gz
bsc                         2.27                  01 Jun 2014   http://www.beesoft.org/download/bsc_2.20_src.tar.gz
btrbk                       0.27.1                06 Dec 2018   https://digint.ch/download/btrbk/releases/btrbk-0.27.1.tar.xz
btrfsprogs                  v5.2                  07 Jul 2019   https://mirrors.edge.kernel.org/pub/linux/kernel/people/kdave/btrfs-progs/btrfs-progs-v5.2.tar.xz
bubblewrap                  0.3.3                 03 May 2019   https://github.com/projectatomic/bubblewrap/releases/download/v0.3.3/bubblewrap-0.3.3.tar.xz
bubbros                     1.6                   01 Jun 2014   http://briquolo.free.fr/en/download.html
budgiedesktop               10.4                  15 Aug 2017   https://github.com/solus-project/budgie-desktop/releases/download/v10.4/budgie-desktop-10.4.tar.xz
bugbuddy                    2.32.0                01 Jun 2014   http://ftp.acc.umu.se/pub/GNOME/sources/bug-buddy/2.32/bug-buddy-2.32.0.tar.bz2
builder                     3.2.3                 03 Dec 2017   https://rubygems.org/downloads/builder-3.2.3.gem
buildroot                   20171011              11 Oct 2017   http://buildroot.uclibc.org/downloads/snapshots/buildroot-20171011.tar.bz2
buildstream                 1.2.7                 20 May 2019   http://ftp.gnome.org/pub/gnome/sources/BuildStream/1.2/BuildStream-1.2.7.tar.xz
bullet                      2.83.6                22 Sep 2015   https://github.com/bulletphysics/bullet3/archive/bullet-2.83.6.tar.gz
bumprace                    1.5.3                 01 Jun 2014   http://www.linux-games.com/bumprace/index.html
bundler                     2.0.1                 18 Feb 2019   https://rubygems.org/downloads/bundler-2.0.1.gem
burgerspace                 1.9.2                 02 Nov 2015   http://perso.b2b2c.ca/sarrazip/dev/burgerspace-1.9.2.tar.gz
busybox                     1.31.0                11 Jun 2019   http://www.busybox.net/downloads/busybox-1.31.0.tar.bz2
butt                        0.1.17                04 Mar 2019   http://sourceforge.net/projects/butt/files/butt/butt-0.1.17/butt-0.1.17.tar.gz
byebug                      11.0.1                14 Apr 2019   https://rubygems.org/downloads/byebug-11.0.1.gem
bytename                    1.9                   01 Jun 2014   http://billposer.org/Software/Downloads/bytename-1.9.tar.bz2
bzip2                       1.0.7                 28 Jun 2019   https://sourceware.org/pub/bzip2/bzip2-1.0.7.tar.gz
bzr                         2.7.0                 14 Mar 2017   http://launchpad.net/bzr/2.7/2.7.0/+download/bzr-2.7.0.tar.gz
cabextract                  1.6                   31 Aug 2017   https://www.cabextract.org.uk/cabextract-1.6.tar.gz
cachetools                  3.0.0                 04 Dec 2018   https://files.pythonhosted.org/packages/e6/28/7cde8e73835ff48b4f35b2d93a509575f7bc02b7d614ada71b820c8d9233/cachetools-3.0.0.tar.gz
cacti                       1.2.0                 03 Jan 2019   http://www.cacti.net/downloads/cacti-1.2.0.tar.gz
cairo                       1.16.0                21 Oct 2018   https://www.cairographics.org/releases/cairo-1.16.0.tar.xz
cairodock                   3.4.1                 04 Mar 2017   https://github.com/Cairo-Dock/cairo-dock-core/releases/download/3.4.1/cairo-dock-3.4.1.tar.gz
cairomm                     1.15.5                11 Oct 2017   https://www.cairographics.org/releases/cairomm-1.15.5.tar.gz
caja                        1.22.1                24 Apr 2019   https://pub.mate-desktop.org/releases/1.22/caja-1.22.1.tar.xz
cajadropbox                 1.22.1                24 Apr 2019   https://pub.mate-desktop.org/releases/1.22/caja-dropbox-1.22.1.tar.xz
cajaextensions              1.22.0                07 Mar 2019   http://pub.mate-desktop.org/releases/1.22/caja-extensions-1.22.0.tar.xz
cajun                       4.2                   01 Jun 2014   http://cajun.sourceforge.net/
cal3d                       0.11.0                28 Jul 2016   http://download.gna.org/cal3d/sources/cal3d-0.11.0.tar.gz
calendarsupport             19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/calendarsupport-19.04.2.tar.xz
calf                        0.90.0                28 Nov 2017   https://github.com/calf-studio-gear/calf/archive/0.90.0.tar.gz
calibre                     3.44.0                31 May 2019   https://download.calibre-ebook.com/3.44.0/calibre-3.44.0.tar.xz
california                  0.4.0                 24 Mar 2015   http://ftp.gnome.org/pub/GNOME/sources/california/0.4/california-0.4.0.tar.xz
callgrind                   0.9.12                01 Jun 2014   http://kcachegrind.sourceforge.net/callgrind-0.9.12.tar.gz
calligra                    3.1.0                 01 Feb 2018   http://download.kde.org/stable/calligra/3.1.0/calligra-3.1.0.tar.xz
camping                     2.1.532               30 Nov 2017   https://rubygems.org/downloads/camping-2.1.532.gem
cantarellfonts              0.111                 04 Sep 2018   http://ftp.gnome.org/pub/gnome/sources/cantarell-fonts/0.111/cantarell-fonts-0.111.tar.xz
cantor                      19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/cantor-19.04.2.tar.xz
captive                     1.1.7                 01 Jun 2014   http://www.jankratochvil.net/project/captive/dist/captive-1.1.7.tar.gz
capybara                    3.22.0                01 Jun 2019   https://rubygems.org/downloads/capybara-3.22.0.gem
cares                       1.15.0                23 Dec 2018   https://c-ares.haxx.se/download/c-ares-1.15.0.tar.gz
caribou                     0.4.20                16 Feb 2016   http://ftp.gnome.org/pub/GNOME/sources/caribou/0.4/caribou-0.4.20.tar.xz
castlecombat                0.8.1                 28 Jul 2016   http://prdownloads.sourceforge.net/castle-combat/castle-combat-0.8.1.tar.gz
cbase                       1.3.5                 01 Jun 2014   http://www.hyperrealm.com/cbase/cbase-1.3.5.tar.gz
ccache                      3.7                   25 Apr 2019   https://github.com/ccache/ccache/releases/download/v3.7/ccache-3.7.tar.xz
ccaudio2                    2.2.0                 19 Mar 2019   http://ftp.gnu.org/pub/gnu/ccaudio/ccaudio2-2.2.0.tar.gz
ccmath                      2.2.1                 01 Jun 2014   http://www.ibiblio.org/pub/Linux/libs/ccmath-2.2.1.tar.gz
cdemuclient                 3.0.1                 28 Jul 2016   http://downloads.sourceforge.net/cdemu/cdemu-client-3.0.1.tar.bz2
cdemudaemon                 3.0.4                 28 Jul 2016   http://downloads.sourceforge.net/cdemu/cdemu-daemon-3.0.4.tar.bz2
cdogs                       13.03.2007            01 Jun 2014   http://lumaki.com/code/cdogs
cdparanoia                  3.10.2                01 Jun 2014   http://xiph.org/paranoia/down.html
cdrdao                      1.2.3                 01 Jun 2014   http://sourceforge.net/projects/cdrdao/files/cdrdao/1.2.3/cdrdao-1.2.3.tar.bz2
cdrkit                      1.1.11                01 Jun 2014   http://cdrkit.org/releases/cdrkit-1.1.11.tar.gz
cdrtools                    3.01a24               16 Aug 2015   http://downloads.sourceforge.net/project/cdrtools/alpha/cdrtools-3.01a24.tar.gz
ceferino                    0.97.8                01 Jun 2014   http://www.loosersjuegos.com.ar/_media/juegos/ceferino/descargas/ceferino-0.97.8.tar.gz
cegui                       0.7.7                 01 Jun 2014   http://prdownloads.sourceforge.net/crayzedsgui/CEGUI-0.7.7.tar.gz
celt                        0.11.3                08 Aug 2016   http://downloads.xiph.org/releases/celt/celt-0.11.3.tar.gz
centericq                   4.21.0                31 Oct 2017   http://thekonst.net/download/centericq-4.21.0.tar.bz2
ceph                        0.47.2                01 Jun 2014   http://ceph.com/download/ceph-0.47.2.tar.bz2
certifi                     2018.11.29            02 Dec 2018   https://files.pythonhosted.org/packages/55/54/3ce77783acba5979ce16674fc98b1920d00b01d337cfaaf5db22543505ed/certifi-2018.11.29.tar.gz
cervisia                    19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/cervisia-19.04.2.tar.xz
cfengine                    3.8.0                 12 Dec 2015   https://cfengine-package-repos.s3.amazonaws.com/tarballs/cfengine-3.8.0.tar.gz
cflow                       1.6                   23 Feb 2019   http://ftp.gnu.org/pub/gnu/cflow/cflow-1.6.tar.xz
cgiirc                      0.5.8                 01 Jun 2014   http://kent.dl.sourceforge.net/sourceforge/cgiirc/cgiirc-0.5.8.tar.gz
cgilib                      0.6                   01 Jun 2014   http://www.infodrom.org/projects/cgilib/download/cgilib-0.6.tar.gz
changepassword              0.9                   01 Jun 2014   http://prdownloads.sourceforge.net/changepassword/changepassword-0.9.tar.gz
char                        1.0                   01 Jun 2014   http://ltsword.allegronetwork.com/char-linux.tar.gz
charlix                     0.4                   01 Jun 2014   http://charlix.sourceforge.net/
check                       0.12.0                01 Jan 2018   https://github.com/libcheck/check/releases/download/0.12.0/check-0.12.0.tar.gz
checkinstall                1.6.2                 01 Jun 2014   http://asic-linux.com.mx/~izto/checkinstall/files/source/checkinstall-1.6.2.tar.gz
cheese                      3.32.1                09 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/cheese/3.32/cheese-3.32.1.tar.xz
cheetah                     2.4.4                 15 Jan 2019   https://files.pythonhosted.org/packages/cd/b0/c2d700252fc251e91c08639ff41a8a5203b627f4e0a2ae18a6b662ab32ea/Cheetah-2.4.4.tar.gz
chemtool                    1.6.14                01 Mar 2014   http://ruby.chemie.uni-freiburg.de/~martin/chemtool/chemtool-1.6.14.tar.gz
cherokee                    1.2.99                01 Apr 2014   http://www.cherokee-project.com/download/1.2/1.2.99/cherokee-1.2.99.tar.gz
childprocess                1.0.1                 18 Feb 2019   https://rubygems.org/downloads/childprocess-1.0.1.gem
chkrootkit                  0.49                  01 Jun 2014   ftp://ftp.pangeia.com.br/pub/seg/pac/chkrootkit-0.49.tar.gz
chmlib                      0.40                  17 Mar 2017   http://www.jedrea.com/chmlib/chmlib-0.40.tar.gz
chromaprint                 1.4.2                 22 Sep 2018   https://bitbucket.org/acoustid/chromaprint/downloads/chromaprint-1.4.2.tar.gz
chromegnomeshell            10.1                  02 Oct 2018   http://ftp.gnome.org/pub/gnome/sources/chrome-gnome-shell/10.1/chrome-gnome-shell-10.1.tar.xz
chromium                    61.0.3163.100         13 Oct 2017   https://commondatastorage.googleapis.com/chromium-browser-official/chromium-61.0.3163.100.tar.xz
chromiumbsu                 0.9.16.1              01 Jan 2017   https://sourceforge.net/projects/chromium-bsu/files/Chromium%20B.S.U.%20source%20code/chromium-bsu-0.9.16.1.tar.gz
chronojump                  1.8.1                 29 Apr 2018   http://ftp.gnome.org/pub/gnome/sources/chronojump/1.8/chronojump-1.8.1.tar.xz
chrpath                     0.16                  31 Oct 2015   https://alioth.debian.org/frs/download.php/latestfile/813/chrpath-0.16.tar.gz
chunkypng                   1.3.11                25 Nov 2018   https://rubygems.org/downloads/chunky_png-1.3.11.gem
cifsutils                   6.9                   09 Apr 2019   https://www.samba.org/ftp/linux-cifs/cifs-utils/cifs-utils-6.9.tar.bz2
cinelerra                   7.1                   04 Feb 2019   https://sourceforge.net/projects/heroines/files/cinelerra-7.1-src.tar.xz
cinepaint                   0.22.1                01 Jun 2014   http://sourceforge.net/projects/cinepaint/files/CinePaint/CinePaint-0.22-1/cinepaint-0.22.1.tar.gz
circuslinux                 1.0.3                 01 Jun 2014   ftp://ftp.tuxpaint.org/unix/x/circus-linux/src/circuslinux-1.0.3.tar.gz
clamav                      0.101.1               12 Jan 2019   https://www.clamav.net/downloads/production/clamav-0.101.1.tar.gz
clang                       7.0.0                 03 Nov 2018   http://releases.llvm.org/7.0.0/cfe-7.0.0.src.tar.xz
clanlib                     24.10.2015            24 Oct 2015   http://www.clanlib.org/download/releases-3.0/ClanLib-24.10.2015
classaccessor               0.51                  28 Oct 2017   http://search.cpan.org/CPAN/authors/id/K/KA/KASEI/Class-Accessor-0.51.tar.gz
classlib                    0.1                   01 Jun 2014   http://students.washington.edu/aemk/cm/releases/classlib-0.1.tar.gz
classtiny                   1.004                 01 Apr 2016   http://search.cpan.org/CPAN/authors/id/D/DA/DAGOLDEN/Class-Tiny-1.004.tar.gz
clawsmail                   2.8.1                 01 Jun 2014   http://www.claws-mail.org/
clearsilver                 0.10.5                01 Jun 2014   http://www.clearsilver.net/downloads/clearsilver-0.10.5.tar.gz
clementine                  1.2.3                 01 Apr 2014   http://clementine-player.googlecode.com/files/clementine-1.2.3.tar.gz
click                       5.1                   09 Sep 2015   https://pypi.python.org/packages/source/c/click/click-5.1.tar.gz#md5=9c5323008cccfe232a8b161fc8196d41
clisp                       2.49                  01 May 2014   https://sourceforge.net/projects/clisp/files/clisp/2.49/clisp-2.49.tar.gz
clive                       1.0.2                 01 Jun 2014   http://download.gna.org/clive/1.0.x/clive-1.0.2.tar.bz2
cliver                      0.3.2                 13 Aug 2018   https://rubygems.org/downloads/cliver-0.3.2.gem
cloog                       0.18.4                11 Feb 2017   http://www.bastoul.net/cloog/pages/download/cloog-0.18.4.tar.gz
cloogparma                  0.16.1                01 Nov 2012   http://www.bastoul.net/cloog/pages/download/cloog-parma-0.16.1.tar.gz
cloogppl                    0.15.11               01 Nov 2012   ftp://gcc.gnu.org/pub/gcc/infrastructure/cloog-ppl-0.15.11.tar.gz
clucenecore                 2.3.3.4               28 Jul 2016   https://sourceforge.net/projects/clucene/files/clucene-core-unstable/2.3/clucene-core-2.3.3.4.tar.gz
clustalomega                1.2.4                 09 Sep 2018   http://www.clustal.org/omega/clustal-omega-1.2.4.tar.gz
clustalw                    2.1                   11 Apr 2016   http://www.clustal.org/download/current/clustalw-2.1.tar.gz
clutter                     1.26.2                13 May 2017   http://ftp.gnome.org/pub/gnome/sources/clutter/1.26/clutter-1.26.2.tar.xz
cluttergst                  3.0.27                07 Feb 2019   http://ftp.gnome.org/pub/gnome/sources/clutter-gst/3.0/clutter-gst-3.0.27.tar.xz
cluttergtk                  1.8.4                 09 Aug 2017   http://ftp.gnome.org/pub/gnome/sources/clutter-gtk/1.8/clutter-gtk-1.8.4.tar.xz
cluttergtkmm                1.6.0                 18 Mar 2015   http://ftp.gnome.org/pub/GNOME/sources/clutter-gtkmm/1.6/clutter-gtkmm-1.6.0.tar.xz
cluttermm                   1.17.3                01 May 2014   http://ftp.gnome.org/pub/GNOME/sources/cluttermm/1.17/cluttermm-1.17.3.tar.xz
cmake                       3.14.5                01 Jun 2019   http://www.cmake.org/files/v3.14/cmake-3.14.5.tar.gz
cmark                       0.28.3                05 Dec 2018   https://github.com/commonmark/cmark/archive/0.28.3.tar.gz
cmdftp                      0.9.7                 01 Jun 2014   http://download.savannah.nongnu.org/releases/cmdftp/cmdftp-0.9.7.tar.gz
codebrowser                 4.9                   01 Dec 2012   http://tibleiz.net/download/code-browser-4.9-src.tar.gz
coderay                     1.1.2                 11 Sep 2017   https://rubygems.org/downloads/coderay-1.1.2.gem
codestriker                 1.9.10                01 Jun 2014   http://sourceforge.net/projects/codestriker/files/codestriker/1.9.10/codestriker-1.9.10.tar.gz
coffeescript                2.4.1                 10 Dec 2017   https://rubygems.org/downloads/coffee-script-2.4.1.gem
cogito                      0.17.2                01 Jun 2014   http://www.kernel.org/pub/software/scm/cogito/cogito-0.17.2.tar.bz2
cogl                        1.22.4                23 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/cogl/1.22/cogl-1.22.4.tar.xz
coin                        2.5.0                 01 Jun 2014   http://www.coin3d.org/lib/coin/releases/2.5.0
collectd                    3.8.4                 01 Jun 2014   http://collectd.org/files/collectd-3.8.4.tar.bz2
collectl                    4.2.0                 14 Jul 2017   https://sourceforge.net/projects/collectl/files/collectl/collectl-4.2.0/collectl-4.2.0.src.tar.gz
colorama                    0.4.1                 02 Dec 2018   https://files.pythonhosted.org/packages/76/53/e785891dce0e2f2b9f4b4ff5bc6062a53332ed28833c7afede841f46a5db/colorama-0.4.1.tar.gz
colord                      1.4.4                 11 Mar 2019   https://www.freedesktop.org/software/colord/releases/colord-1.4.4.tar.xz
colordgtk                   0.2.0                 30 Jun 2019   https://www.freedesktop.org/software/colord/releases/colord-gtk-0.2.0.tar.xz
colortools                  1.3.0                 22 Jul 2017   https://rubygems.org/downloads/color-tools-1.3.0.gem
colour                      0.1.5                 02 Dec 2018   https://files.pythonhosted.org/packages/a0/d4/5911a7618acddc3f594ddf144ecd8a03c29074a540f4494670ad8f153efe/colour-0.1.5.tar.gz
comix                       4.0.4                 11 Oct 2017   https://sourceforge.net/projects/comix/files/comix/comix-4.0.4/comix-4.0.4.tar.gz
compface                    1.5.2                 01 Jan 2013   http://ftp.xemacs.org/pub/xemacs/aux/compface-1.5.2.tar.gz
compiz                      0.9.14.0              17 Feb 2019   https://launchpad.net/compiz/0.9.14/0.9.14.0/+download/compiz-0.9.14.0.tar.xz
compositeproto              0.4.2                 08 Aug 2016   https://www.x.org/releases/individual/proto/compositeproto-0.4.2.tar.gz
comptonconf                 0.14.1                27 Feb 2019   https://downloads.lxqt.org/downloads/compton-conf/0.14.1/compton-conf-0.14.1.tar.xz
conary                      1.1.18                01 Dec 2012   ftp://download.rpath.com/conary/
concurrentruby              1.1.5                 24 Mar 2019   https://rubygems.org/downloads/concurrent-ruby-1.1.5.gem
confuse                     2.7                   01 Dec 2013   http://download.savannah.gnu.org/releases/confuse/confuse-2.7.tar.gz
conky                       1.11.3                03 May 2018   https://github.com/brndnmtthws/conky/archive/v1.11.3.tar.gz
connman                     1.33                  22 Feb 2017   https://www.kernel.org/pub/linux/network/connman/connman-1.33.tar.xz
consolekit                  0.4.6                 01 Jun 2014   http://anduin.linuxfromscratch.org/sources/BLFS/svn/c/ConsoleKit-0.4.6.tar.xz
consolekit2                 1.2.1                 14 Nov 2018   https://github.com/Consolekit2/ConsoleKit2/releases/download/1.2.1/ConsoleKit2-1.2.1.tar.bz2
constype                    1.0.4                 01 May 2014   http://xorg.freedesktop.org/releases/individual/app/constype-1.0.4.tar.bz2
contextfree                 3.1                   03 May 2019   https://www.contextfreeart.org/download/ContextFreeSource3.1.tgz
controlcenter               2.18.1                01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/control-center/2.18/control-center-2.18.1.tar.bz2
convmv                      1.10                  01 Jun 2014   http://www.j3e.de/linux/convmv/
cook                        2.28                  01 Jun 2014   cook-2.28
coreutils                   8.31                  11 Mar 2019   ftp://ftp.gnu.org/gnu/coreutils/coreutils-8.31.tar.xz
cparser                     1.22.0                04 Jan 2016   https://github.com/MatzeB/cparser/archive/cparser-1.22.0.tar.gz
cpio                        2.12                  03 Feb 2016   http://ftp.gnu.org/pub/gnu/cpio/cpio-2.12.tar.bz2
cppunit                     1.12.1                01 Nov 2012   https://sourceforge.net/projects/cppunit/files/cppunit/1.12.1/cppunit-1.12.1.tar.gz
cpufreqd                    2.4.2                 24 Dec 2015   http://sourceforge.net/projects/cpufreqd/files/Source%20packages/2.4.2/cpufreqd-2.4.2.tar.bz2
cracklib                    2.9.7                 05 Mar 2019   https://github.com/cracklib/cracklib/releases/download/v2.9.7/cracklib-2.9.7.tar.gz
crass                       1.0.4                 13 Oct 2018   https://rubygems.org/downloads/crass-1.0.4.gem
cream                       0.43                  01 Jun 2014   http://prdownloads.sourceforge.net/cream/cream-0.43.tar.gz
createtorrent               1.1.4                 01 Jan 2013   http://www.createtorrent.com/createtorrent-1.1.4.tar.gz
crimson                     0.5.3                 01 Jan 2013   http://crimson.seul.org/files/crimson-0.5.3.tar.bz2
crunchyfrog                 0.2.0                 01 Jun 2014   http://crunchyfrog.googlecode.com/files/crunchyfrog-0.2.0.tar.gz
cryopid                     0.5.9.1               01 Jun 2014   http://dagobah.ucc.asn.au/wacky/cryopid-0.5.9.1-i386.tar.gz
cryptkeeper                 0.9.5                 01 Jun 2014   http://tom.noflag.org.uk/cryptkeeper/cryptkeeper-0.9.5.tar.gz
cryptopp                    8.1.0                 09 Apr 2019   https://www.cryptopp.com/cryptopp810.zip
cryptsetup                  2.1.0                 09 Feb 2019   https://mirrors.edge.kernel.org/pub/linux/utils/cryptsetup/v2.1/cryptsetup-2.1.0.tar.xz
crystal                     0.28.0                05 Apr 2016   https://github.com/crystal-lang/crystal/archive/0.28.0.tar.gz
crystalspace                1.2                   01 Jun 2014   http://www.crystalspace3d.org/downloads/release/crystalspace-src-2.0.tar.bz2
csmash                      0.6.6                 01 Jun 2014   http://cannonsmash.sourceforge.net/
ctags                       5.8                   01 Jun 2014   http://prdownloads.sourceforge.net/ctags/ctags-5.8.tar.gz
ctalk                       0.0.77a               01 Jun 2014   http://ctalk-lang.sourceforge.net/
ctioga                      1.11.1                01 Dec 2017   https://rubygems.org/downloads/ctioga-1.11.1.gem
ctorrent                    1.3.4                 01 Jun 2014   https://sourceforge.net/projects/ctorrent/files/CTorrent-1.3.4/CTorrent-1.3.4/ctorrent-1.3.4.tar.bz2
ctris                       0.42                  01 Jun 2014   http://www.hackl.dhs.org/ctris/
ctypes                      1.0.2                 01 Jan 2013   http://downloads.sourceforge.net/project/ctypes/ctypes/1.0.2/ctypes-1.0.2.tar.gz
cuiterm                     0.9.9                 01 Jun 2014   ftp://linux.pte.hu/pub/CUI/cuiterm-0.9.9.tar.gz
cunit                       2.1-3                 20 Feb 2017   https://downloads.sourceforge.net/project/cunit/CUnit/2.1-3/CUnit-2.1-3.tar.bz2
cups                        2.2.11                23 Mar 2019   https://github.com/apple/cups/releases/download/v2.2.11/cups-2.2.11-source.tar.gz
cupsfilters                 1.25.1                07 Jul 2019   https://www.openprinting.org/download/cups-filters/cups-filters-1.25.1.tar.xz
cupspdf                     2.6.1                 01 Jul 2013   http://www.physik.uni-wuerzburg.de/~vrbehr/cups-pdf/src/cups-pdf_2.6.1.tar.gz
curl                        7.65.1                05 Jun 2019   http://curl.haxx.se/download/curl-7.65.1.tar.bz2
cutmp3                      3.0.1                 12 Dec 2015   http://www.puchalla-online.de/cutmp3-3.0.1.tar.bz2
cvs                         1.11.23               01 Mar 2013   http://ftp.gnu.org/non-gnu/cvs/source/stable/1.11.23/cvs-1.11.23.tar.bz2
cvtool                      0.1.0                 01 Jun 2014   http://sourceforge.net/projects/cvtool/
cxxtools                    2.2.1                 02 Aug 2017   http://www.tntnet.org/download/cxxtools-2.2.1.tar.gz
cyrussasl                   2.1.27                20 Nov 2018   ftp://ftp.cyrusimap.org/cyrus-sasl/cyrus-sasl-2.1.27.tar.gz
cython                      0.29.11               02 Jul 2019   https://files.pythonhosted.org/packages/87/26/6f8a63fda4743d9e405c17c2872ac3242b63b5eba79c475ddf1777f17b9e/Cython-0.29.11.tar.gz
daemons                     1.3.1                 15 Dec 2018   https://rubygems.org/downloads/daemons-1.3.1.gem
damageproto                 1.2.1                 08 Aug 2016   http://xorg.freedesktop.org/releases/individual/proto/damageproto-1.2.1.tar.bz2
daq                         2.0.5                 20 May 2015   https://www.snort.org/downloads/snort/daq-2.0.5.tar.gz
darcs                       2.12.5                21 Feb 2017   http://darcs.net/releases/darcs-2.12.5.tar.gz
dash                        0.5.10.2              08 Nov 2018   http://gondor.apana.org.au/~herbert/dash/files/dash-0.5.10.2.tar.gz
dasher                      4.11                  01 Jun 2014   http://ftp.acc.umu.se/pub/GNOME/sources/dasher/4.11/dasher-4.11.tar.bz2
datamash                    1.4                   23 Dec 2018   http://ftp.gnu.org/pub/gnu/datamash/datamash-1.4.tar.gz
dateutils                   0.4.6                 19 Mar 2019   https://bitbucket.org/hroptatyr/dateutils/downloads/dateutils-0.4.6.tar.xz
db                          6.2.32                19 Feb 2019   http://download.oracle.com/berkeley-db/db-6.2.32.tar.gz
dbh                         1.0.24                01 Jun 2014   http://heanet.dl.sourceforge.net/sourceforge/xfce/xfce-4.2.3.2-src.tar.bz2
dblatex                     0.3                   01 Jun 2014   http://prdownloads.sourceforge.net/dblatex/dblatex-0.3.tar.bz2
dbus                        1.12.16               14 Jun 2019   https://dbus.freedesktop.org/releases/dbus/dbus-1.12.16.tar.gz
dbusglib                    0.110                 03 Feb 2018   https://dbus.freedesktop.org/releases/dbus-glib/dbus-glib-0.110.tar.gz
dbuspython                  1.2.8                 22 May 2018   http://dbus.freedesktop.org/releases/dbus-python/dbus-python-1.2.8.tar.gz
dconf                       0.32.0                11 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/dconf/0.32/dconf-0.32.0.tar.xz
dconfeditor                 3.30.2                01 Dec 2018   http://ftp.gnome.org/pub/gnome/sources/dconf-editor/3.30/dconf-editor-3.30.2.tar.xz
dcraw                       9.15                  01 Jun 2014   http://www.cybercom.net/~dcoffin/dcraw/archive/dcraw-9.15.tar.gz
ddclient                    3.8.3                 07 Feb 2016   https://sourceforge.net/projects/ddclient/files/ddclient/ddclient-3.8.3/ddclient-3.8.3.tar.bz2
ddd                         3.3.12                01 Jun 2014   http://ftp.gnu.org/gnu/ddd/ddd-3.3.12.tar.gz
ddrescue                    1.24                  25 Feb 2019   http://ftp.gnu.org/pub/gnu/ddrescue/ddrescue-1.24.tar.lz
defendguin                  0.0.12                05 Jan 2019   ftp://ftp.tuxpaint.org/unix/x/defendguin/src/defendguin-0.0.12.tar.gz
dejagnu                     1.6.2                 28 Nov 2018   http://ftp.gnu.org/pub/gnu/dejagnu/dejagnu-1.6.2.tar.gz
deluge                      2.0.2                 09 Jun 2019   http://download.deluge-torrent.org/source/2.0/deluge-2.0.2.tar.xz
denemo                      2.2.0                 01 Aug 2017   http://ftp.gnu.org/pub/gnu/denemo/denemo-2.2.0.tar.gz
deplate                     0.8.5                 13 Feb 2016   https://sourceforge.net/projects/deplate/files/deplate/0.8.5/deplate-0.8.5.gem
dermixd                     2.0.1                 16 Apr 2016   http://thomas.orgis.org/dermixd/dermixd-2.0.1.tar.bz2
deskbarapplet               2.20.0                01 Jun 2014   http://ftp.acc.umu.se/pub/GNOME/sources/deskbar-applet/2.20/deskbar-applet-2.20.0.tar.bz2
desksanity                  1.1.0                 05 Feb 2016   https://download.enlightenment.org/rel/apps/enlightenment/desksanity-1.1.0.tar.xz
desktopfileutils            0.23                  22 Oct 2016   http://freedesktop.org/software/desktop-file-utils/releases/desktop-file-utils-0.23.tar.xz
detect                      0.9.83                01 Jun 2014   http://www.hylius.fr/
devede                      4.5.0                 02 Feb 2016   https://github.com/rastersoft/devedeng/archive/4.5.0.tar.gz
devhelp                     3.32.0                10 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/devhelp/3.32/devhelp-3.32.0.tar.xz
devil                       1.7.8                 19 Jun 2015   http://downloads.sourceforge.net/openil/DevIL-1.7.8.tar.gz
dfeet                       0.3.14                24 Oct 2018   http://ftp.gnome.org/pub/gnome/sources/d-feet/0.3/d-feet-0.3.14.tar.xz
dhcp                        4.4.1                 28 Feb 2018   ftp://ftp.isc.org/isc/dhcp/4.4.1/dhcp-4.4.1.tar.gz
dhcpcd                      7.2.3                 30 Jun 2019   https://roy.marples.name/downloads/dhcpcd/dhcpcd-7.2.3.tar.xz
dia                         0.97.3                05 Sep 2014   http://ftp.gnome.org/pub/GNOME/sources/dia/0.97/dia-0.97.3.tar.xz
diakonos                    0.9.4                 15 Nov 2015   http://diakonos.pist0s.ca/archives/diakonos-0.9.4.tar.gz
dico                        2.9                   24 Apr 2019   https://ftp.gnu.org/pub/gnu/dico/dico-2.9.tar.xz
dictd                       1.12.1                01 Jun 2014   https://sourceforge.net/projects/dict/files/dictd/dictd-1.12.1/dictd-1.12.1.tar.gz
didyoumean                  1.3.0                 26 Dec 2018   https://rubygems.org/downloads/did_you_mean-1.3.0.gem
dietlibc                    0.33                  11 Oct 2017   https://www.fefe.de/dietlibc/dietlibc-0.33.tar.bz2
difflcs                     1.3                   03 Dec 2017   https://rubygems.org/downloads/diff-lcs-1.3.gem
diffutils                   3.7                   01 Jan 2019   https://ftp.gnu.org/pub/gnu/diffutils/diffutils-3.7.tar.xz
digger                      14.03.2002            01 Jun 2014   http://www.digger.org/unix.html
digikam                     6.1.0                 15 Apr 2019   https://download.kde.org/stable/digikam/6.1.0/digikam-6.1.0.tar.xz
dillo                       3.0.5                 03 Jul 2015   http://www.dillo.org/download/dillo-3.0.5.tar.bz2
ding                        1.8.1                 28 Jul 2018   ftp://ftp.tu-chemnitz.de/pub/Local/urz/ding/ding-1.8.1.tar.gz
dirac                       1.0.0                 01 Jun 2014   https://sourceforge.net/projects/dirac/files/dirac-codec/Dirac-1.0.0/dirac-1.0.0.tar.gz
directfb                    1.7.7                 18 Apr 2015   http://directfb.org/downloads/Core/DirectFB-1.7/DirectFB-1.7.7.tar.gz
direvent                    5.1                   24 Jul 2016   http://ftp.gnu.org/gnu/direvent/direvent-5.1.tar.gz
discount                    2.2.3a                21 Jul 2018   https://www.pell.portland.or.us/~orc/Code/discount/discount-2.2.3a.tar.bz2
discover                    5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/discover-5.16.3.tar.xz
disko                       1.8.0                 01 May 2014   http://www.diskohq.com/repository/sources/disko_1.8.0.tar.gz
distcc                      3.3.2                 05 May 2019   https://github.com/distcc/distcc/releases/download/v3.3.2/distcc-3.3.2.tar.gz
dit                         0.4                   24 Nov 2016   https://hisham.hm/dit/releases/0.4/dit-0.4.tar.gz
diva                        0.0.2                 01 May 2014   http://files.diva-project.org/releases/diva-0.0.2.tar.gz
django                      2.1.4                 30 Dec 2018   https://files.pythonhosted.org/packages/83/f7/4939b60c4127d5f49ccb570e34f4c59ecc222949220234a88e4f363f1456/Django-2.1.4.tar.gz
djview                      4.10.6                25 Dec 2017   http://downloads.sourceforge.net/djvu/djview-4.10.6.tar.gz
djvulibre                   3.5.27                19 Jan 2016   http://sourceforge.net/projects/djvu/files/DjVuLibre/3.5.27/djvulibre-3.5.27.tar.gz
dkms                        2.2.0.3               01 May 2014   http://linux.dell.com/dkms/permalink/dkms-2.2.0.3.tar.gz
dmidecode                   3.2                   19 Sep 2018   http://download.savannah.gnu.org/releases/dmidecode/dmidecode-3.2.tar.xz
dmxproto                    2.3.1                 01 May 2014   https://xorg.freedesktop.org/releases/individual/proto/dmxproto-2.3.1.tar.bz2
dnsmasq                     2.80                  09 Feb 2019   http://www.thekelleys.org.uk/dnsmasq/dnsmasq-2.80.tar.xz
dnspython                   1.15.0                04 Nov 2017   http://www.dnspython.org/kits/1.15.0/dnspython-1.15.0.tar.gz
docbook                     4.5                   01 Oct 2013   http://www.docbook.org/sgml/4.5/docbook-4.5.zip
docbook2x                   0.8.8                 01 May 2014   http://downloads.sourceforge.net/docbook2x/docbook2X-0.8.8.tar.gz
docbookutils                0.6.14                27 Oct 2015   http://sources-redhat.mirrors.redwire.net/docbook-tools/new-trials/SOURCES/docbook-utils-0.6.14.tar.gz
docbookxsl                  1.79.2                30 Dec 2017   https://github.com/docbook/xslt10-stylesheets/releases/download/release/1.79.2/docbook-xsl-1.79.2.tar.bz2
docutils                    0.14                  21 Nov 2017   https://pypi.python.org/packages/84/f4/5771e41fdf52aabebbadecc9381d11dea0fa34e4759b4071244fa094804c/docutils-0.14.tar.gz
docx2txt                    1.4                   24 Oct 2016   http://downloads.sourceforge.net/docx2txt/docx2txt-1.4.tgz
dolphin                     19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/dolphin-19.04.2.tar.xz
dolphinplugins              19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/dolphin-plugins-19.04.2.tar.xz
domainname                  0.5.20180417          01 May 2018   https://rubygems.org/downloads/domain_name-0.5.20180417.gem
dosbox                      0.74-2                07 Sep 2018   http://sourceforge.net/projects/dosbox/files/dosbox/0.74-2/dosbox-0.74-2.tar.gz
dosfstools                  4.1                   20 Sep 2018   https://github.com/dosfstools/dosfstools/releases/download/v4.1/dosfstools-4.1.tar.xz
dotconf                     1.0.13                01 Nov 2013   http://www.azzit.de/dotconf/download/v1.0/dotconf-1.0.13.tar.gz
dovecot                     2.3.6                 30 Apr 2019   http://www.dovecot.org/releases/2.3/dovecot-2.3.6.tar.gz
doxygen                     1.8.15                30 Dec 2018   http://doxygen.nl/files/doxygen-1.8.15.src.tar.gz
dpkg                        1.18.22               25 Feb 2017   http://ftp.debian.org/debian/pool/main/d/dpkg/dpkg_1.18.22.tar.xz
dragon                      19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/dragon-19.04.2.tar.xz
dragonhunt                  3.56                  01 Jun 2014   http://emhsoft.com/dh/Dragon_Hunt-3.56.tar.gz
dri2proto                   2.8                   01 Jan 2014   http://xorg.freedesktop.org/releases/individual/proto/dri2proto-2.8.tar.bz2
dri3proto                   1.0                   01 Nov 2013   http://xorg.freedesktop.org/releases/individual/proto/dri3proto-1.0.tar.bz2
drkonqi                     5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/drkonqi-5.16.3.tar.xz
drupal                      7.38                  06 Aug 2015   http://ftp.drupal.org/files/projects/drupal-7.38.tar.gz
dssi                        1.1.1                 01 Jun 2014   https://sourceforge.net/projects/dssi/files/dssi/1.1.1/dssi-1.1.1.tar.gz
duma                        2.4.27                01 Jun 2014   http://duma.sourceforge.net/
dumb                        0.9.3                 01 May 2014   http://switch.dl.sourceforge.net/sourceforge/dumb/dumb-0.9.3.tar.gz
dunelegacy                  0.96.4                21 Aug 2017   https://sourceforge.net/projects/dunelegacy/files/dunelegacy/0.96.4/dunelegacy-0.96.4-src.tar.bz2
dvd+rwtools                 7.1                   01 May 2014   http://fy.chalmers.se/~appro/linux/DVD+RW/tools/dvd+rw-tools-7.1.tar.gz
dvdauthor                   0.7.1                 01 Sep 2014   https://sourceforge.net/projects/dvdauthor/files/dvdauthor/0.7.1/dvdauthor-0.7.1.tar.gz
dvdrip                      0.98.11               01 May 2014   http://www.exit1.org/dvdrip/dist/dvdrip-0.98.11.tar.gz
dvdripomatic                0.94                  01 May 2014   http://surfnet.dl.sourceforge.net/sourceforge/dvdripomatic/DVDRipOMatic-0.94.tar.bz2
dvdrtools                   0.3.1                 01 May 2014   http://www.arklinux.org/download/dvdrtools-0.3.1.tar.bz2
dvdslideshow                0.7.5                 01 May 2014   http://dvd-slideshow.sourceforge.net/
dvgrab                      1.7                   01 May 2014   https://github.com/ddennedy/dvgrab
dwm                         6.1                   25 Dec 2015   http://dl.suckless.org/dwm/dwm-6.1.tar.gz
dxpy                        0.172.0               06 Dec 2015   https://pypi.python.org/packages/source/d/dxpy/dxpy-0.172.0.tar.gz
dzen2                       0.4.5                 28 Jul 2016   https://github.com/robm/dzen2-0.4.5.tar.xz
e16                         1.0.17                16 Aug 2015   http://sourceforge.net/projects/enlightenment/files/e16/1.0.17/e16-1.0.17.tar.gz
e2fsprogs                   1.45.2                30 May 2019   https://sourceforge.net/projects/e2fsprogs/files/e2fsprogs/v1.45.2/e2fsprogs-1.45.2.tar.gz
easytag                     2.4.3                 10 Sep 2018   https://download.gnome.org/sources/easytag/2.4/easytag-2.4.3.tar.xz
ecasound                    2.9.0                 01 Jun 2014   http://ecasound.seul.org/download/ecasound-2.9.0.tar.gz
ecawave                     0.6.1                 01 Jun 2014   http://ecasound.seul.org/download/ecawave-0.6.1.tar.gz
ecell                       3.2.2                 01 Nov 2012   http://sourceforge.net/projects/ecell/files/E-Cell%203.2/3.2.2/ecell-3.2.2.tar.bz2
echievements                2                     01 Dec 2012   http://download.enlightenment.fr/releases/echievements-2.tar.bz2
econnman                    1.1                   13 Apr 2019   https://download.enlightenment.org/rel/apps/econnman/econnman-1.1.tar.xz
ecore                       1.7.10                01 Jan 2014   http://download.enlightenment.org/releases/ecore-1.7.10.tar.gz
ed                          1.15                  25 Feb 2019   http://ftp.gnu.org/pub/gnu/ed/ed-1.15.tar.lz
edb                         1.0.5.042             01 Jun 2014   http://download.enlightenment.org/snapshots/2008-01-25/edb-1.0.5.042.tar.bz2
edbus                       1.7.10                01 Jan 2014   http://download.enlightenment.org/releases/e_dbus-1.7.10.tar.gz
ede                         2.1                   17 Sep 2014   https://sourceforge.net/projects/ede/files/ede/2.1/ede-2.1.tar.gz
edelib                      2.1                   24 Feb 2017   http://sourceforge.net/projects/ede/files/edelib/2.1/edelib-2.1.tar.gz
editres                     1.0.4                 01 Jun 2014   http://xorg.freedesktop.org/releases/individual/app/editres-1.0.4.tar.bz2
edje                        1.7.10                01 Jan 2014   http://download.enlightenment.org/releases/edje-1.7.10.tar.gz
eel                         2.26.0                01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/eel/2.26/eel-2.26.0.tar.bz2
eet                         1.7.10                01 Jan 2014   http://download.enlightenment.org/releases/eet-1.7.10.tar.gz
eeze                        1.7.10                01 Jan 2014   http://download.enlightenment.org/releases/eeze-1.7.10.tar.gz
efl                         1.22.1                20 Apr 2019   https://download.enlightenment.org/rel/libs/efl/efl-1.22.1.tar.xz
efltk                       2.0.7                 01 Jun 2014   http://ovh.dl.sourceforge.net/sourceforge/ede/efltk-2.0.7.tar.bz2
efreet                      1.7.10                01 Jan 2014   http://download.enlightenment.org/releases/efreet-1.7.10.tar.bz2
eggdbus                     0.6                   01 Nov 2012   http://cgit.freedesktop.org/~david/eggdbus/snapshot/eggdbus-0.6.tar.bz2
eigen                       3.2.10                06 Dec 2017   http://bitbucket.org/eigen/eigen/get/3.2.10.tar.bz2
eina                        1.7.10                01 Jan 2014   https://download.enlightenment.org/releases/eina-1.7.10.tar.gz
eio                         1.7.10                01 Jan 2014   http://download.enlightenment.org/releases/eio-1.7.10.tar.gz
eject                       2.1.5                 01 Nov 2012   http://ca.geocities.com/jefftranter@rogers.com/eject-2.1.5.tar.gz
ekiga                       4.0.1                 01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/ekiga/4.0/ekiga-4.0.1.tar.xz
eldbus                      1.7.9                 01 Dec 2013   http://download.enlightenment.org/releases/eldbus-1.7.9.tar.bz2
elementary                  1.7.10                28 Jan 2016   https://download.enlightenment.org/releases/elementary-1.7.10.tar.gz
elfrc                       0.7                   01 Jun 2014   http://ktown.kde.org/~frerich/elfrc.html
elfutils                    0.176                 17 Feb 2019   https://sourceware.org/elfutils/ftp/0.176/elfutils-0.176.tar.bz2
elinks                      0.11.7                28 Aug 2016   http://elinks.or.cz/download/elinks-0.11.7.tar.bz2
elisa                       0.3.0                 09 Oct 2018   https://download.kde.org/stable/elisa/0.3.0/elisa-0.3.0.tar.xz
elixir                      1.8.1                 08 Jul 2018   https://github.com/elixir-lang/elixir/archive/v1.8.1.tar.gz
elogind                     241.2                 01 Apr 2019   
elph                        1.0.1                 29 Jun 2017   ftp://ftp.cbcb.umd.edu/pub/software/elph/ELPH-1.0.1.tar.gz
emacs                       26.2                  12 Apr 2019   ftp://ftp.gnu.org/gnu/emacs/emacs-26.2.tar.xz
embryo                      1.7.10                01 Jan 2014   http://download.enlightenment.org/releases/embryo-1.7.10.tar.gz
emech                       3.0.99p3              01 Jun 2014   http://www.energymech.net/files/emech-3.0.99p3.tar.gz
emelfm                      0.9.2                 01 Jun 2014   http://emelfm.sourceforge.net/emelfm-0.9.2.tar.gz
emotion                     1.7.10                01 Jan 2014   http://download.enlightenment.org/releases/emotion-1.7.10.tar.gz
emotiongenericplayers       1.15.0                04 Aug 2015   http://download.enlightenment.org/rel/libs/emotion_generic_players/emotion_generic_players-1.15.0.tar.gz
empathy                     3.25.90               17 Aug 2017   http://ftp.gnome.org/pub/gnome/sources/empathy/3.25/empathy-3.25.90.tar.xz
enca                        1.9                   01 Jun 2014   http://trific.ath.cx/Ftp//enca/enca-1.9.tar.bz2
enchant                     2.2.5                 04 Jul 2019   https://github.com/AbiWord/enchant/releases/download/v2.2.5/enchant-2.2.5.tar.gz
encodings                   1.0.5                 19 Jun 2019   http://xorg.freedesktop.org/releases/individual/font/encodings-1.0.5.tar.bz2
engrampa                    1.22.1                24 Apr 2019   https://pub.mate-desktop.org/releases/1.22/engrampa-1.22.1.tar.xz
enigma                      1.01                  01 Jun 2014   http://www.nongnu.org/enigma/
enlightenment               0.22.4                17 Sep 2018   https://download.enlightenment.org/rel/apps/enlightenment/enlightenment-0.22.4.tar.xz
enscript                    1.6.6                 01 Sep 2012   http://ftp.gnu.org/gnu/enscript/enscript-1.6.6.tar.gz
enter                       0.0.9                 13 Feb 2016   https://sourceforge.net/projects/enter/files/enter/0.0.9/enter-0.0.9.tar.gz
entrance                    0.9.0.013             01 Jun 2014   http://download.enlightenment.org/snapshots/2007-08-26/entrance-0.9.0.013.tar.gz
enum34                      1.1.6                 10 Aug 2018   https://files.pythonhosted.org/packages/bf/3e/31d502c25302814a7c2f1d3959d2a3b3f78e509002ba91aea64993936876/enum34-1.1.6.tar.gz
enventor                    0.3.2                 01 Sep 2014   http://download.enlightenment.org/rel/apps/enventor/enventor-0.3.2.tar.gz
eog                         3.32.1                09 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/eog/3.32/eog-3.32.1.tar.xz
eogplugins                  3.26.3                24 Jul 2018   http://ftp.gnome.org/pub/gnome/sources/eog-plugins/3.26/eog-plugins-3.26.3.tar.xz
eom                         1.22.1                24 Apr 2019   https://pub.mate-desktop.org/releases/1.22/eom-1.22.1.tar.xz
epdfview                    0.1.8                 01 Jun 2014   http://trac.emma-soft.com/epdfview/chrome/site/releases/epdfview-0.1.8.tar.bz2
epic5                       2.1.1                 23 May 2019   http://ftp.epicsol.org/pub/epic/EPIC5-PRODUCTION/epic5-2.1.1.tar.xz
epiphany                    3.32.2                03 May 2019   http://ftp.gnome.org/pub/gnome/sources/epiphany/3.32/epiphany-3.32.2.tar.xz
epkg                        2.3.9                 01 Jun 2014   ftp://ftp.encap.org/pub/encap/epkg/epkg-2.3.9.tar.gz
epm                         4.1                   01 Jun 2014   http://www.nu6.org/_/mirror/ftp.easysw.com/pub/epm/4.1/epm-4.1-source.tar.bz2
eps                         1.5                   01 Jun 2014   http://www.inter7.com/eps/eps-1.5.tar.gz
erlang                      21.3                  13 Mar 2019   http://erlang.org/download/otp_src_21.3.tar.gz
error                       0.17026               26 May 2018   http://www.cpan.org/authors/id/S/SH/SHLOMIF/Error-0.17026.tar.gz
erubi                       1.8.0                 26 Dec 2018   https://rubygems.org/downloads/erubi-1.8.0.gem
erubis                      2.7.0                 03 Dec 2017   https://rubygems.org/downloads/erubis-2.7.0.gem
esdl                        1.3.1                 02 Aug 2017   https://sourceforge.net/projects/esdl/files/esdl/esdl-1.3.1/esdl-1.3.1.src.tgz
eselect                     1.4.9                 18 Jul 2017   https://gitweb.gentoo.org/proj/eselect.git/snapshot/eselect-1.4.9.tar.bz2
esmtp                       1.2                   01 Jun 2014   http://sourceforge.net/projects/esmtp/files/esmtp/1.2/esmtp-1.2.tar.bz2
esound                      0.2.41                05 Dec 2018   http://ftp.gnome.org/pub/GNOME/sources/esound/0.2/esound-0.2.41.tar.gz
espeak                      1.46.02               01 Jun 2014   http://sourceforge.net/projects/espeak/files/espeak/espeak-1.46/espeak-1.46.02-source.zip
eterm                       0.9.6                 01 Jul 2013   http://www.eterm.org/download/Eterm-0.9.6.tar.gz
ethtool                     2.6.36                06 Nov 2018   https://sourceforge.net/projects/gkernel/files/ethtool/2.6.36/ethtool-2.6.36.tar.gz
ethumb                      1.7.10                01 Jan 2014   http://download.enlightenment.org/releases/ethumb-1.7.10.tar.gz
etk                         0.1.0.042             01 Nov 2012   http://download.enlightenment.org/snapshots/2008-01-25/etk-0.1.0.042.tar.bz2
eudev                       3.2.8                 01 Jun 2019   https://dev.gentoo.org/~blueness/eudev/eudev-3.2.8.tar.gz
evas                        1.7.10                01 Jan 2014   http://download.enlightenment.org/releases/evas-1.7.10.tar.gz
evasgenericloaders          1.13.2                16 Apr 2015   https://download.enlightenment.org/rel/libs/evas_generic_loaders/evas_generic_loaders-1.13.2.tar.xz
eve                         0.3.0                 01 Dec 2012   http://download.enlightenment.org/snapshots/2010-12-03/eve-0.3.0.tar.bz2
event                       1.03                  01 Jan 2013   Event-1.03.tar.xz
eventmachine                1.2.7                 27 May 2019   https://rubygems.org/downloads/eventmachine-1.2.7.gem
eventviews                  19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/eventviews-19.04.2.tar.xz
evieext                     1.1.1                 01 Jun 2014   http://xorg.freedesktop.org/releases/individual/proto/evieext-1.1.1.tar.bz2
evil                        1.7.10                01 Jan 2014   http://download.enlightenment.org/releases/evil-1.7.10.tar.gz
evilvte                     0.5.2~pre1            01 Jun 2013   http://www.calno.com/evilvte/evilvte-0.5.2~pre1.tar.xz
evilwm                      1.1.1                 24 Feb 2017   http://www.6809.org.uk/evilwm/evilwm-1.1.1.tar.gz
evince                      3.32.0                14 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/evince/3.32/evince-3.32.0.tar.xz
evisum                      0.1.2                 11 Nov 2018   https://download.enlightenment.org/rel/apps/evisum/evisum-0.1.2.tar.xz
evolution                   3.32.3                17 Jun 2019   ftp://ftp.gnome.org/pub/gnome/sources/evolution/3.32/evolution-3.32.3.tar.xz
evolutiondataserver         3.32.1                18 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/evolution-data-server/3.32/evolution-data-server-3.32.1.tar.xz
evolutionews                3.32.1                22 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/evolution-ews/3.32/evolution-ews-3.32.1.tar.xz
evolutionmapi               3.32.1                22 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/evolution-mapi/3.32/evolution-mapi-3.32.1.tar.xz
ewl                         0.5.2.042             01 Dec 2012   http://download.enlightenment.org/snapshots/2008-01-25/ewl-0.5.2.042.tar.bz2
exactimage                  1.0.1                 04 Jan 2018   http://dl.exactcode.de/oss/exact-image/exact-image-1.0.1.tar.bz2
execjs                      2.7.0                 22 Jul 2018   https://rubygems.org/downloads/execjs-2.7.0.gem
exempi                      2.5.0                 07 Jan 2019   https://libopenraw.freedesktop.org/download/exempi-2.5.0.tar.gz
exim                        4.92                  10 Feb 2019   ftp://ftp.exim.org/pub/exim/exim4/exim-4.92.tar.bz2
exiv2                       0.27.1                05 May 2019   https://www.exiv2.org/builds/exiv2-0.27.1-Source.tar.gz
exo                         0.12.6                17 Jun 2019   http://archive.xfce.org/src/xfce/exo/0.12/exo-0.12.6.tar.bz2
expat                       2.2.6                 18 Aug 2018   https://sourceforge.net/projects/expat/files/expat/2.2.6/expat-2.2.6.tar.bz2
expect                      5.45.4                18 Feb 2018   https://sourceforge.net/projects/expect/files/Expect/5.45.4/expect5.45.4.tar.gz
expedite                    1.7.10                22 Dec 2015   http://download.enlightenment.org/releases/expedite-1.7.10.tar.bz2
explore                     11                    01 Jun 2014   http://mesh.dl.sourceforge.net/sourceforge/openexplore/explore_v11_src.zip
extracmakemodules           5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/extra-cmake-modules-5.59.0.tar.xz
extutilsdepends             0.405                 22 Feb 2016   http://search.cpan.org/CPAN/authors/id/X/XA/XAOC/ExtUtils-Depends-0.405.tar.gz
extutilshelpers             0.026                 15 Sep 2016   http://search.cpan.org/CPAN/authors/id/L/LE/LEONT/ExtUtils-Helpers-0.026.tar.gz
extutilspkgconfig           1.16                  30 Aug 2017   http://search.cpan.org/CPAN/authors/id/X/XA/XAOC/ExtUtils-PkgConfig-1.16.tar.gz
exult                       1.2                   01 Jun 2014   http://exult.sourceforge.net/
faac                        1.29.9                11 Nov 2017   https://downloads.sourceforge.net/faac/faac-1.29.9.tar.gz
faad2                       2.8.8                 22 Dec 2017   https://sourceforge.net/projects/faac/files/faad2-src/faad2-2.8.0/faad2-2.8.8.tar.gz
facter                      1.5.4rc1              01 May 2013   http://reductivelabs.com/downloads/facter/facter-1.5.4rc1.tgz
factorygirl                 4.9.0                 06 Jan 2018   https://rubygems.org/downloads/factory_girl-4.9.0.gem
fam                         2.7.0                 01 Jun 2014   http://gd.tuwien.ac.at/opsys/linux/gentoo/distfiles/fam-2.7.0.tar.gz
faraday                     0.15.4                15 Dec 2018   https://rubygems.org/downloads/faraday-0.15.4.gem
farstream                   0.2.7                 16 Sep 2015   http://freedesktop.org/software/farstream/releases/farstream/farstream-0.2.7.tar.gz
fasta                       36.3.8e               10 Jul 2017   http://faculty.virginia.edu/wrpearson/fasta/fasta36/fasta-36.3.8e.tar.gz
fbdesk                      1.4.1                 01 Jun 2014   http://fluxbox.sourceforge.net/download/fbdesk-1.4.1.tar.gz
fbida                       2.09                  01 Jun 2014   http://www.kraxel.org/releases/fbida/fbida-2.09.tar.gz
fcgi                        2.4.0                 01 Jan 2014   http://www.fastcgi.com/dist/fcgi-2.4.0.tar.gz
fdkaac                      0.1.6                 16 Mar 2018   http://downloads.sourceforge.net/opencore-amr/fdk-aac-0.1.6.tar.gz
feh                         3.1.3                 17 Feb 2019   http://feh.finalrewind.org/feh-3.1.3.tar.bz2
ferret                      0.11.8.7              09 Jan 2018   https://rubygems.org/downloads/ferret-0.11.8.7.gem
fetchmail                   6.3.26                01 Mar 2013   http://download.berlios.de/fetchmail/fetchmail-6.3.26.tar.xz
ffe                         0.3.7                 21 Aug 2017   https://sourceforge.net/projects/ff-extractor/files/ff-extractor/0.3.7/ffe-0.3.7.tar.gz
ffmpeg                      4.1.4                 08 Jul 2019   http://ffmpeg.org/releases/ffmpeg-4.1.4.tar.xz
ffmpeg2theora               0.30                  30 Sep 2017   http://v2v.cc/~j/ffmpeg2theora/downloads/ffmpeg2theora-0.30.tar.bz2
ffmpegphp                   0.5.0                 01 Jun 2014   http://ffmpeg-php.sourceforge.net/
ffmpegthumbs                19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/ffmpegthumbs-19.04.2.tar.xz
ffpocket                    0.6.5                 01 Jun 2014   http://members.hellug.gr/djart/ffpocket/ffpocket-0.6.5.tar.bz2
fftw                        3.3.8                 01 Jun 2018   http://www.fftw.org/fftw-3.3.8.tar.gz
fiddle                      1.0.0                 05 Dec 2017   https://rubygems.org/downloads/fiddle-1.0.0.gem
fife                        0.3.4                 01 Mar 2013   http://sourceforge.net/projects/fife/files/active/src/fife_0.3.4.tar.gz
fifoembed                   2.1.1                 01 Jun 2014   https://sourceforge.net/projects/fifoembed/files/fifoembed/2.1.1/fifoembed-2.1.1.tar.bz2
figaro                      1.1.1                 13 Oct 2018   https://rubygems.org/downloads/figaro-1.1.1.gem
figlet                      2.2.4                 01 Jun 2014   ftp://ftp.figlet.org/pub/figlet/program/unix/figlet-2.2.4.tar.gz
file                        5.37                  17 May 2019   ftp://ftp.astron.com/pub/file/file-5.37.tar.gz
filechdir                   0.1010                12 Apr 2019   https://cpan.metacpan.org/authors/id/D/DA/DAGOLDEN/File-chdir-0.1010.tar.gz
filelight                   19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/filelight-19.04.2.tar.xz
fileroller                  3.32.1                08 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/file-roller/3.32/file-roller-3.32.1.tar.xz
filesharedir                1.116                 05 Jun 2018   https://cpan.metacpan.org/authors/id/R/RE/REHSACK/File-ShareDir-1.116.tar.gz
fileslurper                 0.010                 26 Dec 2017   http://search.cpan.org/CPAN/authors/id/L/LE/LEONT/File-Slurper-0.010.tar.gz
filetype                    1.0.5                 06 Apr 2019   https://files.pythonhosted.org/packages/e8/53/298887541ae479f8467d4d23e028c6d15f9811da25c582297fd3869666b7/filetype-1.0.5.tar.gz
filewhich                   1.23                  03 Jan 2019   http://search.cpan.org/CPAN/authors/id/P/PL/PLICEASE/File-Which-1.23.tar.gz
filezilla                   3.5.2                 01 Jun 2014   http://filezilla-project.org/download.php?type=client
fim                         0.6                   30 Dec 2018   http://download.savannah.nongnu.org/releases/fbi-improved/fim-0.6-trunk.tar.gz
findlib                     1.7.1                 11 Mar 2017   http://download.camlcity.org/download/findlib-1.7.1.tar.gz
findutils                   4.6.0                 29 Dec 2015   ftp://ftp.gnu.org/gnu/findutils/findutils-4.6.0.tar.gz
firebird                    3.0.1                 01 Oct 2016   https://sourceforge.net/projects/firebird/files/firebird/3.0.1-Release/Firebird-3.0.1.32609-0.tar.bz2
firefox                     55.0                  12 Aug 2017   https://ftp.mozilla.org/pub/mozilla.org/firefox/releases/55.0/source/firefox-55.0.source.tar.xz
firejail                    0.9.56                19 Sep 2018   https://sourceforge.net/projects/firejail/files/firejail/firejail-0.9.56.tar.xz
fische                      3.2.2                 01 Nov 2012   http://26elf.at/files/fische-3.2.2.tar.gz
fish                        3.0.0                 28 Dec 2018   https://github.com/fish-shell/fish-shell/releases/download/3.0.0/fish-3.0.0.tar.gz
fitd                        0.1                   01 Jun 2014   http://sourceforge.net/project/showfiles.php?group_id=104884
fiveormore                  3.32.0                19 Mar 2019   http://ftp.acc.umu.se/pub/gnome/sources/five-or-more/3.32/five-or-more-3.32.0.tar.xz
fixesproto                  5.0                   01 Jun 2014   http://xorg.freedesktop.org/releases/individual/proto/fixesproto-5.0.tar.bz2
flac                        1.3.2                 02 Jan 2017   http://sourceforge.net/projects/flac/files/flac-src/flac-1.3.2.tar.xz
flagpoll                    0.3.0                 01 Jun 2014   https://crate.io/packages/flagpoll/0.3.0/
flam3                       3.0.1                 01 Aug 2014   http://flam3.googlecode.com/files/flam3-3.0.1.zip
flask                       1.0.2                 06 May 2019   https://files.pythonhosted.org/packages/4b/12/c1fbf4971fda0e4de05565694c9f0c92646223cff53f15b6eb248a310a62/Flask-1.0.2.tar.gz
flatbuffers                 26.12.2018            26 Dec 2018   flatbuffers-26.12.2018
flatpak                     1.1.0                 16 Nov 2018   https://github.com/flatpak/flatpak/releases/download/1.1.0/flatpak-1.1.0.tar.xz
flatzebra                   0.1.6                 02 Nov 2015   http://perso.b2b2c.ca/sarrazip/dev/flatzebra-0.1.6.tar.gz
fldigi                      3.23.12               20 Jul 2016   https://sourceforge.net/projects/fldigi/files/fldigi/fldigi-3.23.12.tar.gz
flex                        2.6.4                 07 May 2017   https://github.com/westes/flex/releases/download/v2.6.4/flex-2.6.4.tar.gz
flimp                       0.0.11                25 Apr 2015   http://www.ecademix.com/JohannesHofmann/flimp-0.0.11.tar.gz
florence                    0.6.3                 25 Apr 2015   http://sourceforge.net/projects/florence/files/florence/0.6.3/florence-0.6.3.tar.bz2
fltk                        1.3.5                 05 Mar 2019   https://www.fltk.org/pub/fltk/1.3.5/fltk-1.3.5-source.tar.bz2
fluidsynth                  2.0.3                 15 Feb 2019   https://github.com/FluidSynth/fluidsynth/archive/v2.0.3.tar.gz
fluxbox                     24.12.2018            17 May 2015   http://sourceforge.net/projects/fluxbox/files/fluxbox/1.3.7/fluxbox-1.3.7.tar.xz
folks                       0.13.1                22 Jun 2019   http://ftp.gnome.org/pub/gnome/sources/folks/0.13/folks-0.13.1.tar.xz
fontadobe100dpi             1.0.0                 01 Jun 2014   http://xorg.freedesktop.org/releases/individual/font/font-adobe-100dpi-1.0.0.tar.bz2
fontadobe75dpi              1.0.0                 01 Jun 2014   http://xorg.freedesktop.org/releases/individual/font/font-adobe-75dpi-1.0.0.tar.bz2
fontadobeutopia100dpi       1.0.1                 01 Jun 2014   http://xorg.freedesktop.org/releases/individual/font/font-adobe-utopia-100dpi-1.0.1.tar.bz2
fontadobeutopia75dpi        1.0.1                 01 Jun 2014   http://xorg.freedesktop.org/releases/individual/font/font-adobe-utopia-75dpi-1.0.1.tar.bz2
fontadobeutopiatype1        1.0.4                 01 May 2014   http://xorg.freedesktop.org/releases/individual/font/font-adobe-utopia-type1-1.0.4.tar.gz
fontalias                   1.0.1                 01 Jun 2014   http://xorg.freedesktop.org/releases/individual/font/font-alias-1.0.1.tar.bz2
fontbh75dpi                 1.0.0                 01 Jan 2013   http://xorg.freedesktop.org/releases/individual/font/font-bh-75dpi-1.0.0.tar.bz2
fontbitstream100dpi         1.0.0                 01 Jun 2014   http://xorg.freedesktop.org/releases/individual/font/font-bitstream-100dpi-1.0.0.tar.bz2
fontbitstream75dpi          1.0.3                 01 Jun 2014   http://xorg.freedesktop.org/releases/individual/font/font-bitstream-75dpi-1.0.3.tar.bz2
fontcacheproto              0.1.2                 01 May 2014   http://xorg.freedesktop.org/releases/individual/proto/fontcacheproto-0.1.2.tar.bz2
fontconfig                  2.13.1                04 Sep 2018   https://www.freedesktop.org/software/fontconfig/release/fontconfig-2.13.1.tar.bz2
fontforge                   29.05.2011            01 May 2014   https://sourceforge.net/projects/fontforge/files/fontforge-source/fontforge_full-20110222.tar.bz2
fontopia                    1.7                   21 Aug 2016   http://ftp.gnu.org/pub/gnu/fontopia/fontopia-1.7.tar.gz
fontsonymisc                1.0.3                 01 Aug 2013   http://xorg.freedesktop.org/releases/individual/font/font-sony-misc-1.0.3.tar.bz2
fontsproto                  2.1.3                 01 Apr 2014   http://xorg.freedesktop.org/releases/individual/proto/fontsproto-2.1.3.tar.bz2
fonttosfnt                  1.0.5                 03 Aug 2018   http://xorg.freedesktop.org/releases/individual/app/fonttosfnt-1.0.5.tar.bz2
fontttf                     1.06                  16 Dec 2018   https://cpan.metacpan.org/authors/id/B/BH/BHALLISSY/Font-TTF-1.06.tar.gz
fontutil                    1.3.1                 26 Jul 2015   http://xorg.freedesktop.org/releases/individual/font/font-util-1.3.1.tar.gz
fontxfree86type1            1.0.4                 01 May 2014   http://xorg.freedesktop.org/releases/individual/font/font-xfree86-type1-1.0.4.tar.bz2
formatador                  0.2.5                 25 Dec 2017   https://rubygems.org/downloads/formatador-0.2.5.gem
fosfat                      0.2.3                 01 May 2014   http://download.gna.org/fosfat/fosfat_0.2.3.orig.tar.gz
fourinarow                  3.22.2                09 Sep 2017   http://ftp.gnome.org/pub/gnome/sources/four-in-a-row/3.22/four-in-a-row-3.22.2.tar.xz
fox                         1.6.57                18 Feb 2019   ftp://ftp.fox-toolkit.org/pub/fox-1.6.57.tar.gz
fpdf                        1.53                  01 Jun 2014   http://www.fpdf.de/downloads/fpdf17.tgz
fpm                         1.11.0                31 Jan 2019   https://rubygems.org/downloads/fpm-1.11.0.gem
fprintd                     0.6.0                 01 Mar 2015   http://people.freedesktop.org/~hadess/fprintd-0.6.0.tar.xz
frameworkintegration        5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/frameworkintegration-5.59.0.tar.xz
freealut                    1.1.0                 01 Jun 2014   http://www.openal.org/openal_webstf/downloads/freealut-1.1.0.tar.gz
freebasic                   0.23.0                01 Jun 2014   http://sourceforge.net/projects/fbc/files/Source%20Code/FreeBASIC-0.23.0-source.tar.gz
freeciv                     2.5.4                 21 Jul 2016   http://prdownloads.sourceforge.net/freeciv/freeciv-2.5.4.tar.bz2
freecol                     0.7.1                 01 Dec 2012   http://www.freecol.org/news/freecol-0.10.5-released.html
freecycle                   0.6.1.1alpha          25 Dec 2015   http://download.savannah.gnu.org/releases/freecycle/freecycle-0.6.1.1alpha.tar.bz2
freeglut                    2.8.0                 01 Nov 2012   http://sourceforge.net/projects/freeglut/files/freeglut/2.8.0/freeglut-2.8.0.tar.gz
freeimage                   3.15.4                01 Nov 2013   http://sourceforge.net/projects/freeimage/files/Source%20Distribution/3.15.4/FreeImage3154.zip
freeipmi                    1.6.3                 25 Jan 2019   http://ftp.gnu.org/pub/gnu/freeipmi/freeipmi-1.6.3.tar.gz
freepop                     0.6.0                 01 Dec 2012   http://sourceforge.net/project/showfiles.php?group_id=43507&package_id=35844&release_id=306784
freesci                     0.3.5                 01 Jun 2014   http://freesci.linuxgames.com/
freesteam                   2.0                   01 Jun 2014   http://sourceforge.net/projects/freesteam/files/freesteam/2.0/freesteam-2.0.tar.bz2
freesynd                    0.7.5                 14 Apr 2017   https://sourceforge.net/projects/freesynd/files/freesynd/freesynd-0.7.5/freesynd-0.7.5.tar.gz
freetds                     1.1.1                 19 Mar 2019   ftp://ftp.freetds.org/pub/freetds/stable/freetds-1.1.1.tar.gz
freetype                    2.10.1                02 Jul 2019   https://sourceforge.net/projects/freetype/files/freetype2/2.10.1/freetype-2.10.1.tar.xz
freevo                      1.9.2b2               11 Dec 2017   https://sourceforge.net/projects/freevo/files/Freevo%20releases/1.9.2b2/freevo-1.9.2b2.tar.bz2
freewrl                     1.18.4                01 Jun 2014   http://ovh.dl.sourceforge.net/sourceforge/freewrl/freewrl-1.18.4.tar.gz
fribidi                     1.0.5                 30 Jul 2018   https://github.com/fribidi/fribidi/releases/download/v1.0.5/fribidi-1.0.5.tar.bz2
friction                    0.1                   01 Jun 2014   http://remar.se/games/files/
frogr                       1.5                   25 Nov 2018   http://ftp.gnome.org/pub/gnome/sources/frogr/1.5/frogr-1.5.tar.xz
frozenbubble                2.2.0                 01 Jan 2013   http://www.frozen-bubble.org/data/frozen-bubble-2.2.0.tar.bz2
fslsfonts                   1.0.4                 01 Jun 2014   http://xorg.freedesktop.org/releases/individual/app/fslsfonts-1.0.4.tar.bz2
fspot                       0.6.2                 01 Jun 2014   http://ftp.gnome.org/Public/GNOME/sources/f-spot/0.6/f-spot-0.6.2.tar.bz2
fstobdf                     1.0.5                 01 Jun 2014   http://xorg.freedesktop.org/releases/individual/app/fstobdf-1.0.5.tar.bz2
ftwin                       0.4.0                 01 Jun 2014   http://jok.is-a-geek.net/ftwin.php
fuse                        3.5.0                 18 Apr 2019   https://github.com/libfuse/libfuse/archive/fuse-3.5.0.tar.gz
fuseiso                     20070708              29 Oct 2014   https://sourceforge.net/projects/fuseiso/files/fuseiso/20070708/fuseiso-20070708.tar.bz2
fvwm                        2.7.0                 01 Jun 2014   ftp://ftp.fvwm.org/pub/fvwm/version-2/fvwm-2.7.0.tar.gz
fxruby                      1.6.40                18 Feb 2019   https://rubygems.org/downloads/fxruby-1.6.40.gem
g3d                         10.00                 01 May 2014   https://sourceforge.net/projects/g3d/files/g3d-cpp/10.00/G3D-10.00-source.zip
gaa                         1.6.6                 01 May 2014   http://sourceforge.net/projects/gaa/files/gaa/gaa-1.6.6/gaa-1.6.6.tar.gz
gabedit                     2.1.8                 01 Jun 2014   http://dfn.dl.sourceforge.net/sourceforge/gabedit/Gabedit218Src.tar.gz
gail                        1.22.3                01 May 2014   http://ftp.gnome.org/pub/GNOME/sources/gail/1.22/gail-1.22.3.tar.bz2
gal                         2.5.3                 01 May 2014   http://ftp.acc.umu.se/pub/GNOME/sources/gal/2.5/gal-2.5.3.tar.bz2
galculator                  2.1.4                 02 Aug 2017   http://galculator.mnim.org/downloads/galculator-2.1.4.tar.bz2
gallipoli                   0.7.0                 01 Dec 2012   http://www.um.com.au/gallipoli/
gama                        2.03                  04 Feb 2019   http://ftp.gnu.org/pub/gnu/gama/gama-2.03.tar.gz
gambas3                     3.10.0                15 Jan 2018   https://sourceforge.net/projects/gambas/files/gambas3/gambas3-3.10.0.tar.bz2
gambit                      4.9.3                 23 Feb 2019   https://github.com/gambit/gambit/archive/v4.9.3.tar.gz
gamgi                       0.13.4                01 Jun 2014   http://www.gamgi.org/changelogs/changelogs13.html
gamin                       0.1.10                01 Jan 2013   http://www.gnome.org/~veillard/gamin/sources/gamin-0.1.10.tar.gz
garcon                      0.6.3                 02 Jul 2019   http://archive.xfce.org/src/xfce/garcon/0.6/garcon-0.6.3.tar.bz2
garnome                     2.24.0                18 Dec 2018   http://ftp.gnome.org/pub/gnome/sources/garnome/2.24/garnome-2.24.0.tar.bz2
gaupol                      0.28.2                29 Aug 2015   http://download.gna.org/gaupol/0.28/gaupol-0.28.2.tar.xz
gav                         0.9.0                 01 Dec 2012   https://sourceforge.net/projects/gav/files/gav/0.9.0/gav-0.9.0.tar.gz
gavl                        1.2.0                 01 Jun 2014   http://sourceforge.net/projects/gmerlin/files/gavl/1.2.0/gavl-1.2.0.tar.gz
gawk                        5.0.1                 19 Jun 2019   ftp://ftp.gnu.org/gnu/gawk/gawk-5.0.1.tar.xz
gbench                      2.6.0                 01 Oct 2012   ftp://ftp.ncbi.nlm.nih.gov/toolbox/gbench/ver-2.6.0/gbench-src-2.6.0.tgz
gc                          8.0.4                 08 Apr 2019   http://www.hboehm.info/gc/gc_source/gc-8.0.4.tar.gz
gcab                        1.2                   09 Dec 2018   http://ftp.gnome.org/pub/gnome/sources/gcab/1.2/gcab-1.2.tar.xz
gcal                        3.6.2                 01 Jun 2014   http://ftpmirror.gnu.org/gcal/gcal-3.6.2.tar.xz
gcalctool                   6.6.2                 24 Sep 2014   http://ftp.gnome.org/pub/GNOME/sources/gcalctool/6.6/gcalctool-6.6.2.tar.xz
gcc                         9.1.0                 03 May 2019   https://ftp.gnu.org/gnu/gcc/gcc-9.1.0/gcc-9.1.0.tar.xz
gccmakedep                  1.0.3                 01 May 2014   http://xorg.freedesktop.org/releases/individual/util/gccmakedep-1.0.3.tar.bz2
gcdemu                      1.2.0                 01 Jun 2014   http://sourceforge.net/projects/cdemu/files/CDemu%20clients/gcdemu-1.2.0/gcdemu-1.2.0.tar.bz2
gchart                      1.0.0                 01 Dec 2017   https://rubygems.org/downloads/gchart-1.0.0.gem
gcide                       0.52                  31 Jan 2018   https://ftp.gnu.org/pub/gnu/gcide/gcide-0.52.tar.xz
gconf                       3.2.6                 16 Sep 2017   http://ftp.acc.umu.se/pub/GNOME/sources/GConf/3.2/GConf-3.2.6.tar.xz
gconfcleaner                0.0.3                 01 Jun 2014   http://code.google.com/p/gconf-cleaner/downloads/list
gconfeditor                 3.0.1                 20 Mar 2015   http://ftp.gnome.org/pub/GNOME/sources/gconf-editor/3.0/gconf-editor-3.0.1.tar.xz
gconfmm                     2.28.3                20 Mar 2015   http://ftp.gnome.org/pub/GNOME/sources/gconfmm/2.28/gconfmm-2.28.3.tar.xz
gcr                         3.28.1                18 Jan 2019   http://ftp.gnome.org/pub/gnome/sources/gcr/3.28/gcr-3.28.1.tar.xz
gdado                       2.2                   01 Jun 2014   http://sourceforge.net/projects/gdado/files/souce%20code/gdado-2.2/gdado-2.2.tar.gz
gdal                        2.1.4                 20 Sep 2017   http://download.osgeo.org/gdal/2.1.4/gdal-2.1.4.tar.xz
gdb                         8.3                   12 May 2019   http://ftp.gnu.org/pub/gnu/gdb/gdb-8.3.tar.xz
gdbm                        1.18.1                27 Oct 2018   https://ftp.gnu.org/pub/gnu/gdbm/gdbm-1.18.1.tar.gz
gdesklets                   0.35.4                01 Jun 2014   http://www.gdesklets.de/files/gDesklets-0.35.4.tar.bz2
gdk3                        3.3.6                 24 Mar 2019   https://rubygems.org/downloads/gdk3-3.3.6.gem
gdkpixbuf                   2.38.1                28 Feb 2019   http://ftp.gnome.org/pub/gnome/sources/gdk-pixbuf/2.38/gdk-pixbuf-2.38.1.tar.xz
gdl                         3.28.0                02 Oct 2018   http://ftp.gnome.org/pub/gnome/sources/gdl/3.28/gdl-3.28.0.tar.xz
gdm                         3.32.0                05 May 2019   http://ftp.gnome.org/pub/gnome/sources/gdm/3.32/gdm-3.32.0.tar.xz
gdsl                        1.8                   25 Apr 2015   http://download.gna.org/gdsl/gdsl-1.8.tar.gz
geany                       1.35                  06 May 2019   http://download.geany.org/geany-1.35.tar.bz2
geanyplugins                1.35                  06 May 2019   https://plugins.geany.org/geany-plugins/geany-plugins-1.35.tar.gz
geary                       3.32.1                28 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/geary/3.32/geary-3.32.1.tar.xz
gedit                       3.32.2                15 May 2019   http://ftp.gnome.org/pub/gnome/sources/gedit/3.32/gedit-3.32.2.tar.xz
geditcodeassistance         3.16.0                30 Apr 2015   http://ftp.gnome.org/pub/GNOME/sources/gedit-code-assistance/3.16/gedit-code-assistance-3.16.0.tar.xz
geditplugins                3.32.2                15 May 2019   http://ftp.gnome.org/pub/gnome/sources/gedit-plugins/3.32/gedit-plugins-3.32.2.tar.xz
gegl                        0.4.16                09 May 2019   https://download.gimp.org/pub/gegl/0.4/gegl-0.4.16.tar.bz2
geglmm                      0.1.6                 20 Mar 2015   http://ftp.gnome.org/pub/GNOME/sources/geglmm/0.1/geglmm-0.1.6.tar.xz
geis                        2.2.16                01 Sep 2014   https://launchpad.net/geis/trunk/2.2.16/+download/geis-2.2.16.tar.xz
gengetopt                   2.23                  09 Jun 2019   ftp://ftp.gnu.org/gnu/gengetopt/gengetopt-2.23.tar.xz
genius                      1.0.24                20 May 2018   http://ftp.gnome.org/pub/gnome/sources/genius/1.0/genius-1.0.24.tar.xz
genshi                      0.7                   10 Aug 2018   https://ftp.edgewall.org/pub/genshi/Genshi-0.7.tar.gz
genx4r                      0.05                  01 Jun 2014   https://rubygems.org/downloads/genx4r-0.05.gem
geoclue                     2.5.3                 19 May 2019   https://gitlab.freedesktop.org/geoclue/geoclue/-/archive/2.5.3/geoclue-2.5.3.tar.bz2
geocodeglib                 3.26.1                14 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/geocode-glib/3.26/geocode-glib-3.26.1.tar.xz
geos                        3.6.2                 20 Sep 2017   http://download.osgeo.org/geos/geos-3.6.2.tar.bz2
geshi                       1.0.9.0               20 Sep 2017   https://sourceforge.net/projects/geshi/files/geshi/GeSHi%201.0.9.0/GeSHi-1.0.9.0.tar.bz2
getmail                     5.10                  13 Feb 2019   http://pyropus.ca/software/getmail/old-versions/getmail-5.10.tar.gz
gettext                     0.20.1                12 May 2019   ftp://ftp.gnu.org/gnu/gettext/gettext-0.20.1.tar.xz
gexiv2                      0.12.0                13 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/gexiv2/0.12/gexiv2-0.12.0.tar.xz
gfbgraph                    0.2.3                 15 Jul 2015   http://ftp.gnome.org/pub/GNOME/sources/gfbgraph/0.2/gfbgraph-0.2.3.tar.xz
gftp                        2.0.19                01 Jan 2013   http://gftp.seul.org/gftp-2.0.19.tar.bz2
ggv                         2.12.0                01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/ggv/2.12/ggv-2.12.0.tar.bz2
ghemical                    3.0.0                 01 Nov 2012   http://bioinformatics.org/ghemical/download/current/ghemical-3.0.0.tar.gz
gherkin                     6.0.17                17 Apr 2019   https://rubygems.org/downloads/gherkin-6.0.17.gem
ghex                        3.18.2                13 Jun 2016   http://ftp.gnome.org/pub/GNOME/sources/ghex/3.18/ghex-3.18.2.tar.xz
ghostscript                 9.27                  04 Apr 2019   https://github.com/ArtifexSoftware/ghostpdl-downloads/releases/download/gs927/ghostscript-9.27.tar.xz
giblib                      1.2.4                 01 Jun 2014   http://linuxbrit.co.uk/downloads/giblib-1.2.4.tar.gz
gif2apng                    1.6                   01 Jun 2014   https://sourceforge.net/projects/gif2apng/files/1.6/gif2apng-1.6
giflib                      5.2.1                 26 Jun 2019   https://sourceforge.net/projects/giflib/files/giflib-5.2.1.tar.gz
gifsicle                    1.88                  18 Mar 2017   http://www.lcdf.org/gifsicle/gifsicle-1.88.tar.gz
gift                        0.1.14                01 Jun 2014   ftp://ftp.gnu.org/gnu/gift/gift-0.1.14.tar.gz
gimp                        2.10.12               13 Jun 2019   https://download.gimp.org/pub/gimp/stable/gimp-2.10.12.tar.bz2
gimpsharp                   0.13                  01 Jun 2014   http://gimp-sharp.sourceforge.net/
gingerblue                  0.1.1                 01 Jan 2019   http://ftp.gnome.org/pub/gnome/sources/gingerblue/0.1/gingerblue-0.1.1.tar.xz
giostandalone               0.1.2                 01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/gio-standalone/0.1/gio-standalone-0.1.2.tar.bz2
girara                      0.2.6                 18 Aug 2016   https://pwmt.org/projects/girara/download/girara-0.2.6.tar.gz
girl                        9.9.8                 27 Apr 2017   http://ftp.gnome.org/pub/gnome/sources/girl/9.9/girl-9.9.8.tar.xz
gist                        5.0.0                 23 May 2018   https://rubygems.org/downloads/gist-5.0.0.gem
git                         2.22.0                08 Jun 2019   https://mirrors.edge.kernel.org/pub/software/scm/git/git-2.22.0.tar.xz
gitcola                     2.10                  27 Jul 2017   https://github.com/git-cola/git-cola/archive/v2.10.tar.gz
gitdb                       0.6.4                 10 Aug 2017   https://pypi.python.org/packages/e3/95/7e5d7261feb46c0539ac5e451be340ddd64d78c5118f2d893b052c76fe8c/gitdb-0.6.4.tar.gz#md5=44e4366b8bdfd306b075c3a52c96ae1a
gitg                        3.32.0                23 May 2019   http://ftp.gnome.org/pub/gnome/sources/gitg/3.32/gitg-3.32.0.tar.xz
giv                         0.9.15                01 Jun 2014   http://downloads.sourceforge.net/giv/giv-0.9.15.tar.gz
gjs                         1.57.3                22 Jun 2019   http://ftp.gnome.org/pub/gnome/sources/gjs/1.57/gjs-1.57.3.tar.xz
gkrellm                     2.3.10                20 Jul 2017   http://gkrellm.srcbox.net/releases/gkrellm-2.3.10.tar.bz2
glabels                     3.4.1                 29 Apr 2018   http://ftp.gnome.org/pub/gnome/sources/glabels/3.4/glabels-3.4.1.tar.xz
glade                       3.22.1                03 Apr 2018   http://ftp.gnome.org/pub/gnome/sources/glade/3.22/glade-3.22.1.tar.xz
glade3                      3.8.6                 08 Aug 2017   http://ftp.gnome.org/pub/gnome/sources/glade3/3.8/glade3-3.8.6.tar.xz
glamoregl                   0.6.0                 01 Jan 2014   https://www.x.org/releases/individual/driver/glamor-egl-0.6.0.tar.bz2
glark                       1.7.10                28 Jul 2016   https://sourceforge.net/projects/glark/files/glark/1.7.10/glark-1.7.10.tar.gz
glfw                        3.2.1                 20 May 2018   https://github.com/glfw/glfw/archive/3.2.1.zip
glib                        2.60.4                11 Jun 2019   http://ftp.gnome.org/pub/gnome/sources/glib/2.60/glib-2.60.4.tar.xz
glibc                       2.29                  31 Jan 2019   http://ftp.gnu.org/gnu/glibc/glibc-2.29.tar.xz
glibclibidn                 2.10.1                01 Jun 2014   http://ftp.gnu.org/gnu/libc/glibc-libidn-2.10.1.tar.bz2
glibmm                      2.60.0                19 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/glibmm/2.60/glibmm-2.60.0.tar.xz
glibnetworking              2.60.3                10 Jun 2019   http://ftp.gnome.org/pub/gnome/sources/glib-networking/2.60/glib-networking-2.60.3.tar.xz
glibopenssl                 2.50.7                27 Feb 2018   http://ftp.gnome.org/pub/gnome/sources/glib-openssl/2.50/glib-openssl-2.50.7.tar.xz
glimmer                     3.02b                 24 Jun 2017   http://ccb.jhu.edu/software/glimmer/glimmer302b.tar.gz
glob2                       0.9.4.4               01 Jul 2013   http://dl.sv.nongnu.org/releases/glob2/0.9.4/glob2-0.9.4.4.tar.gz
globalid                    0.4.2                 18 Feb 2019   https://rubygems.org/downloads/globalid-0.4.2.gem
glom                        1.30.4                11 Jun 2016   http://ftp.gnome.org/pub/GNOME/sources/glom/1.30/glom-1.30.4.tar.xz
glpk                        4.65                  23 Sep 2018   https://ftp.gnu.org/pub/gnu/glpk/glpk-4.65.tar.gz
glproto                     1.4.17                01 Dec 2013   http://xorg.freedesktop.org/releases/individual/proto/glproto-1.4.17.tar.bz2
glu                         9.0.0                 01 Aug 2015   ftp://ftp.freedesktop.org/pub/mesa/glu/glu-9.0.0.tar.bz2
glue                        1.1.1                 01 Dec 2017   https://rubygems.org/downloads/glue-1.1.1.gem
glut                        3.7                   01 Dec 2012   http://www.opengl.org/resources/libraries/glut/glut-3.7.tar.gz
gmail                       0.7.1                 22 Jul 2018   https://rubygems.org/downloads/gmail-0.7.1.gem
gmailxoauth                 0.4.2                 27 Dec 2017   https://rubygems.org/downloads/gmail_xoauth-0.4.2.gem
gmerlin                     0.3.5                 01 Jun 2014   http://gmerlin.sourceforge.net/
gmic                        1.5.7.1               01 Oct 2013   http://downloads.sourceforge.net/project/gmic/gmic_1.5.7.1.tar.gz
gmime                       3.2.3                 30 Nov 2018   http://ftp.gnome.org/pub/gnome/sources/gmime/3.2/gmime-3.2.3.tar.xz
gmp                         6.1.2                 17 Nov 2017   https://gmplib.org/download/gmp/gmp-6.1.2.tar.xz
gmsh                        1.65.0                01 Jun 2014   http://geuz.org/gmsh/src/gmsh-1.65.0-source.tgz
gmtk                        1.0.9a                01 Nov 2013   http://gmtk.googlecode.com/files/gmtk-1.0.9a.tar.gz
gmuehle                     0.2                   01 Jan 2013   http://stpohle.dyndns.org/index.php?page=progs#gmuehle
gnash                       0.8.10                03 Jun 2014   http://ftp.gnu.org/gnu/gnash/0.8.10/gnash-0.8.10.tar.bz2
gnet                        2.0.8                 01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/gnet/2.0/gnet-2.0.8.tar.bz2
gneutronica                 0.33                  01 Jun 2014   http://sourceforge.net/projects/gneutronica/files/gneutronica/gneutronica-0.33/gneutronica-0.33.tar.gz
gno3dtet                    1.96.1                28 Jul 2016   http://www.eseb.net/ftp/gno3dtet/gno3dtet-1.96.1.tgz
gnoise                      0.1.15                01 Jun 2014   http://peterhost.dl.sourceforge.net/sourceforge/gnoise/gnoise-0.1.15.tar.gz
gnomeapplets                3.32.0                05 May 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-applets/3.32/gnome-applets-3.32.0.tar.xz
gnomeappletsensors          1.7.12                01 Jun 2014   http://sensors-applet.sourceforge.net/
gnomeaudio                  2.22.2                01 Sep 2012   http://ftp.gnome.org/pub/GNOME/sources/gnome-audio/2.22/gnome-audio-2.22.2.tar.bz2
gnomeautoar                 0.2.2                 31 Jan 2018   https://ftp.gnome.org/pub/gnome/sources/gnome-autoar/0.2/gnome-autoar-0.2.2.tar.xz
gnomebackgrounds            3.32.0                19 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-backgrounds/3.32/gnome-backgrounds-3.32.0.tar.xz
gnomebaker                  0.6.2                 01 Jun 2014   http://sourceforge.net/projects/gnomebaker/
gnomebatterybench           3.15.4                07 Mar 2015   http://ftp.gnome.org/pub/GNOME/sources/gnome-battery-bench/3.15/gnome-battery-bench-3.15.4.tar.xz
gnomebluetooth              3.32.1                27 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-bluetooth/3.32/gnome-bluetooth-3.32.1.tar.xz
gnomeboxes                  3.32.1                11 May 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-boxes/3.32/gnome-boxes-3.32.1.tar.xz
gnomebtdownload             0.0.32                01 Jun 2014   http://sourceforge.net/projects/gnome-bt/files/gnome-bt/0.0.32/gnome-btdownload-0.0.32.tar.gz
gnomebuild                  0.2.4                 01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/gnome-build/0.2/gnome-build-0.2.4.tar.bz2
gnomebuilder                3.32.1                11 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-builder/3.32/gnome-builder-3.32.1.tar.xz
gnomecalculator             3.32.1                09 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-calculator/3.32/gnome-calculator-3.32.1.tar.xz
gnomecalendar               3.32.2                11 May 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-calendar/3.32/gnome-calendar-3.32.2.tar.xz
gnomecharacters             3.32.0                15 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-characters/3.32/gnome-characters-3.32.0.tar.xz
gnomechemistryutils         0.9.2                 01 Jun 2014   http://download.savannah.nongnu.org/releases/gchemutils/0.9/gnome-chemistry-utils-0.9.2.tar.bz2
gnomechess                  3.28.1                10 Apr 2018   http://ftp.gnome.org/pub/gnome/sources/gnome-chess/3.28/gnome-chess-3.28.1.tar.xz
gnomeclocks                 3.32.0                10 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-clocks/3.32/gnome-clocks-3.32.0.tar.xz
gnomecodeassistance         3.16.0                30 Apr 2015   http://ftp.gnome.org/pub/GNOME/sources/gnome-code-assistance/3.16/gnome-code-assistance-3.16.0.tar.xz
gnomecolormanager           3.30.0                05 Sep 2018   http://ftp.gnome.org/pub/gnome/sources/gnome-color-manager/3.30/gnome-color-manager-3.30.0.tar.xz
gnomecommander              1.10.2                17 May 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-commander/1.10/gnome-commander-1.10.2.tar.xz
gnomecommon                 3.18.0                22 Sep 2015   http://ftp.gnome.org/pub/GNOME/sources/gnome-common/3.18/gnome-common-3.18.0.tar.xz
gnomecontacts               3.30                  04 Sep 2018   http://ftp.gnome.org/pub/gnome/sources/gnome-contacts/3.30/gnome-contacts-3.30.tar.xz
gnomecontrolcenter          3.32.2                25 May 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-control-center/3.32/gnome-control-center-3.32.2.tar.xz
gnomecupsmanager            0.33                  01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/gnome-cups-manager/0.33/gnome-cups-manager-0.33.tar.bz2
gnomedesktop                3.32.2                08 May 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-desktop/3.32/gnome-desktop-3.32.2.tar.xz
gnomedictionary             3.26.1                02 Oct 2017   http://ftp.gnome.org/pub/gnome/sources/gnome-dictionary/3.26/gnome-dictionary-3.26.1.tar.xz
gnomedirectorythumbnailer   0.1.10                26 Jan 2018   http://ftp.gnome.org/pub/gnome/sources/gnome-directory-thumbnailer/0.1/gnome-directory-thumbnailer-0.1.10.tar.xz
gnomediskutility            3.30.2                23 Oct 2018   http://ftp.gnome.org/pub/gnome/sources/gnome-disk-utility/3.30/gnome-disk-utility-3.30.2.tar.xz
gnomedocuments              3.30.1                19 Jan 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-documents/3.30/gnome-documents-3.30.1.tar.xz
gnomedocutils               0.20.10               20 Mar 2015   http://ftp.gnome.org/pub/GNOME/sources/gnome-doc-utils/0.20/gnome-doc-utils-0.20.10.tar.xz
gnomedvbdaemon              0.2.90                01 Aug 2014   http://ftp.gnome.org/pub/GNOME/sources/gnome-dvb-daemon/0.2/gnome-dvb-daemon-0.2.90.tar.xz
gnomeflashback              3.32.0                05 May 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-flashback/3.32/gnome-flashback-3.32.0.tar.xz
gnomefontviewer             3.32.0                15 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-font-viewer/3.32/gnome-font-viewer-3.32.0.tar.xz
gnomegames                  3.32.1                10 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-games/3.32/gnome-games-3.32.1.tar.xz
gnomehearts                 0.2                   01 Jun 2014   http://www.jejik.com/files/gnome-hearts/gnome-hearts-0.2.tar.gz
gnomeicontheme              3.12.0                01 Apr 2014   http://ftp.gnome.org/pub/gnome/sources/gnome-icon-theme/3.12/gnome-icon-theme-3.12.0.tar.xz
gnomeiconthemeextras        3.12.0                01 Apr 2014   http://ftp.gnome.org/pub/gnome/sources/gnome-icon-theme-extras/3.12/gnome-icon-theme-extras-3.12.0.tar.xz
gnomeiconthemesymbolic      3.12.0                01 Apr 2014   http://ftp.gnome.org/pub/gnome/sources/gnome-icon-theme-symbolic/3.12/gnome-icon-theme-symbolic-3.12.0.tar.xz
gnomeinitialsetup           3.32.1                05 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-initial-setup/3.32/gnome-initial-setup-3.32.1.tar.xz
gnomeinternetradiolocator   2.0.4                 25 May 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-internet-radio-locator/2.0/gnome-internet-radio-locator-2.0.4.tar.xz
gnomekeyring                3.28.2                10 May 2018   http://ftp.gnome.org/pub/GNOME/sources/gnome-keyring/3.28/gnome-keyring-3.28.2.tar.xz
gnomekeyringmanager         2.20.0                01 Jun 2014   ftp://ftp.gnome.org/pub/gnome/sources/gnome-keyring-manager/2.20/gnome-keyring-manager-2.20.0.tar.bz2
gnomeklotski                3.22.2                09 Sep 2017   http://ftp.gnome.org/pub/gnome/sources/gnome-klotski/3.22/gnome-klotski-3.22.2.tar.xz
gnomekraorathumbnailer      1.3                   01 Dec 2013   http://ftp.gnome.org/pub/GNOME/sources/gnome-kra-ora-thumbnailer/1.3/gnome-kra-ora-thumbnailer-1.3.tar.xz
gnomelatex                  3.32.0                10 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-latex/3.32/gnome-latex-3.32.0.tar.xz
gnomelogs                   3.32.1                23 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-logs/3.32/gnome-logs-3.32.1.tar.xz
gnomemag                    0.15.1                01 Jun 2014   http://ftp.gnome.org/pub/gnome/sources/gnome-mag/0.15/gnome-mag-0.15.1.tar.bz2
gnomemahjongg               3.21.2                01 Jun 2016   http://ftp.gnome.org/pub/GNOME/sources/gnome-mahjongg/3.21/gnome-mahjongg-3.21.2.tar.xz
gnomemainmenu               1.8.0                 01 Jun 2014   http://pub.mate-desktop.org/releases/1.8/gnome-main-menu-1.8.0.tar.xz
gnomemaps                   3.32.1                01 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-maps/3.32/gnome-maps-3.32.1.tar.xz
gnomemedia                  3.4.0                 24 Sep 2014   http://ftp.gnome.org/pub/GNOME/sources/gnome-media/3.4/gnome-media-3.4.0.tar.xz
gnomemeeting                1.2.2                 01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/gnomemeeting
gnomemenus                  3.31.4                22 Jan 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-menus/3.31/gnome-menus-3.31.4.tar.xz
gnomemimedata               2.18.0                01 Jul 2012   http://ftp.gnome.org/pub/gnome/sources/gnome-mime-data/2.18/gnome-mime-data-2.18.0.tar.bz2
gnomemines                  3.30.1                24 Sep 2018   http://ftp.gnome.org/pub/gnome/sources/gnome-mines/3.30/gnome-mines-3.30.1.tar.xz
gnomemount                  0.6                   01 Jun 2014   http://people.freedesktop.org/~david/dist/gnome-mount-0.6.tar.gz
gnomemplayer                1.0.9b                01 Nov 2013   https://storage.googleapis.com/google-code-archive-downloads/v2/code.google.com/gnome-mplayer/gnome-mplayer-1.0.9b.tar.gz
gnomemultiwriter            3.30.0                05 Sep 2018   http://ftp.gnome.org/pub/gnome/sources/gnome-multi-writer/3.30/gnome-multi-writer-3.30.0.tar.xz
gnomemusic                  3.32.1                09 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-music/3.32/gnome-music-3.32.1.tar.xz
gnomenetstatus              2.28.2                02 Aug 2017   http://ftp.gnome.org/pub/GNOME/sources/gnome-netstatus/2.28/gnome-netstatus-2.28.2.tar.gz
gnomenettool                3.8.1                 01 Jun 2013   http://ftp.gnome.org/pub/GNOME/sources/gnome-nettool/3.8/gnome-nettool-3.8.1.tar.xz
gnomenetwork                1.99.5                01 Jun 2014   ftp://ftp.gnome.org/pub/GNOME/sources/gnome-network/1.99/gnome-network-1.99.5.tar.bz2
gnomenibbles                3.24.1                09 Sep 2018   http://ftp.gnome.org/pub/gnome/sources/gnome-nibbles/3.24/gnome-nibbles-3.24.1.tar.xz
gnomeonlineaccounts         3.32.0                25 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-online-accounts/3.32/gnome-online-accounts-3.32.0.tar.xz
gnomeonlineminers           3.30.0                04 Sep 2018   http://ftp.gnome.org/pub/gnome/sources/gnome-online-miners/3.30/gnome-online-miners-3.30.0.tar.xz
gnomepackagekit             3.30.0                05 Sep 2018   http://ftp.gnome.org/pub/gnome/sources/gnome-packagekit/3.30/gnome-packagekit-3.30.0.tar.xz
gnomepanel                  3.32.0                05 May 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-panel/3.32/gnome-panel-3.32.0.tar.xz
gnomephonemanager           0.69                  24 Sep 2014   http://ftp.gnome.org/pub/GNOME/sources/gnome-phone-manager/0.69/gnome-phone-manager-0.69.tar.xz
gnomephotos                 3.32.0                19 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-photos/3.32/gnome-photos-3.32.0.tar.xz
gnomepkgtool                0.5.2                 01 Jun 2014   http://gnome-pkgtool.sourceforge.net/
gnomepowermanager           3.30.0                05 Sep 2018   http://ftp.gnome.org/pub/gnome/sources/gnome-power-manager/3.30/gnome-power-manager-3.30.0.tar.xz
gnomepython                 2.28.1                01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/gnome-python/2.28/gnome-python-2.28.1.tar.gz
gnomepythondesktop          2.23.0                01 Jan 2013   http://ftp.gnome.org/pub/GNOME/sources/gnome-python-desktop/2.23/gnome-python-desktop-2.23.0.tar.bz2
gnomerecipes                1.2.0                 08 May 2017   http://ftp.gnome.org/pub/gnome/sources/gnome-recipes/1.2/gnome-recipes-1.2.0.tar.xz
gnomerobots                 3.22.2                09 Sep 2017   http://ftp.gnome.org/pub/gnome/sources/gnome-robots/3.22/gnome-robots-3.22.2.tar.xz
gnomescan                   0.5.94                01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/gnome-scan/0.5/gnome-scan-0.5.94.tar.bz2
gnomeschedule               2.2.2                 01 Jun 2014   http://sourceforge.net/projects/gnome-schedule/files/gnome-schedule-2/gnome-schedule-2-2-2/gnome-schedule-2.2.2.tar.gz
gnomescreensaver            3.6.1                 24 Sep 2014   http://ftp.gnome.org/pub/GNOME/sources/gnome-screensaver/3.6/gnome-screensaver-3.6.1.tar.xz
gnomescreenshot             3.32.0                15 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-screenshot/3.32/gnome-screenshot-3.32.0.tar.xz
gnomesession                3.32.0                14 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-session/3.32/gnome-session-3.32.0.tar.xz
gnomesettingsdaemon         3.32.0                30 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-settings-daemon/3.32/gnome-settings-daemon-3.32.0.tar.xz
gnomesharp                  2.24.0                01 Jun 2014   http://ftp.gnome.org/pub/gnome/sources/gnome-sharp/2.24/gnome-sharp-2.24.0.tar.bz2
gnomeshell                  3.32.2                18 May 2019   ftp://ftp.gnome.org/pub/gnome/sources/gnome-shell/3.32/gnome-shell-3.32.2.tar.xz
gnomeshellextensions        3.32.1                18 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-shell-extensions/3.32/gnome-shell-extensions-3.32.1.tar.xz
gnomesoftware               3.32.3                25 May 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-software/3.32/gnome-software-3.32.3.tar.xz
gnomesoundrecorder          3.32.1                04 Jun 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-sound-recorder/3.32/gnome-sound-recorder-3.32.1.tar.xz
gnomespeech                 0.4.16                01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/gnome-speech/
gnomesubtitles              1.1                   01 Jun 2014   http://downloads.sourceforge.net/gnome-subtitles/gnome-subtitles-1.1.tar.gz
gnomesudoku                 3.30.0                04 Sep 2018   http://ftp.gnome.org/pub/gnome/sources/gnome-sudoku/3.30/gnome-sudoku-3.30.0.tar.xz
gnomesystemmonitor          3.32.1                09 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-system-monitor/3.32/gnome-system-monitor-3.32.1.tar.xz
gnomesystemtools            3.0.0                 20 Mar 2015   http://ftp.gnome.org/pub/GNOME/sources/gnome-system-tools/3.0/gnome-system-tools-3.0.0.tar.bz2
gnometaquin                 3.30.0                05 Sep 2018   http://ftp.gnome.org/pub/gnome/sources/gnome-taquin/3.30/gnome-taquin-3.30.0.tar.xz
gnometerminal               3.32.1                09 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-terminal/3.32/gnome-terminal-3.32.1.tar.xz
gnometetravex               3.19.92               16 Mar 2016   http://ftp.gnome.org/pub/GNOME/sources/gnome-tetravex/3.19/gnome-tetravex-3.19.92.tar.xz
gnomethemes                 3.0.0                 01 Jun 2014   ftp://ftp.gnome.org/pub/GNOME/sources/gnome-themes/3.0/gnome-themes-3.0.0.tar.bz2
gnomethemesextra            3.28                  24 Mar 2018   http://ftp.gnome.org/pub/gnome/sources/gnome-themes-extra/3.28/gnome-themes-extra-3.28.tar.xz
gnomethemesstandard         3.27.90               14 Feb 2018   http://ftp.gnome.org/pub/gnome/sources/gnome-themes-standard/3.27/gnome-themes-standard-3.27.90.tar.xz
gnometodo                   3.91.1                24 Sep 2018   http://ftp.gnome.org/pub/gnome/sources/gnome-todo/3.91/gnome-todo-3.91.1.tar.xz
gnometweaks                 3.30.2                10 Dec 2018   http://ftp.gnome.org/pub/gnome/sources/gnome-tweaks/3.30/gnome-tweaks-3.30.2.tar.xz
gnometweaktool              3.26.4                30 Dec 2017   http://ftp.gnome.org/pub/gnome/sources/gnome-tweak-tool/3.26/gnome-tweak-tool-3.26.4.tar.xz
gnomeusage                  3.32.0                22 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-usage/3.32/gnome-usage-3.32.0.tar.xz
gnomeusershare              3.18.0                21 Sep 2015   http://ftp.gnome.org/pub/GNOME/sources/gnome-user-share/3.18/gnome-user-share-3.18.0.tar.xz
gnomeutils                  3.2.1                 20 Mar 2015   http://ftp.gnome.org/pub/GNOME/sources/gnome-utils/3.2/gnome-utils-3.2.1.tar.xz
gnomevfs                    2.24.4                01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/gnome-vfs/2.24/gnome-vfs-2.24.4.tar.bz2
gnomevfsmm                  2.26.0                01 Jun 2014   http://ftp.acc.umu.se/pub/GNOME/sources/gnome-vfsmm/2.26/gnome-vfsmm-2.26.0.tar.bz2
gnomevideoarcade            0.8.8                 31 Oct 2017   http://ftp.gnome.org/pub/gnome/sources/gnome-video-arcade/0.8/gnome-video-arcade-0.8.8.tar.xz
gnomevideoeffects           0.4.1                 01 Mar 2014   http://ftp.gnome.org/pub/GNOME/sources/gnome-video-effects/0.4/gnome-video-effects-0.4.1.tar.xz
gnomevoicecontrol           0.4                   01 Jun 2014   http://prdownload.berlios.de/festlang/gnome-voice-control-0.4.tar.gz
gnomevolumemanager          2.17.0                01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/gnome-volume-manager/
gnomeweather                3.32.2                22 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/gnome-weather/3.32/gnome-weather-3.32.2.tar.xz
gnonlin                     1.2.0                 01 Apr 2014   http://gstreamer.freedesktop.org/src/gnonlin/gnonlin-1.2.0.tar.xz
gnopernicus                 1.1.2                 01 Jun 2014   http://ftp.gnome.org/pub/gnome/sources/gnopernicus/1.1/gnopernicus-1.1.2.tar.bz2
gnormalize                  0.63                  01 Jun 2014   http://prdownloads.sourceforge.net/gnormalize/gnormalize-0.63.tar.gz
gnote                       3.32.1                14 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/gnote/3.32/gnote-3.32.1.tar.xz
gnuastro                    0.9                   18 Apr 2019   https://ftp.gnu.org/pub/gnu/gnuastro/gnuastro-0.9.tar.lz
gnucash                     3.6                   01 Jul 2019   http://downloads.sourceforge.net/gnucash/gnucash-3.6.tar.bz2
gnuchess                    6.2.5                 28 Jul 2017   http://ftp.gnu.org/gnu/chess/gnuchess-6.2.5.tar.gz
gnucobol                    2.2                   11 Sep 2017   https://sourceforge.net/projects/open-cobol/files/gnu-cobol/2.2/gnucobol-2.2.tar.xz
gnugo                       3.8                   01 Jan 2013   http://ftp.gnu.org/gnu/gnugo/gnugo-3.8.tar.gz
gnuhealth                   3.4.0                 27 Nov 2018   http://ftp.gnu.org/pub/gnu/health/gnuhealth-3.4.0.tar.gz
gnujump                     1.0.5                 01 Dec 2012   http://download.savannah.gnu.org/releases/gnujump/gnujump-1.0.5.tar.gz
gnumeric                    1.12.45               04 Jun 2019   ftp://ftp.gnome.org/pub/gnome/sources/gnumeric/1.12/gnumeric-1.12.45.tar.xz
gnump3d                     3.0                   01 Jun 2014   http://savannah.gnu.org/download/gnump3d/gnump3d-3.0.tar.bz2
gnunet                      0.11.3                10 Apr 2019   https://ftp.gnu.org/pub/gnu/gnunet/gnunet-0.11.3.tar.gz
gnunetgtk                   0.7.2b                01 Jun 2014   http://gnunet.org/download/gnunet-gtk-0.7.0e.tar.bz2
gnupdf                      16.10.2013            16 Oct 2013   https://www.gnu.org/software/pdf/
gnupg                       2.2.17                09 Jul 2019   ftp://ftp.gnupg.org/gcrypt/gnupg/gnupg-2.2.17.tar.bz2
gnuplot                     5.2.7                 30 May 2019   https://sourceforge.net/projects/gnuplot/files/gnuplot/5.2.7/gnuplot-5.2.7.tar.gz
gnuradio                    3.7.9                 15 Jan 2019   https://www.gnuradio.org/releases/gnuradio/gnuradio-3.7.9.tar.gz
gnurl                       7.65.1                09 Jun 2019   http://ftp.gnu.org/pub/gnu/gnunet/gnurl-7.65.1.tar.gz
gnustepbase                 1.24.0                01 Jun 2014   ftp://ftp.gnustep.org/pub/gnustep/core/gnustep-base-1.24.0.tar.gz
gnustepmake                 1.12.0                01 Jun 2014   ftp://ftp.gnustep.org/pub/gnustep/core/gnustep-make-1.12.0.tar.gz
gnutls                      3.6.8                 29 May 2019   ftp://ftp.gnupg.org/gcrypt/gnutls/v3.6/gnutls-3.6.8.tar.xz
gnuuid                      0.5.1                 14 Oct 2018   https://rubygems.org/downloads/gn_uuid-0.5.1.gem
go                          1.4.2                 19 May 2015   https://storage.googleapis.com/golang/go1.4.2.src.tar.gz
gob2                        2.0.20                01 Dec 2013   http://ftp.gnome.org/pub/GNOME/sources/gob2/2.0/gob2-2.0.20.tar.xz
gobby                       0.5.0                 02 Jun 2018   http://releases.0x539.de/gobby/gobby-0.5.0.tar.gz
gobjectintrospection        1.60.2                17 Jun 2019   http://ftp.gnome.org/pub/gnome/sources/gobject-introspection/1.60/gobject-introspection-1.60.2.tar.xz
gocr                        0.47                  02 Aug 2017   https://sourceforge.net/projects/jocr/files/gocr/0.47/gocr-0.47.tar.gz
god                         0.7.3                 01 May 2013   http://godrb.com/
goffice                     0.10.45               04 Jun 2019   http://ftp.gnome.org/pub/gnome/sources/goffice/0.10/goffice-0.10.45.tar.xz
gok                         2.30.1                20 Aug 2017   http://ftp.acc.umu.se/pub/GNOME/sources/gok/2.30/gok-2.30.1.tar.gz
golly                       3.0                   03 Sep 2017   https://sourceforge.net/projects/golly/files/golly/golly-3.0/golly-3.0-src.tar.gz
gom                         0.3.3                 28 Jun 2017   http://ftp.gnome.org/pub/gnome/sources/gom/0.3/gom-0.3.3.tar.xz
gonzui                      1.2                   01 Jun 2014   https://rubygems.org/downloads/gonzui-1.2.gem
goobox                      3.6.0                 10 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/goobox/3.6/goobox-3.6.0.tar.xz
goocanvas                   2.0.4                 10 Oct 2017   http://ftp.gnome.org/pub/gnome/sources/goocanvas/2.0/goocanvas-2.0.4.tar.xz
googleauth                  0.8.1                 17 Apr 2019   https://rubygems.org/downloads/googleauth-0.8.1.gem
gosu                        0.14.5                09 Jan 2019   https://rubygems.org/downloads/gosu-0.14.5.gem
gpa                         0.10.0                30 Oct 2018   https://www.gnupg.org/ftp/gcrypt/gpa/gpa-0.10.0.tar.bz2
gpac                        0.7.1                 27 Feb 2018   https://github.com/gpac/gpac/archive/v0.7.1.tar.gz
gpaint                      2.0.3.3               01 Jun 2014   ftp://alpha.gnu.org/gnu/gpaint/gpaint-2-0.3.3.tar.gz
gparted                     1.0.0                 30 May 2019   http://downloads.sourceforge.net/gparted/gparted-1.0.0.tar.gz
gpass                       0.5.1                 01 Jun 2014   http://projects.netlab.jp/gpass/
gpdf                        2.10.0                01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/gpdf/2.10/gpdf-2.10.0.tar.bz2
gperf                       3.1                   20 Jul 2017   ftp://ftp.gnu.org/gnu/gperf/gperf-3.1.tar.gz
gperiodic                   2.0.10                01 Jun 2014   http://www.frantz.fi/software/gperiodic-2.0.10.tar.gz
gpgme                       1.13.1                22 Jun 2019   https://www.gnupg.org/ftp/gcrypt/gpgme/gpgme-1.13.1.tar.bz2
gphoto2                     2.5.23                05 Jun 2019   https://sourceforge.net/projects/gphoto/files/gphoto/2.5.23/gphoto2-2.5.23.tar.bz2
gpm                         1.20.7                18 Feb 2019   http://anduin.linuxfromscratch.org/BLFS/gpm/gpm-1.20.7.tar.bz2
gpredict                    0.9.0                 01 Jun 2014   http://groundstation.sourceforge.net/gpredict/
gprolog                     1.4.5                 07 Feb 2019   http://ftp.gnu.org/pub/gnu/gprolog/gprolog-1.4.5.tar.gz
gpsd                        2.36                  01 Jun 2014   http://download.berlios.de/gpsd/gpsd-2.36.tar.gz
gptfdisk                    1.0.4                 30 Jul 2018   https://sourceforge.net/projects/gptfdisk/files/gptfdisk/1.0.4/gptfdisk-1.0.4.tar.gz
gqview                      2.1.5                 02 Jan 2018   http://prdownloads.sourceforge.net/gqview/gqview-2.1.5.tar.gz
grabc                       1.1                   01 Jun 2014   http://freshmeat.net/projects/grabc
grace                       5.1.25                19 Oct 2018   ftp://plasma-gate.weizmann.ac.il/pub/grace/src/grace5/grace-5.1.25.tar.gz
gramps                      4.2.6                 20 Sep 2017   https://sourceforge.net/projects/gramps/files/Stable/4.2.6/gramps-4.2.6.tar.gz
granatier                   19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/granatier-19.04.2.tar.xz
grantlee                    5.1.0                 10 Jun 2018   http://downloads.grantlee.org/grantlee-5.1.0.tar.gz
grantleeeditor              19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/grantlee-editor-19.04.2.tar.xz
grantleetheme               19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/grantleetheme-19.04.2.tar.xz
grape                       1.2.3                 18 Feb 2019   https://rubygems.org/downloads/grape-1.2.3.gem
graphene                    1.9.4                 19 Jun 2019   http://ftp.gnome.org/pub/gnome/sources/graphene/1.9/graphene-1.9.4.tar.xz
graphicsmagick              1.3.27                16 Jan 2018   https://sourceforge.net/projects/graphicsmagick/files/graphicsmagick/1.3.27/GraphicsMagick-1.3.27.tar.xz
graphite2                   1.3.13                22 Dec 2018   https://github.com/silnrsi/graphite/releases/download/1.3.13/graphite2-1.3.13.tgz
graphviz                    2.40.1                29 Dec 2016   http://www.graphviz.org/pub/graphviz/ARCHIVE/graphviz-2.40.1.tar.gz
graphvizcairo               2.8                   01 Jun 2014   http://www.graphviz.org/pub/graphviz/ARCHIVE/graphviz-cairo-2.8.tar.gz
greenfish                   0.0.4                 15 Apr 2015   greenfish-0.0.4
grep                        3.3                   21 Dec 2018   http://ftp.gnu.org/gnu/grep/grep-3.3.tar.xz
grhino                      0.16.0                01 Jan 2013   http://rhino.sourceforge.net/
grilo                       0.3.7                 19 Nov 2018   http://ftp.gnome.org/pub/gnome/sources/grilo/0.3/grilo-0.3.7.tar.xz
griloplugins                0.3.8                 24 Sep 2018   http://ftp.gnome.org/pub/gnome/sources/grilo-plugins/0.3/grilo-plugins-0.3.8.tar.xz
grism                       0.9.0                 16 Feb 2017   http://www.grism.org/grism-0.9.0.tar.gz
groff                       1.22.4                23 Dec 2018   http://ftp.gnu.org/pub/gnu/groff/groff-1.22.4.tar.gz
groonga                     5.1.0                 29 Nov 2015   http://packages.groonga.org/source/groonga/groonga-5.1.0.tar.gz
grub                        2.04                  05 Jul 2019   ftp://ftp.gnu.org/gnu/grub/grub-2.04.tar.xz
gruff                       0.7.0                 16 Oct 2017   https://rubygems.org/downloads/gruff-0.7.0.gem
gsambad                     0.1.9                 01 Jun 2014   http://directory.fsf.org/wiki/GSAMBAD
gscan2pdf                   2.2.0                 01 Dec 2018   https://sourceforge.net/projects/gscan2pdf/files/gscan2pdf/2.2.0/gscan2pdf-2.2.0.tar.xz
gscanbus                    0.8                   04 Nov 2017   https://sourceforge.net/projects/gscanbus.berlios/files/gscanbus-0.8.tar.gz
gsettingsdesktopschemas     3.32.0                13 Mar 2019   https://ftp.gnome.org/pub/GNOME/sources/gsettings-desktop-schemas/3.32/gsettings-desktop-schemas-3.32.0.tar.xz
gsl                         2.5                   10 Sep 2018   https://ftp.gnu.org/pub/gnu/gsl/gsl-2.5.tar.gz
gslapt                      0.5.4c                16 Oct 2017   http://software.jaos.org/source/gslapt/gslapt-0.5.4c.tar.gz
gsmartcontrol               0.8.7                 11 Apr 2015   http://sourceforge.net/projects/gsmartcontrol/files/0.8.7/gsmartcontrol-0.8.7.tar.bz2
gsound                      1.0.2                 01 Nov 2015   http://ftp.gnome.org/pub/GNOME/sources/gsound/1.0/gsound-1.0.2.tar.xz
gspell                      1.8.1                 17 Jun 2018   http://ftp.gnome.org/pub/gnome/sources/gspell/1.8/gspell-1.8.1.tar.xz
gssdp                       1.2.1                 03 May 2019   http://ftp.gnome.org/pub/gnome/sources/gssdp/1.2/gssdp-1.2.1.tar.xz
gstdebugger                 0.90.0                09 Oct 2015   http://ftp.gnome.org/pub/GNOME/sources/gst-debugger/0.90/gst-debugger-0.90.0.tar.xz
gstlibav                    1.6.1                 31 Oct 2015   http://gstreamer.freedesktop.org/src/gst-libav/gst-libav-1.6.1.tar.xz
gstm                        1.2                   01 Jun 2014   https://sourceforge.net/projects/gstm/files/gSTM/gSTM-1.2/gstm-1.2.tar.gz
gstplugins                  0.8.12                01 May 2013   http://gstreamer.freedesktop.org/src/gst-plugins/gst-plugins-0.8.12.tar.bz2
gstpluginsbad               1.14.4                10 Feb 2019   https://gstreamer.freedesktop.org/src/gst-plugins-bad/gst-plugins-bad-1.14.4.tar.xz
gstpluginsbase              1.16.0                19 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/gst-plugins-base/1.16/gst-plugins-base-1.16.0.tar.xz
gstpluginsgood              1.16.0                19 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/gst-plugins-good/1.16/gst-plugins-good-1.16.0.tar.xz
gstpluginsugly              1.12.2                17 Sep 2017   https://gstreamer.freedesktop.org/src/gst-plugins-ugly/gst-plugins-ugly-1.12.2.tar.xz
gstpython                   0.10.22               20 Mar 2015   http://ftp.gnome.org/pub/GNOME/sources/gst-python/0.10/gst-python-0.10.22.tar.xz
gstreamer                   1.16.0                19 Apr 2019   https://gstreamer.freedesktop.org/src/gstreamer/gstreamer-1.16.0.tar.xz
gstreamermm                 1.10.0                21 Oct 2017   http://ftp.gnome.org/pub/gnome/sources/gstreamermm/1.10/gstreamermm-1.10.0.tar.xz
gstrtspserver               1.4.5                 02 Mar 2015   http://gstreamer.freedesktop.org/src/gst-rtsp/gst-rtsp-server-1.4.5.tar.xz
gsview                      4.9                   01 Jun 2014   http://mirror.cs.wisc.edu/pub/mirrors/ghost/ghostgum/gsview-4.9.tar.gz
gtef                        1.99.2                12 Mar 2017   http://ftp.gnome.org/pub/GNOME/sources/gtef/1.99/gtef-1.99.2.tar.xz
gthumb                      3.8.0                 27 May 2019   http://ftp.gnome.org/pub/gnome/sources/gthumb/3.8/gthumb-3.8.0.tar.xz
gtk+                        3.24.10               04 Jul 2019   ftp://ftp.gnome.org/pub/GNOME/sources/gtk+/3.24/gtk+-3.24.10.tar.xz
gtk                         3.96.0                08 May 2019   http://ftp.acc.umu.se/pub/gnome/sources/gtk/3.96/gtk-3.96.0.tar.xz
gtkam                       0.2.0                 01 Apr 2014   http://sourceforge.net/projects/gphoto/files/gtkam/0.2.0/gtkam-0.2.0.tar.gz
gtkchtheme                  0.3.1                 01 Dec 2012   http://plasmasturm.org/code/gtk-chtheme/gtk-chtheme-0.3.1.tar.bz2
gtkclearlooks               0.5                   01 Dec 2012   http://ftp.gnome.org/pub/GNOME/teams/art.gnome.org/themes/gtk_engines/gtk-clearlooks-0.5.tar.bz2
gtkdatabox                  0.8.0.0               01 Dec 2012   http://www.eudoxos.net/gtk/gtkdatabox/download/gtkdatabox-0.8.0.0.tar.gz
gtkdoc                      1.30                  11 May 2019   http://ftp.gnome.org/pub/gnome/sources/gtk-doc/1.30/gtk-doc-1.30.tar.xz
gtkengines                  2.91.1                01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/gtk-engines/2.91/gtk-engines-2.91.1.tar.bz2
gtkglarea                   2.1.0                 01 Apr 2014   http://ftp.gnome.org/pub/gnome/sources/gtkglarea/2.1/gtkglarea-2.1.0.tar.xz
gtkglext                    1.2.0                 01 Nov 2012   http://downloads.sourceforge.net/gtkglext/gtkglext-1.2.0.tar.gz
gtkhtml                     4.10.0                21 Sep 2015   http://ftp.gnome.org/pub/GNOME/sources/gtkhtml/4.10/gtkhtml-4.10.0.tar.xz
gtkinternetradiolocator     0.0.3                 14 Jul 2018   http://ftp.gnome.org/pub/gnome/sources/gtk-internet-radio-locator/0.0/gtk-internet-radio-locator-0.0.3.tar.xz
gtkiptables                 0.5.0                 01 Jun 2014   http://gtk-iptables.sourceforge.net/
gtkmm                       3.24.1                20 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/gtkmm/3.24/gtkmm-3.24.1.tar.xz
gtkpacman                   1.2                   01 Dec 2012   https://gna.org/projects/gtkpacman/
gtkperl                     0.7010                21 Aug 2017   http://search.cpan.org/CPAN/authors/id/M/ML/MLEHMANN/Gtk-Perl-0.7010.tar.gz
gtkrecordmydesktop          0.3.8                 29 Sep 2018   https://sourceforge.net/projects/recordmydesktop/files/gtk-recordMyDesktop/0.3.8/gtk-recordmydesktop-0.3.8.tar.gz
gtksharp                    2.12.45               21 Aug 2017   https://download.mono-project.com/sources/gtk-sharp212/gtk-sharp-2.12.45.tar.gz
gtksourceview               3.24.11               08 May 2019   http://ftp.acc.umu.se/pub/gnome/sources/gtksourceview/3.24/gtksourceview-3.24.11.tar.xz
gtksourceview2              3.3.6                 24 Mar 2019   https://rubygems.org/downloads/gtksourceview2-3.3.6.gem
gtksourceview3              3.3.6                 24 Mar 2019   https://rubygems.org/downloads/gtksourceview3-3.3.6.gem
gtksourceviewmm             3.91.1                04 Dec 2018   http://ftp.gnome.org/pub/gnome/sources/gtksourceviewmm/3.91/gtksourceviewmm-3.91.1.tar.xz
gtkspell                    2.0.16                01 Dec 2012   http://gtkspell.sourceforge.net/download/gtkspell-2.0.16.tar.gz
gtkspell3                   3.0.10                03 Jan 2019   https://sourceforge.net/projects/gtkspell/files/3.0.10/gtkspell3-3.0.10.tar.xz
gtkvnc                      0.9.0                 17 Aug 2018   http://ftp.gnome.org/pub/gnome/sources/gtk-vnc/0.9/gtk-vnc-0.9.0.tar.xz
gtkwave                     3.3.98                02 Jan 2019   http://gtkwave.sourceforge.net/gtkwave-3.3.98.tar.gz
gtkxfceengine               3.2.0                 30 Aug 2017   http://archive.xfce.org/src/xfce/gtk-xfce-engine/3.2/gtk-xfce-engine-3.2.0.tar.bz2
gtranslator                 3.32.1                22 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/gtranslator/3.32/gtranslator-3.32.1.tar.xz
gucharmap                   12.0.1                13 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/gucharmap/12.0/gucharmap-12.0.1.tar.xz
guichan                     0.8.2                 01 Jun 2014   http://guichan.googlecode.com/files/guichan-0.8.2.tar.gz
guile                       2.2.6                 02 Jul 2019   http://ftp.gnu.org/pub/gnu/guile/guile-2.2.6.tar.xz
guilib                      1.2.1                 01 Jun 2014   http://www.libsdl.org/projects/GUIlib/src/GUIlib-1.2.1.tar.gz
guiloader                   2.99.0                01 Jun 2014   http://nothing-personal.googlecode.com/files/guiloader-2.99.0.tar.xz
gujin                       2.8.7                 08 Mar 2017   https://sourceforge.net/projects/gujin/files/gujin/2.8.7/gujin-2.8.7.tar.gz
gumboparser                 14.08.2013            01 Aug 2013   gumboparser-14.08.2013.tar.xz
gupnp                       1.2.1                 03 May 2019   http://ftp.gnome.org/pub/gnome/sources/gupnp/1.2/gupnp-1.2.1.tar.xz
gupnpav                     0.12.11               13 Nov 2018   http://ftp.gnome.org/pub/gnome/sources/gupnp-av/0.12/gupnp-av-0.12.11.tar.xz
gupnpdlna                   0.10.5                10 Aug 2017   http://ftp.acc.umu.se/pub/GNOME/sources/gupnp-dlna/0.10/gupnp-dlna-0.10.5.tar.xz
gupnpigd                    0.2.5                 21 Nov 2017   http://ftp.gnome.org/pub/gnome/sources/gupnp-igd/0.2/gupnp-igd-0.2.5.tar.xz
gupnptools                  0.10.0                30 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/gupnp-tools/0.10/gupnp-tools-0.10.0.tar.xz
gutenprint                  5.2.15                17 Jun 2019   https://downloads.sourceforge.net/gimp-print/gutenprint-5.2.15.tar.bz2
guyum                       0.4.1                 01 Jun 2014   http://sourceforge.net/projects/guyum/files/guyum/guyum-0.4/guyum-0.4.1.tar.gz
gv                          3.7.4                 21 Jan 2019   http://ftp.gnu.org/gnu/gv/gv-3.7.4.tar.gz
gvfs                        1.41.3                19 Jun 2019   http://ftp.gnome.org/pub/gnome/sources/gvfs/1.41/gvfs-1.41.3.tar.xz
gvpe                        3.1                   26 Oct 2018   https://ftp.gnu.org/pub/gnu/gvpe/gvpe-3.1.tar.gz
gwenhywfar                  4.20.1                28 Oct 2018   https://github.com/cstim/gwenhywfar/archive/4.20.1.tar.gz
gwenview                    19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/gwenview-19.04.2.tar.xz
gwget                       1.0.4                 01 Jun 2014   http://ftp.gnome.org/pub/gnome/sources/gwget/1.0/gwget-1.0.4.tar.bz2
gwyddion                    2.52                  19 Dec 2018   https://sourceforge.net/projects/gwyddion/files/gwyddion/2.52/gwyddion-2.52.tar.xz
gxine                       0.5.910               13 Jan 2018   http://sourceforge.net/projects/xine/files/gxine/0.5.910/gxine-0.5.910.tar.xz
gxml                        0.18.0                05 Jul 2019   http://ftp.gnome.org/pub/gnome/sources/gxml/0.18/gxml-0.18.0.tar.xz
gzip                        1.10                  30 Dec 2018   ftp://ftp.gnu.org/gnu/gzip/gzip-1.10.tar.gz
hal                         0.5.14                01 Jun 2014   http://hal.freedesktop.org/releases/hal-0.5.14.tar.bz2
haml                        5.1.1                 01 Jun 2019   https://rubygems.org/downloads/haml-5.1.1.gem
hanami                      1.3.1                 18 Feb 2019   https://rubygems.org/downloads/hanami-1.3.1.gem
hanamiassets                1.3.1                 04 May 2019   https://rubygems.org/downloads/hanami-assets-1.3.1.gem
harfbuzz                    2.5.3                 28 Jun 2019   https://www.freedesktop.org/software/harfbuzz/release/harfbuzz-2.5.3.tar.xz
hashie                      3.6.0                 26 Sep 2018   https://rubygems.org/downloads/hashie-3.6.0.gem
haveged                     1.9.3                 03 Nov 2018   http://downloads.sourceforge.net/project/haveged/haveged-1.9.3.tar.gz
hawknl                      1.70                  01 Jun 2014   http://hawksoft.com/download/files/HawkNL17b1src.zip
hd2u                        1.0.3                 01 Jan 2014   http://hany.sk/~hany/_data/hd2u/hd2u-1.0.3.tgz
hdf5                        1.10.4                30 Jan 2019   https://support.hdfgroup.org/ftp/HDF5/releases/hdf5-1.10/hdf5-1.10.4/src/hdf5-1.10.4.tar.gz
hdparm                      9.58                  30 Oct 2018   http://downloads.sourceforge.net/hdparm/hdparm-9.58.tar.gz
heckle                      1.4.3                 30 Nov 2017   https://rubygems.org/downloads/heckle-1.4.3.gem
heimdal                     1.6rc2                22 Sep 2015   http://www.h5l.org/dist/src/heimdal-1.6rc2.tar.gz
hello                       2.10                  09 Jul 2018   http://ftp.gnu.org/gnu/hello/hello-2.10.tar.gz
help2man                    1.47.10               23 Mar 2019   http://ftp.gnu.org/pub/gnu/help2man/help2man-1.47.10.tar.xz
herbstluftwm                0.7.1                 11 Sep 2018   https://www.herbstluftwm.org/tarballs/herbstluftwm-0.7.1.tar.gz
hexahop                     1.1.0                 26 Feb 2017   https://sourceforge.net/projects/hexahop/files/1.1.0/hex-a-hop-1.1.0.tar.gz
hexchat                     2.14.2                30 Aug 2018   https://dl.hexchat.net/hexchat/hexchat-2.14.2.tar.xz
hexedit                     0.9.7                 01 Jun 2014   http://www.rogoyski.com/adam/programs/hexedit/hexedit-0.9.7.tar.gz
hicoloricontheme            0.17                  15 Sep 2017   https://icon-theme.freedesktop.org/releases/hicolor-icon-theme-0.17.tar.xz
hiera                       3.5.0                 22 Dec 2018   https://rubygems.org/downloads/hiera-3.5.0.gem
highlight                   3.52                  13 Jun 2019   http://www.andre-simon.de/zip/highlight-3.52.tar.bz2
highline                    2.0.2                 17 Apr 2019   https://rubygems.org/downloads/highline-2.0.2.gem
highmoon                    1.2.4                 01 Jun 2014   http://highmoon.gerdsmeier.net/highmoon-1.2.4.tar.gz
hiki                        0.8.7                 01 Feb 2013   http://rubyforge.org/frs/?group_id=1444&release_id=4578
hitori                      3.22.4                02 Oct 2017   http://ftp.gnome.org/pub/gnome/sources/hitori/3.22/hitori-3.22.4.tar.xz
hmmer                       3.2.1                 31 Oct 2018   http://eddylab.org/software/hmmer/hmmer-3.2.1.tar.gz
hmount                      0.2.2                 01 Jan 2013   http://sourceforge.net/project/showfiles.php?group_id=186602&package_id=217560
hoe                         3.18.0                01 Jun 2019   https://rubygems.org/downloads/hoe-3.18.0.gem
hornetseyeffmpeg            1.2.5                 10 Oct 2017   https://rubygems.org/downloads/hornetseye-ffmpeg-1.2.5.gem
hotbabe                     0.2.2                 01 Jun 2014   http://dindinx.net/hotbabe/download.php
hplip                       3.19.5                16 May 2019   https://sourceforge.net/projects/hplip/files/hplip/3.19.5/hplip-3.19.5.tar.gz
hpricot                     0.8.6                 17 Oct 2018   https://rubygems.org/downloads/hpricot-0.8.6.gem
hsakmt                      1.0.0                 26 Oct 2015   http://xorg.freedesktop.org/releases/individual/lib/hsakmt-1.0.0.tar.bz2
hspell                      1.4                   31 Dec 2018   http://hspell.ivrix.org.il/hspell-1.4.tar.gz
htop                        2.2.0                 02 Dec 2018   http://hisham.hm/htop/releases/2.2.0/htop-2.2.0.tar.gz
httparty                    0.17.0                20 Apr 2019   https://rubygems.org/downloads/httparty-0.17.0.gem
httpclient                  2.8.3                 13 Nov 2017   https://rubygems.org/downloads/httpclient-2.8.3.gem
httpcookie                  1.0.3                 29 Sep 2017   https://rubygems.org/downloads/http-cookie-1.0.3.gem
httpd                       2.4.39                01 Apr 2019   http://www.apache.org/dist/httpd/httpd-2.4.39.tar.gz
hugin                       2017.0.0              21 Aug 2017   https://sourceforge.net/projects/hugin/files/hugin/hugin-2017.0/hugin-2017.0.0.tar.bz2
humphrey                    24.06.2014            01 Jun 2014   http://retrospec.sgn.net/users/ignacio/downloads/humphrey-24.06.2014
hunspell                    1.7.0                 31 Dec 2018   https://github.com/hunspell/hunspell/archive/v1.7.0.tar.gz
hwd                         4.8.1                 01 Jun 2014   http://user-contributions.org/projects/hwd/src/hwd-4.8.1.tar.gz
hwloc                       1.11.7                28 Dec 2018   https://www.open-mpi.org/software/hwloc/v1.11/downloads/hwloc-1.11.7.tar.gz
hydrogen                    0.9.5.1               01 Nov 2015   http://sourceforge.net/projects/hydrogen/files/Hydrogen/0.9.5%20Sources/hydrogen-0.9.5.1.tar.gz
hyperbole                   7.0.3                 12 May 2019   http://ftp.gnu.org/pub/gnu/hyperbole/hyperbole-7.0.3.tar.gz
hypermammut                 0.0.1                 01 Jun 2014   http://switch.dl.sourceforge.net/sourceforge/hypermammut/hypermammut-0.0.1.tar.bz2
hyphen                      2.8.8                 11 Jun 2015   http://sourceforge.net/projects/hunspell/files/Hyphen/2.8/hyphen-2.8.8.tar.gz
hypothesis                  4.7.7                 21 Feb 2019   https://files.pythonhosted.org/packages/02/52/0b683679bc27200d3fbc51beebf39075b0b28e139e56858b0c7be1b27cd5/hypothesis-4.7.7.tar.gz
i18n                        1.6.0                 24 Mar 2019   https://rubygems.org/downloads/i18n-1.6.0.gem
i3                          4.16.1                10 Feb 2019   https://i3wm.org/downloads/i3-4.16.1.tar.bz2
iagno                       3.32.0                23 May 2019   http://ftp.gnome.org/pub/gnome/sources/iagno/3.32/iagno-3.32.0.tar.xz
ianaetc                     2.20                  01 Jun 2014   http://sethwklein.net/projects/iana-etc/
iat                         0.1.7                 12 Apr 2019   https://sourceforge.net/projects/iat.berlios/files/iat-0.1.7.tar.bz2
ibus                        1.5.20                01 Mar 2019   https://github.com/ibus/ibus/releases/download/1.5.20/ibus-1.5.20.tar.gz
ice                         3.4.2                 01 Jun 2014   http://www.zeroc.com/download/Ice/3.4/Ice-3.4.2.tar.gz
iceauth                     1.0.8                 08 Mar 2018   http://xorg.freedesktop.org/releases/individual/app/iceauth-1.0.8.tar.bz2
icebreaker                  1.9.8                 01 Jun 2014   http://mattdm.org/icebreaker/1.9.x/icebreaker-1.9.8.tgz
icecast                     2.4.4                 02 Nov 2018   http://downloads.xiph.org/releases/icecast/icecast-2.4.4.tar.gz
icedtea                     2.4.7                 01 May 2014   http://icedtea.classpath.org/download/source/icedtea-2.4.7.tar.xz
icewm                       1.5.5                 07 Jun 2019   https://github.com/ice-wm/icewm/releases/download/1.5.5/icewm-1.5.5.tar.xz
icinga                      1.3.1                 01 Jun 2014   https://www.icinga.org/
ico                         1.0.4                 01 Jan 2013   http://xorg.freedesktop.org/releases/individual/app/ico-1.0.4.tar.bz2
iconnamingutils             0.8.90                01 Jun 2014   http://tango.freedesktop.org/releases/icon-naming-utils-0.8.90.tar.bz2
icoutils                    0.31.2                08 Mar 2017   http://savannah.nongnu.org/download/icoutils/icoutils-0.31.2.tar.bz2
icu4c                       64.2                  19 Apr 2019   http://download.icu-project.org/files/icu4c/64.2/icu4c-64_2-src.tgz
id3ed                       1.10.4                01 Jun 2014   http://www.ibiblio.org/pub/Linux/apps/sound/editors/id3ed-1.10.4.tar.gz
id3lib                      3.8.3                 01 May 2014   https://sourceforge.net/projects/id3lib/files/id3lib/3.8.3/id3lib-3.8.3.tar.gz
idesk                       0.7.5                 04 Sep 2015   http://nchc.dl.sourceforge.net/sourceforge/idesk/idesk-0.7.5.tar.bz2
idjc                        0.7.2                 01 Jun 2014   http://worldforum.pardus-linux.nl/index.php?topic=1501.5;wap2
iftop                       0.17                  01 Jun 2014   http://www.ex-parrot.com/pdw/iftop/download/iftop-0.17.tar.gz
ijs                         0.35                  01 Jan 2013   http://www.openprinting.org/download/ijs/download/ijs-0.35.tar.bz2
ilmbase                     2.2.0                 09 Jul 2015   http://download.savannah.nongnu.org/releases/openexr/ilmbase-2.2.0.tar.gz
imageexiftool               11.34                 04 Apr 2019   https://www.sno.phy.queensu.ca/~phil/exiftool/Image-ExifTool-11.34.tar.gz
imagemagick                 7.0.8.45              13 May 2019   ftp://ftp.imagemagick.org/pub/ImageMagick/ImageMagick-7.0.8-45.tar.xz
imageworsener               1.3.0                 02 Dec 2015   http://entropymine.com/imageworsener/imageworsener-1.3.0.tar.gz
imaging                     1.1.6                 01 Jun 2014   http://effbot.org/downloads/Imaging-1.1.6.tar.gz
imake                       1.0.8                 17 Mar 2019   http://xorg.freedesktop.org/releases/individual/util/imake-1.0.8.tar.bz2
imlib                       1.9.15                01 Sep 2013   http://ftp.gnome.org/pub/GNOME/sources/imlib/1.9/imlib-1.9.15.tar.bz2
imlib2                      1.5.1                 26 Mar 2018   https://downloads.sourceforge.net/enlightenment/imlib2-1.5.1.tar.bz2
inch                        0.8.0                 01 May 2018   https://rubygems.org/downloads/inch-0.8.0.gem
incidenceeditor             19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/incidenceeditor-19.04.2.tar.xz
incron                      0.5.10                20 Sep 2018   http://inotify.aiken.cz/download/incron/incron-0.5.10.tar.bz2
indent                      2.2.12                08 Sep 2018   http://mirror.rit.edu/gnu/indent/indent-2.2.12.tar.xz
inetutils                   1.9.4                 18 Oct 2015   http://ftp.gnu.org/gnu/inetutils/inetutils-1.9.4.tar.xz
initdtools                  0.1.3                 24 Jul 2017   http://people.freedesktop.org/~dbn/initd-tools/releases/initd-tools-0.1.3.tar.gz
initng                      0.6.10.2              01 Jun 2014   http://download.initng.org/initng/v0.6/initng-0.6.10.2.tar.bz2
inkscape                    0.92.4                18 Jan 2019   https://inkscape.org/en/gallery/item/12187/inkscape-0.92.4.tar.bz2
inotifytools                3.14                  01 Jun 2014   http://github.com/downloads/rvoicilas/inotify-tools/inotify-tools-3.14.tar.gz
inputproto                  2.3.2                 31 Jul 2016   https://www.x.org/releases/individual/proto/inputproto-2.3.2.tar.bz2
inspec                      4.3.2                 03 May 2019   https://rubygems.org/downloads/inspec-4.3.2.gem
instiki                     0.10.2                01 Jan 2013   https://rubygems.org/downloads/instiki-0.10.2.gem
intelgputools               1.21                  23 Jan 2018   https://www.x.org/releases/individual/app/intel-gpu-tools-1.21.tar.xz
intelvaapidriver            2.3.0                 18 Dec 2018   https://github.com/intel/intel-vaapi-driver/releases/download/2.3.0/intel-vaapi-driver-2.3.0.tar.xz
intltool                    0.51.0                04 Apr 2015   https://launchpad.net/intltool/trunk/0.51.0/+download/intltool-0.51.0.tar.gz
iostring                    1.08                  16 Dec 2018   https://cpan.metacpan.org/authors/id/G/GA/GAAS/IO-String-1.08.tar.gz
iotop                       0.6                   01 Jun 2014   http://guichaz.free.fr/iotop/files/iotop-0.6.tar.gz
iowa                        0.99.2.14             01 Jul 2013   https://github.com/wyhaines/iowa
iproute2                    5.1.0                 15 May 2019   https://www.kernel.org/pub/linux/utils/net/iproute2/iproute2-5.1.0.tar.xz
ipset                       7.2                   13 Jun 2019   http://ipset.netfilter.org/ipset-7.2.tar.bz2
iptables                    1.8.3                 27 May 2019   http://ftp.netfilter.org/pub/iptables/iptables-1.8.3.tar.bz2
iptraf                      3.0.0                 01 Jul 2011   ftp://iptraf.seul.org/pub/iptraf/iptraf-3.0.0.tar.gz
ipython                     6.4.0                 15 May 2018   https://files.pythonhosted.org/packages/ee/01/2a85cd07f5a43fa2e86d60001c213647252662d44a0c2e3d69471a058f1b/ipython-6.4.0.tar.gz
ire                         0.93                  01 Jan 2013   http://ire.sourceforge.net/
irrlicht                    1.8.3                 18 Mar 2016   http://downloads.sourceforge.net/irrlicht/irrlicht-1.8.3.zip
irssi                       1.2.1                 30 Jun 2019   https://github.com/irssi-import/irssi/releases/download/1.2.1/irssi-1.2.1.tar.xz
isbf                        0.1                   01 May 2014   http://freshmeat.net/redir/isbf/66799/url_tgz/isbf-0.1.tar.gz
isl                         0.21                  23 Jun 2019   http://isl.gforge.inria.fr/isl-0.21.tar.xz
isocodes                    4.1                   03 Mar 2019   https://salsa.debian.org/iso-codes-team/iso-codes/uploads/049ce6aac94d842be809f4063950646c/iso-codes-4.1.tar.xz
isomaster                   1.1                   01 Jun 2014   http://littlesvr.ca/isomaster/
ispell                      3.4.00                07 Feb 2016   http://www.cs.hmc.edu/~geoff/tars/ispell-3.4.00.tar.gz
itstool                     2.0.6                 09 Apr 2019   http://files.itstool.org/itstool/itstool-2.0.6.tar.bz2
ivtv                        1.0.2                 01 Jun 2014   http://dl.ivtvdriver.org/ivtv/archive/
iw                          5.0.1                 06 Feb 2019   https://mirrors.edge.kernel.org/pub/software/network/iw/iw-5.0.1.tar.xz
jack2                       17.01.2019            08 Oct 2018   https://github.com/jackaudio/jack2/releases/download/v1.9.12/jack2-1.9.12.tar.gz
jackaudioconnectionkit      0.125.0               15 Nov 2017   http://jackaudio.org/downloads/jack-audio-connection-kit-0.125.0.tar.gz
jailkit                     2.17                  25 Apr 2015   http://olivier.sessink.nl/jailkit/jailkit-2.17.tar.bz2
jam                         1.05                  01 Jun 2014   http://www.disi.unige.it/person/LagorioG/jam/jam105.zip
jansson                     2.12                  12 Jan 2019   http://www.digip.org/jansson/releases/jansson-2.12.tar.gz
jasper                      2.0.16                14 Mar 2019   https://github.com/mdadams/jasper/archive/version-2.0.16.tar.gz
jawavedit                   1.16beta              01 Jun 2014   http://www.bome.com/JaWavedit/JaWavedit.tar.gz
jbuilder                    2.9.1                 01 Jun 2019   https://rubygems.org/downloads/jbuilder-2.9.1.gem
jed                         0.99-19               11 Mar 2017   http://www.jedsoft.org/releases/jed/jed-0.99-19.tar.bz2
jekyll                      3.8.5                 06 Nov 2018   https://rubygems.org/downloads/jekyll-3.8.5.gem
jemalloc                    5.0.1                 27 Nov 2017   https://github.com/jemalloc/jemalloc/releases/download/5.0.1/jemalloc-5.0.1.tar.bz2
jfsutils                    1.1.15                11 Jun 2015   http://jfs.sourceforge.net/project/pub/jfsutils-1.1.15.tar.gz
jhbuild                     3.15.92               19 Mar 2015   http://ftp.gnome.org/pub/GNOME/sources/jhbuild/3.15/jhbuild-3.15.92.tar.xz
jigdo                       0.7.3                 21 Aug 2017   http://atterer.org/sites/atterer/files/2009-08/jigdo/jigdo-0.7.3.tar.bz2
jikes                       1.22                  01 Jun 2014   https://ovh.dl.sourceforge.net/sourceforge/jikes/jikes-1.22.tar.bz2
joe                         4.6                   13 Jan 2018   https://downloads.sourceforge.net/joe-editor/joe-4.6.tar.gz
john                        1.8.0                 18 Mar 2017   http://www.openwall.com/john/j/john-1.8.0.tar.xz
jokosher                    0.10.1                01 Jun 2014   http://edge.launchpad.net/jokosher/0.10/0.10.1/+download/jokosher-0.10.1.tar.gz
jp2a                        1.0.6                 01 Jun 2014   http://sourceforge.net/projects/jp2a/files/jp2a/1.0.6/jp2a-1.0.6.tar.bz2
jpeg                        9c                    09 Dec 2018   http://www.ijg.org/files/jpegsr9c.zip
jpegoptim                   1.4.3                 19 Jun 2015   http://www.kokkonen.net/tjko/src/jpegoptim-1.4.3.tar.gz
jquery                      3.3.1                 03 Aug 2018   https://code.jquery.com/jquery-3.3.1.js
jqueryui                    1.11.4                27 Oct 2015   http://code.jquery.com/jquery-ui-1.11.4.js
jre                         1.8.0_45              01 May 2014   http://www.java.com/en/download/manual.jsp?locale=en
jruby                       1.6.5                 01 Jan 2013   http://jruby.org.s3.amazonaws.com/downloads/1.6.5/jruby-src-1.6.5.tar.gz
json                        2.2.0                 24 Mar 2019   https://rubygems.org/downloads/json-2.2.0.gem
jsonc                       0.13.1                16 Mar 2018   https://s3.amazonaws.com/json-c_releases/releases/json-c-0.13.1.tar.gz
jsoncpp                     19.09.2017            19 Sep 2017   https://github.com/open-source-parsers/jsoncpp/archive/master.zip
jsonglib                    1.4.4                 18 Sep 2018   http://ftp.gnome.org/pub/gnome/sources/json-glib/1.4/json-glib-1.4.4.tar.xz
jsonpure                    2.2.0                 24 Mar 2019   https://rubygems.org/downloads/json_pure-2.2.0.gem
jsonrpcglib                 3.32.0                13 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/jsonrpc-glib/3.32/jsonrpc-glib-3.32.0.tar.xz
jthread                     1.3.1                 01 Jun 2014   http://research.edm.uhasselt.be/jori/jthread/jthread-1.3.1.tar.bz2
judy                        1.0.5                 01 Jun 2014   http://sourceforge.net/projects/judy/files/judy/Judy-1.0.5/Judy-1.0.5.tar.gz
juk                         19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/juk-19.04.2.tar.xz
julia                       1.1.0                 19 Feb 2019   https://github.com/JuliaLang/julia/releases/download/v1.1.0/julia-1.1.0.tar.gz
jumpstart                   0.3                   01 Jun 2014   http://bitwagon.com/jumpstart/jumpstart-0.3.tgz
jwm                         2.2.2                 24 May 2015   http://joewing.net/projects/jwm/releases/jwm-2.2.2.tar.xz
k3b                         19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/k3b-19.04.2.tar.xz
kaccessible                 17.08.3               09 Nov 2017   https://download.kde.org/stable/applications/17.08.3/src/kaccessible-17.08.3.tar.xz
kaccountsintegration        19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kaccounts-integration-19.04.2.tar.xz
kaccountsproviders          19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kaccounts-providers-19.04.2.tar.xz
kactivities                 5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kactivities-5.59.0.tar.xz
kactivitiesstats            5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kactivities-stats-5.59.0.tar.xz
kactivitymanagerd           5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/kactivitymanagerd-5.16.3.tar.xz
kaddressbook                19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kaddressbook-19.04.2.tar.xz
kaffeine                    1.2.2                 01 Jun 2014   http://downloads.sourceforge.net/kaffeine/kaffeine-1.2.2.tar.gz
kajaanikombat               0.7                   01 Jan 2013   http://kombat.kajaani.net/dl/kajaani-kombat-0.7.tar.gz
kajongg                     18.10.2018            06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kajongg-19.04.2.tar.xz
kalarm                      19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kalarm-19.04.2.tar.xz
kalarmcal                   19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kalarmcal-19.04.2.tar.xz
kalgebra                    19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kalgebra-19.04.2.tar.xz
kalzium                     19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kalzium-19.04.2.tar.xz
kamera                      19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kamera-19.04.2.tar.xz
kamikaze                    0.2.2                 01 Jun 2014   http://kamikaze.coolprojects.org/download/kamikaze-0.2.2.tar.gz
kamoso                      19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kamoso-19.04.2.tar.xz
kanagram                    19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kanagram-19.04.2.tar.xz
kapidox                     5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kapidox-5.59.0.tar.xz
kapman                      19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kapman-19.04.2.tar.xz
kapptemplate                19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kapptemplate-19.04.2.tar.xz
karchive                    5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/karchive-5.59.0.tar.xz
karmen                      0.11                  01 Jun 2014   karmen.sourceforge.net
kasablanca                  0.4.0.2               01 Jan 2013   http://download.berlios.de/kasablanca/kasablanca-0.4.0.2.tar.gz
katchtv                     0.91                  01 Jun 2014   http://www.digitalunleashed.com/downloads/katchtv/katchtv_latest_release.tar.bz2
kate                        19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kate-19.04.2.tar.xz
katomic                     19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/katomic-19.04.2.tar.xz
kauth                       5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kauth-5.59.0.tar.xz
kbackup                     19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kbackup-19.04.2.tar.xz
kbd                         2.0.4                 11 Nov 2018   ftp://ftp.altlinux.org/pub/people/legion/kbd/kbd-2.0.4.tar.xz
kbfx                        0.4.9.3.1             01 Jun 2014   https://www.linux-apps.com/p/1081006/
kblackbox                   19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kblackbox-19.04.2.tar.xz
kblocks                     19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kblocks-19.04.2.tar.xz
kblog                       19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kblog-19.04.2.tar.xz
kblogger                    1.0-alpha2            01 Jun 2014   http://kblogger.pwsp.net/files/kblogger-1.0-alpha2.tar.bz2
kbookmarks                  5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kbookmarks-5.59.0.tar.xz
kbounce                     19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kbounce-19.04.2.tar.xz
kbproto                     1.0.7                 04 May 2015   http://xorg.freedesktop.org/releases/individual/proto/kbproto-1.0.7.tar.bz2
kbreakout                   19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kbreakout-19.04.2.tar.xz
kbruch                      19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kbruch-19.04.2.tar.xz
kcachegrind                 19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kcachegrind-19.04.2.tar.xz
kcalc                       19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kcalc-19.04.2.tar.xz
kcalcore                    19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kcalcore-19.04.2.tar.xz
kcalutils                   19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kcalutils-19.04.2.tar.xz
kcharselect                 19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kcharselect-19.04.2.tar.xz
kchmviewer                  7.7                   16 Sep 2017   https://sourceforge.net/projects/kchmviewer/files/kchmviewer/7.7/kchmviewer-7.7.tar.gz
kcmutils                    5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kcmutils-5.59.0.tar.xz
kcodecs                     5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kcodecs-5.59.0.tar.xz
kcolorchooser               19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kcolorchooser-19.04.2.tar.xz
kcompletion                 5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kcompletion-5.59.0.tar.xz
kconfig                     5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kconfig-5.59.0.tar.xz
kconfigwidgets              5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kconfigwidgets-5.59.0.tar.xz
kcontacts                   19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kcontacts-19.04.2.tar.xz
kcoreaddons                 5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kcoreaddons-5.59.0.tar.xz
kcrash                      5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kcrash-5.59.0.tar.xz
kcron                       19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kcron-19.04.2.tar.xz
kdav                        19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kdav-19.04.2.tar.xz
kdbg                        2.1.0                 01 Jun 2014   http://prdownloads.sourceforge.net/kdbg/kdbg-2.1.0.tar.gz
kdbusaddons                 5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kdbusaddons-5.59.0.tar.xz
kdeadmin                    4.3.0                 01 Jun 2014   ftp://gd.tuwien.ac.at/kde/stable/4.3.0/src/kdeadmin-4.3.0.tar.bz2
kdebaseruntime              4.3.90                08 Mar 2015   http://ftp.gwdg.de/pub/x11/kde/stable/4.0.2/src/kdebase-runtime-4.3.90.tar.xz
kdebugsettings              19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kdebugsettings-19.04.2.tar.xz
kdeclarative                5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kdeclarative-5.59.0.tar.xz
kdeclitools                 5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/kde-cli-tools-5.16.3.tar.xz
kdecoration                 5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/kdecoration-5.16.3.tar.xz
kded                        5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kded-5.59.0.tar.xz
kdedevscripts               19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kde-dev-scripts-19.04.2.tar.xz
kdedevutils                 19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kde-dev-utils-19.04.2.tar.xz
kdeedudata                  19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kdeedu-data-19.04.2.tar.xz
kdegraphicsmobipocket       19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kdegraphics-mobipocket-19.04.2.tar.xz
kdegraphicsstrigianalyzer   4.12.97               01 Apr 2014   ftp://gd.tuwien.ac.at/kde/unstable/4.12.97/src/kdegraphics-strigi-analyzer-4.12.97.tar.xz
kdegraphicsthumbnailers     19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kdegraphics-thumbnailers-19.04.2.tar.xz
kdegtkconfig                5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/kde-gtk-config-5.16.3.tar.xz
kdelibs4support             5.58.0                07 Jun 2019   https://download.kde.org/stable/frameworks/5.58/portingAids/kdelibs4support-5.58.0.tar.xz
kdenetworkfilesharing       19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kdenetwork-filesharing-19.04.2.tar.xz
kdenlive                    19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kdenlive-19.04.2.tar.xz
kdepimaddons                19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kdepim-addons-19.04.2.tar.xz
kdepimappslibs              19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kdepim-apps-libs-19.04.2.tar.xz
kdepimruntime               19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kdepim-runtime-19.04.2.tar.xz
kdeplasmaaddons             5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/kdeplasma-addons-5.16.3.tar.xz
kdesdkkioslaves             19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kdesdk-kioslaves-19.04.2.tar.xz
kdesdkthumbnailers          19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kdesdk-thumbnailers-19.04.2.tar.xz
kdesignerplugin             5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kdesignerplugin-5.59.0.tar.xz
kdesu                       5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kdesu-5.59.0.tar.xz
kdesvn                      0.14.6                01 Jun 2014   http://www.alwins-world.de/programs/download/kdesvn/0.14.x/kdesvn-0.14.6.tar.bz2
kdevelop                    5.3.2                 07 Mar 2019   http://download.kde.org/stable/kdevelop/5.3.2/src/kdevelop-5.3.2.tar.xz
kdewebkit                   5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kdewebkit-5.59.0.tar.xz
kdf                         19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kdf-19.04.2.tar.xz
kdialog                     19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kdialog-19.04.2.tar.xz
kdiamond                    19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kdiamond-19.04.2.tar.xz
kdnssd                      5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kdnssd-5.59.0.tar.xz
kdoctools                   5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kdoctools-5.59.0.tar.xz
keditbookmarks              19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/keditbookmarks-19.04.2.tar.xz
kemoticons                  5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kemoticons-5.59.0.tar.xz
kexectools                  2.0.18                31 Oct 2018   https://mirrors.edge.kernel.org/pub/linux/utils/kernel/kexec/kexec-tools-2.0.18.tar.xz
keytouch                    2.4.0beta             01 Jun 2014   http://keytouch.sourceforge.net/keytouch-2.4.0beta.tar.xz
keyutils                    1.6                   25 Nov 2018   http://people.redhat.com/~dhowells/keyutils/keyutils-1.6.tar.bz2
kfilemetadata               5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kfilemetadata-5.59.0.tar.xz
kfind                       19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kfind-19.04.2.tar.xz
kflickr                     20140907              24 Apr 2016   http://sourceforge.net/projects/kflickr/files/kflickr/20140907/kflickr-20140907.tar.bz2
kfloppy                     19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kfloppy-19.04.2.tar.xz
kfourinline                 19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kfourinline-19.04.2.tar.xz
kgamma5                     5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/kgamma5-5.16.3.tar.xz
kgeography                  19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kgeography-19.04.2.tar.xz
kget                        19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kget-19.04.2.tar.xz
kglobalaccel                5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kglobalaccel-5.59.0.tar.xz
kgoldrunner                 19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kgoldrunner-19.04.2.tar.xz
kgpg                        19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kgpg-19.04.2.tar.xz
kgraphviewer                1.99.1                01 Jun 2014   http://download.gna.org/kgraphviewer/kgraphviewer-1.99.1.tar.bz2
kguiaddons                  5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kguiaddons-5.59.0.tar.xz
khangman                    19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/khangman-19.04.2.tar.xz
khard                       0.13.0                24 Dec 2018   https://files.pythonhosted.org/packages/75/17/5b97ce7dd15e835e801f513dd80fb4fee8c6f45f5c5be7c3a2b8b839121a/khard-0.13.0.tar.gz
khelpcenter                 19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/khelpcenter-19.04.2.tar.xz
kholidays                   5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kholidays-5.59.0.tar.xz
khotkeys                    5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/khotkeys-5.16.3.tar.xz
khtml                       5.58.0                07 Jun 2019   https://download.kde.org/stable/frameworks/5.58/portingAids/khtml-5.58.0.tar.xz
ki18n                       5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/ki18n-5.59.0.tar.xz
kiconthemes                 5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kiconthemes-5.59.0.tar.xz
kid                         0.9.6                 17 Dec 2018   https://files.pythonhosted.org/packages/85/59/5256812b4b90cf379be2947142aa384f0b2f739643933ffab66914c79332/kid-0.9.6.tar.gz
kid3                        3.7.0                 10 Jan 2019   https://prdownloads.sourceforge.net/kid3/kid3-3.7.0.tar.gz
kidentitymanagement         19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kidentitymanagement-19.04.2.tar.xz
kidletime                   5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kidletime-5.59.0.tar.xz
kig                         19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kig-19.04.2.tar.xz
kigo                        19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kigo-19.04.2.tar.xz
kildclient                  2.7.0                 01 Jun 2014   http://heanet.dl.sourceforge.net/sourceforge/kildclient/kildclient-2.7.0.tar.gz
kile                        2.9.92                26 Jul 2018   https://sourceforge.net/projects/kile/files/unstable/kile-3.0b2/kile-2.9.92.tar.bz2
killbots                    19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/killbots-19.04.2.tar.xz
kimageformats               5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kimageformats-5.59.0.tar.xz
kimagemapeditor             19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kimagemapeditor-19.04.2.tar.xz
kimap                       19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kimap-19.04.2.tar.xz
kimurai                     1.4.0                 18 Feb 2019   https://rubygems.org/downloads/kimurai-1.4.0.gem
kimwitu++                   2.3.11                26 Sep 2017   https://www2.informatik.hu-berlin.de/sam/kimwitu++/kimwitu++-2.3.11.tar.gz
kinfocenter                 5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/kinfocenter-5.16.3.tar.xz
kinit                       5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kinit-5.59.0.tar.xz
kio                         5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kio-5.59.0.tar.xz
kioextras                   19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kio-extras-19.04.2.tar.xz
kirigami2                   5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kirigami2-5.59.0.tar.xz
kirigamigallery             19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kirigami-gallery-19.04.2.tar.xz
kiriki                      19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kiriki-19.04.2.tar.xz
kismet                      07-R1                 27 Jan 2018   https://www.kismetwireless.net/code/kismet-2016-07-R1.tar.xz
kite                        1.0.4                 01 Jun 2014   http://www.kite-language.org/files/kite-1.0.4.tar.gz
kitemmodels                 5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kitemmodels-5.59.0.tar.xz
kitemviews                  5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kitemviews-5.59.0.tar.xz
kiten                       19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kiten-19.04.2.tar.xz
kitinerary                  19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kitinerary-19.04.2.tar.xz
kiwi                        1.9.29                20 Mar 2015   http://ftp.gnome.org/pub/GNOME/sources/kiwi/1.9/kiwi-1.9.29.tar.xz
kjobwidgets                 5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kjobwidgets-5.59.0.tar.xz
kjs                         5.58.0                07 Jun 2019   https://download.kde.org/stable/frameworks/5.58/portingAids/kjs-5.58.0.tar.xz
kjsembed                    5.58.0                07 Jun 2019   https://download.kde.org/stable/frameworks/5.58/portingAids/kjsembed-5.58.0.tar.xz
kjumpingcube                19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kjumpingcube-19.04.2.tar.xz
klavaro                     3.03                  01 Jun 2014   http://downloads.sourceforge.net/project/klavaro/klavaro-3.03.tar.bz2
kldap                       19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kldap-19.04.2.tar.xz
klear                       0.6.1                 01 Jun 2014   http://www.klear.org/sources/
kleopatra                   19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kleopatra-19.04.2.tar.xz
klettres                    19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/klettres-19.04.2.tar.xz
klickety                    19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/klickety-19.04.2.tar.xz
klineakconfig               0.8beta2              01 Jun 2014   http://nchc.dl.sourceforge.net/sourceforge/lineak/klineakconfig-0.8-beta2.tar.gz
klines                      19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/klines-19.04.2.tar.xz
klininfo                    0.2                   01 Jun 2014   http://freshmeat.net/projects/klininfo/?branch_id=65372&release_id=231523
kmag                        19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kmag-19.04.2.tar.xz
kmahjongg                   19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kmahjongg-19.04.2.tar.xz
kmail                       19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kmail-19.04.2.tar.xz
kmailaccountwizard          19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kmail-account-wizard-19.04.2.tar.xz
kmailtransport              19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kmailtransport-19.04.2.tar.xz
kmarkdownwebview            0.3.0                 01 Nov 2017   https://download.kde.org/stable/kmarkdownwebview/0.3.0/src/kmarkdownwebview-0.3.0.tar.xz
kmbox                       19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kmbox-19.04.2.tar.xz
kmediaplayer                5.58.0                07 Jun 2019   https://download.kde.org/stable/frameworks/5.58/portingAids/kmediaplayer-5.58.0.tar.xz
kmenuedit                   5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/kmenuedit-5.16.3.tar.xz
kmime                       19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kmime-19.04.2.tar.xz
kmines                      19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kmines-19.04.2.tar.xz
kmix                        19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kmix-19.04.2.tar.xz
kmod                        26                    08 Feb 2019   https://www.kernel.org/pub/linux/utils/kernel/kmod/kmod-26.tar.xz
kmousetool                  19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kmousetool-19.04.2.tar.xz
kmouth                      19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kmouth-19.04.2.tar.xz
kmp3burn                    0.3.2                 01 Jun 2014   http://www.martin-keil.de/
kmpg2                       1.95                  01 Jun 2014   http://nchc.dl.sourceforge.net/sourceforge/kmpg2/KmPg2-1.95.tar.bz2
kmplayer                    0.12.0b               15 Jun 2018   http://mirror.freedif.org/KDE/ftp/stable/kmplayer/0.12/kmplayer-0.12.0b.tar.bz2
kmplot                      19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kmplot-19.04.2.tar.xz
kmuddy                      22.03.2016            22 Mar 2016   http://www.kmuddy.com/releases/stable/kmuddy-1.0.1.tar.gz
kmymoney                    5.0.5                 10 Jul 2019   http://download.kde.org/stable/kmymoney/5.0.5/src/kmymoney-5.0.5.tar.xz
knavalbattle                19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/knavalbattle-19.04.2.tar.xz
knetdockapp                 0.8                   01 Jun 2014   http://www.kde-apps.org/content/show.php/KNetDockApp?content=29398
knetwalk                    19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/knetwalk-19.04.2.tar.xz
knewstuff                   5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/knewstuff-5.59.0.tar.xz
knights                     19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/knights-19.04.2.tar.xz
knotes                      19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/knotes-19.04.2.tar.xz
knotifications              5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/knotifications-5.59.0.tar.xz
knotifyconfig               5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/knotifyconfig-5.59.0.tar.xz
kobodeluxe                  0.5.1                 01 Aug 2013   http://www.olofson.net/kobodl/download/KoboDeluxe-0.5.1.tar.bz2
kolf                        19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kolf-19.04.2.tar.xz
kollision                   19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kollision-19.04.2.tar.xz
kolourpaint                 19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kolourpaint-19.04.2.tar.xz
kompare                     19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kompare-19.04.2.tar.xz
konforka                    0.0                   01 Jun 2014   http://kin.klever.net/dist/konforka-0.0.tar.bz2
konqueror                   19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/konqueror-19.04.2.tar.xz
konquest                    19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/konquest-19.04.2.tar.xz
konsole                     19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/konsole-19.04.2.tar.xz
kontact                     19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kontact-19.04.2.tar.xz
kontactinterface            19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kontactinterface-19.04.2.tar.xz
konversation                1.7.5                 17 Nov 2018   http://download.kde.org/download.php?url=stable/konversation/1.7.5/src/konversation-1.7.5.tar.xz
kopete                      19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kopete-19.04.2.tar.xz
korganizer                  19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/korganizer-19.04.2.tar.xz
korundum                    4.14.3                16 Mar 2015   ftp://gd.tuwien.ac.at/kde/stable/4.14.3/src/korundum-4.14.3.tar.xz
kpackage                    5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kpackage-5.59.0.tar.xz
kparts                      5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kparts-5.59.0.tar.xz
kpat                        19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kpat-19.04.2.tar.xz
kpeople                     5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kpeople-5.59.0.tar.xz
kpimtextedit                19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kpimtextedit-19.04.2.tar.xz
kpkpass                     19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kpkpass-19.04.2.tar.xz
kplayer                     0.6.3                 01 Jun 2014   http://heanet.dl.sourceforge.net/sourceforge/kplayer/kplayer-0.6.3.tar.bz2
kplotting                   5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kplotting-5.59.0.tar.xz
kpty                        5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kpty-5.59.0.tar.xz
kqtquickcharts              19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kqtquickcharts-19.04.2.tar.xz
kramdown                    2.1.0                 18 Feb 2019   https://rubygems.org/downloads/kramdown-2.1.0.gem
kraptor                     03APR2004             01 Jun 2014   http://kraptor.sourceforge.net/
krb5                        1.17                  11 Jan 2019   http://web.mit.edu/kerberos/dist/krb5/1.17/krb5-1.17.tar.gz
krb5authdialog              3.26.1                12 Nov 2017   http://ftp.gnome.org/pub/gnome/sources/krb5-auth-dialog/3.26/krb5-auth-dialog-3.26.1.tar.xz
krdc                        19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/krdc-19.04.2.tar.xz
krecord                     1.16                  01 Jun 2014   http://dl.bytesex.org/releases/krecord/
kreversi                    19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kreversi-19.04.2.tar.xz
krfb                        19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/krfb-19.04.2.tar.xz
krita                       4.2.2                 24 Jun 2019   https://download.kde.org/stable/krita/4.2.2/krita-4.2.2.tar.gz
kross                       5.58.0                07 Jun 2019   https://download.kde.org/stable/frameworks/5.58/portingAids/kross-5.58.0.tar.xz
krossinterpreters           19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kross-interpreters-19.04.2.tar.xz
kruler                      19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kruler-19.04.2.tar.xz
krunner                     5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/krunner-5.59.0.tar.xz
krusader                    2.7.1                 12 Aug 2018   http://download.kde.org/stable/krusader/2.7.1/krusader-2.7.1.tar.xz
ksaneplugin                 17.08.3               12 Nov 2017   https://download.kde.org/stable/applications/17.08.3/src/ksaneplugin-17.08.3.tar.xz
kscreen                     5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/kscreen-5.16.3.tar.xz
kscreenlocker               5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/kscreenlocker-5.16.3.tar.xz
kservice                    5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kservice-5.59.0.tar.xz
kshisen                     19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kshisen-19.04.2.tar.xz
kshutdown                   1.0.2                 01 Jun 2014   http://superb-west.dl.sourceforge.net/sourceforge/kshutdown/kshutdown-1.0.2.tar.bz2
ksirk                       19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/ksirk-19.04.2.tar.xz
ksmoothdock                 5.11                  22 Dec 2018   https://github.com/dangvd/ksmoothdock/archive/v5.11.tar.gz
ksmtp                       19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/ksmtp-19.04.2.tar.xz
ksnakeduel                  19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/ksnakeduel-19.04.2.tar.xz
ksniffer                    0.3.2                 01 Jun 2014   http://www.ksniffer.org/
kspaceduel                  19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kspaceduel-19.04.2.tar.xz
ksquares                    19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/ksquares-19.04.2.tar.xz
ksquirrel                   0.8.0                 01 Jun 2014   http://downloads.sourceforge.net/ksquirrel/ksquirrel-0.8.0.tar.bz2
ksquirrellibs               0.7.1                 01 Jun 2014   http://ksquirrel.sourceforge.net/download.php
ksshaskpass                 5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/ksshaskpass-5.16.3.tar.xz
kstars                      2.9.8                 14 Dec 2018   http://mirror.easyname.at/kde/stable/kstars/kstars-2.9.8.tar.xz
ksubtile                    1.3                   01 Jun 2014   http://ksubtile.sourceforge.net/
ksudoku                     19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/ksudoku-19.04.2.tar.xz
ksysguard                   5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/ksysguard-5.16.3.tar.xz
ksystemlog                  19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/ksystemlog-19.04.2.tar.xz
kteatime                    19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kteatime-19.04.2.tar.xz
ktexteditor                 5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/ktexteditor-5.59.0.tar.xz
ktextwidgets                5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/ktextwidgets-5.59.0.tar.xz
ktimer                      19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/ktimer-19.04.2.tar.xz
ktnef                       19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/ktnef-19.04.2.tar.xz
ktorrent                    4.3.1                 01 Jan 2013   http://ktorrent.pwsp.net/downloads/4.3.1/ktorrent-4.3.1.tar.bz2
ktouch                      19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/ktouch-19.04.2.tar.xz
ktpaccountskcm              19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/ktp-accounts-kcm-19.04.2.tar.xz
ktpapprover                 19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/ktp-approver-19.04.2.tar.xz
ktpauthhandler              19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/ktp-auth-handler-19.04.2.tar.xz
ktpcallui                   19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/ktp-call-ui-19.04.2.tar.xz
ktpcommoninternals          19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/ktp-common-internals-19.04.2.tar.xz
ktpcontactlist              19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/ktp-contact-list-19.04.2.tar.xz
ktpcontactrunner            19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/ktp-contact-runner-19.04.2.tar.xz
ktpdesktopapplets           19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/ktp-desktop-applets-19.04.2.tar.xz
ktpfiletransferhandler      19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/ktp-filetransfer-handler-19.04.2.tar.xz
ktpkdedmodule               19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/ktp-kded-module-19.04.2.tar.xz
ktpsendfile                 19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/ktp-send-file-19.04.2.tar.xz
ktptextui                   19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/ktp-text-ui-19.04.2.tar.xz
ktuberling                  19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/ktuberling-19.04.2.tar.xz
kturtle                     19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kturtle-19.04.2.tar.xz
kubrick                     19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kubrick-19.04.2.tar.xz
kuklomenos                  0.4.5                 10 Oct 2017   http://mbays.freeshell.org/kuklomenos/src/kuklomenos-0.4.5.tar.gz
kunitconversion             5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kunitconversion-5.59.0.tar.xz
kuser                       4.14.3                15 Jul 2015   ftp://gd.tuwien.ac.at/kde/stable/4.14.3/src/kuser-4.14.3.tar.xz
kvideoencoder               0.08                  01 Jun 2014   http://kde-apps.org/content/show.php?content=31385
kvkbd                       0.5                   01 Jun 2014   http://www.kde-apps.org/content/show.php/Kvkbd?content=56019
kwalify                     0.7.2                 01 Dec 2017   https://rubygems.org/downloads/kwalify-0.7.2.gem
kwallet                     5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kwallet-5.59.0.tar.xz
kwalletmanager              19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kwalletmanager-19.04.2.tar.xz
kwalletpam                  5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/kwallet-pam-5.16.3.tar.xz
kwave                       19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kwave-19.04.2.tar.xz
kwayland                    5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kwayland-5.59.0.tar.xz
kwaylandintegration         5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/kwayland-integration-5.16.3.tar.xz
kwidgetsaddons              5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kwidgetsaddons-5.59.0.tar.xz
kwifiselector               0.8                   01 Jun 2014   http://sourceforge.net/projects/kwifiselector
kwin                        5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/kwin-5.16.3.tar.xz
kwindowsystem               5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kwindowsystem-5.59.0.tar.xz
kwlan                       0.6.2                 01 Jun 2014   http://www.kde-apps.org/content/show.php?content=37041
kwordquiz                   19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/kwordquiz-19.04.2.tar.xz
kwrited                     5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/kwrited-5.16.3.tar.xz
kxmlgui                     5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kxmlgui-5.59.0.tar.xz
kxmlrpcclient               5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/kxmlrpcclient-5.59.0.tar.xz
kye                         0.9.3                 01 Jun 2014   http://games.moria.org.uk/kye/pygtk
lablgtk                     2.10.0                01 Jun 2014   http://wwwfun.kurims.kyoto-u.ac.jp/soft/olabl/dist/lablgtk-2.10.0.tar.gz
labplot                     2.6.0                 23 Apr 2019   http://mirror.hs-esslingen.de/Mirrors/ftp.kde.org/pub/kde/stable/labplot/2.6.0/labplot-2.6.0.tar.xz
ladspasdk                   1.13                  01 Nov 2012   http://www.ladspa.org/download/ladspa_sdk_1.13.tgz
lame                        3.100                 26 Oct 2017   https://sourceforge.net/projects/lame/files/lame/3.100/lame-3.100.tar.gz
lapack                      3.8.0                 11 May 2019   http://www.netlib.org/lapack/lapack-3.8.0.tar.gz
larswm                      7.5.3                 27 Jul 2017   http://porneia.free.fr/larswm/larswm-7.5.3.tar.gz
lasem                       0.5.1                 24 May 2019   http://ftp.gnome.org/pub/gnome/sources/lasem/0.5/lasem-0.5.1.tar.xz
latexila                    3.27.1                09 Dec 2017   http://ftp.gnome.org/pub/gnome/sources/latexila/3.27/latexila-3.27.1.tar.xz
lbreakout                   010315                01 Dec 2012   http://prdownloads.sourceforge.net/lgames/lbreakout-010315.tar.gz
lbreakout2                  2.6.5                 07 Jun 2018   http://prdownloads.sourceforge.net/lgames/lbreakout2-2.6.5.tar.gz
lbxproxy                    1.0.3                 09 Jun 2018   https://www.x.org/releases/individual/app/lbxproxy-1.0.3.tar.bz2
lcms                        1.19                  01 Jan 2013   http://sourceforge.net/projects/lcms/files/lcms/1.19/lcms-1.19.tar.gz
lcms2                       2.9                   11 Nov 2017   https://sourceforge.net/projects/lcms/files/lcms/2.9/lcms2-2.9.tar.gz
ldmud                       3.5.0                 22 Feb 2017   https://github.com/ldmud/ldmud/tarball/master/ldmud-3.5.0
leafpad                     0.8.17                24 Nov 2017   http://savannah.nongnu.org/download/leafpad/leafpad-0.8.17.tar.gz
leptonica                   1.77.0                13 Jan 2019   http://www.leptonica.com/source/leptonica-1.77.0.tar.gz
less                        551                   11 Jun 2019   http://www.greenwoodsoftware.com/less/less-551.tar.gz
lesstif                     0.95.2                01 Jun 2014   http://sourceforge.net/projects/lesstif/files/lesstif/0.95.2/lesstif-0.95.2.tar.gz
lfhex                       0.4                   01 Jun 2014   http://stoopidsimple.com/files/lfhex-0.4.tar.gz
lftp                        4.8.4                 01 Aug 2018   http://lftp.yar.ru/ftp/lftp-4.8.4.tar.xz
lgeneral                    1.4.3                 07 Jun 2018   http://prdownloads.sourceforge.net/lgeneral/lgeneral-1.4.3.tar.gz
libabigail                  1.5                   30 Oct 2018   http://mirrors.kernel.org/sourceware/libabigail/libabigail-1.5.tar.gz
libao                       1.2.0                 14 Aug 2017   https://ftp.osuosl.org/pub/xiph/releases/ao/libao-1.2.0.tar.gz
libapplewm                  1.3.0                 01 Jun 2014   http://xorg.freedesktop.org/releases/individual/lib/libAppleWM-1.3.0.tar.bz2
libapreq2                   2.08                  01 Jun 2014   http://apache.4any.org/httpd/libapreq/libapreq2-2.08.tar.gz
libarchive                  3.4.0                 18 Jun 2019   https://github.com/libarchive/libarchive/releases/download/v3.4.0/libarchive-3.4.0.tar.gz
libartlgpl                  2.3.21                01 Jun 2014   ftp://ftp.gnome.org/pub/GNOME/sources/libart_lgpl/2.3/libart_lgpl-2.3.21.tar.bz2
libass                      0.14.0                11 Nov 2017   https://github.com/libass/libass/releases/download/0.14.0/libass-0.14.0.tar.xz
libassuan                   2.5.3                 13 Feb 2019   ftp://ftp.gnupg.org/gcrypt/libassuan/libassuan-2.5.3.tar.bz2
libast                      0.7                   09 Jul 2015   http://www.eterm.org/download/libast-0.7.tar.gz
libatasmart                 0.19                  01 Jun 2014   http://0pointer.de/public/libatasmart-0.19.tar.xz
libatomicops                7.6.10                03 Mar 2019   https://github.com/ivmai/libatomic_ops/releases/download/v7.6.10/libatomic_ops-7.6.10.tar.gz
libav                       11.3                  22 May 2015   https://libav.org/releases/libav-11.3.tar.xz
libavc1394                  0.5.4                 01 Jun 2014   https://sourceforge.net/projects/libavc1394/files/libavc1394/libavc1394-0.5.4.tar.gz
libavg                      0.7.0                 01 Jun 2014   http://www.libavg.de/libavg-0.7.0.tar.gz
libbash                     0.9.10                01 Jun 2014   http://surfnet.dl.sourceforge.net/sourceforge/libbash/libbash-0.9.10.tar.bz2
libblockdev                 2.22                  22 Jun 2019   https://github.com/storaged-project/libblockdev/releases/download/2.22-1/libblockdev-2.22.tar.gz
libbluray                   1.1.2                 18 Jun 2019   ftp://ftp.videolan.org/pub/videolan/libbluray/1.1.2/libbluray-1.1.2.tar.bz2
libbonobo                   2.32.1                01 May 2012   http://ftp.gnome.org/pub/GNOME/sources/libbonobo/2.32/libbonobo-2.32.1.tar.bz2
libbonoboui                 2.24.5                01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/libbonoboui/2.24/libbonoboui-2.24.5.tar.bz2
libbpg                      0.9.8                 10 Nov 2018   https://bellard.org/bpg/libbpg-0.9.8.tar.gz
libbtctl                    0.11.1                01 Jun 2014   http://ftp.gnome.org/pub/gnome/sources/libbtctl/0.11/libbtctl-0.11.1.tar.bz2
libburn                     1.5.0                 08 Oct 2018   http://files.libburnia-project.org/releases/libburn-1.5.0.tar.gz
libbytesize                 2.1                   06 Jul 2019   https://github.com/storaged-project/libbytesize/releases/download/2.1/libbytesize-2.1.tar.gz
libcaca                     0.99.beta19           12 Mar 2015   http://caca.zoy.org/files/libcaca/libcaca-0.99.beta19.tar.gz
libcacard                   2.5.2                 17 Dec 2015   http://www.spice-space.org/download/libcacard/libcacard-2.5.2.tar.xz
libcanberra                 0.30                  01 Jan 2013   http://0pointer.de/lennart/projects/libcanberra/libcanberra-0.30.tar.xz
libcap                      2.27                  18 Apr 2019   https://www.kernel.org/pub/linux/libs/security/linux-privs/libcap2/libcap-2.27.tar.xz
libcap2                     2.24                  08 Mar 2015   ftp://ftp.de.debian.org/debian/pool/main/libc/libcap2/libcap2_2.24.orig.tar.xz
libcapng                    0.7.7                 21 May 2015   https://people.redhat.com/sgrubb/libcap-ng/libcap-ng-0.7.7.tar.gz
libcddb                     1.3.2                 01 Jun 2014   https://sourceforge.net/projects/libcddb/files/libcddb/libcddb-1.3.2.tar.bz2
libcdio                     2.1.0                 18 Apr 2019   https://ftp.gnu.org/pub/gnu/libcdio/libcdio-2.1.0.tar.bz2
libcerf                     1.3                   03 Jan 2016   https://sourceforge.net/projects/libcerf/files/libcerf-1.3.tgz
libchamplain                0.12.18               21 Feb 2019   http://ftp.gnome.org/pub/gnome/sources/libchamplain/0.12/libchamplain-0.12.18.tar.xz
libcm                       0.1.1                 01 Jun 2014   http://ftp.acc.umu.se/pub/GNOME/sources/libcm/0.1/libcm-0.1.1.tar.bz2
libconfig                   1.7.2                 08 Apr 2018   https://github.com/hyperrealm/libconfig/archive/v1.7.2.tar.gz
libcroco                    0.6.13                06 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/libcroco/0.6/libcroco-0.6.13.tar.xz
libcryptui                  3.12.2                01 May 2014   http://ftp.acc.umu.se/pub/GNOME/sources/libcryptui/3.12/libcryptui-3.12.2.tar.xz
libdaemon                   0.14                  01 Aug 2013   http://0pointer.de/lennart/projects/libdaemon/libdaemon-0.14.tar.gz
libdap                      3.20.3                19 Apr 2019   https://www.opendap.org/pub/source/libdap-3.20.3.tar.gz
libdazzle                   3.32.2                08 May 2019   http://ftp.gnome.org/pub/gnome/sources/libdazzle/3.32/libdazzle-3.32.2.tar.xz
libdbi                      0.9.0                 04 Jul 2017   https://sourceforge.net/projects/libdbi/files/libdbi/libdbi-0.9.0/libdbi-0.9.0.tar.gz
libdc1394                   2.1.4                 01 Jun 2014   http://sourceforge.net/projects/libdc1394/files/libdc1394-2/libdc1394-2.1.4.tar.gz
libdca                      0.0.5                 01 Jun 2014   http://download.videolan.org/pub/videolan/libdca/0.0.5/libdca-0.0.5.tar.bz2
libdisasm                   0.22                  01 Jun 2014   http://freshmeat.net/redir/libdisasm/72406/url_tgz/libdisasm-0.22.tar.gz
libdiscid                   0.6.1                 14 Apr 2015   http://ftp.musicbrainz.org/pub/musicbrainz/libdiscid/libdiscid-0.6.1.tar.gz
libdivecomputer             0.4.2                 02 Jun 2015   http://www.libdivecomputer.org/releases/libdivecomputer-0.4.2.tar.gz
libdmtx                     0.7.4                 19 Sep 2017   https://sourceforge.net/projects/libdmtx/files/libdmtx/0.7.4/libdmtx-0.7.4.tar.gz
libdmx                      1.1.4                 02 Jun 2018   https://www.x.org/releases/individual/lib/libdmx-1.1.4.tar.bz2
libdnet                     1.12                  01 Jun 2014   http://libdnet.googlecode.com/files/libdnet-1.12.tgz
libdrm                      2.4.99                06 Jul 2019   https://dri.freedesktop.org/libdrm/libdrm-2.4.99.tar.bz2
libdv                       1.0.0                 01 Jun 2014   http://sourceforge.net/projects/libdv/files/libdv/1.0.0/libdv-1.0.0.tar.gz
libdvbpsi                   0.2.0                 01 Jun 2014   http://download.videolan.org/pub/libdvbpsi/0.2.0/libdvbpsi-0.2.0.tar.bz2
libdvdcss                   1.4.1                 27 Mar 2018   https://get.videolan.org/libdvdcss/1.4.1/libdvdcss-1.4.1.tar.bz2
libdvdnav                   6.0.0                 20 Jan 2018   http://download.videolan.org/pub/videolan/libdvdnav/6.0.0/libdvdnav-6.0.0.tar.bz2
libdvdread                  6.0.1                 06 Feb 2019   http://download.videolan.org/videolan/libdvdread/6.0.1/libdvdread-6.0.1.tar.bz2
libebml                     1.3.7                 14 Mar 2019   https://dl.matroska.org/downloads/libebml/libebml-1.3.7.tar.xz
libebt                      1.3.0                 01 Jun 2014   http://libebt.berlios.de/
libedit                     3.1                   11 Sep 2017   http://thrysoee.dk/editline/libedit-20170329-3.1.tar.gz
libelf                      0.8.13                01 Jun 2014   http://www.mr511.de/software/libelf-0.8.13.tar.gz
libepc                      0.4.5                 04 Dec 2018   http://ftp.gnome.org/pub/gnome/sources/libepc/0.4/libepc-0.4.5.tar.xz
libepoxy                    1.5.3                 05 Oct 2018   http://ftp.gnome.org/pub/gnome/sources/libepoxy/1.5/libepoxy-1.5.3.tar.xz
liberror                    2.1.80011             01 Jun 2014   http://www.theiling.de/projects/liberror.html#download
libesmtp                    1.0.6                 01 Jun 2014   http://www.stafford.uklinux.net/libesmtp/libesmtp-1.0.6.tar.bz2
libetc                      0.4                   31 Oct 2017   http://ordiluc.net/fs/libetc/libetc-0.4.tar.gz
libetpan                    1.1                   25 Aug 2016   https://sourceforge.net/projects/libetpan/files/libetpan/1.1/libetpan-1.1.tar.gz
libevdev                    1.7.0                 18 Jun 2019   https://www.freedesktop.org/software/libevdev/libevdev-1.7.0.tar.xz
libevent                    2.1.10                26 May 2019   https://github.com/libevent/libevent/releases/download/release-2.1.10-stable/libevent-2.1.10-stable.tar.gz
libexif                     0.6.21                01 Mar 2013   http://downloads.sourceforge.net/libexif/libexif-0.6.21.tar.bz2
libextractor                1.9                   11 Feb 2019   http://ftp.gnu.org/pub/gnu/libextractor/libextractor-1.9.tar.gz
libfakekey                  0.1                   01 Sep 2014   http://downloads.yoctoproject.org/releases/matchbox/libfakekey/0.1/libfakekey-0.1.tar.bz2
libfame                     0.9.1                 01 Jun 2014   https://puzzle.dl.sourceforge.net/sourceforge/fame/libfame-0.9.1.tar.gz
libferris                   1.5.19                11 Nov 2017   https://sourceforge.net/projects/witme/files/libferris/1.5.19/libferris-1.5.19.tar.xz
libfetch                    2.33                  01 Jun 2014   ftp://ftp.archlinux.org/other/libfetch/libfetch-2.33.tar.gz
libffado                    2.3.0                 13 Apr 2019   http://www.ffado.org/files/libffado-2.3.0.tgz
libffi                      3.2.1                 24 May 2015   ftp://sourceware.org/pub/libffi/libffi-3.2.1.tar.gz
libfirm                     1.22.0                04 Jan 2016   https://github.com/MatzeB/libfirm/archive/libfirm-1.22.0.tar.gz
libfixbuf                   1.5.0                 01 Jul 2014   http://tools.netsa.cert.org/releases/libfixbuf-1.5.0.tar.gz
libflowmanager              3.0.0                 20 May 2018   http://research.wand.net.nz/software/libflowmanager/libflowmanager-2.0.2.tar.gz
libfm                       1.2.5                 11 Nov 2017   https://downloads.sourceforge.net/pcmanfm/libfm-1.2.5.tar.xz
libfmqt                     0.14.1                27 Feb 2019   https://downloads.lxqt.org/downloads/libfm-qt/0.14.1/libfm-qt-0.14.1.tar.xz
libfontenc                  1.1.4                 21 Feb 2019   http://xorg.freedesktop.org/releases/individual/lib/libfontenc-1.1.4.tar.gz
libfreebob                  1.0.11                01 Jun 2014   https://sourceforge.net/projects/freebob/files/libfreebob-1.x/libfreebob-1.0.11/libfreebob-1.0.11.tar.gz
libfs                       1.0.8                 13 Mar 2019   http://xorg.freedesktop.org/releases/individual/lib/libFS-1.0.8.tar.bz2
libft                       0.4                   01 Jun 2014   http://defiant.homedns.org/~erik/ft/libft/files/libft-0.4.tar.gz
libftdi                     0.13                  01 Jun 2014   http://www.intra2net.com/de/produkte/opensource/ftdi/TGZ/libftdi-0.13.tar.gz
libgadu                     1.12.2                19 Sep 2017   http://github.com/wojtekka/libgadu/releases/download/1.12.2/libgadu-1.12.2.tar.gz
libgailgnome                1.20.4                01 Nov 2012   http://ftp.gnome.org/pub/GNOME/sources/libgail-gnome/1.20/libgail-gnome-1.20.4.tar.bz2
libgamessupport             1.0.2                 07 May 2016   http://ftp.gnome.org/pub/GNOME/sources/libgames-support/1.0/libgames-support-1.0.2.tar.xz
libgcrypt                   1.8.4                 30 Oct 2018   ftp://ftp.gnupg.org/gcrypt/libgcrypt/libgcrypt-1.8.4.tar.gz
libgd                       2.2.5                 12 Jan 2018   https://github.com/libgd/libgd/releases/download/gd-2.2.5/libgd-2.2.5.tar.xz
libgda                      5.2.9                 08 May 2019   http://ftp.gnome.org/pub/gnome/sources/libgda/5.2/libgda-5.2.9.tar.xz
libgdamm                    4.99.11               04 Nov 2015   http://ftp.gnome.org/pub/GNOME/sources/libgdamm/4.99/libgdamm-4.99.11.tar.xz
libgdata                    0.17.10               26 Jun 2019   http://ftp.gnome.org/pub/gnome/sources/libgdata/0.17/libgdata-0.17.10.tar.xz
libgdiplus                  2.10.9                01 Jan 2013   http://download.mono-project.com/sources/libgdiplus/libgdiplus-2.10.9.tar.bz2
libgee                      0.20.1                12 Feb 2018   http://ftp.gnome.org/pub/gnome/sources/libgee/0.20/libgee-0.20.1.tar.xz
libgepub                    0.6.0                 02 Oct 2018   http://ftp.gnome.org/pub/gnome/sources/libgepub/0.6/libgepub-0.6.0.tar.xz
libggi                      2.2.2                 01 Jun 2014   http://prdownloads.sourceforge.net/ggi/libggi-2.2.2.src.tar.gz
libghemical                 3.0.0                 01 Jun 2014   http://bioinformatics.org/ghemical/download/current/libghemical-3.0.0.tar.gz
libgig                      3.3.0                 01 Jun 2014   http://download.linuxsampler.org/packages/libgig-3.3.0.tar.bz2
libgii                      1.0.2.html            01 Jun 2014   http://www.ggi-project.org/packages/libgii-1.0.2.html
libgit2                     0.28.1                08 Aug 2017   https://github.com/libgit2/libgit2/archive/v0.28.1.tar.gz
libgit2glib                 0.27.8                13 Feb 2019   http://ftp.gnome.org/pub/gnome/sources/libgit2-glib/0.27/libgit2-glib-0.27.8.tar.xz
libgksu                     2.0.12                27 Dec 2017   http://people.debian.org/~kov/gksu/libgksu-2.0.12.tar.gz
libglade                    2.6.4                 01 Jun 2014   http://ftp.acc.umu.se/pub/GNOME/sources/libglade/2.6/libglade-2.6.4.tar.bz2
libglademm                  2.6.6                 01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/libglademm/2.6/libglademm-2.6.6.tar.bz2
libgnome                    2.32.1                01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/libgnome/2.32/libgnome-2.32.1.tar.bz2
libgnomecanvas              2.30.3                01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/libgnomecanvas/2.30/libgnomecanvas-2.30.3.tar.bz2
libgnomecanvasmm            2.20.0                01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/libgnomecanvasmm/2.20/libgnomecanvasmm-2.20.0.tar.bz2
libgnomecups                0.2.3                 01 Jun 2014   http://ftp.acc.umu.se/pub/gnome/sources/libgnomecups/0.2/libgnomecups-0.2.3.tar.bz2
libgnomedb                  3.99.3                01 Jun 2014   http://www.gnome-db.org/
libgnomegamessupport        1.4.3                 12 Dec 2018   http://ftp.gnome.org/pub/gnome/sources/libgnome-games-support/1.4/libgnome-games-support-1.4.3.tar.xz
libgnomekbd                 3.26.1                07 Feb 2019   http://ftp.gnome.org/pub/gnome/sources/libgnomekbd/3.26/libgnomekbd-3.26.1.tar.xz
libgnomekeyring             3.12.0                10 Sep 2014   http://ftp.gnome.org/pub/GNOME/sources/libgnome-keyring/3.12/libgnome-keyring-3.12.0.tar.xz
libgnomemm                  2.30.0                01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/libgnomemm/2.30/libgnomemm-2.30.0.tar.bz2
libgnomeprintui             2.18.6                01 Jun 2014   http://ftp.acc.umu.se/pub/GNOME/sources/libgnomeprintui/2.18/libgnomeprintui-2.18.6.tar.bz2
libgnomeui                  2.24.5                01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/libgnomeui/2.24/libgnomeui-2.24.5.tar.bz2
libgnomeuimm                2.28.0                01 Jun 2014   http://ftp.acc.umu.se/pub/GNOME/sources/libgnomeuimm/2.28/libgnomeuimm-2.28.0.tar.bz2
libgovirt                   0.3.3                 09 Apr 2015   http://ftp.gnome.org/pub/GNOME/sources/libgovirt/0.3/libgovirt-0.3.3.tar.xz
libgpgerror                 1.36                  20 Mar 2019   ftp://ftp.gnupg.org/gcrypt/libgpg-error/libgpg-error-1.36.tar.gz
libgphoto2                  2.5.23                05 Jun 2019   https://sourceforge.net/projects/gphoto/files/libgphoto/2.5.23/libgphoto2-2.5.23.tar.bz2
libgravatar                 19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/libgravatar-19.04.2.tar.xz
libgrss                     0.7.0                 17 Jul 2015   http://ftp.gnome.org/pub/GNOME/sources/libgrss/0.7/libgrss-0.7.0.tar.xz
libgsasl                    1.7.6                 01 Aug 2013   ftp://alpha.gnu.org/pub/gnu/gsasl/libgsasl-1.7.6.tar.gz
libgsf                      1.14.46               05 May 2019   http://ftp.gnome.org/pub/gnome/sources/libgsf/1.14/libgsf-1.14.46.tar.xz
libgtkhtml                  2.11.1                08 Apr 2009   http://ftp.gnome.org/pub/GNOME/sources/libgtkhtml/2.11/libgtkhtml-2.11.1.tar.bz2
libgtop                     2.40.0                11 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/libgtop/2.40/libgtop-2.40.0.tar.xz
libgudev                    232                   01 Sep 2017   http://ftp.gnome.org/pub/gnome/sources/libgudev/232/libgudev-232.tar.xz
libgusb                     0.3.0                 20 Feb 2018   http://people.freedesktop.org/~hughsient/releases/libgusb-0.3.0.tar.xz
libgweather                 3.32.1                20 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/libgweather/3.32/libgweather-3.32.1.tar.xz
libgxps                     0.3.1                 05 Jan 2019   http://ftp.gnome.org/pub/gnome/sources/libgxps/0.3/libgxps-0.3.1.tar.xz
libhttpseverywhere          0.8.3                 11 Sep 2018   http://ftp.gnome.org/pub/GNOME/sources/libhttpseverywhere/0.8/libhttpseverywhere-0.8.3.tar.xz
libical                     3.0.5                 17 May 2019   https://github.com/libical/libical/releases/download/v3.0.5/libical-3.0.5.tar.gz
libicalglib                 1.0.4                 23 Jan 2016   http://ftp.gnome.org/pub/GNOME/sources/libical-glib/1.0/libical-glib-1.0.4.tar.xz
libice                      1.0.9                 01 Jun 2014   https://www.x.org/releases/individual/lib/libICE-1.0.9.tar.bz2
libiconv                    1.16                  27 Apr 2019   https://ftp.gnu.org/pub/gnu/libiconv/libiconv-1.16.tar.gz
libid3tag                   0.15.1b               01 Jun 2014   https://sourceforge.net/projects/mad/files/libid3tag/0.15.1b/libid3tag-0.15.1b.tar.gz
libidl                      0.8.14                01 Jan 2013   http://ftp.gnome.org/pub/gnome/sources/libIDL/0.8/libIDL-0.8.14.tar.bz2
libidn                      1.35                  26 May 2018   http://ftp.gnu.org/gnu/libidn/libidn-1.35.tar.gz
libidn2                     2.2.0                 24 May 2019   https://ftp.gnu.org/pub/gnu/libidn/libidn2-2.2.0.tar.gz
libinput                    1.13.4                06 Jul 2019   https://www.freedesktop.org/software/libinput/libinput-1.13.4.tar.xz
libiodbc                    3.52.10               15 Jun 2015   http://downloads.sourceforge.net/project/iodbc/iodbc/3.52.10/libiodbc-3.52.10.tar.gz
libisoburn                  1.5.0                 08 Oct 2018   http://files.libburnia-project.org/releases/libisoburn-1.5.0.tar.gz
libisofs                    1.5.0                 08 Oct 2018   http://files.libburnia-project.org/releases/libisofs-1.5.0.tar.gz
libjpeg                     8d                    07 Mar 2015   http://www.ijg.org/files/jpegsrc.v8d.tar.gz
libjpegturbo                2.0.2                 17 Feb 2019   https://downloads.sourceforge.net/libjpeg-turbo/libjpeg-turbo-2.0.2.tar.gz
libkcddb                    19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/libkcddb-19.04.2.tar.xz
libkcompactdisc             19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/libkcompactdisc-19.04.2.tar.xz
libkdcraw                   19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/libkdcraw-19.04.2.tar.xz
libkdegames                 19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/libkdegames-19.04.2.tar.xz
libkdepim                   19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/libkdepim-19.04.2.tar.xz
libkeduvocdocument          19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/libkeduvocdocument-19.04.2.tar.xz
libkexif                    0.2.5                 01 Jun 2014   http://prdownloads.sourceforge.net/kipi/libkexif-0.2.5.tar.bz2
libkexiv2                   19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/libkexiv2-19.04.2.tar.xz
libkface                    17.08.1               01 Jan 2018   https://download.kde.org/stable/applications/17.08.1/src/libkface-17.08.1.tar.xz
libkgapi                    19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/libkgapi-19.04.2.tar.xz
libkgeomap                  19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/libkgeomap-19.04.2.tar.xz
libkipi                     19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/libkipi-19.04.2.tar.xz
libkleo                     19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/libkleo-19.04.2.tar.xz
libkmahjongg                19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/libkmahjongg-19.04.2.tar.xz
libkomparediff2             19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/libkomparediff2-19.04.2.tar.xz
libksane                    19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/libksane-19.04.2.tar.xz
libksba                     1.3.5                 03 Nov 2018   ftp://ftp.gnupg.org/gcrypt/libksba/libksba-1.3.5.tar.bz2
libkscreen                  5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/libkscreen-5.16.3.tar.xz
libksieve                   19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/libksieve-19.04.2.tar.xz
libksysguard                5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/libksysguard-5.16.3.tar.xz
liblbxutil                  1.1.0                 01 Jun 2014   http://xorg.freedesktop.org/releases/individual/lib/liblbxutil-1.1.0.tar.bz2
liblinear                   221                   08 Oct 2018   https://github.com/cjlin1/liblinear/archive/v221/liblinear-221.tar.gz
liblinebreak                2.1                   18 Jan 2018   https://sourceforge.net/projects/vimgadgets/files/liblinebreak/2.1/liblinebreak-2.1.tar.gz
liblo                       0.29                  26 Oct 2017   https://sourceforge.net/projects/liblo/files/liblo-0.29.tar.gz
liblrdf                     0.4.0                 01 Dec 2012   https://sourceforge.net/projects/lrdf/files/liblrdf/0.4.0/liblrdf-0.4.0.tar.gz
liblxqt                     0.14.1                27 Feb 2019   https://downloads.lxqt.org/downloads/liblxqt/0.14.1/liblxqt-0.14.1.tar.xz
libmad                      0.15.1b               01 Sep 2013   ftp://ftp.mars.org/pub/mpeg/libmad-0.15.1b.tar.gz
libmanette                  0.2.2                 11 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/libmanette/0.2/libmanette-0.2.2.tar.xz
libmatekbd                  1.22.0                07 Mar 2019   http://pub.mate-desktop.org/releases/1.22/libmatekbd-1.22.0.tar.xz
libmatemixer                1.22.0                07 Mar 2019   http://pub.mate-desktop.org/releases/1.22/libmatemixer-1.22.0.tar.xz
libmateweather              1.22.0                07 Mar 2019   http://pub.mate-desktop.org/releases/1.22/libmateweather-1.22.0.tar.xz
libmatheval                 1.1.8                 01 Jun 2012   http://ftp.gnu.org/gnu/libmatheval/libmatheval-1.1.8.tar.gz
libmatroska                 1.5.0                 14 Mar 2019   https://dl.matroska.org/downloads/libmatroska/libmatroska-1.5.0.tar.xz
libmbim                     1.18.2                13 Apr 2019   https://www.freedesktop.org/software/libmbim/libmbim-1.18.2.tar.xz
libmcs                      0.7.2                 12 Dec 2015   http://distfiles.atheme.org/libmcs-0.7.2.tgz
libmd                       0.3                   21 Jul 2016   ftp://ftp.penguin.cz/pub/users/mhi/libmd/libmd-0.3.tar.bz2
libmediaart                 1.9.4                 23 Aug 2017   http://ftp.gnome.org/pub/gnome/sources/libmediaart/1.9/libmediaart-1.9.4.tar.xz
libmemcached                0.22                  01 Jun 2014   http://download.tangent.org/libmemcached-0.22.tar.gz
libmicrohttpd               0.9.65                06 Jul 2019   http://ftp.gnu.org/pub/gnu/libmicrohttpd/libmicrohttpd-0.9.65.tar.gz
libmikmod                   3.1.12                01 Jun 2014   http://sourceforge.net/projects/mikmod/files/libmikmod%20%28source%29/3.1.12/libmikmod-3.1.12.tar.gz
libmirage                   1.5.0                 01 Jun 2014   http://downloads.sourceforge.net/cdemu/libmirage-1.5.0.tar.bz2
libmng                      2.0.3                 22 Oct 2015   http://sourceforge.net/projects/libmng/files/libmng-devel/2.0.3/libmng-2.0.3.tar.xz
libmodplug                  0.8.8.5               26 Feb 2017   https://sourceforge.net/projects/modplug-xmms/files/libmodplug/0.8.8.5/libmodplug-0.8.8.5.tar.gz
libmowgli                   2.1.3                 09 Mar 2017   https://github.com/atheme/libmowgli-2/archive/v2.1.3.tar.gz
libmp3splt                  0.7.2                 01 Jun 2014   http://downloads.sourceforge.net/project/mp3splt/libmp3splt/0.7.2/libmp3splt-0.7.2.tar.gz
libmpcdec                   1.2.6                 01 Jun 2014   http://www.musepack.net/index.php?pg=src
libmpeg2                    0.5.1                 01 Feb 2013   http://libmpeg2.sourceforge.net/files/libmpeg2-0.5.1.tar.gz
libmpeg3                    1.8                   01 Feb 2013   http://downloads.sourceforge.net/heroines/libmpeg3-1.8-src.tar.bz2
libmspack                   0.0.20060920alpha     01 Jun 2014   http://www.kyz.uklinux.net/downloads/libmspack-0.0.20040308alpha.tar.gz
libmtp                      1.1.16                30 Oct 2018   https://sourceforge.net/projects/libmtp/files/libmtp/1.1.16/libmtp-1.1.16.tar.gz
libmusicbrainz              5.1.0                 26 Nov 2018   https://github.com/metabrainz/libmusicbrainz/releases/download/release-5.1.0/libmusicbrainz-5.1.0.tar.gz
libmxp                      0.2.2                 01 Jun 2014   http://www.kmuddy.net/libmxp/files/libmxp-0.2.2.tar.gz
libmypaint                  1.3.0                 10 Sep 2017   https://github.com/mypaint/libmypaint/releases/download/v1.3.0/libmypaint-1.3.0.tar.xz
libndp                      1.7                   31 Jul 2018   http://libndp.org/files/libndp-1.7.tar.gz
libnet                      1.1.6                 22 Feb 2019   https://sourceforge.net/projects/libnet-dev/files/libnet-1.1.6.tar.gz
libnftnl                    1.1.3                 30 May 2019   ftp://ftp.netfilter.org/pub/libnftnl/libnftnl-1.1.3.tar.bz2
libnice                     0.1.13                16 Sep 2015   http://nice.freedesktop.org/releases/libnice-0.1.13.tar.gz
libnids                     1.24                  01 May 2013   https://sourceforge.net/projects/libnids/files/libnids/1.24/libnids-1.24.tar.gz
libnih                      1.0.3                 01 Jan 2013   https://launchpad.net/libnih/1.0/1.0.3/+download/libnih-1.0.3.tar.gz
libnl                       3.2.25                19 May 2015   http://www.infradead.org/~tgr/libnl/files/libnl-3.2.25.tar.gz
libnotify                   0.7.8                 06 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/libnotify/0.7/libnotify-0.7.8.tar.xz
libnsl                      1.2.0                 18 Nov 2017   https://github.com/thkukuk/libnsl/archive/v1.2.0/libnsl-1.2.0.tar.gz
libnxml                     0.18.3                31 Oct 2017   http://www.autistici.org/bakunin/libnxml/libnxml-0.18.3.tar.gz
liboauth                    1.0.3                 19 Apr 2015   http://sourceforge.net/projects/liboauth/files/liboauth-1.0.3.tar.gz
libogg                      1.3.3                 11 Nov 2017   http://downloads.xiph.org/releases/ogg/libogg-1.3.3.tar.xz
liboggz                     1.1.1                 01 Jun 2014   http://downloads.xiph.org/releases/liboggz/liboggz-1.1.1.tar.gz
liboil                      0.3.17                23 Aug 2017   https://liboil.freedesktop.org/download/liboil-0.3.17.tar.gz
liboobs                     3.0.0                 20 Mar 2015   http://ftp.gnome.org/pub/GNOME/sources/liboobs/3.0/liboobs-3.0.0.tar.bz2
libopenglrecorder           26.09.2018            26 Sep 2018   http://hisham.hm/htop/releases/2.0.2/htop-2.0.2.tar.gz
libopenmodeller             0.4.2                 20 Sep 2017   http://sourceforge.net/projects/openmodeller/files/openModeller/1.5.0/libopenmodeller-src-1.5.0.zip
libopenraw                  0.0.8                 01 Jun 2014   http://libopenraw.freedesktop.org/download/libopenraw-0.0.8.tar.gz
libopensync                 0.36                  01 Jun 2014   http://www.opensync.org/download/releases/?C=M;O=D
libopusenc                  0.2.1                 20 Oct 2018   https://archive.mozilla.org/pub/opus/libopusenc-0.2.1.tar.gz
libosinfo                   1.5.0                 11 May 2019   http://releases.pagure.org/libosinfo/libosinfo-1.5.0.tar.gz
libosip2                    5.1.0                 29 Mar 2019   http://ftp.gnu.org/gnu/osip/libosip2-5.1.0.tar.gz
libosmium                   2.13.0                16 Aug 2017   https://github.com/osmcode/libosmium/archive/v2.13.0.tar.gz
libpaper                    1.1.23                01 Jun 2014   http://packages.debian.org/unstable/source/libpaper
libpcap                     1.9.0                 03 Aug 2018   http://www.tcpdump.org/release/libpcap-1.9.0.tar.gz
libpciaccess                0.14                  24 Oct 2017   https://www.x.org/releases/individual/lib/libpciaccess-0.14.tar.gz
libpeas                     1.22.0                10 Sep 2017   http://ftp.gnome.org/pub/gnome/sources/libpeas/1.22/libpeas-1.22.0.tar.xz
libpipeline                 1.2.3                 01 May 2013   http://download.savannah.gnu.org/releases/libpipeline/libpipeline-1.2.3.tar.gz
libpng                      1.6.37                17 Apr 2019   https://sourceforge.net/projects/libpng/files/libpng16/1.6.37/libpng-1.6.37.tar.xz
libpod                      1.1.1                 02 Mar 2019   https://github.com/containers/libpod/archive/v1.1.1.tar.gz
libproxy                    0.4.15                23 Sep 2017   https://github.com/libproxy/libproxy/releases/download/0.4.15/libproxy-0.4.15.tar.xz
libpsl                      0.21.0                18 Apr 2019   https://github.com/rockdaboot/libpsl/releases/download/libpsl-0.21.0/libpsl-0.21.0.tar.gz
libpthreadstubs             0.4                   18 Mar 2017   https://xorg.freedesktop.org/releases/individual/xcb/libpthread-stubs-0.4.tar.bz2
libptytty                   1.2                   01 Jun 2014   http://software.schmorp.de/pkg/libptytty
libpwquality                1.4.0                 28 Dec 2018   https://github.com/libpwquality/libpwquality/releases/download/libpwquality-1.4.0/libpwquality-1.4.0.tar.bz2
libqglviewer                2.3.17                01 Jun 2014   http://www.libqglviewer.com/src/libQGLViewer-2.3.17.tar.gz
libqmi                      1.22.4                19 Apr 2019   https://www.freedesktop.org/software/libqmi/libqmi-1.22.4.tar.xz
libqtxdg                    3.3.1                 27 Feb 2019   https://downloads.lxqt.org/downloads/libqtxdg/3.3.1/libqtxdg-3.3.1.tar.xz
libquicktime                1.2.4                 01 Jun 2014   https://sourceforge.net/projects/libquicktime/files/libquicktime/1.2.4/libquicktime-1.2.4.tar.gz
libquvi                     0.9.4                 09 Jun 2016   https://sourceforge.net/projects/quvi/files/0.9/libquvi/libquvi-0.9.4.tar.xz
libquviscripts              0.9.20131130          09 Jun 2016   https://sourceforge.net/projects/quvi/files/0.9/libquvi-scripts/libquvi-scripts-0.9.20131130.tar.xz
libraw                      0.19.2                04 Jan 2019   http://www.libraw.org/data/LibRaw-0.19.2.tar.gz
libraw1394                  2.0.5                 01 Jun 2014   http://www.linux1394.org/dl/libraw1394-2.0.5.tar.gz
libredwg                    0.7                   06 Dec 2018   http://ftp.gnu.org/pub/gnu/libredwg/libredwg-0.7.tar.xz
librejs                     7.18.1                14 Feb 2019   http://ftp.gnu.org/pub/gnu/librejs/librejs-7.18.1.tar.gz
libreoffice                 6.2.3.2               03 May 2019   http://download.documentfoundation.org/libreoffice/src/6.2.3/libreoffice-6.2.3.2.tar.xz
libreofficebinfilter        3.6.4.1               08 Mar 2015   http://download.documentfoundation.org/libreoffice/src/4.2.2/libreoffice-binfilter-3.6.4.1.tar.xz
libreofficecore             3.6.4.3               06 Mar 2014   http://download.documentfoundation.org/libreoffice/src/4.2.2/libreoffice-core-3.6.4.3.tar.xz
libreofficehelp             6.1.0.3               16 Aug 2018   http://download.documentfoundation.org/libreoffice/src/6.1.0/libreoffice-help-6.1.0.3.tar.xz
librep                      0.92.6                25 Jul 2017   http://download.tuxfamily.org/librep/librep_0.92.6.tar.xz
libressl                    2.9.2                 21 May 2019   http://ftp.openbsd.org/pub/OpenBSD/LibreSSL/libressl-2.9.2.tar.gz
librevenge                  0.0.4                 13 Mar 2016   https://sourceforge.net/projects/libwpd/files/librevenge/librevenge-0.0.4/librevenge-0.0.4.tar.xz
librsvg                     2.45.7                03 Jul 2019   http://ftp.gnome.org/pub/gnome/sources/librsvg/2.45/librsvg-2.45.7.tar.xz
libsafe                     2.0-16                01 Jun 2014   http://pubs.research.avayalabs.com/src/libsafe-2.0-16.tgz
libsamplerate               0.1.8                 01 Jun 2014   http://www.mega-nerd.com/SRC/libsamplerate-0.1.8.tar.gz
libseccomp                  2.4.1                 18 Apr 2019   https://github.com/seccomp/libseccomp/releases/download/v2.4.1/libseccomp-2.4.1.tar.gz
libsecret                   0.18.8                02 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/libsecret/0.18/libsecret-0.18.8.tar.xz
libselinux                  2.9                   15 Mar 2019   https://github.com/SELinuxProject/selinux/archive/libselinux-2.9.tar.gz
libsexy                     0.1.11                01 Jun 2014   http://releases.chipx86.com/libsexy/libsexy/libsexy-0.1.11.tar.gz
libshout                    2.3.1                 01 Jun 2014   http://downloads.us.xiph.org/releases/libshout/libshout-2.3.1.tar.gz
libsigc++                   2.10.2                17 Jun 2019   http://ftp.gnome.org/pub/gnome/sources/libsigc++/2.10/libsigc++-2.10.2.tar.xz
libsigsegv                  2.12                  18 Jan 2018   http://ftp.gnu.org/gnu/libsigsegv/libsigsegv-2.12.tar.gz
libslab                     2.30.0                01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/libslab/2.30/libslab-2.30.0.tar.gz
libsm                       1.2.3                 11 Oct 2018   http://xorg.freedesktop.org/releases/individual/lib/libSM-1.2.3.tar.bz2
libsndfile                  1.0.28                28 Dec 2017   http://www.mega-nerd.com/libsndfile/files/libsndfile-1.0.28.tar.gz
libsodium                   1.0.18                01 Jun 2019   https://download.libsodium.org/libsodium/releases/libsodium-1.0.18.tar.gz
libsoup                     2.67.1                24 May 2019   http://ftp.gnome.org/pub/gnome/sources/libsoup/2.67/libsoup-2.67.1.tar.xz
libspectre                  0.2.8                 20 Jan 2019   http://libspectre.freedesktop.org/releases/libspectre-0.2.8.tar.gz
libspnav                    0.2.3                 04 Jun 2016   https://sourceforge.net/projects/spacenav/files/spacenav%20library%20%28SDK%29/libspnav%200.2.3/libspnav-0.2.3.tar.gz
libspng                     0.4.0                 29 Nov 2018   https://gitlab.com/randy408/libspng/uploads/45fc951c6c5c612ed9df9447c47ef9a7/libspng-0.4.0.tar.xz
libsrtp                     2.2                   22 Sep 2018   https://github.com/cisco/libsrtp/archive/v2.2.tar.gz
libssh                      0.9.0                 30 Jun 2019   https://www.libssh.org/files/0.9/libssh-0.9.0.tar.xz
libssh2                     1.9.0                 22 Jun 2019   https://www.libssh2.org/download/libssh2-1.9.0.tar.gz
libstatgrab                 0.91                  09 Aug 2017   http://www.mirrorservice.org/sites/ftp.i-scream.org/pub/i-scream/libstatgrab/libstatgrab-0.91.tar.gz
libstemmer                  24.12.2018            24 Dec 2018   https://github.com/zvelo/libstemmer
libsysstat                  0.4.2                 27 Feb 2019   https://downloads.lxqt.org/downloads/libsysstat/0.4.2/libsysstat-0.4.2.tar.xz
libtar                      1.2.11                01 Jun 2014   ftp://ftp.feep.net/pub/software/libtar/libtar-1.2.11.tar.gz
libtasn1                    4.13                  18 Jan 2018   http://ftp.gnu.org/gnu/libtasn1/libtasn1-4.13.tar.gz
libtheora                   1.1.1                 12 Aug 2017   http://downloads.xiph.org/releases/theora/libtheora-1.1.1.tar.xz
libtirpc                    1.1.4                 03 Sep 2018   https://downloads.sourceforge.net/libtirpc/libtirpc-1.1.4.tar.bz2
libtool                     2.4.6                 18 Feb 2015   ftp://ftp.gnu.org/gnu/libtool/libtool-2.4.6.tar.gz
libtorrent                  0.13.7                30 May 2019   https://github.com/rakshasa/rtorrent/releases/download/v0.9.7/libtorrent-0.13.7.tar.gz
libtorrentrasterbar         1.2.1                 30 May 2019   https://github.com/arvidn/libtorrent/releases/download/libtorrent-1_2_1/libtorrent-rasterbar-1.2.1.tar.gz
libtrace                    3.0.14                01 Jun 2014   http://research.wand.net.nz/software/libtrace/libtrace-3.0.14.tar.bz2
libtranslate                0.99                  01 Jun 2014   http://savannah.nongnu.org/download/libtranslate/libtranslate-0.99.tar.gz
libungif                    4.1.4                 01 Jun 2014   http://sourceforge.net/projects/libungif
libunibreak                 3.0                   30 Aug 2016   https://github.com/adah1972/libunibreak/releases/download/libunibreak_3_0/libunibreak-3.0.tar.gz
libunicap                   0.9.12                30 Dec 2015   http://unicap-imaging.org/downloads/libunicap-0.9.12.tar.gz
libuninum                   2.6                   01 Jun 2014   http://billposer.org/Software/libuninum.html#downloads
libunique                   3.0.2                 01 Aug 2013   http://ftp.acc.umu.se/pub/GNOME/sources/libunique/3.0/libunique-3.0.2.tar.xz
libunistring                0.9.10                29 May 2018   http://ftp.gnu.org/gnu/libunistring/libunistring-0.9.10.tar.xz
libunwind                   1.3.1                 11 Jun 2019   https://download.savannah.gnu.org/releases/libunwind/libunwind-1.3.1.tar.gz
libupnp                     1.6.21                17 Apr 2017   https://sourceforge.net/projects/pupnp/files/pupnp/libUPnP%201.6.21/libupnp-1.6.21.tar.bz2
libusb                      1.0.22                08 Apr 2018   https://github.com//libusb/libusb/releases/download/v1.0.22/libusb-1.0.22.tar.bz2
libusbcompat                0.1.5                 12 Jan 2019   http://downloads.sourceforge.net/libusb/libusb-compat-0.1.5.tar.bz2
libutempter                 1.1.6                 24 Feb 2019   ftp://ftp.altlinux.org/pub/people/ldv/utempter/libutempter-1.1.6.tar.bz2
libuv                       v1.30.0               02 Jul 2019   https://dist.libuv.org/dist/v1.30.0/libuv-v1.30.0.tar.gz
libva                       2.5.0                 07 Jul 2019   https://github.com/intel/libva/releases/download/2.5.0/libva-2.5.0.tar.bz2
libvautils                  2.4.0                 21 Sep 2017   https://www.freedesktop.org/software/vaapi/releases/libva/libva-utils-1.8.2.tar.bz2
libvdpau                    1.2                   03 Mar 2019   https://gitlab.freedesktop.org/vdpau/libvdpau/uploads/14b620084c027d546fa0b3f083b800c6/libvdpau-1.2.tar.bz2
libvdpauvagl                0.3.4                 11 Apr 2015   https://github.com/i-rinat/libvdpau-va-gl/releases/download/v0.3.4/libvdpau-va-gl-0.3.4.tar.gz
libviper                    1.4.6                 01 Jun 2014   http://sourceforge.net/projects/libviper/files/libviper-1.4.6.tar.gz
libvirt                     5.5.0                 03 Jul 2019   ftp://libvirt.org/libvirt/libvirt-5.5.0.tar.xz
libvisio                    0.1.6                 31 Mar 2019   https://dev-www.libreoffice.org/src/libvisio/libvisio-0.1.6.tar.xz
libvisual                   0.4.0                 01 Jan 2013   http://sourceforge.net/projects/libvisual/files/libvisual/libvisual-0.4.0/libvisual-0.4.0.tar.bz2
libvolumeid                 0.81.0                01 Jun 2014   http://www.marcuscom.com/downloads/libvolume_id-0.81.0.tar.bz2
libvorbis                   1.3.6                 17 Mar 2018   http://downloads.xiph.org/releases/vorbis/libvorbis-1.3.6.tar.gz
libvpx                      v1.8.0                06 Feb 2019   https://github.com/webmproject/libvpx/archive/v1.8.0/libvpx-v1.8.0.tar.gz
libwacom                    0.33                  13 Apr 2019   https://github.com/linuxwacom/libwacom/releases/download/libwacom-0.33/libwacom-0.33.tar.bz2
libwebp                     1.0.2                 19 Jan 2019   http://downloads.webmproject.org/releases/webp/libwebp-1.0.2.tar.gz
libwindowswm                1.0.0                 01 Jun 2014   http://ftp.cica.es/pub/X/pub/X11R7.3/src/lib/libWindowsWM-1.0.0.tar.bz2
libwmf                      0.2.8.4               01 Jun 2014   https://sourceforge.net/projects/wvware/files/libwmf/0.2.8.4/libwmf-0.2.8.4.tar.gz
libwnck                     3.32.0                01 May 2019   http://ftp.gnome.org/pub/gnome/sources/libwnck/3.32/libwnck-3.32.0.tar.xz
libwps                      0.4.10                17 Sep 2018   https://sourceforge.net/projects/libwps/files/libwps/libwps-0.4.10/libwps-0.4.10.tar.xz
libwrapiter                 1.2.0                 01 Jun 2014   http://libwrapiter.berlios.de/
libwwwperl                  6.35                  29 Jul 2018   http://search.cpan.org/CPAN/authors/id/O/OA/OALDERS/libwww-perl-6.35.tar.gz
libx11                      1.6.8                 19 Jun 2019   https://www.x.org/releases/individual/lib/libX11-1.6.8.tar.gz
libxau                      1.0.9                 13 Feb 2019   http://xorg.freedesktop.org/releases/individual/lib/libXau-1.0.9.tar.gz
libxaw                      1.0.13                03 May 2015   https://www.x.org/releases/individual/lib/libXaw-1.0.13.tar.bz2
libxaw3d                    1.6.3                 02 Jul 2018   https://www.x.org/releases/individual/lib/libXaw3d-1.6.3.tar.bz2
libxaw3dxft                 1.6.2d                15 Nov 2015   http://sourceforge.net/projects/sf-xpaint/files/libxaw3dxft/libXaw3dXft-1.6.2d.tar.bz2
libxcb                      1.13.1                28 Sep 2018   https://www.x.org/archive/individual/xcb/libxcb-1.13.1.tar.gz
libxcm                      0.5.3                 10 Oct 2015   http://sourceforge.net/projects/oyranos/files/libXcm/libXcm-0.5/libXcm-0.5.3.tar.gz
libxcomposite               0.4.5                 13 Mar 2019   https://www.x.org/releases/individual/lib/libXcomposite-0.4.5.tar.bz2
libxcursor                  1.2.0                 13 Mar 2019   http://xorg.freedesktop.org/releases/individual/lib/libXcursor-1.2.0.tar.bz2
libxdamage                  1.1.5                 11 Mar 2019   http://xorg.freedesktop.org/releases/individual/lib/libXdamage-1.1.5.tar.bz2
libxdmcp                    1.1.3                 17 Mar 2019   https://www.x.org/releases/individual/lib/libXdmcp-1.1.3.tar.gz
libxevie                    1.0.3                 01 Jun 2014   http://xorg.freedesktop.org/releases/individual/lib/libXevie-1.0.3.tar.bz2
libxext                     1.3.4                 17 Mar 2019   https://www.x.org/releases/individual/lib/libXext-1.3.4.tar.gz
libxfce4ui                  4.12.1                30 Aug 2017   http://archive.xfce.org/src/xfce/libxfce4ui/4.12/libxfce4ui-4.12.1.tar.bz2
libxfce4util                4.13.2                07 Feb 2019   http://archive.xfce.org/src/xfce/libxfce4util/4.13/libxfce4util-4.13.2.tar.bz2
libxfixes                   5.0.3                 08 Oct 2016   https://www.x.org/releases/individual/lib/libXfixes-5.0.3.tar.gz
libxfont                    1.5.3                 22 Oct 2017   https://www.x.org/releases/individual/lib/libXfont-1.5.3.tar.gz
libxfont2                   2.0.3                 30 Nov 2017   https://www.x.org/releases/individual/lib/libXfont2-2.0.3.tar.gz
libxfontcache               1.0.4                 01 Jun 2014   https://www.x.org/releases/individual/lib/libXfontcache-1.0.4.tar.bz2
libxft                      2.3.3                 17 Mar 2019   https://www.x.org/releases/individual/lib/libXft-2.3.3.tar.gz
libxi                       1.7.10                20 Jun 2019   https://www.x.org/releases/individual/lib/libXi-1.7.10.tar.gz
libxinerama                 1.1.4                 06 Jul 2018   https://www.x.org/releases/individual/lib/libXinerama-1.1.4.tar.bz2
libxkbcommon                0.8.4                 23 Feb 2019   https://xkbcommon.org/download/libxkbcommon-0.8.4.tar.xz
libxkbfile                  1.1.0                 17 Mar 2019   https://www.x.org/releases/individual/lib/libxkbfile-1.1.0.tar.bz2
libxkbui                    1.0.2                 01 Jun 2014   https://www.x.org/releases/individual/lib/libxkbui-1.0.2.tar.bz2
libxklavier                 5.3                   01 Sep 2013   http://ftp.gnome.org/pub/GNOME/sources/libxklavier/5.3/libxklavier-5.3.tar.xz
libxls                      1.5.1                 15 Apr 2019   https://github.com/libxls/libxls/releases/download/v1.5.1/libxls-1.5.1.tar.gz
libxml++                    3.0.0                 03 Feb 2016   http://ftp.gnome.org/pub/GNOME/sources/libxml++/3.0/libxml++-3.0.0.tar.xz
libxml2                     2.9.9                 05 Jan 2019   http://xmlsoft.org/sources/libxml2-2.9.9.tar.gz
libxml2python               2.6.15                01 Jan 2013   ftp://xmlsoft.org/libxml2/python/libxml2-python-2.6.15.tar.gz
libxmlb                     0.1.7                 08 Mar 2019   https://github.com/hughsie/libxmlb/archive/0.1.7.tar.gz
libxmp                      4.4.1                 11 Sep 2018   https://sourceforge.net/projects/xmp/files/libxmp/4.4.1/libxmp-4.4.1.tar.gz
libxmu                      1.1.3                 17 Mar 2019   https://www.x.org/releases/individual/lib/libXmu-1.1.3.tar.gz
libxo                       1.0.4                 15 May 2019   https://github.com/Juniper/libxo/releases/download/1.0.4/libxo-1.0.4.tar.gz
libxp                       1.0.3                 26 Feb 2015   http://xorg.freedesktop.org/releases/individual/lib/libXp-1.0.3.tar.gz
libxpm                      3.5.12                16 Dec 2016   https://www.x.org/releases/individual/lib/libXpm-3.5.12.tar.bz2
libxpresent                 1.0.0                 14 Apr 2015   http://xorg.freedesktop.org/releases/individual/lib/libXpresent-1.0.0.tar.gz
libxprintapputil            1.0.1                 13 Sep 2018   https://www.x.org/releases/individual/lib/libXprintAppUtil-1.0.1.tar.gz
libxprintutil               1.0.1                 01 Jun 2014   http://xorg.freedesktop.org/releases/individual/lib/libXprintUtil-1.0.1.tar.bz2
libxrandr                   1.5.2                 17 Mar 2019   https://www.x.org/releases/individual/lib/libXrandr-1.5.2.tar.gz
libxrender                  0.9.10                08 Oct 2016   https://www.x.org/releases/individual/lib/libXrender-0.9.10.tar.gz
libxres                     1.2.0                 14 Oct 2017   https://www.x.org/releases/individual/lib/libXres-1.2.0.tar.gz
libxscrnsaver               1.2.3                 06 Jul 2018   https://www.x.org/releases/individual/lib/libXScrnSaver-1.2.3.tar.gz
libxshmfence                1.3                   02 Jun 2018   https://www.x.org/releases/individual/lib/libxshmfence-1.3.tar.gz
libxslt                     1.1.33                05 Jan 2019   http://xmlsoft.org/sources/libxslt-1.1.33.tar.gz
libxspf                     1.2.0                 16 Feb 2017   http://downloads.xiph.org/releases/xspf/libxspf-1.2.0.tar.bz2
libxt                       1.2.0                 23 Jun 2019   http://xorg.freedesktop.org/releases/individual/lib/libXt-1.2.0.tar.gz
libxtrap                    1.0.1                 01 Jun 2014   http://xorg.freedesktop.org/releases/individual/lib/libXTrap-1.0.1.tar.bz2
libxtst                     1.2.3                 08 Oct 2016   https://www.x.org/releases/individual/lib/libXtst-1.2.3.tar.gz
libxv                       1.0.11                08 Oct 2016   https://www.x.org/releases/individual/lib/libXv-1.0.11.tar.gz
libxvmc                     1.0.11                17 Mar 2019   https://www.x.org/releases/individual/lib/libXvMC-1.0.11.tar.gz
libxxf86dga                 1.1.5                 17 Mar 2019   https://www.x.org/releases/individual/lib/libXxf86dga-1.1.5.tar.gz
libxxf86misc                1.0.4                 06 Jul 2018   https://www.x.org/releases/individual/lib/libXxf86misc-1.0.4.tar.bz2
libxxf86vm                  1.1.4                 26 Feb 2015   http://xorg.freedesktop.org/releases/individual/lib/libXxf86vm-1.1.4.tar.gz
libzapojit                  0.0.3                 17 Apr 2015   http://ftp.gnome.org/pub/GNOME/sources/libzapojit/0.0/libzapojit-0.0.3.tar.xz
libzeitgeist                0.3.18                01 Jan 2014   https://launchpad.net/libzeitgeist/0.3/0.3.18/+download/libzeitgeist-0.3.18.tar.gz
libzip                      1.5.2                 14 Mar 2019   https://libzip.org/download/libzip-1.5.2.tar.xz
libzrtpcpp                  2.0.0                 01 Jun 2014   http://ftp.gnu.org/gnu/ccrtp/libzrtpcpp-2.0.0.tar.gz
licq                        1.8.2                 09 Jul 2015   http://downloads.sourceforge.net/licq/licq-1.8.2.tar.bz2
liferea                     1.12.6b               10 Dec 2018   https://github.com/lwindolf/liferea/releases/download/v1.12.6/liferea-1.12.6b.tar.bz2
lightdm                     1.30.0                09 Jul 2019   https://github.com/CanonicalLtd/lightdm/releases/download/1.30.0/lightdm-1.30.0.tar.xz
lightfeather                0.5.5                 01 Jun 2014   http://lightfeather.de/news.php
lightsoff                   3.30.0                24 Sep 2018   http://ftp.gnome.org/pub/gnome/sources/lightsoff/3.30/lightsoff-3.30.0.tar.xz
lighttpd                    1.4.54                28 May 2019   http://download.lighttpd.net/lighttpd/releases-1.4.x/lighttpd-1.4.54.tar.xz
lilo                        24.2                  23 Nov 2015   http://lilo.alioth.debian.org/ftp/sources/lilo-24.2.tar.gz
lilykde                     0.6.1                 01 Jun 2014   http://lilykde.googlecode.com/files/lilykde-0.6.1.tar.gz
lilypond                    2.18.2                16 Jul 2017   http://download.linuxaudio.org/lilypond/sources/v2.18/lilypond-2.18.2.tar.gz
lilyterm                    0.9.9.4               01 Jul 2013   http://lilyterm.luna.com.tw/file/lilyterm-0.9.9.4.tar.gz
linc                        1.0.3                 03 Mar 2015   http://hisham.hm/htop/releases/1.0.3/linc-1.0.3.tar.gz
lincityng                   2.0                   01 Jan 2013   http://download.berlios.de/lincity-ng/lincity-ng-2.0.tar.bz2
lineakconfig                0.3.2                 01 Jun 2014   http://prdownloads.sourceforge.net/lineak/lineakconfig-0.3.2.tar.gz
lineakd                     0.9                   01 May 2014   http://prdownloads.sourceforge.net/lineak/lineakd-0.9.tar.gz
linkgrammar                 5.2.5                 28 Sep 2015   http://www.abiword.org/downloads/link-grammar/5.2.5/link-grammar-5.2.5.tar.gz
links                       2.19                  05 Apr 2019   http://links.twibright.com/download/links-2.19.tar.bz2
linlibertine                2.0.9                 01 Jun 2014   http://sourceforge.net/projects/linuxlibertine
linux                       5.2                   08 Jul 2019   https://www.kernel.org/pub/linux/kernel/v5.x/linux-5.2.tar.xz
linuxlive                   6.2.4                 01 Jun 2014   ftp://ftp.slax.org/Linux-Live/linux-live-6.2.4.tar.gz
linuxpam                    1.3.0                 18 Jul 2017   http://linux-pam.org/library/Linux-PAM-1.3.0.tar.bz2
liquidwar                   5.6.4                 01 Jun 2014   http://www.gnu.org/software/liquidwar6/#downloading
listmoreutils               0.428                 17 Dec 2017   https://www.cpan.org/authors/id/R/RE/REHSACK/List-MoreUtils-0.428.tar.gz
listres                     1.0.4                 06 Dec 2018   https://www.x.org/releases/individual/app/listres-1.0.4.tar.bz2
littleutils                 1.0.37                18 Jul 2017   https://sourceforge.net/projects/littleutils/files/littleutils-source/1.0.37/littleutils-1.0.37.tar.xz
lives                       0.9.8.12              01 Jun 2014   http://www.xs4all.nl/%7Esalsaman/lives/current/LiVES-0.9.8.12.tar.bz2
liveslak                    1.1.9.1               01 Oct 2017   http://bear.alienbase.nl/cgit/liveslak/snapshot/liveslak-1.1.9.1.tar.xz
lksctptools                 1.0.17                14 Oct 2018   https://sourceforge.net/projects/lksctp/files/lksctp-tools/lksctp-tools-1.0.17.tar.gz
llvm                        8.0.0                 27 Mar 2019   http://llvm.org/releases/8.0.0/llvm-8.0.0.src.tar.xz
lmarbles                    1.0.8                 26 Mar 2016   https://sourceforge.net/projects/lgames/files/lmarbles/lmarbles-1.0.8.tar.gz
lmdb                        02.04.2019            04 Jan 2019   https://github.com/LMDB/lmdb/archive/LMDB_0.9.23.tar.gz
lmsensors                   3.4.0                 26 Jul 2017   http://pkgs.fedoraproject.org/repo/pkgs/lm_sensors/lm_sensors-3.4.0.tar.bz2/c03675ae9d43d60322110c679416901a/lm_sensors-3.4.0.tar.bz2
lndir                       1.0.3                 13 Mar 2016   http://xorg.freedesktop.org/releases/individual/util/lndir-1.0.3.tar.gz
log4cpp                     1.1.2rc1              08 Dec 2015   http://sourceforge.net/projects/log4cpp/files/log4cpp-1.1.x%20%28new%29/log4cpp-1.1/log4cpp-1.1.2rc1.tar.gz
log4r                       1.0.5                 01 Mar 2013   http://log4r.sourceforge.net/
logrotate                   3.14.0                26 Aug 2018   https://github.com/logrotate/logrotate/releases/download/3.14.0/logrotate-3.14.0.tar.xz
lokalize                    19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/lokalize-19.04.2.tar.xz
lollypop                    0.9.925               11 Mar 2019   https://gitlab.gnome.org/World/lollypop/uploads/ef81ac8a37493bb94af9fdc63357ab4b/lollypop-0.9.925.tar.xz
longomatch                  1.0.2                 28 Feb 2015   http://ftp.gnome.org/pub/GNOME/sources/longomatch/1.0/longomatch-1.0.2.tar.xz
loofah                      2.2.3                 06 Nov 2018   https://rubygems.org/downloads/loofah-2.2.3.gem
loopaes                     v3.7d                 25 Jul 2015   http://sourceforge.net/projects/loop-aes/files/loop-aes/v3.7d/loop-AES-v3.7d.tar.bz2
lordsawar                   0.2.0                 01 Jun 2014   http://download.savannah.gnu.org/releases-noredirect/lordsawar/lordsawar-0.2.0.tar.gz
loudmouth                   1.5.3                 05 Mar 2016   https://github.com/mcabber/loudmouth/archive/1.5.3.tar.gz
lpairs                      1.0.5                 12 May 2019   http://prdownloads.sourceforge.net/lgames/lpairs-1.0.5.tar.gz
lrzsz                       0.12.20               13 Jul 2017   https://ohse.de/uwe/releases/lrzsz-0.12.20.tar.gz
lsdvd                       0.17                  15 Nov 2015   http://sourceforge.net/projects/lsdvd/files/lsdvd/lsdvd-0.17.tar.gz
lshw                        B.02.18               15 Nov 2018   https://www.ezix.org/software/files/lshw-B.02.18.tar.gz
lshwd                       1.1.3.1               01 Jun 2014   http://user-contributions.org/projects/lshwd/
lskat                       19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/lskat-19.04.2.tar.xz
lsof                        4.89                  31 Oct 2015   ftp://sunsite.ualberta.ca/pub/Mirror/lsof/lsof_4.89.tar.bz2
lsscsi                      0.30                  27 Mar 2019   http://sg.danny.cz/scsi/lsscsi-0.30.tgz
ltris                       1.0.19                02 Jul 2016   http://prdownloads.sourceforge.net/lgames/ltris-1.0.19.tar.gz
lua                         5.3.5                 09 Sep 2018   http://www.lua.org/ftp/lua-5.3.5.tar.gz
luajit                      2.0.5                 09 Oct 2017   http://luajit.org/download/LuaJIT-2.0.5.tar.gz
luarocks                    2.4.1                 20 Oct 2016   http://www.luarocks.org/releases/luarocks-2.4.1.tar.gz
luit                        1.1.1                 01 Nov 2012   http://xorg.freedesktop.org/releases/individual/app/luit-1.1.1.tar.bz2
lumina                      1.5.0                 30 Apr 2019   https://github.com/lumina-desktop/lumina/archive/v1.5.0.tar.gz
lxappearance                0.6.3                 06 Oct 2017   https://sourceforge.net/projects/lxde/files/LXAppearance/lxappearance-0.6.3.tar.xz
lxc                         2.1.1                 28 Dec 2017   https://linuxcontainers.org/downloads/lxc/lxc-2.1.1.tar.gz
lxdecommon                  0.99.2                24 Feb 2017   https://sourceforge.net/projects/lxde/files/lxde-common%20%28default%20config%29/lxde-common%200.99/lxde-common-0.99.2.tar.xz
lxdm                        0.5.3                 24 Feb 2017   https://sourceforge.net/projects/lxde/files/lxdm/lxdm%200.5.3/lxdm-0.5.3.tar.xz
lxdvdrip                    1.60                  01 Jun 2014   http://lxdvdrip.berlios.de/
lximageqt                   0.14.1                27 Feb 2019   https://downloads.lxqt.org/downloads/lximage-qt/0.14.1/lximage-qt-0.14.1.tar.xz
lxml                        4.3.3                 28 Mar 2019   https://files.pythonhosted.org/packages/7d/29/174d70f303016c58bd790c6c86e6e86a9d18239fac314d55a9b7be501943/lxml-4.3.3.tar.gz
lxpanel                     0.10.0                27 Feb 2019   https://sourceforge.net/projects/lxde/files/LXPanel%20%28desktop%20panel%29/LXPanel%200.10.x/lxpanel-0.10.0.tar.xz
lxqtabout                   0.14.1                27 Feb 2019   https://downloads.lxqt.org/downloads/lxqt-about/0.14.1/lxqt-about-0.14.1.tar.xz
lxqtadmin                   0.14.1                27 Feb 2019   https://downloads.lxqt.org/downloads/lxqt-admin/0.14.1/lxqt-admin-0.14.1.tar.xz
lxqtbuildtools              0.6.0                 27 Feb 2019   https://downloads.lxqt.org/downloads/lxqt-build-tools/0.6.0/lxqt-build-tools-0.6.0.tar.xz
lxqtconfig                  0.14.1                27 Feb 2019   https://downloads.lxqt.org/downloads/lxqt-config/0.14.1/lxqt-config-0.14.1.tar.xz
lxqtglobalkeys              0.14.1                27 Feb 2019   https://downloads.lxqt.org/downloads/lxqt-globalkeys/0.14.1/lxqt-globalkeys-0.14.1.tar.xz
lxqtl10n                    0.13.0                17 Jan 2019   https://downloads.lxqt.org/downloads/lxqt-l10n/0.13.0/lxqt-l10n-0.13.0.tar.xz
lxqtnotificationd           0.14.1                27 Feb 2019   https://downloads.lxqt.org/downloads/lxqt-notificationd/0.14.1/lxqt-notificationd-0.14.1.tar.xz
lxqtopensshaskpass          0.14.1                27 Feb 2019   https://downloads.lxqt.org/downloads/lxqt-openssh-askpass/0.14.1/lxqt-openssh-askpass-0.14.1.tar.xz
lxqtpanel                   0.14.1                27 Feb 2019   https://downloads.lxqt.org/downloads/lxqt-panel/0.14.1/lxqt-panel-0.14.1.tar.xz
lxqtpolicykit               0.14.1                27 Feb 2019   https://downloads.lxqt.org/downloads/lxqt-policykit/0.14.1/lxqt-policykit-0.14.1.tar.xz
lxqtpowermanagement         0.14.1                27 Feb 2019   https://downloads.lxqt.org/downloads/lxqt-powermanagement/0.14.1/lxqt-powermanagement-0.14.1.tar.xz
lxqtqtplugin                0.14.0                27 Feb 2019   https://downloads.lxqt.org/downloads/lxqt-qtplugin/0.14.0/lxqt-qtplugin-0.14.0.tar.xz
lxqtrunner                  0.14.1                27 Feb 2019   https://downloads.lxqt.org/downloads/lxqt-runner/0.14.1/lxqt-runner-0.14.1.tar.xz
lxqtsession                 0.14.1                27 Feb 2019   https://downloads.lxqt.org/downloads/lxqt-session/0.14.1/lxqt-session-0.14.1.tar.xz
lxqtsudo                    0.14.1                27 Feb 2019   https://downloads.lxqt.org/downloads/lxqt-sudo/0.14.1/lxqt-sudo-0.14.1.tar.xz
lxqtthemes                  0.14.0                26 Jan 2019   https://downloads.lxqt.org/downloads/lxqt-themes/0.14.0/lxqt-themes-0.14.0.tar.xz
lxsession                   0.5.4                 03 Mar 2019   https://downloads.sourceforge.net/lxde/lxsession-0.5.4.tar.xz
lxterminal                  0.3.2                 08 Oct 2018   https://downloads.sourceforge.net/lxde/lxterminal-0.3.2.tar.xz
lyntin                      4.2                   01 Jun 2014   http://sourceforge.net/projects/lyntin/files/lyntin/4.2/lyntin-4.2.tar.gz
lynx                        2.8.9rel1             26 Jan 2019   http://invisible-mirror.net/archives/lynx/tarballs/lynx2.8.9rel.1.tar.bz2
lyx                         2.3.3                 20 Jun 2019   ftp://ftp.lyx.org/pub/lyx/stable/2.3.x/lyx-2.3.3.tar.xz
lz4                         1.8.3                 05 Oct 2018   https://github.com/lz4/lz4/archive/v1.8.3.tar.gz
lzip                        1.20                  13 Jan 2019   http://download.savannah.gnu.org/releases/lzip/lzip-1.20.tar.lz
lzlib                       1.11                  12 Jan 2019   http://download-mirror.savannah.gnu.org/releases/lzip/lzlib/lzlib-1.11.tar.gz
lzma                        4.32.7                01 Dec 2012   http://tukaani.org/lzma/lzma-4.32.7.tar.bz2
lzo                         2.10                  07 Jan 2018   http://www.oberhumer.com/opensource/lzo/download/lzo-2.10.tar.gz
m17nlib                     1.7.0                 03 Jan 2018   http://download.savannah.gnu.org/releases/m17n/m17n-lib-1.7.0.tar.gz
m2crypto                    0.34.0                01 Jun 2019   https://files.pythonhosted.org/packages/23/93/1c1a887f956ec38d691af110d38059e234fc941019e0c074a66a10846813/M2Crypto-0.34.0.tar.gz
m4                          1.4.18                01 Jan 2017   ftp://ftp.gnu.org/gnu/m4/m4-1.4.18.tar.xz
madwifi                     0.9.4                 01 Jun 2014   http://downloads.sourceforge.net/madwifi/madwifi-0.9.4.tar.bz2
mafft                       7.310                 20 Sep 2017   http://mafft.cbrc.jp/alignment/software/mafft-7.310-with-extensions-src.tgz
mail                        2.7.1                 14 Oct 2018   https://rubygems.org/downloads/mail-2.7.1.gem
mailcommon                  19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/mailcommon-19.04.2.tar.xz
mailimporter                19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/mailimporter-19.04.2.tar.xz
mailman                     2.1.29                09 Sep 2018   https://ftp.gnu.org/pub/gnu/mailman/mailman-2.1.29.tgz
mailutils                   3.6                   23 Feb 2019   http://ftp.gnu.org/pub/gnu/mailutils/mailutils-3.6.tar.xz
make                        4.2.1                 11 Jun 2016   ftp://ftp.gnu.org/gnu/make/make-4.2.1.tar.bz2
makedepend                  1.0.5                 01 Jun 2014   http://xorg.freedesktop.org/releases/individual/util/makedepend-1.0.5.tar.bz2
mako                        1.0.13                09 Jul 2019   https://files.pythonhosted.org/packages/fa/29/8016763284d8fab844224f7cc5675cb4a388ebda0eb5a403260187e48435/Mako-1.0.13.tar.gz
mandb                       2.8.5                 06 Jan 2019   http://download.savannah.nongnu.org/releases/man-db/man-db-2.8.5.tar.xz
mandvd                      2.4                   01 Jun 2014   http://csgib36.ifrance.com/FTP/mandvd-2.4.tar.gz
manpages                    5.01                  09 May 2019   https://www.kernel.org/pub/linux/docs/man-pages/man-pages-5.01.tar.xz
mantisbt                    1.2.5                 08 Mar 2015   http://sourceforge.net/projects/mantisbt/files/mantis-stable/1.2.5/mantisbt-1.2.5.tar.gz
mapacman                    1.00                  01 Jan 2013   http://prdownloads.sourceforge.net/arianne/mapacman-1.00.tar.gz
mapmvideo                   5.32                  01 May 2012   http://sourceforge.net/project/showfiles.php?group_id=215566
marble                      19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/marble-19.04.2.tar.xz
marcel                      0.3.3                 13 Oct 2018   https://rubygems.org/downloads/marcel-0.3.3.gem
marco                       1.22.2                17 Jun 2019   https://pub.mate-desktop.org/releases/1.22/marco-1.22.2.tar.xz
mariadb                     10.4.6                06 Jul 2019   ftp://ftp.osuosl.org/pub/mariadb/mariadb-10.4.6/source/mariadb-10.4.6.tar.gz
markdown                    1.0.1                 01 Jul 2011   http://daringfireball.net/projects/downloads/markdown-1.0.1.tar.bz2
markupsafe                  1.1.0                 17 Dec 2018   https://files.pythonhosted.org/packages/ac/7e/1b4c2e05809a4414ebce0892fe1e32c14ace86ca7d50c70f00979ca9b3a3/MarkupSafe-1.1.0.tar.gz
mars                        0.2.1                 01 Dec 2012   http://www.marsnomercy.org/cms2/
massxpert                   3.6.0                 12 Dec 2015   http://download.tuxfamily.org/massxpert/source/massxpert_3.6.0.tar.gz
mateapplets                 1.22.0                07 Mar 2019   http://pub.mate-desktop.org/releases/1.22/mate-applets-1.22.0.tar.xz
matebackgrounds             1.22.0                07 Mar 2019   http://pub.mate-desktop.org/releases/1.22/mate-backgrounds-1.22.0.tar.xz
matecalc                    1.22.1                07 May 2019   https://pub.mate-desktop.org/releases/1.22/mate-calc-1.22.1.tar.xz
matecommon                  1.22.0                07 Mar 2019   http://pub.mate-desktop.org/releases/1.22/mate-common-1.22.0.tar.xz
matecontrolcenter           1.22.1                24 Apr 2019   https://pub.mate-desktop.org/releases/1.22/mate-control-center-1.22.1.tar.xz
matedesktop                 1.23.0                16 May 2019   https://pub.mate-desktop.org/releases/1.23/mate-desktop-1.23.0.tar.xz
mateicontheme               1.23.0                16 May 2019   https://pub.mate-desktop.org/releases/1.23/mate-icon-theme-1.23.0.tar.xz
mateiconthemefaenza         1.20.0                07 Feb 2018   http://pub.mate-desktop.org/releases/1.20/mate-icon-theme-faenza-1.20.0.tar.xz
mateindicatorapplet         1.22.0                07 Mar 2019   http://pub.mate-desktop.org/releases/1.22/mate-indicator-applet-1.22.0.tar.xz
matemedia                   1.22.1                02 Apr 2019   https://pub.mate-desktop.org/releases/1.22/mate-media-1.22.1.tar.xz
matemenus                   1.22.0                07 Mar 2019   http://pub.mate-desktop.org/releases/1.22/mate-menus-1.22.0.tar.xz
matenetbook                 1.22.1                24 Apr 2019   http://pub.mate-desktop.org/releases/1.22/mate-netbook-1.22.1.tar.xz
matenotificationdaemon      1.22.0                07 Mar 2019   http://pub.mate-desktop.org/releases/1.22/mate-notification-daemon-1.22.0.tar.xz
matepanel                   1.23.0                16 May 2019   https://pub.mate-desktop.org/releases/1.23/mate-panel-1.23.0.tar.xz
matepolkit                  1.22.0                07 Mar 2019   http://pub.mate-desktop.org/releases/1.22/mate-polkit-1.22.0.tar.xz
matepowermanager            1.22.1                24 Apr 2019   http://pub.mate-desktop.org/releases/1.22/mate-power-manager-1.22.1.tar.xz
matescreensaver             1.22.1                24 Apr 2019   https://pub.mate-desktop.org/releases/1.22/mate-screensaver-1.22.1.tar.xz
matesensorsapplet           1.22.1                24 Apr 2019   https://pub.mate-desktop.org/releases/1.22/mate-sensors-applet-1.22.1.tar.xz
matesessionmanager          1.22.1                28 Apr 2019   https://pub.mate-desktop.org/releases/1.22/mate-session-manager-1.22.1.tar.xz
matesettingsdaemon          1.22.0                07 Mar 2019   http://pub.mate-desktop.org/releases/1.22/mate-settings-daemon-1.22.0.tar.xz
matesystemmonitor           1.22.1                24 Apr 2019   http://pub.mate-desktop.org/releases/1.22/mate-system-monitor-1.22.1.tar.xz
mateterminal                1.22.1                24 Apr 2019   https://pub.mate-desktop.org/releases/1.22/mate-terminal-1.22.1.tar.xz
matethemes                  3.22.19               11 Mar 2019   http://pub.mate-desktop.org/releases/themes/3.22/mate-themes-3.22.19.tar.xz
mateuserguide               1.22.0                07 Mar 2019   http://pub.mate-desktop.org/releases/1.22/mate-user-guide-1.22.0.tar.xz
mateusershare               1.22.0                07 Mar 2019   http://pub.mate-desktop.org/releases/1.22/mate-user-share-1.22.0.tar.xz
mateutils                   1.22.1                24 Apr 2019   https://pub.mate-desktop.org/releases/1.22/mate-utils-1.22.1.tar.xz
mathomatic                  15.8.4                01 Jun 2014   http://mathomatic.org/mathomatic-15.8.4.tar.bz2
matplotlib                  3.0.3                 03 Mar 2019   https://files.pythonhosted.org/packages/26/04/8b381d5b166508cc258632b225adbafec49bbe69aa9a4fa1f1b461428313/matplotlib-3.0.3.tar.gz
maxemumtvguide              7.3.2                 01 Jun 2014   http://sourceforge.net/projects/mtvg/files/mtvg/7.3.2/maxemumtvguide-7.3.2.tar.gz
maxima                      5.42.2                28 Apr 2019   https://sourceforge.net/projects/maxima/files/Maxima-source/5.42.2-source/maxima-5.42.2.tar.gz
mboximporter                19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/mbox-importer-19.04.2.tar.xz
mc                          4.8.23                23 Jun 2019   http://www.midnight-commander.org/downloads/mc-4.8.23.tar.bz2
mcabber                     1.1.0                 25 Oct 2018   https://mcabber.com/files/mcabber-1.1.0.tar.bz2
mcelog                      158                   01 Jun 2018   https://github.com/andikleen/mcelog/archive/v158.tar.gz
mcron                       1.1.1                 15 Sep 2018   https://ftp.gnu.org/pub/gnu/mcron/mcron-1.1.1.tar.gz
mcs                         0.4.1                 01 Jun 2014   mcs-0.4.1
mcsim                       6.1.0                 20 Feb 2019   http://ftp.gnu.org/pub/gnu/mcsim/mcsim-6.1.0.tar.gz
mdadm                       4.0                   08 Oct 2018   https://mirrors.edge.kernel.org/pub/linux/utils/raid/mdadm/mdadm-4.0.tar.xz
meanwhile                   1.0.2                 01 Jun 2014   http://meanwhile.sourceforge.net/
mechanize                   2.7.6                 07 Jun 2018   https://rubygems.org/downloads/mechanize-2.7.6.gem
mediatomb                   0.12.1                01 Jun 2014   https://sourceforge.net/projects/mediatomb/files/MediaTomb/0.12.1/mediatomb-0.12.1.tar.gz
mediawiki                   1.25.1                28 May 2015   http://releases.wikimedia.org/mediawiki/1.25/mediawiki-1.25.1.tar.gz
medit                       1.2.0                 09 May 2017   https://sourceforge.net/projects/mooedit/files/medit/1.2.0/medit-1.2.0.tar.bz2
megapov                     1.2.1                 01 Jan 2013   http://megapov.inetart.net/packages/unix/megapov-1.2.1.tar.bz2
megatools                   1.10.2                16 Sep 2018   https://megatools.megous.com/builds/megatools-1.10.2.tar.gz
meld                        3.20.1                30 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/meld/3.20/meld-3.20.1.tar.xz
melons                      1.1.1                 01 Jun 2014   http://www.imitationpickles.org/melons/melons.tgz
meme                        5.0.3                 19 Jan 2019   http://meme-suite.org/meme-software/5.0.3/meme-5.0.3.tar.gz
memtester                   4.3.0                 01 Jun 2014   http://pyropus.ca/software/memtester/old-versions/memtester-4.3.0.tar.gz
menucache                   1.1.0                 11 Nov 2017   https://downloads.sourceforge.net/lxde/menu-cache-1.1.0.tar.xz
merb                        0.9.4                 01 Jun 2014   https://github.com/merb/merb
mercurial                   5.0                   03 May 2019   https://www.mercurial-scm.org/release/mercurial-5.0.tar.gz
mercurycompiler             0.13.1                01 Jun 2014   http://www.mercury.csse.unimelb.edu.au/download.html
mesa                        19.1.2                09 Jul 2019   https://mesa.freedesktop.org/archive/mesa-19.1.2.tar.xz
mesademos                   6.5                   01 Jun 2014   http://mesh.dl.sourceforge.net/sourceforge/mesa3d/MesaDemos-6.5.tar.bz2
mesaglut                    7.10                  01 Jun 2014   ftp://ftp.freedesktop.org/pub/mesa/7.10/MesaGLUT-7.10.tar.bz2
mesk                        0.1.0                 01 Jun 2014   http://mesk.nicfit.net/releases/mesk-0.1.0.tgz
meson                       0.51.0                19 Jun 2019   https://files.pythonhosted.org/packages/0b/93/ce7dc081e742d4b87e2d591e709f427992cdefdf0bac3f7638cb4f2bb14b/meson-0.51.0.tar.gz
messagelib                  19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/messagelib-19.04.2.tar.xz
metacity                    3.32.0                05 May 2019   http://ftp.gnome.org/pub/gnome/sources/metacity/3.32/metacity-3.32.0.tar.xz
metaclass                   0.0.4                 04 Aug 2018   https://rubygems.org/downloads/metaclass-0.0.4.gem
methodsource                0.9.2                 17 Nov 2018   https://rubygems.org/downloads/method_source-0.9.2.gem
mftrace                     1.2.14                01 Jun 2013   http://www.xs4all.nl/~hanwen/mftrace/mftrace-1.2.14.tar.gz
mgaview                     0.0.30                01 Jun 2014   http://sourceforge.net/projects/mgaview/mgaview-0.0.30
midilib                     2.0.5                 16 Jun 2015   https://rubygems.org/downloads/midilib-2.0.5.gem
midillo                     0.0                   01 Jun 2014   http://kin.klever.net/konforka/
midori                      0.5.1                 01 Feb 2012   http://archive.xfce.org/src/apps/midori/0.5/midori-0.5.1.tar.bz2
milkytracker                0.90.85               01 Jun 2014   http://milkytracker.org/files/milkytracker-0.90.85.tar.bz2
milou                       5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/milou-5.16.3.tar.xz
mimetypes                   3.2.2                 26 Sep 2018   https://rubygems.org/downloads/mime-types-3.2.2.gem
mimetypesdata               3.2019.0331           17 Apr 2019   https://rubygems.org/downloads/mime-types-data-3.2019.0331.gem
ming                        0.4.0rc1              01 Jun 2014   http://sourceforge.net/project/showfiles.php?group_id=18365&package_id=187304&release_id=611489
mingetty                    1.07                  01 Jun 2014   http://ftp.de.debian.org/debian/pool/main/m/mingetty/mingetty-1.07.orig.tar.gz
minimagick                  4.9.3                 24 Mar 2019   https://rubygems.org/downloads/mini_magick-4.9.3.gem
minimime                    1.0.1                 26 Sep 2018   https://rubygems.org/downloads/mini_mime-1.0.1.gem
miniportile2                2.4.0                 15 Dec 2018   https://rubygems.org/downloads/mini_portile2-2.4.0.gem
minit                       0.10                  03 Mar 2015   http://hisham.hm/htop/releases/1.0.3/minit-0.10.tar.xz
minitar                     0.8                   18 Feb 2019   https://rubygems.org/downloads/minitar-0.8.gem
minitest                    5.11.3                27 Jan 2018   https://rubygems.org/downloads/minitest-5.11.3.gem
minitestsprint              1.2.0                 03 Jan 2019   https://rubygems.org/downloads/minitest-sprint-1.2.0.gem
minuet                      19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/minuet-19.04.2.tar.xz
mirage                      0.8                   01 Jun 2014   http://raschix.de/article/mirage-image-viewer
missile                     1.0.1                 01 Jun 2014   http://missile.sourceforge.net/dl/missile-1.0.1.tar.gz
mjpegtools                  1.8.0                 01 Jun 2014   http://citkit.dl.sourceforge.net/sourceforge/mjpeg/mjpegtools-1.8.0.tar.gz
mkcfm                       1.0.1                 01 Jun 2014   http://xorg.freedesktop.org/releases/individual/app/mkcfm-1.0.1.tar.bz2
mkcomposecache              1.2.1                 01 Jun 2014   http://xorg.freedesktop.org/releases/individual/app/mkcomposecache-1.2.1.tar.bz2
mkfontdir                   1.0.7                 01 Jun 2014   http://xorg.freedesktop.org/releases/individual/app/mkfontdir-1.0.7.tar.bz2
mkfontscale                 1.2.1                 22 Mar 2019   http://xorg.freedesktop.org/archive/individual/app/mkfontscale-1.2.1.tar.bz2
mkrf                        0.2.3                 29 Nov 2017   https://rubygems.org/downloads/mkrf-0.2.3.gem
mktemp                      1.7                   01 Dec 2012   ftp://ftp.mktemp.org/pub/mktemp/mktemp-1.7.tar.gz
mkvtoolnix                  30.1.0                05 Jan 2019   https://mkvtoolnix.download/sources/mkvtoolnix-30.1.0.tar.xz
mlr                         4.4.0                 14 Aug 2016   https://github.com/johnkerl/miller/releases/download/v4.4.0/mlr-4.4.0.tar.gz
mlt                         6.16.0                09 May 2019   https://github.com/mltframework/mlt/releases/download/v6.16.0/mlt-6.16.0.tar.gz
mmcommon                    0.9.12                08 Apr 2018   http://ftp.gnome.org/pub/gnome/sources/mm-common/0.9/mm-common-0.9.12.tar.xz
mobs                        2.3.2                 01 Jun 2014   http://www.dervishd.net/Projects/mobs-2.3.2.tar.gz
moc                         2.5.2                 09 Jun 2017   http://ftp.daper.net/pub/soft/moc/stable/moc-2.5.2.tar.bz2
mocha                       1.8.0                 18 Feb 2019   https://rubygems.org/downloads/mocha-1.8.0.gem
mochikit                    1.3.1                 01 Jun 2014   http://mochikit.com/
modemmanager                1.8.2                 12 Oct 2018   https://www.freedesktop.org/software/ModemManager/ModemManager-1.8.2.tar.xz
modemmanagerqt              5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/modemmanager-qt-5.59.0.tar.xz
modfastcgi                  2.4.2                 01 Jun 2014   http://freshmeat.net/redir/mod_fastcgi/6562/url_tgz/mod_fastcgi-2.4.2.tar.gz
modmono                     1.2.5                 01 Jun 2014   http://go-mono.com/sources/mod_mono/mod_mono-1.2.5.tar.bz2
modperl                     2.0.10                28 Oct 2016   http://www-eu.apache.org/dist/perl/mod_perl-2.0.10.tar.gz
modpython                   3.5.0                 31 Oct 2017   http://dist.modpython.org/dist/mod_python-3.5.0.tgz
modspin                     1.1.6                 01 Jun 2014   http://freecode.com/projects/mod_spin/releases
modulebuild                 0.4224                13 Oct 2018   https://cpan.metacpan.org/authors/id/L/LE/LEONT/Module-Build-0.4224.tar.gz
moduleinittools             3.16                  01 Jun 2014   ftp://ftp.kernel.org/pub/linux/utils/kernel/module-init-tools/module-init-tools-3.16.tar.bz2
modules                     3.2.10                27 Aug 2015   http://sourceforge.net/projects/modules/files/Modules/modules-3.2.10/modules-3.2.10.tar.bz2
moe                         1.10                  02 Jan 2019   https://ftp.gnu.org/pub/gnu/moe/moe-1.10.tar.lz
mojolicious                 2.0                   01 Jan 2013   http://cpan.metacpan.org/authors/id/S/SR/SRI/Mojolicious-2.0.tar.gz
momomoto                    0.2.1                 01 Dec 2017   https://rubygems.org/downloads/momomoto-0.2.1.gem
moneta                      1.1.1                 17 Apr 2019   https://rubygems.org/downloads/moneta-1.1.1.gem
money                       6.13.3                17 Apr 2019   https://rubygems.org/downloads/money-6.13.3.gem
mongodb                     r3.4.6                20 Apr 2015   https://fastdl.mongodb.org/src/mongodb-src-r3.4.6.tar.gz
mongrel                     1.1.5                 01 Aug 2017   https://rubygems.org/downloads/mongrel-1.1.5.gem
mongrel2                    v1.10.0               22 Dec 2015   https://github.com/mongrel2/mongrel2/releases/download/v1.10.0/mongrel2-v1.10.0.tar.bz2
monit                       4.10                  01 Jun 2014   http://www.tildeslash.com/monit/dist/monit-4.10.tar.gz
mono                        6.5.0.356             30 Jun 2019   http://download.mono-project.com/sources/mono/nightly/mono-6.5.0.356.tar.xz
monoaddins                  0.4                   01 Jun 2014   http://ftp.novell.com/pub/mono/sources/mono-addins/mono-addins-0.4.tar.bz2
monodevelop                 6.0.0.4761            28 Mar 2016   http://download.mono-project.com/sources/monodevelop/monodevelop-6.0.0.4761.tar.bz2
monsterz                    0.7.1                 10 Oct 2017   http://sam.zoy.org/monsterz/monsterz-0.7.1.tar.gz
moodle                      3.4                   08 Dec 2017   https://sourceforge.net/projects/moodle/files/Moodle/stable34/moodle-3.4.tgz
moonbuggy                   1.0.51                24 Mar 2016   http://m.seehuhn.de/programs/moon-buggy-1.0.51.tar.gz
moserial                    3.0.12                08 Jan 2018   http://ftp.gnome.org/pub/gnome/sources/moserial/3.0/moserial-3.0.12.tar.xz
mosh                        1.3.2                 07 Apr 2019   https://mosh.org/mosh-1.3.2.tar.gz
most                        5.0.0                 01 Jun 2014   ftp://space.mit.edu/pub/davis/most/most-5.0.0a.tar.bz2
motif                       2.3.8                 09 Dec 2017   https://sourceforge.net/projects/motif/files/Motif%202.3.8%20Source%20Code/motif-2.3.8.tar.gz
mousepad                    0.4.1                 25 Aug 2018   http://archive.xfce.org/src/apps/mousepad/0.4/mousepad-0.4.1.tar.bz2
mousetrap                   3.17.3                23 Jun 2015   http://ftp.gnome.org/pub/GNOME/sources/mousetrap/3.17/mousetrap-3.17.3.tar.xz
mozart2                     13.12.2015            13 Dec 2015   http://www.mozart-oz.org/download/mozart-ftp/store/1.3.1-2004-06-16/mozart2-13.12.2015.tar.xz
mozjs                       60.1.0                09 Sep 2018   http://ftp.gnome.org/pub/gnome/teams/releng/tarballs-needing-help/mozjs/mozjs-60.1.0.tar.bz2
mozo                        1.22.1                24 Apr 2019   https://pub.mate-desktop.org/releases/1.22/mozo-1.22.1.tar.xz
mp                          5.2.2                 01 Jun 2014   http://triptico.com/download/mp-5.2.2.tar.gz
mp3blaster                  3.2.6                 17 Nov 2017   https://sourceforge.net/projects/mp3blaster/files/mp3blaster/mp3blaster-3.2.6/mp3blaster-3.2.6.tar.gz
mp3cat                      0.4                   01 Jun 2014   http://tomclegg.net/software/mp3cat-0.4.tar.gz
mp3daemon                   0.63                  01 May 2014   http://search.cpan.org/CPAN/authors/id/B/BE/BEPPU/MP3-Daemon-0.63.tar.gz
mp3diags                    1.0.12.079            01 Jan 2013   http://sourceforge.net/projects/mp3diags/files/mp3diags-src/MP3Diags-1.0.12.079.tar.gz
mp3gain                     1.4.6                 01 Jun 2014   http://mp3gain.sourceforge.net/download.php
mp3info                     0.8.5                 01 Jun 2014   ftp://ftp.ibiblio.org/pub/linux/apps/sound/mp3-utils/mp3info/mp3info-0.8.5.tgz
mp3splt                     2.4.2                 01 Jun 2014   http://downloads.sourceforge.net/project/mp3splt/mp3splt/2.4.2/mp3splt-2.4.2.tar.gz
mp3val                      0.1.8                 01 Jan 2013   http://downloads.sourceforge.net/mp3val/mp3val-0.1.8-src.tar.gz
mp3wrap                     0.5                   01 Jun 2014   http://prdownloads.sourceforge.net/mp3wrap/mp3wrap-0.5-src.tar.gz
mpage                       2.5.6                 07 Feb 2016   http://www.mesa.nl/pub/mpage/mpage-2.5.6.tgz
mpc                         1.1.0                 19 Jan 2018   https://ftp.gnu.org/gnu/mpc/mpc-1.1.0.tar.gz
mpd                         0.20.15               20 Jan 2018   http://www.musicpd.org/download/mpd/0.20/mpd-0.20.15.tar.xz
mpeg2dec                    0.4.1                 01 Jun 2014   http://libmpeg2.sourceforge.net/files/mpeg2dec-0.4.1.tar.gz
mpeg4ip                     1.5.0.1               01 Jun 2014   http://wiki.ubuntuusers.de/Archiv/mpeg4ip
mpfr                        4.0.2                 01 Feb 2019   http://ftp.gnu.org/pub/gnu/mpfr/mpfr-4.0.2.tar.xz
mpg123                      1.25.10               16 Mar 2018   http://www.mpg123.de/download/mpg123-1.25.10.tar.bz2
mpg321                      0.2.13.3              01 Jun 2014   http://sourceforge.net/projects/mpg321/files/mpg321/0.2.13/mpg321-0.2.13.3.tar.gz
mpgedit                     0.75dev2              01 Jun 2014   http://mpgedit.org/mpgedit/download/0_75/mpgedit_0-75dev2_src.tgz
mpgtx                       1.3.1                 01 Jun 2014   http://ovh.dl.sourceforge.net/sourceforge/mpgtx/mpgtx-1.3.1.tgz
mplayer                     1.4                   19 Apr 2019   ftp://ftp1.mplayerhq.hu/MPlayer/releases/MPlayer-1.4.tar.xz
mpop                        1.4.3                 12 Feb 2019   https://marlam.de/mpop/releases/mpop-1.4.3.tar.xz
mppenc                      1.16                  01 Jun 2014   http://files.musepack.net/source/mppenc-1.16.tar.bz2
mpv                         12.03.2019            12 Mar 2019   mpv-05.02.2019
mrepo                       0.8.4                 01 Jun 2014   http://dag.wieers.com/home-made/mrepo/
mrxvt                       0.5.4                 01 Jun 2014   http://prdownloads.sourceforge.net/materm/mrxvt-0.5.4.tar.gz
msgpack                     2.1.5                 01 Jan 2018   https://github.com/msgpack/msgpack-c/releases/download/cpp-2.1.5/msgpack-2.1.5.tar.gz
msitools                    0.99                  15 Feb 2019   http://ftp.gnome.org/pub/gnome/sources/msitools/0.99/msitools-0.99.tar.xz
msleep                      0.1.0                 25 Jul 2009   https://rubygems.org/downloads/msleep-0.1.0.gem
msmtp                       1.8.1                 09 Dec 2018   https://marlam.de/msmtp/releases/msmtp-1.8.1.tar.xz
mssys                       2.1.2                 01 Jun 2014   http://ms-sys.sourceforge.net/
mtail                       1.1.1                 01 Jan 2013   http://matt.immute.net/src/mtail/mtail-1.1.1.tgz
mtdev                       1.1.5                 28 May 2015   http://bitmath.org/code/mtdev/mtdev-1.1.5.tar.bz2
mtools                      4.0.23                10 Dec 2018   http://ftp.gnu.org/pub/gnu/mtools/mtools-4.0.23.tar.bz2
mtpaint                     3.31                  01 Jun 2014   http://sourceforge.net/projects/mtpaint/files/mtpaint/3.31/mtpaint-3.31.tar.bz2
mtr                         0.87                  19 Oct 2016   ftp://ftp.bitwizard.nl/mtr/mtr-0.87.tar.gz
mudlet                      1.0.5                 01 Jun 2014   mudlet-1.0.5.tar.gz
mudmagic                    1.9                   01 Jun 2014   https://sourceforge.net/projects/kyndig/files/Mud%20Magic%20Source/1.9/mudmagic-1.9.tar.gz
multiaterm                  0.2.1                 01 Jun 2014   http://www.nongnu.org/materm/multi-aterm-0.2.1.tar.gz
multijson                   1.13.1                16 Jan 2018   https://rubygems.org/downloads/multi_json-1.13.1.gem
multipartpost               2.1.1                 01 Jun 2019   https://rubygems.org/downloads/multipart-post-2.1.1.gem
mummer                      3.23                  01 Nov 2013   https://sourceforge.net/projects/mummer/files/mummer/3.23/MUMmer3.23.tar.gz
muparser                    2.2.6.1               19 Jan 2019   https://github.com/beltoforion/muparser/archive/muparser-2.2.6.1.tar.gz
mupdf                       1.15.0                08 May 2019   http://www.mupdf.com/downloads/archive/mupdf-1.15.0-source.tar.gz
muse                        2.2.1                 25 Sep 2015   http://sourceforge.net/projects/lmuse/files/muse-2.2/muse-2.2.1.tar.gz
musepack                    r475                  01 Nov 2012   http://files.musepack.net/source/musepack_src_r475.tar.gz
musescore                   3.0.1                 17 Jan 2019   https://github.com/musescore/MuseScore/archive/v3.0.1.tar.gz
musikcube                   0.61.0                14 Jan 2019   https://github.com/clangen/musikcube/archive/0.61.0.tar.gz
musl                        1.1.22                01 Jun 2019   https://www.musl-libc.org/releases/musl-1.1.22.tar.gz
mustermann                  1.0.3                 26 Sep 2018   https://rubygems.org/downloads/mustermann-1.0.3.gem
mutagen                     1.31                  05 Mar 2016   https://pypi.python.org/packages/source/m/mutagen/mutagen-1.31.tar.gz
mutt                        1.9.5                 30 Jun 2019   ftp://ftp.mutt.org/pub/mutt/mutt-1.9.5.tar.gz
mutter                      3.32.2                04 Jul 2019   http://ftp.gnome.org/pub/gnome/sources/mutter/3.32/mutter-3.32.2.tar.xz
mutterwayland               3.12.1                01 Apr 2014   http://ftp.gnome.org/pub/gnome/sources/mutter-wayland/3.12/mutter-wayland-3.12.1.tar.xz
mxk                         1.10                  01 Jun 2014   http://welz.org.za/projects/mxk/mxk-1.10.tar.gz
mxml                        2.7                   01 Jun 2014   http://ftp.easysw.com/pub/mxml/2.7/mxml-2.7.tar.gz
myghty                      1.0                   01 Jun 2014   http://easynews.dl.sourceforge.net/sourceforge/myghty/Myghty-1.0.tar.gz
myman                       0.7.0                 10 Oct 2017   https://sourceforge.net/projects/myman/files/myman/myman-0.7.0/myman-0.7.0.tar.gz
mypaint                     1.2.1                 10 Sep 2017   https://github.com/mypaint/mypaint/releases/download/v1.2.1/mypaint-1.2.1.tar.xz
mypaintbrushes              2.0.1                 09 May 2018   https://github.com/mypaint/mypaint-brushes/archive/v2.0.1.tar.gz
mysql                       8.0.16                24 Apr 2019   https://cdn.mysql.com/Downloads/MySQL-8.0/mysql-8.0.16.tar.gz
mythplugins                 0.20a                 01 Jun 2014   http://www.mythtv.org/modules.php?name=Downloads&d_op=getit&lid=124/mythplugins-0.20a
nagios                      4.3.1                 01 Jun 2014   https://assets.nagios.com/downloads/nagioscore/releases/nagios-4.3.1.tar.gz#_ga=1.110313093.1455961778.1489835372
naim                        0.11.8.3.1            01 Jun 2014   http://naim.googlecode.com/files/naim-0.11.8.3.1.tar.bz2
najitool                    0.8.4                 07 Oct 2017   https://sourceforge.net/projects/najitool/files/najitool/0.8.4/najitool-0.8.4.src.tar.bz2
nana                        1.5.4                 08 Aug 2017   https://sourceforge.net/projects/nanapro/files/Nana/Nana%201.x/nana%201.5.4.zip
nano                        4.3                   18 Jun 2019   http://ftp.gnu.org/pub/gnu/nano/nano-4.3.tar.xz
nap                         1.5.4                 01 Jun 2014   http://nap.sourceforge.net/dist/nap-1.5.4.tar.gz
nas                         1.9.3                 01 Aug 2013   http://sourceforge.net/projects/nas/files/nas/nas.1.9.3%20%28stable%29/nas-1.9.3.src.tar.gz
nasm                        2.14.02               28 Dec 2018   https://www.nasm.us/pub/nasm/releasebuilds/2.14.02/nasm-2.14.02.tar.xz
nast                        0.2.0                 05 Jun 2014   http://download.berlios.de/nast/nast-0.2.0.tar.gz
nativepackageinstaller      1.0.7                 24 Mar 2019   https://rubygems.org/downloads/native-package-installer-1.0.7.gem
nautilus                    3.32.1                05 May 2019   ftp://ftp.gnome.org/pub/gnome/sources/nautilus/3.32/nautilus-3.32.1.tar.xz
nautilusactions             3.2.4                 01 Aug 2014   http://ftp.gnome.org/pub/GNOME/sources/nautilus-actions/3.2/nautilus-actions-3.2.4.tar.xz
nautiluscdburner            2.22.1                01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/nautilus-cd-burner/2.22/nautilus-cd-burner-2.22.1.tar.bz2
nautilusmedia               0.8.1                 01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/nautilus-media/0.8/nautilus-media-0.8.1.tar.bz2
nautiluspython              1.2.2                 15 Jan 2018   http://ftp.gnome.org/pub/gnome/sources/nautilus-python/1.2/nautilus-python-1.2.2.tar.xz
nautilussendto              3.8.6                 09 Aug 2017   http://ftp.gnome.org/pub/gnome/sources/nautilus-sendto/3.8/nautilus-sendto-3.8.6.tar.xz
nazghul                     0.7.1                 31 Mar 2015   https://downloads.sourceforge.net/project/nazghul/nazghul/nazghul-0.7.1/nazghul-0.7.1.tar.gz
ncbiblast                   2.7.1+                09 Sep 2018   ftp://ftp.ncbi.nlm.nih.gov/blast/executables/blast+/2.7.1/ncbi-blast-2.7.1+-src.tar.gz
ncbicxx                     22.0.0                19 Apr 2019   ftp://ftp.ncbi.nlm.nih.gov/toolbox/ncbi_tools++/2019/Mar_28_2019/ncbi_cxx--22_0_0.tar.gz
ncc                         2.8                   01 Jun 2014   http://students.ceid.upatras.gr/~sxanth/ncc/ncc-2.8.tar.gz
ncdu                        1.4                   01 Jun 2014   http://dev.yorhel.nl/ncdu/
ncftp                       3.2.6                 28 Nov 2016   ftp://ftp.ncftp.com/ncftp/ncftp-3.2.6-src.tar.xz
ncmpcpp                     0.5.10                01 Jun 2014   http://unkart.ovh.org/ncmpcpp/ncmpcpp-0.5.10.tar.bz2
ncompress                   4.2.4.5               29 Apr 2018   https://sourceforge.net/projects/ncompress/files/ncompress-4.2.4.5.tar.gz
ncurses                     6.1                   30 Jan 2018   ftp://ftp.gnu.org/gnu/ncurses/ncurses-6.1.tar.gz
ndeskdbus                   0.6.0                 01 Jun 2014   http://www.ndesk.org/DBusSharp
ndeskdbusglib               0.4.1                 01 Feb 2013   http://www.ndesk.org/archive/dbus-sharp/ndesk-dbus-glib-0.4.1.tar.gz
ndiswrapper                 1.61                  26 Jul 2017   https://sourceforge.net/projects/ndiswrapper/files/stable/ndiswrapper-1.61.tar.gz
nemesis                     1.5                   22 Feb 2019   https://github.com/troglobit/nemesis/releases/download/v1.5/nemesis-1.5.tar.xz
nemiver                     0.9.6                 24 Sep 2015   http://ftp.gnome.org/pub/GNOME/sources/nemiver/0.9/nemiver-0.9.6.tar.xz
neon                        0.30.2                22 Nov 2016   https://fossies.org/linux/www/neon-0.30.2.tar.gz
net6                        1.3.14                16 Dec 2018   http://releases.0x539.de/net6/net6-1.3.14.tar.gz
netatalk                    3.1.12                22 Dec 2018   https://sourceforge.net/projects/netatalk/files/netatalk/3.1.12/netatalk-3.1.12.tar.gz
netcat                      0.7.1                 01 Jun 2014   http://netcat.sourceforge.net/
netdns                      1.19                  15 Dec 2018   https://cpan.metacpan.org/authors/id/N/NL/NLNETLABS/Net-DNS-1.19.tar.gz
nethack                     3.6.0                 01 Jun 2014   https://sourceforge.net/projects/nethack/files/nethack/3.6.0/nethack-360-src.tgz
nethttp                     6.17                  27 Sep 2017   http://search.cpan.org/CPAN/authors/id/O/OA/OALDERS/Net-HTTP-6.17.tar.gz
netkitbase                  0.17                  01 Jun 2014   ftp://ftp.uk.linux.org/pub/linux/Networking/netkit/netkit-base-0.17.tar.gz
netkitrsh                   0.17                  01 Jun 2014   ftp://ftp.uk.linux.org/pub/linux/Networking/netkit/netkit-rsh-0.17.tar.gz
netpbm                      10.47.71              10 Jan 2019   https://sourceforge.net/projects/netpbm/files/super_stable/10.47.71/netpbm-10.47.71.tgz
netrc                       0.11.0                29 Sep 2017   https://rubygems.org/downloads/netrc-0.11.0.gem
netscp                      2.0.0                 24 Mar 2019   https://rubygems.org/downloads/net-scp-2.0.0.gem
nettle                      3.5.1                 28 Jun 2019   http://ftp.gnu.org/pub/gnu/nettle/nettle-3.5.1.tar.gz
networkmanager              1.19.5                03 Jul 2019   http://ftp.gnome.org/pub/gnome/sources/NetworkManager/1.19/NetworkManager-1.19.5.tar.xz
networkmanagerapplet        1.8.22                29 May 2019   http://ftp.gnome.org/pub/gnome/sources/network-manager-applet/1.8/network-manager-applet-1.8.22.tar.xz
networkmanagerfortisslvpn   1.2.10                21 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/NetworkManager-fortisslvpn/1.2/NetworkManager-fortisslvpn-1.2.10.tar.xz
networkmanagerlibreswan     1.2.10                15 Oct 2018   http://ftp.gnome.org/pub/gnome/sources/NetworkManager-libreswan/1.2/NetworkManager-libreswan-1.2.10.tar.xz
networkmanageropenconnect   1.2.2                 14 May 2016   http://ftp.gnome.org/pub/GNOME/sources/NetworkManager-openconnect/1.2/NetworkManager-openconnect-1.2.2.tar.xz
networkmanageropenvpn       1.8.10                08 Feb 2019   http://ftp.gnome.org/pub/gnome/sources/NetworkManager-openvpn/1.8/NetworkManager-openvpn-1.8.10.tar.xz
networkmanagerpptp          1.2.8                 04 Oct 2018   http://ftp.gnome.org/pub/gnome/sources/NetworkManager-pptp/1.2/NetworkManager-pptp-1.2.8.tar.xz
networkmanagerqt            5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/networkmanager-qt-5.59.0.tar.xz
networkmanagervpnc          1.2.6                 20 Jul 2018   http://ftp.gnome.org/pub/gnome/sources/NetworkManager-vpnc/1.2/NetworkManager-vpnc-1.2.6.tar.xz
networkx                    2.3                   12 Apr 2019   https://files.pythonhosted.org/packages/85/08/f20aef11d4c343b557e5de6b9548761811eb16e438cee3d32b1c66c8566b/networkx-2.3.zip
neverball                   1.6.0                 27 Mar 2016   http://neverball.org/neverball-1.6.0.tar.gz
newlib                      2.0.0                 01 Jun 2014   ftp://sourceware.org/pub/newlib/newlib-2.0.0.tar.gz
newrelicrpm                 6.4.0                 01 Jun 2019   https://rubygems.org/downloads/newrelic_rpm-6.4.0.356.gem
newsbeuter                  2.9                   03 Oct 2017   http://www.newsbeuter.org/downloads/newsbeuter-2.9.tar.gz
newt                        0.52.21               31 May 2019   https://releases.pagure.org/newt/newt-0.52.21.tar.gz
nfsutils                    2.4.1                 30 Jun 2019   https://sourceforge.net/projects/nfs/files/nfs-utils/2.4.1/nfs-utils-2.4.1.tar.xz
nftables                    0.9.1                 28 Jun 2019   https://netfilter.org/projects/nftables/files/nftables-0.9.1.tar.bz2
nghttp2                     1.38.0                19 Apr 2019   https://github.com/nghttp2/nghttp2/releases/download/v1.38.0/nghttp2-1.38.0.tar.xz
nginx                       1.16.0                23 Apr 2019   http://nginx.org/download/nginx-1.16.0.tar.gz
nglview                     2.5.2                 09 Jul 2019   https://files.pythonhosted.org/packages/40/03/3d45ce1a34729bfd90c8c98e3d0e18483211408ae990feb0b4b0de5bf49b/nglview-2.5.2.tar.gz
ngstar                      1.3.3                 01 Jun 2014   http://cycojesus.free.fr/faire/coder/jouer/ng-star/files/
nim                         0.19.2                04 Jan 2019   https://nim-lang.org/download/nim-0.19.2.tar.xz
ninja                       1.9.0                 30 Jan 2018   https://github.com/ninja-build/ninja/archive/v1.9.0.tar.gz
nio4r                       2.3.1                 17 Nov 2018   https://rubygems.org/downloads/nio4r-2.3.1.gem
nmap                        7.70                  21 Mar 2018   http://download.insecure.org/nmap/dist/nmap-7.70.tgz
node                        v10.16.0              05 Jun 2019   https://nodejs.org/dist/v10.16.0/node-v10.16.0.tar.gz
nokogiri                    1.10.3                01 May 2019   https://rubygems.org/downloads/nokogiri-1.10.3.gem
normalize                   0.7.7                 01 Jun 2014   http://savannah.nongnu.org/download/normalize/normalize-0.7.7.tar.bz2
notificationdaemon          3.18.2                16 Feb 2016   http://ftp.gnome.org/pub/GNOME/sources/notification-daemon/3.18/notification-daemon-3.18.2.tar.xz
notify2                     0.3.1                 04 Sep 2017   https://pypi.python.org/packages/aa/e8/d4b335aa739dc299a77766ecc5f1972d1de1993524aa94acef3219bba315/notify2-0.3.1.tar.gz
notifypython                0.1.1                 25 Apr 2015   http://www.galago-project.org/files/releases/source/notify-python/notify-python-0.1.1.tar.bz2
npth                        1.6                   30 Jul 2018   ftp://ftp.gnupg.org/gcrypt/npth/npth-1.6.tar.bz2
nsd                         4.1.17                25 Jul 2017   https://www.nlnetlabs.nl/downloads/nsd/nsd-4.1.17.tar.gz
nspluginwrapper             1.2.2                 01 Jun 2014   http://gwenole.beauchesne.info/projects/nspluginwrapper/files/nspluginwrapper-1.2.2.tar.bz2
nspr                        4.21                  09 Mar 2019   https://archive.mozilla.org/pub/nspr/releases/v4.21/src/nspr-4.21.tar.gz
nss                         3.44                  12 May 2019   https://archive.mozilla.org/pub/security/nss/releases/NSS_3_44_RTM/src/nss-3.44.tar.gz
ntfs3gntfsprogs             2017.3.23             12 Sep 2017   https://tuxera.com/opensource/ntfs-3g_ntfsprogs-2017.3.23.tgz
ntp                         4.2.8p13              09 Mar 2019   https://www.eecis.udel.edu/~ntp/ntp_spool/ntp4/ntp-4.2.8p13.tar.gz
ntrack                      017                   09 Jul 2015   https://launchpad.net/ntrack/main/017/+download/ntrack-017.tar.gz
numactl                     2.0.11                02 Sep 2017   https://github.com/numactl/numactl/archive/numactl-2.0.11.tar.gz
numeric                     24.2                  01 Jun 2014   Numeric-24.2
numpy                       1.16.3                10 May 2019   https://files.pythonhosted.org/packages/93/48/956b9dcdddfcedb1705839280e02cbfeb2861ed5d7f59241210530867d5b/numpy-1.16.3.zip
nuvie                       0.5                   19 Oct 2014   http://sourceforge.net/projects/nuvie/files/Nuvie/0.5/nuvie-0.5.tgz
nvclock                     0.8                   01 Jun 2014   http://www.linuxhardware.org/nvclock/nvclock0.8b4.tar.gz
nvidia                                            14 Dec 2015   http://us.download.nvidia.com/XFree86/Linux-x86/352.63/NVIDIA-Linux-x86-352.63.run
nwcc                        0.8.2                 01 Jun 2014   http://sourceforge.net/projects/nwcc/files/nwcc/nwcc%200.8.2/nwcc_0.8.2.tar.gz
nzbget                      17.1                  06 Sep 2016   https://github.com/nzbget/nzbget/releases/download/v17.1/nzbget-17.1-src.tar.gz
obby                        0.4.6                 01 Jun 2014   http://releases.0x539.de/obby/obby-0.4.6.tar.gz
obconf                      2.0.4                 23 Aug 2017   http://openbox.org/dist/obconf/obconf-2.0.4.tar.gz
obconfqt                    0.14.1                27 Feb 2019   https://downloads.lxqt.org/downloads/obconf-qt/0.14.1/obconf-qt-0.14.1.tar.xz
obexdataserver              0.4.6                 01 Dec 2012   http://tadas.dailyda.com/software/obex-data-server-0.4.6.tar.gz
ocaml                       4.04.0                25 Feb 2017   http://caml.inria.fr/pub/distrib/ocaml-4.04/ocaml-4.04.0.tar.gz
oclock                      1.0.1                 01 Jun 2014   http://xorg.freedesktop.org/releases/individual/app/oclock-1.0.1.tar.bz2
ocra                        1.3.10                01 Dec 2017   https://rubygems.org/downloads/ocra-1.3.10.gem
ocrad                       0.27                  11 Jan 2019   https://ftp.gnu.org/pub/gnu/ocrad/ocrad-0.27.tar.lz
ocrfeeder                   0.8.1                 12 Mar 2015   http://ftp.gnome.org/pub/GNOME/sources/ocrfeeder/0.8/ocrfeeder-0.8.1.tar.xz
octave                      5.1.0                 25 Feb 2019   http://ftp.gnu.org/gnu/octave/octave-5.1.0.tar.xz
ogmrip                      1.0.1                 30 Aug 2015   http://sourceforge.net/projects/ogmrip/files/ogmrip/1.0/1.0.1/ogmrip-1.0.1.tar.gz
ogmtools                    1.5                   01 Jun 2014   http://www.bunkus.org/videotools/ogmtools/ogmtools-1.5.tar.bz2
ogre                        1.11.2                09 Sep 2018   https://github.com/OGRECave/ogre/archive/v1.11.2.zip
oki                         0.1.6                 01 Jun 2014   http://free.of.pl/s/szatkus/soft/gry/oki-0.1.6.tar.gz
okular                      19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/okular-19.04.2.tar.xz
omake                       0.9.8.5.3             01 Jun 2014   http://omake.metaprl.org/download.html
onig                        6.9.2                 23 Jun 2019   https://github.com/kkos/oniguruma/releases/download/v6.9.2/onig-6.9.2.tar.gz
oniguruma                   6.9.1                 12 Dec 2018   https://github.com/kkos/oniguruma/releases/download/v6.9.1/onig-6.9.1.tar.gz
opal                        3.10.10               24 Sep 2014   http://ftp.gnome.org/pub/GNOME/sources/opal/3.10/opal-3.10.10.tar.xz
openal                      0.0.8                 01 Jun 2014   http://www.openal.org/openal_webstf/downloads/openal-0.0.8.tar.gz
openalsoft                  1.19.1                13 Dec 2018   https://kcat.strangesoft.net/openal-releases/openal-soft-1.19.1.tar.bz2
openbabel                   19.12.2018            20 Sep 2017   https://sourceforge.net/projects/openbabel/files/openbabel/2.4.1/openbabel-2.4.1.tar.gz
openblas                    0.3.5                 19 Apr 2019   https://github.com/xianyi/OpenBLAS/archive/v0.3.5.tar.gz
openbox                     3.6.1                 01 Jul 2015   http://openbox.org/dist/openbox/openbox-3.6.1.tar.gz
opencobol                   1.1                   01 Dec 2013   https://downloads.sourceforge.net/project/open-cobol/open-cobol/1.1/open-cobol-1.1.tar.gz
openconnect                 8.01                  11 Jan 2019   ftp://ftp.infradead.org/pub/openconnect/openconnect-8.01.tar.gz
opencv                      4.1.0                 17 Apr 2019   https://github.com/opencv/opencv/archive/4.1.0/opencv-4.1.0.tar.gz
openexr                     10.05.2019            26 Nov 2018   https://github.com/openexr/openexr/releases/download/v2.3.0/openexr-2.3.0.tar.gz
openinvaders                0.3                   01 Dec 2012   http://www.jamyskis.net/invaders.php
openjade                    1.3.2                 01 Jun 2014   http://prdownloads.sourceforge.net/openjade/openjade-1.3.2.tar.gz
openjpeg                    2.3.1                 06 Apr 2019   https://github.com/uclouvain/openjpeg/archive/v2.3.1.tar.gz
openldap                    2.4.47                19 Dec 2018   ftp://ftp.openldap.org/pub/OpenLDAP/openldap-release/openldap-2.4.47.tgz
openldev                    1.0                   21 Aug 2017   https://sourceforge.net/projects/openldev/files/openldev/1.0/openldev-1.0.tar.gz
openmovieeditor             0.0.20090105          01 Jun 2014   http://openmovieeditor.sourceforge.net/HomePage
openmpi                     4.0.0                 26 Jan 2019   https://download.open-mpi.org/release/open-mpi/v4.0/openmpi-4.0.0.tar.bz2
openobex                    1.7.2                 12 Jan 2019   https://sourceforge.net/projects/openobex/files/openobex/1.7.2/openobex-1.7.2-Source.tar.gz
openscenegraph              2.4.0                 01 Jun 2014   OpenSceneGraph-2.4.0
openshot                    1.4.3                 01 Jun 2014   http://launchpad.net/openshot/1.4/1.4.3/+download/openshot-1.4.3.tar.gz
openshotqt                  2.4.3                 23 Sep 2018   http://launchpad.net/openshot/2.4/2.4.3/+download/openshot-qt-2.4.3.tar.gz
opensp                      1.5.2                 01 Jun 2014   http://ovh.dl.sourceforge.net/sourceforge/openjade/OpenSP-1.5.2.tar.gz
openssh                     7.9p1                 19 Oct 2018   https://ftp3.usa.openbsd.org/pub/OpenBSD/OpenSSH/portable/openssh-7.9p1.tar.gz
openssl                     1.1.1c                29 May 2019   ftp://ftp.openssl.org/source/openssl-1.1.1c.tar.gz
openssn                     1.3                   01 Dec 2012   http://sourceforge.net/projects/openssn/files/openssn-1.3/openssn-1.3.tar.gz
openttd                     1.2.3                 08 Aug 2015   https://binaries.openttd.org/releases/1.5.1/openttd-1.5.1-source.tar.gz
openvrml                    0.18.9                01 Jun 2014   http://sourceforge.net/projects/openvrml/files/openvrml/0.18.9/openvrml-0.18.9.tar.gz
opera                       12.00                 01 Jun 2014   http://snapshot.opera.com/unix/bonkers_12.00-1440/opera-next-12.00-1440.x86_64.linux.tar.xz
ophcrack                    3.6.0                 01 Jul 2013   http://downloads.sourceforge.net/project/ophcrack/ophcrack/3.6.0/ophcrack-3.6.0.tar.bz2
oprofile                    1.3.0                 03 Feb 2019   http://prdownloads.sourceforge.net/oprofile/oprofile-1.3.0.tar.gz
optipng                     0.7.7                 31 Jul 2018   https://sourceforge.net/projects/optipng/files/OptiPNG/optipng-0.7.7/optipng-0.7.7.tar.gz
opus                        1.3.1                 13 Apr 2019   https://ftp.osuosl.org/pub/xiph/releases/opus/opus-1.3.1.tar.gz
opusfile                    0.11                  21 Sep 2018   https://archive.mozilla.org/pub/opus/opusfile-0.11.tar.gz
opustools                   0.1.10                25 Dec 2017   https://archive.mozilla.org/pub/opus/opus-tools-0.1.10.tar.gz
orbit2                      2.14.19               01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/ORBit2/2.14/ORBit2-2.14.19.tar.bz2
orc                         0.4.29                16 Apr 2019   https://github.com/GStreamer/orc/archive/orc-0.4.29.tar.gz
orca                        3.32.0                14 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/orca/3.32/orca-3.32.0.tar.xz
ostree                      2014.4                01 Apr 2014   http://ftp.gnome.org/pub/gnome/sources/ostree/2014.4/ostree-2014.4.tar.xz
oxygen                      5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/oxygen-5.16.3.tar.xz
oxygengtk                   1.1.6                 01 Aug 2013   http://download.kde.org/stable/oxygen-gtk/1.1.6/src/oxygen-gtk-1.1.6.tar.bz2
oxygenicons5                5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/oxygen-icons5-5.59.0.tar.xz
p11kit                      0.23.16.1             24 May 2019   https://github.com/p11-glue/p11-kit/releases/download/0.23.16.1/p11-kit-0.23.16.1.tar.gz
p7zip                       16.02                 28 Sep 2017   https://sourceforge.net/projects/p7zip/files/p7zip/16.02/p7zip_16.02_src_all.tar.bz2
packagekit                  1.1.11                05 Oct 2018   https://www.freedesktop.org/software/PackageKit/releases/PackageKit-1.1.11.tar.xz
pacman                      18.12.2018            18 Dec 2018   ftp://ftp.archlinux.org/other/pacman/pacman-18.12.2018.tar.xz
padrino                     0.14.4                06 Nov 2018   https://rubygems.org/downloads/padrino-0.14.4.gem
padrinoadmin                0.14.4                06 Nov 2018   https://rubygems.org/downloads/padrino-admin-0.14.4.gem
padrinocache                0.14.4                06 Nov 2018   https://rubygems.org/downloads/padrino-cache-0.14.4.gem
padrinocore                 0.14.4                06 Nov 2018   https://rubygems.org/downloads/padrino-core-0.14.4.gem
padrinogen                  0.14.4                06 Nov 2018   https://rubygems.org/downloads/padrino-gen-0.14.4.gem
padrinohelpers              0.14.4                06 Nov 2018   https://rubygems.org/downloads/padrino-helpers-0.14.4.gem
padrinomailer               0.14.4                06 Nov 2018   https://rubygems.org/downloads/padrino-mailer-0.14.4.gem
padrinosupport              0.14.4                06 Nov 2018   https://rubygems.org/downloads/padrino-support-0.14.4.gem
paint                       2.1.0                 18 Feb 2019   https://rubygems.org/downloads/paint-2.1.0.gem
paintown                    3.6.0                 01 May 2013   http://sourceforge.net/projects/paintown/files/paintown/3.6.0/paintown-3.6.0.tar.bz2
palapeli                    19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/palapeli-19.04.2.tar.xz
paludis                     2.6.0                 18 Jul 2017   http://paludis.exherbo.org/download/paludis-2.6.0.tar.bz2
panda3d                     1.8.0                 01 Dec 2012   http://www.panda3d.org/download/panda3d-1.8.0/panda3d-1.8.0.tar.gz
pandas                      0.20.3                09 Jul 2017   https://pypi.python.org/packages/ee/aa/90c06f249cf4408fa75135ad0df7d64c09cf74c9870733862491ed5f3a50/pandas-0.20.3.tar.gz
pandoc                      2.7.2                 14 Apr 2019   https://hackage.haskell.org/package/pandoc-2.7.2/pandoc-2.7.2.tar.gz
pandora                     0.1.2                 01 Jun 2014   https://rubygems.org/downloads/pandora-0.1.2.gem
pango                       1.43.0                12 Dec 2018   http://ftp.gnome.org/pub/gnome/sources/pango/1.43/pango-1.43.0.tar.xz
pangomm                     2.42.0                06 Dec 2018   http://ftp.gnome.org/pub/gnome/sources/pangomm/2.42/pangomm-2.42.0.tar.xz
pangzero                    1.3                   01 Dec 2012   http://sourceforge.net/projects/pangzero/files/pangzero/1.3/pangzero-1.3.tar.gz
papyrus                     0.13.3                28 Mar 2016   https://sourceforge.net/projects/libpapyrus/files/papyrus/0.13.3/papyrus-0.13.3.tar.gz
paragui                     1.1.8                 01 Jun 2014   https://github.com/pipelka/paragui
parallel                    20190622              22 Jun 2019   http://ftp.gnu.org/pub/gnu/parallel/parallel-20190622.tar.bz2
paratim                     0.2.0                 01 Dec 2012   http://paratim.sourceforge.net/
parley                      19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/parley-19.04.2.tar.xz
parole                      1.0.2                 08 Apr 2019   http://archive.xfce.org/src/apps/parole/1.0/parole-1.0.2.tar.bz2
parser                      2.6.3.0               03 May 2019   https://rubygems.org/downloads/parser-2.6.3.0.gem
parsetree                   3.0.9                 01 Aug 2017   https://rubygems.org/downloads/ParseTree-3.0.9.gem
parseyapp                   1.21                  07 Aug 2017   https://cpan.metacpan.org/authors/id/W/WB/WBRASWELL/Parse-Yapp-1.21.tar.gz
parted                      3.2                   01 Jul 2014   ftp://ftp.gnu.org/gnu/parted/parted-3.2.tar.xz
partimage                   0.6.9                 28 Mar 2016   https://sourceforge.net/projects/partimage/files/stable/0.6.9/partimage-0.6.9.tar.bz2
passenger                   6.0.2                 24 Mar 2019   https://rubygems.org/downloads/passenger-6.0.2.gem
patch                       2.7.6                 06 Feb 2018   https://ftp.gnu.org/pub/gnu/patch/patch-2.7.6.tar.xz
patchelf                    0.10                  01 Apr 2019   https://nixos.org/releases/patchelf/patchelf-0.10/patchelf-0.10.tar.gz
pathexpander                1.0.4                 01 Jun 2019   https://rubygems.org/downloads/path_expander-1.0.4.gem
pathlib                     1.0.1                 27 Dec 2017   https://pypi.python.org/packages/ac/aa/9b065a76b9af472437a0059f77e8f962fe350438b927cb80184c32f075eb/pathlib-1.0.1.tar.gz
pavucontrol                 3.0                   18 Jul 2017   http://freedesktop.org/software/pulseaudio/pavucontrol/pavucontrol-3.0.tar.xz
pavucontrolqt               0.14.1                27 Feb 2019   https://downloads.lxqt.org/downloads/pavucontrol-qt/0.14.1/pavucontrol-qt-0.14.1.tar.xz
pawm                        2.3.0                 01 Jun 2014   http://www.pleyades.net/david/projects/pawm/pawm-2.3.0.tar.bz2
paxutils                    1.2.3                 16 Dec 2018   http://distfiles.gentoo.org/distfiles/pax-utils-1.2.3.tar.xz
pbzip2                      1.1.13                21 Sep 2017   https://launchpad.net/pbzip2/1.1/1.1.13/+download/pbzip2-1.1.13.tar.gz
pcc                         02.10.2007            01 Jun 2014   ftp://226.net120.skekraft.net/pcc/
pciutils                    3.6.2                 14 Aug 2018   https://www.kernel.org/pub/software/utils/pciutils/pciutils-3.6.2.tar.xz
pcmanfmqt                   0.14.1                27 Feb 2019   https://downloads.lxqt.org/downloads/pcmanfm-qt/0.14.1/pcmanfm-qt-0.14.1.tar.xz
pcmciautils                 014                   01 May 2012   http://kernel.org/pub/linux/utils/kernel/pcmcia/pcmciautils-014.tar.bz2
pcre                        8.43                  24 Feb 2019   ftp://ftp.csx.cam.ac.uk/pub/software/programming/pcre/pcre-8.43.tar.gz
pcre2                       10.33                 18 Apr 2019   ftp://ftp.csx.cam.ac.uk/pub/software/programming/pcre/pcre2-10.33.tar.gz
pcsclite                    1.8.18                11 Aug 2016   https://alioth.debian.org/frs/download.php/file/4173/pcsc-lite-1.8.18.tar.bz2
pdal                        1.5.0                 21 Sep 2017   http://download.osgeo.org/pdal/PDAL-1.5.0-src.tar.gz
pdfcore                     0.8.1                 01 May 2018   https://rubygems.org/downloads/pdf-core-0.8.1.gem
pdfcrack                    0.14                  12 Sep 2014   http://sourceforge.net/projects/pdfcrack/files/pdfcrack/pdfcrack-0.14/pdfcrack-0.14.tar.gz
pdfedit                     0.4.5                 01 Jun 2014   http://sourceforge.net/projects/pdfedit/files/pdfedit/0.4.5/pdfedit-0.4.5.tar.bz2
pdfgrep                     2.0.1                 25 Mar 2017   https://pdfgrep.org/download/pdfgrep-2.0.1.tar.gz
pdfpc                       4.3.0                 22 Dec 2018   https://github.com/pdfpc/pdfpc/archive/v4.3.0.tar.gz
pdfreader                   2.2.0                 26 Dec 2018   https://rubygems.org/downloads/pdf-reader-2.2.0.gem
pdftk                       2.02                  10 May 2015   https://www.pdflabs.com/tools/pdftk-the-pdf-toolkit/pdftk-2.02-src.zip
pdl                         2.016                 29 Jun 2016   https://sourceforge.net/projects/pdl/files/PDL-2.016/PDL-2.016.tar.gz/
ped                         0.1.6                 01 Jun 2014   http://schwarz-mode.de/files/ped/
peewee                      3.8.0                 18 Dec 2018   https://files.pythonhosted.org/packages/b2/38/fe304c84dd7b771e6c313e2905d54944b9577b212e3247cfa58fac2ac935/peewee-3.8.0.tar.gz
peg                         0.1.18                30 Dec 2018   http://piumarta.com/software/peg/peg-0.1.18.tar.gz
pekwm                       0.1.17                01 Jan 2014   https://www.pekwm.org/projects/pekwm/files/pekwm-0.1.17.tar.bz2
penguincommand              1.6.11                01 Jan 2013   http://user.cs.tu-berlin.de/~karlb/penguin-command/
pengupop                    2.2.4                 01 Jun 2014   http://www.junoplay.com/pengupop
pentagram                   16.02.2008            01 Dec 2012   http://pentagram.sourceforge.net/download.php
perl                        5.30.0                23 May 2019   http://www.cpan.org/src/5.0/perl-5.30.0.tar.gz
perlgettext                 1.05                  01 Jun 2014   http://search.cpan.org/dist/gettext/
perlglib                    1.327                 05 Jun 2018   http://search.cpan.org/CPAN/authors/id/X/XA/XAOC/Glib-1.327.tar.gz
pg                          1.1.4                 18 Feb 2019   https://rubygems.org/downloads/pg-1.1.4.gem
phodav                      2.3                   27 Feb 2019   http:/ftp.gnome.org/pub/gnome/sources/phodav/2.3/phodav-2.3.tar.xz
phonon                      4.10.2                10 Feb 2019   http://download.kde.org/stable/phonon/4.10.2/phonon-4.10.2.tar.xz
phononbackendgstreamer      4.9.0                 13 Oct 2017   http://download.kde.org/stable/phonon/phonon-backend-gstreamer/4.9.0/phonon-backend-gstreamer-4.9.0.tar.xz
phononbackendvlc            0.10.2                10 Feb 2019   http://download.kde.org/stable/phonon/phonon-backend-vlc/0.10.2/phonon-backend-vlc-0.10.2.tar.xz
php                         7.3.7                 02 Jul 2019   http://us3.php.net/distributions/php-7.3.7.tar.xz
phpbb                       3.2.3                 30 Sep 2018   https://www.phpbb.com/files/release/phpBB-3.2.3.zip
phpunit                     6.3                   24 Sep 2017   https://phar.phpunit.de/phpunit-6.3.phar
phpwiki                     1.5.5                 02 Aug 2017   https://sourceforge.net/projects/phpwiki/files/PhpWiki%201.5%20%28current%29/phpwiki-1.5.5.zip
phylip                      3.68                  01 Jun 2014   http://evolution.gs.washington.edu/phylip/download/phylip-3.68.tar.gz
physfs                      3.0.1                 13 Nov 2017   https://icculus.org/physfs/downloads/physfs-3.0.1.tar.bz2
picmi                       19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/picmi-19.04.2.tar.xz
pidgin                      2.13.0                09 Mar 2018   http://downloads.sourceforge.net/pidgin/pidgin-2.13.0.tar.bz2
pidify                                            01 Dec 2017   https://rubygems.org/downloads/pidify-0.2-i686-linux.gem
pillow                      6.0.0                 03 Apr 2019   https://files.pythonhosted.org/packages/81/1a/6b2971adc1bca55b9a53ed1efa372acff7e8b9913982a396f3fa046efaf8/Pillow-6.0.0.tar.gz
pimcommon                   19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/pimcommon-19.04.2.tar.xz
pimdataexporter             19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/pim-data-exporter-19.04.2.tar.xz
pimsieveeditor              19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/pim-sieve-editor-19.04.2.tar.xz
pine                        4.64                  01 Jun 2014   ftp://ftp.cac.washington.edu/pine/pine.tar.bz2
pinentry                    1.1.0                 07 Dec 2017   https://www.gnupg.org/ftp/gcrypt/pinentry/pinentry-1.1.0.tar.bz2
pinfo                       0.6.8                 01 Jun 2014   http://freshmeat.net/redir/pinfo/8107/url_tgz/pinfo-0.6.8.tar.gz
pingus                      0.7.6                 01 Jul 2013   http://pingus.googlecode.com/files/pingus-0.7.6.tar.bz2
pinpoint                    0.1.8                 23 Sep 2015   http://ftp.gnome.org/pub/GNOME/sources/pinpoint/0.1/pinpoint-0.1.8.tar.xz
pinta                       1.6                   13 Jan 2019   https://github.com/PintaProject/Pinta/releases/download/1.6/pinta-1.6.tar.gz
pioneers                    15.4                  01 Nov 2017   https://sourceforge.net/projects/pio/files/Source/pioneers-15.4.tar.gz
pip                         19.1.1                05 Jul 2019   https://files.pythonhosted.org/packages/93/ab/f86b61bef7ab14909bd7ec3cd2178feb0a1c86d451bc9bccd5a1aedcde5f/pip-19.1.1.tar.gz
pipemeter                   1.1.2.1               01 Jun 2014   http://spamaps.org/files/pipemeter/pipemeter-1.1.2-1.tar.gz
pipenv                      2018.11.26            12 Feb 2019   https://files.pythonhosted.org/packages/fd/e9/01822318551caa0d62a181ba3b10f0f3757bb1e270da97165bd52db92776/pipenv-2018.11.26.tar.gz
pipepanic                   0.1.3                 01 Dec 2012   http://www.users.waitrose.com/~thunor/pipepanic/#downloads
pipewalker                  0.4.3                 01 Dec 2012   http://sourceforge.net/project/showfiles.php?group_id=198283
pipewire                    0.2.6                 31 May 2019   https://github.com/PipeWire/pipewire/archive/0.2.6/pipewire-0.2.6.tar.gz
pitivi                      0.999                 30 Aug 2018   http://ftp.gnome.org/pub/gnome/sources/pitivi/0.999/pitivi-0.999.tar.xz
pixman                      0.38.4                11 Apr 2019   https://www.x.org/releases/individual/lib/pixman-0.38.4.tar.gz
pjproject                   1.14.2                01 Jun 2014   http://www.pjsip.org/release/1.14.2/pjproject-1.14.2.tar.bz2
pkgconfig                   0.29.2                26 Mar 2017   https://pkgconfig.freedesktop.org/releases/pkg-config-0.29.2.tar.gz
pl                          6.2.2                 01 Jun 2014   http://www.swi-prolog.org/download/stable/src/pl-6.2.2.tar.gz
plasmabrowserintegration    5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/plasma-browser-integration-5.16.3.tar.xz
plasmadesktop               5.16.3                09 Jul 2019   http://download.kde.org/stable/plasma/5.16.3/plasma-desktop-5.16.3.tar.xz
plasmaframework             5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/plasma-framework-5.59.0.tar.xz
plasmaintegration           5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/plasma-integration-5.16.3.tar.xz
plasmanm                    5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/plasma-nm-5.16.3.tar.xz
plasmapa                    5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/plasma-pa-5.16.3.tar.xz
plasmasdk                   5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/plasma-sdk-5.16.3.tar.xz
plasmatests                 5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/plasma-tests-5.16.3.tar.xz
plasmavault                 5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/plasma-vault-5.16.3.tar.xz
plasmaworkspace             5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/plasma-workspace-5.16.3.tar.xz
plasmaworkspacewallpapers   5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/plasma-workspace-wallpapers-5.16.3.tar.xz
plib                        1.8.4                 01 Jun 2014   http://plib.sourceforge.net/download.html
ploticus                    2.3.2                 01 Jun 2014   http://ploticus.sourceforge.net/download/pl232src.tar.gz
pluma                       1.23.0                03 Jun 2019   https://pub.mate-desktop.org/releases/1.23/pluma-1.23.0.tar.xz
ply                         3.11                  22 Nov 2018   https://files.pythonhosted.org/packages/e5/69/882ee5c9d017149285cab114ebeab373308ef0f874fcdac9beb90e0ac4da/ply-3.11.tar.gz
plymouth                    0.9.4                 04 Dec 2018   https://www.freedesktop.org/software/plymouth/releases/plymouth-0.9.4.tar.xz
plymouthkcm                 5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/plymouth-kcm-5.16.3.tar.xz
plzip                       1.6                   30 Nov 2017   http://download.savannah.gnu.org/releases/lzip/plzip/plzip-1.6.tar.gz
pmacct                      0.14.1                01 Jun 2014   http://www.pmacct.net/pmacct-0.14.1.tar.gz
pmk                         0.10.4                01 Jun 2014   http://prdownloads.sourceforge.net/pmk/pmk-0.10.4.tar.gz
png++                       0.2.9                 30 Jul 2016   http://download.savannah.gnu.org/releases/pngpp/png++-0.2.9.tar.gz
pngcrush                    1.7.41                01 Dec 2012   http://sourceforge.net/projects/pmt/files/pngcrush/1.7.41/pngcrush-1.7.41.tar.xz
pngquant                    2.12.0                22 Jun 2018   http://pngquant.org/pngquant-2.12.0-src.tar.gz
pnmixer                     v0.7.2                10 Sep 2018   https://github.com/nicklan/pnmixer/releases/download/v0.7.2/pnmixer-v0.7.2.tar.gz
poco                        1.1.2                 01 Jun 2014   http://www.basha.org/poco.shtml
podofo                      0.9.5                 31 Oct 2017   http://sourceforge.net/projects/podofo/files/podofo/0.9.5/podofo-0.9.5.tar.gz
polari                      3.32.1                18 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/polari/3.32/polari-3.32.1.tar.xz
policykit                   0.9                   01 Jun 2014   PolicyKit-0.9
polkit                      0.115                 19 Nov 2018   https://www.freedesktop.org/software/polkit/releases/polkit-0.115.tar.gz
polkitgnome                 0.105                 01 Jun 2014   ftp://ftp.gnome.org/pub/gnome/sources/polkit-gnome/0.105/polkit-gnome-0.105.tar.xz
polkitkdeagent1             5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/polkit-kde-agent-1-5.16.3.tar.xz
polkitqt1                   0.112.0               03 Mar 2015   http://download.kde.org/stable/apps/KDE4.x/admin/polkit-qt-1-0.112.0.tar.bz2
poltergeist                 1.18.1                13 Aug 2018   https://rubygems.org/downloads/poltergeist-1.18.1.gem
polyglot                    0.3.5                 14 Oct 2018   https://rubygems.org/downloads/polyglot-0.3.5.gem
polylib                     5.22.5                01 Nov 2012   http://icps.u-strasbg.fr/polylib/polylib_src/polylib-5.22.5.tar.gz
poppler                     0.78.0                06 Jul 2019   https://poppler.freedesktop.org/poppler-0.78.0.tar.xz
popplerdata                 0.4.9                 29 Apr 2018   http://poppler.freedesktop.org/poppler-data-0.4.9.tar.gz
popt                        1.16                  01 Jun 2014   http://rpm5.org/files/popt/popt-1.16.tar.gz
porcupine                   0.0.7                 01 Jun 2014   http://www.innoscript.org/component/option,com_remository/Itemid,33/
porg                        0.10                  01 Sep 2016   https://sourceforge.net/projects/porg/files/porg-0.10.tar.gz
portaudio                   0.19                  01 Jun 2014   http://www.portaudio.com/download.html
postfix                     3.4.6                 03 Jul 2019   ftp://ftp.porcupine.org/mirrors/postfix-release/official/postfix-3.4.6.tar.gz
postgis                     2.3.1                 18 Mar 2017   http://download.osgeo.org/postgis/source/postgis-2.3.1.tar.gz
postgresql                  11.4                  20 Jun 2019   http://ftp.postgresql.org/pub/source/v11.4/postgresql-11.4.tar.bz2
postr                       0.13.1                05 Sep 2014   http://ftp.gnome.org/pub/GNOME/sources/postr/0.13/postr-0.13.1.tar.xz
potrace                     1.15                  20 Aug 2017   https://downloads.sourceforge.net/potrace/potrace-1.15.tar.gz
povray                      3.7.0beta25b          01 May 2013   http://www.povray.org/redirect/www.povray.org/ftp/pub/povray/Official/Linux/povlinux-3.6.tgz
powerassert                 1.1.4                 03 May 2019   https://rubygems.org/downloads/power_assert-1.1.4.gem
powerdevil                  5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/powerdevil-5.16.3.tar.xz
powermanga                  0.93                  10 Oct 2017   http://linux.tlk.fr/games/Powermanga/download/powermanga-0.93.tgz
powerpack                   0.1.2                 14 Oct 2018   https://rubygems.org/downloads/powerpack-0.1.2.gem
powertop                    v2.10                 07 Jan 2019   https://01.org/sites/default/files/downloads//powertop-v2.10.tar.gz
poxml                       19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/poxml-19.04.2.tar.xz
ppl                         1.2                   18 Feb 2016   ftp://ftp.cs.unipr.it/pub/ppl/releases/1.2/ppl-1.2.tar.gz
ppp                         2.4.5                 28 Aug 2015   http://samba.org/ftp/ppp/ppp-2.4.5.tar.gz
prawn                       2.2.2                 06 Jul 2017   https://rubygems.org/downloads/prawn-2.2.2.gem
prefixsuffix                0.6.9                 08 Dec 2015   http://ftp.gnome.org/pub/GNOME/sources/prefixsuffix/0.6/prefixsuffix-0.6.9.tar.xz
presentproto                1.1                   18 Mar 2017   https://www.x.org/releases/individual/proto/presentproto-1.1.tar.bz2
primer3                     2.3.7                 12 Apr 2016   https://sourceforge.net/projects/primer3/files/primer3/2.3.7/primer3-2.3.7.tar.gz
printmanager                19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/print-manager-19.04.2.tar.xz
printproto                  1.0.5                 01 Jun 2014   http://xorg.freedesktop.org/releases/individual/proto/printproto-1.0.5.tar.bz2
prison                      5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/prison-5.59.0.tar.xz
processing                  0111                  01 Jun 2014   http://processing.org/download/processing-0111.tgz
procinfong                  2.0.304               18 Apr 2019   https://sourceforge.net/projects/procinfo-ng/files/procinfo-ng/2.0.304/procinfo-ng-2.0.304.tar.bz2
procps                      3.2.8                 13 Dec 2015   http://procps.sourceforge.net/procps-3.2.8.tar.gz
procpsng                    3.3.15                15 Dec 2018   https://sourceforge.net/projects/procps-ng/files/Production/procps-ng-3.3.15.tar.xz
proftpd                     1.3.6                 10 Apr 2017   ftp://ftp.proftpd.org/distrib/source/proftpd-1.3.6.tar.gz
progsreiserfs               0.3.0.4               01 Jun 2014   http://reiserfs.osdn.org.ua/progsreiserfs-0.3.0.4.tar.gz
projectx                    0.90.3.01             01 Jun 2014   http://peterhost.dl.sourceforge.net/sourceforge/project-x/ProjectX_Source_0.82.1.00.zip
protobufcpp                 3.7.0                 08 Mar 2019   https://github.com/protocolbuffers/protobuf/releases/download/v3.7.0/protobuf-cpp-3.7.0.tar.gz
protocol                    1.0.1                 01 Dec 2017   https://rubygems.org/downloads/protocol-1.0.1.gem
protoeditor                 1.0                   09 Feb 2018   https://sourceforge.net/projects/protoeditor/files/protoeditor/1.0/protoeditor-1.0.tar.gz
proxymngr                   1.0.4                 18 Apr 2015   http://xorg.freedesktop.org/releases/individual/app/proxymngr-1.0.4.tar.gz
pry                         0.12.2                17 Nov 2018   https://rubygems.org/downloads/pry-0.12.2.gem
prybyebug                   3.7.0                 14 Apr 2019   https://rubygems.org/downloads/pry-byebug-3.7.0.gem
pscpug                      0.3.2                 01 Jun 2014   http://www.diablonet.net/~mercadal/projects/pscpug/pscpug030.tgz
psftools                    1.0.12                25 Mar 2019   http://www.seasip.info/Unix/PSF/psftools-1.0.12.tar.gz
psmisc                      23.2                  27 May 2019   https://sourceforge.net/projects/psmisc/files/psmisc/psmisc-23.2.tar.xz
pspp                        1.2.0                 06 Nov 2018   https://ftp.gnu.org/pub/gnu/pspp/pspp-1.2.0.tar.gz
psych                       3.1.0                 30 Dec 2018   https://rubygems.org/downloads/psych-3.1.0.gem
pth                         2.0.7                 01 Jun 2014   ftp://ftp.gnu.org/gnu/pth/pth-2.0.7.tar.gz
ptlib                       2.10.11               07 Nov 2015   http://ftp.gnome.org/pub/GNOME/sources/ptlib/2.10/ptlib-2.10.11.tar.xz
publicsuffix                3.0.3                 26 Sep 2018   https://rubygems.org/downloads/public_suffix-3.0.3.gem
pulseaudio                  12.2                  16 Jul 2018   http://freedesktop.org/software/pulseaudio/releases/pulseaudio-12.2.tar.xz
puma                        3.12.1                03 May 2019   https://rubygems.org/downloads/puma-3.12.1.gem
puppet                      6.4.2                 03 May 2019   https://rubygems.org/downloads/puppet-6.4.2.gem
purpose                     5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/purpose-5.59.0.tar.xz
putty                       0.70                  12 Dec 2018   https://the.earth.li/~sgtatham/putty/latest/putty-0.70.tar.gz
pwdutils                    3.0                   01 Jun 2014   http://www.thkukuk.de/pam/pwdutils/
pwlib                       1.10.0                01 Jun 2014   http://www.voxgratia.org/modules/news/
py2cairo                    1.10.0                25 Apr 2015   http://cairographics.org/releases/py2cairo-1.10.0.tar.bz2
pyatspi                     2.32.1                09 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/pyatspi/2.32/pyatspi-2.32.1.tar.xz
pybliographer               1.4.0                 05 Apr 2018   http://ftp.gnome.org/pub/gnome/sources/pybliographer/1.4/pybliographer-1.4.0.tar.xz
pycairo                     1.18.1                21 Apr 2019   https://github.com/pygobject/pycairo/releases/download/v1.18.1/pycairo-1.18.1.tar.gz
pycdio                      2.0.0                 06 Feb 2018   https://ftp.gnu.org/pub/gnu/libcdio/pycdio-2.0.0.tar.gz
pycha                       0.7.0                 01 Jun 2014   https://pypi.python.org/packages/source/p/pycha/pycha-0.7.0.tar.gz
pycp                        8.0.6                 18 Jan 2019   https://files.pythonhosted.org/packages/fa/2c/b01afb9eacaaeef8c9a733080da62c68545bdbced3c07ae78faad95d3af0/pycp-8.0.6.tar.gz
pycups                      1.9.74                13 Oct 2018   https://files.pythonhosted.org/packages/00/ce/836a0deb8b24bcd5f850f8fb97f99fb4abd7374e078b9e6df5a0838f8eb5/pycups-1.9.74.tar.bz2
pycurl                      7.43.0.2              25 Sep 2018   https://dl.bintray.com/pycurl/pycurl/pycurl-7.43.0.2.tar.gz
pydisplay                   0.2.0                 01 Jun 2014   http://downloads.sourceforge.net/pydisplay/pydisplay-0.2.0.tar.gz
pygame                      1.9.4                 05 Dec 2018   https://www.pygame.org/ftp/pygame-1.9.4.tar.gz
pyglet                      1.0                   01 Nov 2012   http://pyglet.googlecode.com/files/pyglet-1.0.tar.gz
pygobject                   3.32.2                23 Jun 2019   http://ftp.gnome.org/pub/gnome/sources/pygobject/3.32/pygobject-3.32.2.tar.xz
pygraphviz                  0.22                  01 Jun 2014   http://surfnet.dl.sourceforge.net/sourceforge/networkx/pygraphviz-0.22.tar.gz
pygtk                       2.24.0                01 Aug 2012   http://ftp.gnome.org/pub/GNOME/sources/pygtk/2.24/pygtk-2.24.0.tar.bz2
pygtksourceview             2.10.1                12 May 2019   http://ftp.gnome.org/pub/gnome/sources/pygtksourceview/2.10/pygtksourceview-2.10.1.tar.gz
pygui                       2.2                   01 Jun 2014   http://www.cosc.canterbury.ac.nz/greg.ewing/python_gui/PyGUI-2.2.tar.gz
pymol                       v2.1.0                14 Jan 2019   https://sourceforge.net/projects/pymol/files/pymol/2/pymol-v2.1.0.tar.bz2
pymysql                     0.9.2                 05 Dec 2018   https://files.pythonhosted.org/packages/d9/ab/bc9be64876fc9b1590d8ae62b471981b4489c23d6c3f0319af570748d408/PyMySQL-0.9.2.tar.gz
pyorbit                     2.24.0                01 Jun 2014   http://ftp.gnome.org/pub/GNOME/sources/pyorbit/2.24/pyorbit-2.24.0.tar.bz2
pyparsing                   2.4.0                 18 Apr 2019   https://files.pythonhosted.org/packages/5d/3a/24d275393f493004aeb15a1beae2b4a3043526e8b692b65b4a9341450ebe/pyparsing-2.4.0.tar.gz
pype                        2.9.4                 01 Jan 2013   http://sourceforge.net/projects/pype/files/pype/PyPE%202.9.4/PyPE-2.9.4-src.zip
pypy                        v6.0.0                31 Jul 2018   https://bitbucket.org/pypy/pypy/downloads/pypy3-v6.0.0-src.tar.bz2
pyqt                        4.12.1                03 Sep 2017   http://sourceforge.net/projects/pyqt/files/PyQt4/PyQt-4.12.1/PyQt4_gpl_x11-4.12.1.tar.gz
pyrex                       0.9.9                 06 Oct 2017   http://www.cosc.canterbury.ac.nz/greg.ewing/python/Pyrex/Pyrex-0.9.9.tar.gz
pyserial                    2.2                   01 Jan 2013   http://sourceforge.net/project/showfiles.php?group_id=46487&package_id=39324&release_id=346093
pysqlite                    2.6.3                 01 Jun 2014   http://pysqlite.googlecode.com/files/pysqlite-2.6.3.tar.gz
python                      3.7.4                 09 Jul 2019   https://www.python.org/ftp/python/3.7.4/Python-3.7.4.tgz
pythoncaja                  1.22.0                07 Mar 2019   http://pub.mate-desktop.org/releases/1.22/python-caja-1.22.0.tar.xz
pythondbusmock              0.16.9                24 Jul 2016   https://pypi.python.org/packages/dd/c6/3d1979a07190ec9242966112ce134d3c32cf081f90a4f67bc9385d08ba77/python-dbusmock-0.16.9.tar.gz
pythonefl                   1.20.0                06 Jul 2018   https://download.enlightenment.org/rel/bindings/python/python-efl-1.20.0.tar.xz
pytone                      3.0.3                 01 Jun 2014   http://www.luga.de/pytone/download/PyTone-3.0.3.tar.gz
pyvorbis                    1.4                   01 Aug 2013   http://ekyo.nerim.net/software/pyogg/index.html
pywal                       1.7.4                 25 Jan 2016   https://launchpad.net/wicd/1.7/1.7.4/+download/wicd-1.7.4.tar.gz
pyxdg                       0.26                  24 Feb 2019   https://files.pythonhosted.org/packages/47/6e/311d5f22e2b76381719b5d0c6e9dc39cd33999adae67db71d7279a6d70f4/pyxdg-0.26.tar.gz
pyxfce                      4.3.0                 01 Jun 2014   http://pyxfce.xfce.org/pyxfce-4.3.0.tar.gz
pyxml                       0.8.4                 01 Jun 2014   http://mesh.dl.sourceforge.net/sourceforge/pyxml/PyXML-0.8.4.tar.gz
pyyaml                      5.1                   28 Mar 2019   https://pyyaml.org/download/pyyaml/PyYAML-5.1.tar.gz
qbittorrent                 4.1.6                 30 May 2019   https://sourceforge.net/projects/qbittorrent/files/qbittorrent/qbittorrent-4.1.6/qbittorrent-4.1.6.tar.xz
qca                         2.2.0                 19 Apr 2019   https://download.kde.org/stable/qca/2.2.0/qca-2.2.0.tar.xz
qcaqt5                      2.1.0.3               22 Sep 2017   https://download.kde.org/stable/qca-qt5/2.1.0.3/src/qca-qt5-2.1.0.3.tar.xz
qdbm                        1.8.77                12 Sep 2015   http://sourceforge.net/projects/qdbm/files/qdbm/1.8.77/qdbm-1.8.77.tar.gz
qdvdauthor                  2.3.1                 01 Sep 2014   http://sourceforge.net/projects/qdvdauthor/files/qdvdauthor/2.3.1/qdvdauthor-2.3.1.tar.gz
qemu                        4.0.0                 24 Apr 2019   http://wiki.qemu.org/download/qemu-4.0.0.tar.bz2
qgis                        2.18.13               20 Sep 2017   http://qgis.org/downloads/qgis-2.18.13.tar.bz2
qimageblitz                 0.0.6                 01 Jan 2014   http://download.kde.org/stable/qimageblitz/qimageblitz-0.0.6.tar.bz2
qimgv                       0.7.3                 01 Feb 2019   https://github.com/easymodo/qimgv/archive/v0.7.3.tar.gz
qingy                       1.0.0                 01 Jun 2014   http://sourceforge.net/projects/qingy/files/qingy/qingy%201.0.0/
qiv                         2.1pre12              01 Jun 2014   http://www.klografx.net/qiv/download/
qjackctl                    0.3.11                01 Apr 2014   http://downloads.sourceforge.net/qjackctl/qjackctl-0.3.11.tar.gz
qjson                       0.9.0                 24 May 2015   https://github.com/flavio/qjson/archive/0.9.0.tar.gz
qpdf                        8.4.2                 20 May 2019   https://sourceforge.net/projects/qpdf/files/qpdf/8.4.2/qpdf-8.4.2.tar.gz
qps                         1.10.20               27 Feb 2019   https://downloads.lxqt.org/downloads/qps/1.10.20/qps-1.10.20.tar.xz
qqc2desktopstyle            5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/qqc2-desktop-style-5.59.0.tar.xz
qqwing                      1.3.4                 09 Aug 2015   https://github.com/stephenostermiller/qqwing/archive/v1.3.4.zip
qrencode                    4.0.0                 31 Jan 2018   https://fukuchi.org/works/qrencode/qrencode-4.0.0.tar.bz2
qsynth                      0.5.5                 16 Mar 2019   http://downloads.sourceforge.net/qsynth/qsynth-0.5.5.tar.gz
qt                          5.12.3                17 Jun 2019   http://download.qt-project.org/official_releases/qt/5.12/5.12.4/single/qt-everywhere-src-5.12.4.tar.xz
qterminal                   0.14.1                27 Feb 2019   https://downloads.lxqt.org/downloads/qterminal/0.14.1/qterminal-0.14.1.tar.xz
qtermwidget                 0.14.1                27 Feb 2019   https://downloads.lxqt.org/downloads/qtermwidget/0.14.1/qtermwidget-0.14.1.tar.xz
qtgstreamer                 1.2.0                 10 Jul 2015   http://gstreamer.freedesktop.org/src/qt-gstreamer/qt-gstreamer-1.2.0.tar.xz
qtwebengineopensourcesrc    5.12.3                20 Apr 2019   https://download.qt.io/archive/qt/5.12/5.12.3/submodules/qtwebengine-everywhere-src-5.12.3.tar.xz
quadkonsole                 2.0.3                 01 Jun 2014   http://nomis80.org/quadkonsole/quadkonsole-2.0.3.tar.bz2
quadrapassel                3.31.3                06 Dec 2018   http://ftp.gnome.org/pub/gnome/sources/quadrapassel/3.31/quadrapassel-3.31.3.tar.xz
quagga                      1.2.2                 02 Jan 2018   http://download.savannah.gnu.org/releases/quagga/quagga-1.2.2.tar.gz
quazip                      0.8.1                 01 Jun 2019   https://github.com/stachenov/quazip/archive/v0.8.1.tar.gz
quelcom                     0.4.0                 01 Nov 2012   http://www.etse.urv.es/~dmanye/quelcom/src/quelcom-0.4.0.tar.gz
quodlibet                   3.9.1                 28 Jul 2017   https://github.com/quodlibet/quodlibet/releases/download/release-3.9.1/quodlibet-3.9.1.tar.gz
quota                       4.05                  01 Apr 2019   https://sourceforge.net/projects/linuxquota/files/quota-tools/4.05/quota-4.05.tar.gz
quvi                        0.9.5                 09 Jun 2016   https://sourceforge.net/projects/quvi/files/0.9/quvi/quvi-0.9.5.tar.xz
qwt                         6.0.2                 01 Dec 2012   http://sourceforge.net/projects/qwt/files/qwt/6.0.2/qwt-6.0.2.tar.bz2
r                           3.6.1                 09 Jul 2019   https://cran.r-project.org/src/base/R-3/R-3.6.1.tar.gz
rabbit                      2.2.1                 31 Oct 2017   https://rubygems.org/downloads/rabbit-2.2.1.gem
racc                        1.4.15                03 May 2019   https://rubygems.org/downloads/racc-1.4.15.gem
rack                        2.0.7                 14 Apr 2019   https://rubygems.org/downloads/rack-2.0.7.gem
rackaccept                  0.4.5                 04 Aug 2018   https://rubygems.org/downloads/rack-accept-0.4.5.gem
rackcomponent               0.5.0                 18 Feb 2019   https://rubygems.org/downloads/rack-component-0.5.0.gem
rackprotection              2.0.5                 26 Dec 2018   https://rubygems.org/downloads/rack-protection-2.0.5.gem
racktest                    1.1.0                 13 Oct 2018   https://rubygems.org/downloads/rack-test-1.1.0.gem
rage                        0.3.0                 13 Apr 2019   https://download.enlightenment.org/rel/apps/rage/rage-0.3.0.tar.xz
ragel                       6.10                  27 Mar 2017   http://www.colm.net/files/ragel/ragel-6.10.tar.gz
rails                       5.2.3                 03 May 2019   https://rubygems.org/downloads/rails-5.2.3.gem
railsdomtesting             2.0.3                 13 Oct 2018   https://rubygems.org/downloads/rails-dom-testing-2.0.3.gem
railshtmlsanitizer          1.0.4                 13 Oct 2018   https://rubygems.org/downloads/rails-html-sanitizer-1.0.4.gem
railties                    5.2.3                 03 May 2019   https://rubygems.org/downloads/railties-5.2.3.gem
rainbow                     3.0.0                 09 Jan 2018   https://rubygems.org/downloads/rainbow-3.0.0.gem
rak                         1.0.21                01 Jun 2014   rak-1.0.21.tar.xz
rake                        12.3.2                15 Dec 2018   https://rubygems.org/downloads/rake-12.3.2.gem
randrproto                  1.5.0                 17 May 2015   http://xorg.freedesktop.org/releases/individual/proto/randrproto-1.5.0.tar.gz
rant                        0.5.7                 01 Dec 2017   https://rubygems.org/downloads/rant-0.5.7.gem
raptor2                     2.0.15                22 May 2015   http://download.librdf.org/source/raptor2-2.0.15.tar.gz
rarian                      0.8.1                 01 Jan 2013   http://ftp.gnome.org/pub/gnome/sources/rarian/0.8/rarian-0.8.1.tar.bz2
rasmol                      2.7.5.2               01 Jun 2014   http://www.rasmol.org/software/RasMol_2.7.5.2.tar.gz
rasqal                      0.9.30                01 Jan 2014   http://download.librdf.org/source/rasqal-0.9.30.tar.gz
ratpoison                   1.4.9                 31 Jul 2018   http://download.savannah.nongnu.org/releases/ratpoison/ratpoison-1.4.9.tar.xz
rawrec                      0.9.991               01 Jun 2014   https://sourceforge.net/projects/rawrec/files/rawrec/rawrec-0.9.991/rawrec-0.9.991.tar.gz
rcairo                      1.15.14               27 Sep 2018   https://www.cairographics.org/releases/rcairo-1.15.14.tar.gz
rcov                        1.0.0                 31 Oct 2017   https://rubygems.org/downloads/rcov-1.0.0.gem
rdesktop                    1.8.6                 24 May 2019   https://github.com/rdesktop/rdesktop/releases/download/v1.8.6/rdesktop-1.8.6.tar.gz
rdtool                      0.6.38                17 Jan 2019   https://rubygems.org/downloads/rdtool-0.6.38.gem
re2c                        1.0.3                 17 Nov 2017   https://github.com/skvadrik/re2c/releases/download/1.0.3/re2c-1.0.3.tar.gz
readedid                    2.0.0                 01 Jun 2014   http://polypux.org/projects/read-edid/read-edid-2.0.0.tar.gz
readline                    8.0                   07 Jan 2019   https://ftp.gnu.org/pub/gnu/readline/readline-8.0.tar.gz
recode                      3.7.1                 12 May 2019   https://github.com/rrthomas/recode/releases/download/v3.7.1/recode-3.7.1.tar.gz
recordmydesktop             0.3.8.1               01 Jun 2014   http://sourceforge.net/projects/recordmydesktop/files/recordmydesktop/0.3.8.1/recordmydesktop-0.3.8.1.tar.gz
recordproto                 1.14.2                01 Jun 2014   http://xorg.freedesktop.org/releases/individual/proto/recordproto-1.14.2.tar.bz2
recutils                    1.8                   03 Jan 2019   https://ftp.gnu.org/pub/gnu/recutils/recutils-1.8.tar.gz
redcloth                    4.3.2                 21 Sep 2017   https://rubygems.org/downloads/RedCloth-4.3.2.gem
redis                       4.0.2                 21 Sep 2017   http://download.redis.io/releases/redis-4.0.2.tar.gz
redland                     1.0.17                01 Aug 2015   http://download.librdf.org/source/redland-1.0.17.tar.gz
redmine                     3.4.10                23 Apr 2019   http://www.redmine.org/releases/redmine-3.4.10.tar.gz
reft                        0.11                  01 Jun 2014   http://www.rafkind.com/jon/tar/reft-0.11.tar.gz
reiserfsprogs               3.6.27                03 Aug 2017   https://www.kernel.org/pub/linux/kernel/people/jeffm/reiserfsprogs/v3.6.27/reiserfsprogs-3.6.27.tar.xz
relion                      2.1                   08 Jan 2018   http://www2.mrc-lmb.cam.ac.uk/groups/scheres/1sep15/relion-2.1.tar.xz
rendercheck                 1.5                   12 Jun 2015   http://xorg.freedesktop.org/releases/individual/app/rendercheck-1.5.tar.bz2
renderproto                 0.11                  01 Jun 2014   http://xorg.freedesktop.org/releases/individual/proto/renderproto-0.11.tar.bz2
reportbuilder               1.8                   26 Sep 2018   https://rubygems.org/downloads/report_builder-1.8.gem
reportlab                   1.20                  01 Jun 2014   http://www.reportlab.org/ftp/ReportLab_1_20.tgz
resmgr                      1.0                   01 Jun 2014   ftp://ftp.lst.de/pub/people/okir/resmgr/resmgr-1.0.tar.bz2
resourceproto               1.2.0                 01 Jun 2014   http://xorg.freedesktop.org/releases/individual/proto/resourceproto-1.2.0.tar.bz2
rest                        0.8.1                 17 Oct 2017   http://ftp.gnome.org/pub/gnome/sources/rest/0.8/rest-0.8.1.tar.xz
restclient                  2.0.2                 21 Sep 2017   https://rubygems.org/downloads/rest-client-2.0.2.gem
retrogtk                    0.16.1                11 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/retro-gtk/0.16/retro-gtk-0.16.1.tar.xz
rex                         1.4.0                 09 Mar 2016   http://www.cpan.org/authors/id/F/FE/FERKI/Rex-1.4.0.tar.gz
rezound                     0.13.1beta            31 Oct 2017   https://sourceforge.net/projects/rezound/files/ReZound/0.13.1beta/rezound-0.13.1beta.tar.gz
rfkill                      0.5                   24 Oct 2018   https://mirrors.edge.kernel.org/pub/software/network/rfkill/rfkill-0.5.tar.xz
rfpdf                       1.5.3f                01 Jun 2014   http://zeropluszero.com/software/fpdf/rfpdf153g.tar.gz
rgb                         1.0.6                 31 Jul 2016   http://xorg.freedesktop.org/archive/individual/app/rgb-1.0.6.tar.bz2
rhythmbox                   3.4.3                 06 Jan 2019   http://ftp.gnome.org/pub/gnome/sources/rhythmbox/3.4/rhythmbox-3.4.3.tar.xz
rili                        2.0.1                 28 May 2015   https://sourceforge.net/projects/ri-li/files/Ri-li%20Linux_Unix/Ri-li%20V2.0.1/Ri-li-2.0.1.tar.bz2
rinruby                     2.1.0                 19 Aug 2018   https://rubygems.org/downloads/rinruby-2.1.0.gem
ripoff                      0.8.3                 01 Jan 2013   http://sourceforge.net/projects/ripoffc/files/RipOff/0.8.3/ripoff-0.8.3.tar.gz
ripperx                     2.7.2                 01 Jun 2014   http://sourceforge.net/projects/ripperx/
rison                       2.1.0                 01 Dec 2017   https://rubygems.org/downloads/rison-2.1.0.gem
ristretto                   0.8.4                 09 Apr 2019   http://archive.xfce.org/src/apps/ristretto/0.8/ristretto-0.8.4.tar.bz2
rkward                      0.6.5                 16 Sep 2017   http://download.kde.org/stable/rkward/0.6.5/src/rkward-0.6.5.tar.gz
rlog                        1.3.7                 01 Jun 2014   http://www.arg0.net/rlog
rlwrap                      0.37                  01 Jun 2014   http://utopia.knoware.nl/~hlub/uck/rlwrap/rlwrap-0.37.tar.gz
rmagick                     3.1.0                 03 May 2019   https://rubygems.org/downloads/rmagick-3.1.0.gem
rman                        3.2                   01 Jun 2014   http://switch.dl.sourceforge.net/sourceforge/polyglotman/rman-3.2.tar.gz
rmovie                      0.5.1                 01 Dec 2017   https://rubygems.org/downloads/rmovie-0.5.1.gem
rnalila                     0.2                   26 Jan 2018   https://github.com/mtw/RNAlila/releases/RNAlila-0.2.tar.xz
rnaz                        2.1                   29 Jun 2017   https://www.tbi.univie.ac.at/software/RNAz/RNAz-2.1.tar.gz
robodoc                     4.99.34               01 May 2014   http://sourceforge.net/projects/robodoc/
rocksndiamonds              3.2.4                 01 Nov 2012   http://www.artsoft.org/rocksndiamonds/download.html
rocs                        19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/rocs-19.04.2.tar.xz
roo                         2.8.2                 18 Feb 2019   https://rubygems.org/downloads/roo-2.8.2.gem
rope                        0.11.0                24 Dec 2018   https://files.pythonhosted.org/packages/52/8d/2ebe70e55742a46813952493f8fc86ff2800ccc105e2dfcb25298f7eeb72/rope-0.11.0.tar.gz
rosegarden                  19.06                 07 Jun 2019   https://sourceforge.net/projects/rosegarden/files/rosegarden/19.06/rosegarden-19.06.tar.bz2
rouge                       3.3.0                 12 Oct 2018   https://rubygems.org/downloads/rouge-3.3.0.gem
roxclib                     2.1.10                21 Feb 2016   http://www.kerofin.demon.co.uk/rox/ROX-CLib-2.1.10.tar.gz
roxsession                  0.1.25                01 Jun 2014   http://sourceforge.net/project/showfiles.php?group_id=7023
roxterm                     3.3.2                 19 Sep 2018   https://sourceforge.net/projects/roxterm/files/roxterm/3.3.2/roxterm-3.3.2.tar.xz
rpcbind                     1.2.5                 03 Sep 2018   https://sourceforge.net/projects/rpcbind/files/rpcbind/1.2.5/rpcbind-1.2.5.tar.bz2
rpcsvcproto                 1.4                   30 May 2018   https://github.com/thkukuk/rpcsvc-proto/archive/v1.4/rpcsvc-proto-1.4.tar.gz
rpm                         4.14.2.1              22 Oct 2018   http://ftp.rpm.org/releases/rpm-4.14.x/rpm-4.14.2.1.tar.bz2
rrdtool                     1.2.11                01 Jun 2014   http://people.ee.ethz.ch/~oetiker/webtools/rrdtool/pub/rrdtool-1.2.11.tar.gz
rspec                       3.8.0                 06 Aug 2018   https://rubygems.org/downloads/rspec-3.8.0.gem
rspeccore                   3.8.0                 06 Aug 2018   https://rubygems.org/downloads/rspec-core-3.8.0.gem
rspecexpectations           3.8.3                 03 May 2019   https://rubygems.org/downloads/rspec-expectations-3.8.3.gem
rspecsupport                3.8.0                 06 Aug 2018   https://rubygems.org/downloads/rspec-support-3.8.0.gem
rstart                      1.0.5                 01 Feb 2013   http://xorg.freedesktop.org/releases/individual/app/rstart-1.0.5.tar.bz2
rsync                       3.1.3                 29 Jan 2018   http://rsync.samba.org/ftp/rsync/rsync-3.1.3.tar.gz
rtorrent                    01.10.2017            30 Sep 2017   https://github.com/rakshasa/rtorrent-01.10.2017
rtranscoder                 0.1.3                 01 Dec 2017   https://rubygems.org/downloads/rtranscoder-0.1.3.gem
rubberband                  1.8.1                 17 Apr 2018   https://github.com/falkTX/rubberband/archive/v1.8.1.tar.gz
rubinius                    4.1                   01 Jun 2019   https://rubinius-releases-rubinius-com.s3.amazonaws.com/rubinius-4.1.tar.bz2
rubocop                     0.68.1                03 May 2019   https://rubygems.org/downloads/rubocop-0.68.1.gem
ruby                        2.6.3                 17 Apr 2019   https://cache.ruby-lang.org/pub/ruby/2.6/ruby-2.6.3.tar.gz
rubygame                    2.6.4                 01 Dec 2017   https://rubygems.org/downloads/rubygame-2.6.4.gem
rubygems                    3.0.3                 05 Mar 2019   https://rubygems.org/rubygems/rubygems-3.0.3.tgz
rubygnome2                  3.2.1                 15 Dec 2017   https://sourceforge.net/projects/ruby-gnome2/files/ruby-gnome2/ruby-gnome2-3.2.1/ruby-gnome2-all-3.2.1.tar.gz
rubygraphviz                1.2.4                 17 Nov 2018   https://rubygems.org/downloads/ruby-graphviz-1.2.4.gem
rubyinline                  3.12.4                02 Apr 2018   https://rubygems.org/downloads/RubyInline-3.12.4.gem
rubyjs                      0.8.0                 01 Dec 2017   https://rubygems.org/downloads/rubyjs-0.8.0.gem
rubyprogressbar             1.10.0                08 Aug 2018   https://rubygems.org/downloads/ruby-progressbar-1.10.0.gem
rubystats                   0.2.3                 01 May 2014   http://rubyforge.org/frs/download.php/39488/rubystats-0.2.3.tgz
rubytorrent                 0.3                   02 Dec 2017   https://rubygems.org/downloads/rubytorrent-0.3.gem
rubyxosd                    1.1.0                 01 May 2014   https://rubyforge.org/projects/ruby-xosd/
rubyzip                     1.2.2                 26 Sep 2018   https://rubygems.org/downloads/rubyzip-1.2.2.gem
rungetty                    1.2                   01 Jun 2014   http://packages.debian.org/testing/admin/rungetty.html
runit                       1.8.0                 01 May 2014   http://smarden.sunsite.dk/runit/runit-1.8.0.tar.gz
ruport                      1.7.1                 01 Dec 2017   https://rubygems.org/downloads/ruport-1.7.1.gem
rust                        1.34.1                29 Apr 2019   https://github.com/rust-lang/rust
ruvi                        0.4.12                01 Jun 2014   https://rubygems.org/downloads/ruvi-0.4.12.gem
rxvt                        2.7.10                01 May 2014   https://sourceforge.net/projects/rxvt/files/rxvt-dev/2.7.10/rxvt-2.7.10.tar.gz
rxvtunicode                 9.22                  07 Jun 2018   http://dist.schmorp.de/rxvt-unicode/rxvt-unicode-9.22.tar.bz2
rygel                       0.38.1                25 May 2019   http://ftp.gnome.org/pub/gnome/sources/rygel/0.38/rygel-0.38.1.tar.xz
sakura                      3.6.0                 10 May 2018   https://launchpad.net/sakura/trunk/3.6.0/+download/sakura-3.6.0.tar.bz2
samba                       4.10.6                08 Jul 2019   http://us1.samba.org/samba/ftp/stable/samba-4.10.6.tar.gz
samtools                    1.9                   30 Nov 2018   https://sourceforge.net/projects/samtools/files/samtools/1.9/samtools-1.9.tar.bz2
sanebackends                1.0.27                23 May 2017   https://alioth.debian.org/frs/download.php/file/4224/sane-backends-1.0.27.tar.gz
sanefrontends               1.0.14                01 Jun 2014   http://www.sane-project.org/snapshots/sane-frontends-git20180826.tar.gz
sash                        3.7                   01 May 2014   http://members.tip.net.au/~dbell/programs/sash-3.7.tar.gz
sass                        3.7.4                 03 May 2019   https://rubygems.org/downloads/sass-3.7.4.gem
sassrails                   5.0.7                 09 Jan 2018   https://rubygems.org/downloads/sass-rails-5.0.7.gem
sawfish                     1.3.3                 01 Jan 2014   http://download.tuxfamily.org/sawfish/sawfish-1.9.1.tar.xz
schroedinger                1.0.11                19 Jan 2016   https://launchpad.net/schroedinger/trunk/1.0.11/+download/schroedinger-1.0.11.tar.gz
scim                        1.4.18                19 Mar 2019   https://sourceforge.net/projects/scim/files/scim/1.4.17/scim-1.4.17.tar.gz
scintilla                   4.1.4                 10 Oct 2018   https://www.scintilla.org/scintilla414.tgz
scipy                       1.0.0                 26 Oct 2017   https://github.com/scipy/scipy/releases/download/v1.0.0/scipy-1.0.0.tar.xz
scite                       4.1.4                 10 Mar 2019   https://www.scintilla.org/scite414.tgz
scons                       3.0.5                 28 Mar 2019   http://prdownloads.sourceforge.net/scons/scons-3.0.5.tar.gz
scourge                     0.15                  01 Dec 2012   http://scourge.svn.sourceforge.net/viewvc/scourge/scourge2.tar.gz
screen                      4.6.2                 23 Oct 2017   http://ftp.gnu.org/gnu/screen/screen-4.6.2.tar.gz
screengrab                  1.101                 27 Feb 2019   https://downloads.lxqt.org/downloads/screengrab/1.101/screengrab-1.101.tar.xz
scribus                     1.4.8                 06 Mar 2019   https://downloads.sourceforge.net/scribus/scribus-1.4.8.tar.xz
scripts                     1.0.1                 01 May 2014   http://xorg.freedesktop.org/releases/individual/app/scripts-1.0.1.tar.bz2
scrnsaverproto              1.2.2                 01 May 2014   http://xorg.freedesktop.org/releases/individual/proto/scrnsaverproto-1.2.2.tar.bz2
scrollkeeper                0.3.14                01 Mar 2014   ftp://ftp.gnome.org/pub/GNOME/sources/scrollkeeper/0.3/scrollkeeper-0.3.14.tar.bz2
scrot                       0.8                   01 May 2014   http://linuxbrit.co.uk/downloads/scrot-0.8.tar.gz
scruffy                     0.2.6                 02 Dec 2017   https://rubygems.org/downloads/scruffy-0.2.6.gem
scummvm                     2.0.0                 15 Jan 2018   http://www.scummvm.org/frs/scummvm/2.0.0/scummvm-2.0.0.tar.xz
sddm                        0.16.0                10 Oct 2017   https://github.com/sddm/sddm/releases/download/v0.16.0/sddm-0.16.0.tar.xz
sddmkcm                     5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/sddm-kcm-5.16.3.tar.xz
sdl                         1.2.15                01 Jun 2013   http://www.libsdl.org/release/SDL-1.2.15.tar.gz
sdl2                        2.0.9                 01 Nov 2018   http://www.libsdl.org/release/SDL2-2.0.9.tar.gz
sdl2gfx                     1.0.4                 01 Nov 2018   https://sourceforge.net/projects/sdl2gfx/files/SDL2_gfx-1.0.4.tar.gz
sdl2image                   2.0.4                 01 Nov 2018   http://www.libsdl.org/projects/SDL_image/release/SDL2_image-2.0.4.tar.gz
sdl2mixer                   2.0.4                 01 Nov 2018   http://www.libsdl.org/projects/SDL_mixer/release/SDL2_mixer-2.0.4.tar.gz
sdl2net                     2.0.1                 16 Mar 2019   https://www.libsdl.org/projects/SDL_net/release/SDL2_net-2.0.1.tar.gz
sdl2ttf                     2.0.15                27 Jan 2019   http://www.libsdl.org/projects/SDL_ttf/release/SDL2_ttf-2.0.15.tar.gz
sdlgfx                      2.0.22                01 May 2014   http://www.ferzkopp.net/Software/SDL_gfx-2.0/SDL_gfx-2.0.22.tar.gz
sdlimage                    1.2.12                01 May 2014   http://www.libsdl.org/projects/SDL_image/release/SDL_image-1.2.12.tar.gz
sdlmixer                    1.2.12                01 Dec 2012   http://www.libsdl.org/projects/SDL_mixer/release/SDL_mixer-1.2.12.tar.gz
sdlmm                       0.1.8                 01 May 2014   http://sdlmm.sourceforge.net/
sdlnet                      1.2.8                 01 Nov 2012   https://www.libsdl.org/projects/SDL_net/release/SDL_net-1.2.8.tar.gz
sdlpango                    0.1.2                 01 Nov 2012   http://sourceforge.net/projects/sdlpango/
sdlperl                     1.20.3                16 Jul 2016   http://backpan.perl.org/authors/id/D/DG/DGOEHRIG/SDL_Perl-2.2.0.tar.gz
sdlpop                      18.06.2018            16 Jun 2018   https://github.com/NagyD/SDLPoP
sdltty                      0.0.1                 01 May 2014   http://www.libsdl.org/projects/
sdoc                        1.0.0                 13 Feb 2018   https://rubygems.org/downloads/sdoc-1.0.0.gem
sdparm                      1.02                  01 May 2014   http://sg.torque.net/sg/sdparm.html#mozTocId166213
seaborn                     0.8.1                 28 Jun 2018   https://files.pythonhosted.org/packages/10/01/dd1c7838cde3b69b247aaeb61016e238cafd8188a276e366d36aa6bcdab4/seaborn-0.8.1.tar.gz
seahorse                    3.32.2                17 May 2019   http://ftp.gnome.org/pub/gnome/sources/seahorse/3.32/seahorse-3.32.2.tar.xz
seahorsenautilus            3.11.92               01 Mar 2014   http://ftp.gnome.org/pub/GNOME/sources/seahorse-nautilus/3.11/seahorse-nautilus-3.11.92.tar.xz
seamonkey                   2.49.3                03 May 2018   https://ftp.mozilla.org/pub/seamonkey/releases/2.49.3/source/seamonkey-2.49.3.source.tar.xz
seatris                     0.0.14                01 May 2014   http://www.earth.li/projectpurple/files/seatris-0.0.14.tar.gz
sed                         4.7                   21 Dec 2018   ftp://ftp.gnu.org/gnu/sed/sed-4.7.tar.xz
segatex                     5.70                  01 May 2014   https://sourceforge.net/projects/segatex/files/?source=navbar
semanticpuppet              1.0.2                 06 Nov 2018   https://rubygems.org/downloads/semantic_puppet-1.0.2.gem
semantik                    1.0.3                 15 Jun 2018   https://waf.io/semantik-1.0.3.tar.bz2
sendmail                    8.15.2                17 Jul 2015   ftp://ftp.sendmail.org/pub/sendmail/sendmail.8.15.2.tar.gz
sequel                      5.20.0                03 May 2019   https://rubygems.org/downloads/sequel-5.20.0.gem
serf                        1.3.9                 03 Jan 2018   https://archive.apache.org/dist/serf/serf-1.3.9.tar.bz2
sessreg                     1.1.2                 11 Jun 2019   https://www.x.org/releases/individual/app/sessreg-1.1.2.tar.gz
setconf                     0.7.6                 24 Dec 2018   https://files.pythonhosted.org/packages/e2/13/e3d15834bdbbfcfff4a93639541817be2db36263bfb92868187e2db35df2/setconf-0.7.6.tar.gz
setuptools                  41.0.1                24 Apr 2019   https://files.pythonhosted.org/packages/1d/64/a18a487b4391a05b9c7f938b94a16d80305bf0369c6b0b9509e86165e1d3/setuptools-41.0.1.zip
setxkbmap                   1.3.1                 18 Apr 2015   http://xorg.freedesktop.org/releases/individual/app/setxkbmap-1.3.1.tar.gz
sg3utils                    1.44                  05 Oct 2018   http://sg.danny.cz/sg/p/sg3_utils-1.44.tar.xz
sgmlcommon                  0.6.3                 10 Jul 2015   ftp://sources.redhat.com/pub/docbook-tools/new-trials/SOURCES/sgml-common-0.6.3.tgz
shadow                      4.7                   18 Jun 2019   https://github.com/shadow-maint/shadow/releases/download/4.7/shadow-4.7.tar.xz
shapelib                    1.3.0                 01 Nov 2013   http://download.osgeo.org/shapelib/shapelib-1.3.0.tar.gz
shareddesktopontologies     0.11.0                01 Oct 2013   http://sourceforge.net/projects/oscaf/files/shared-desktop-ontologies/0.11.0/shared-desktop-ontologies-0.11.0.tar.bz2
sharedmimeinfo              1.12                  26 Mar 2019   https://gitlab.freedesktop.org/xdg/shared-mime-info/uploads/80c7f1afbcad2769f38aeb9ba6317a51/shared-mime-info-1.12.tar.xz
sharutils                   4.14                  01 Jul 2014   http://ftp.gnu.org/gnu/sharutils/sharutils-4.14.tar.xz
shed                        1.15                  01 May 2014   http://sourceforge.net/projects/shed/files/shed/shed%201.15/shed-1.15.tar.gz
shepherd                    0.6.0                 23 Apr 2019   https://ftp.gnu.org/pub/gnu/shepherd/shepherd-0.6.0.tar.gz
shindo                      0.3.8                 25 Dec 2017   https://rubygems.org/downloads/shindo-0.3.8.gem
shine                       21.05.2015            20 May 2015   shine-21.05.2015
shipping                    1.6.0                 01 Dec 2017   https://rubygems.org/downloads/shipping-1.6.0.gem
shmux                       1.0.2                 01 Nov 2012   http://web.taranis.org/shmux/dist/shmux-1.0.2.tgz
shntool                     3.0.7                 01 May 2014   http://www.etree.org/shnutils/shntool/dist/src/shntool-3.0.7.tar.gz
shoes                       1.0.0                 01 May 2014   https://github.com/shoes/shoes
shorturl                    1.0.0                 01 May 2014   https://rubygems.org/downloads/shorturl-1.0.0.gem
shotcut                     18.12.23              18 Jan 2019   https://github.com/mltframework/shotcut/archive/v18.12.23.tar.gz
shotgun                     0.9.2                 02 Aug 2017   https://rubygems.org/downloads/shotgun-0.9.2.gem
shotwell                    0.30.4                25 Apr 2019   https://download.gnome.org/sources/shotwell/0.30/shotwell-0.30.4.tar.xz
showfont                    1.0.5                 31 Jul 2016   http://xorg.freedesktop.org/archive/individual/app/showfont-1.0.5.tar.bz2
sidekiq                     5.2.7                 03 May 2019   https://rubygems.org/downloads/sidekiq-5.2.7.gem
signonkwalletextension      19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/signon-kwallet-extension-19.04.2.tar.xz
sigopt                      1.3.0                 04 Apr 2016   https://pypi.python.org/packages/source/s/sigopt/sigopt-1.3.0.tar.gz#md5=ccd67d5c514c1a20659f6217686f0782
silk                        3.10.1                08 Mar 2015   https://tools.netsa.cert.org/releases/silk-3.10.1.tar.gz
simplescan                  3.32.2                10 Apr 2019   http://ftp.gnome.org/pub/gnome/sources/simple-scan/3.32/simple-scan-3.32.2.tar.xz
simsoup                     0.3                   01 May 2014   http://www.simsoup.info/SimSoup-0.3-src.tar.gz
sinatra                     2.0.5                 26 Dec 2018   https://rubygems.org/downloads/sinatra-2.0.5.gem
sinatracontrib              2.0.5                 13 Apr 2019   https://rubygems.org/downloads/sinatra-contrib-2.0.5.gem
sinatrareloader             1.0                   26 Dec 2018   https://rubygems.org/downloads/sinatra-reloader-1.0.gem
sip                         4.19.13               08 Nov 2018   https://sourceforge.net/projects/pyqt/files/sip/sip-4.19.13/sip-4.19.13.tar.gz
sipwitch                    1.9.1                 01 May 2014   http://ftp.gnu.org/gnu/sipwitch/sipwitch-1.9.1.tar.gz
sisiya                      0.4-21                01 May 2014   http://ovh.dl.sourceforge.net/sourceforge/sisiya/sisiya-0.4-21.tar.gz
sitemapgenerator            6.0.2                 18 Feb 2019   https://rubygems.org/downloads/sitemap_generator-6.0.2.gem
sithwm                      1.2.3                 01 May 2014   http://sithwm.darkside.no/sn/sithwm-1.2.3.tgz
six                         1.12.0                14 Dec 2018   https://files.pythonhosted.org/packages/dd/bf/4138e7bfb757de47d1f4b6994648ec67a51efe58fa907c1e11e350cddfca/six-1.12.0.tar.gz
skijump                     0.2.0                 01 May 2014   http://prdownloads.sourceforge.net/skijump/skijump-0.2.0.tar.gz
slang                       2.3.1a                25 Jul 2017   http://www.jedsoft.org/releases/slang/slang-2.3.1a.tar.bz2
slaptget                    0.10.2f               01 May 2014   http://software.jaos.org/source/slapt-get/slapt-get-0.10.2f.tar.gz
slave                       0.2.0                 01 May 2014   http://www.codeforpeople.com/lib/ruby/slave/slave-0.2.0.tgz
sleepypenguin               3.1.0                 01 Nov 2012   http://bogomips.org/sleepy_penguin.git/snapshot/sleepy_penguin-3.1.0.tar.gz
slib                        3b5                   16 Dec 2018   http://groups.csail.mit.edu/mac/ftpdir/scm/slib-3b5.tar.gz
slideshow                   0.1.0                 01 May 2014   https://github.com/ext/slideshow
slim                        4.0.1                 26 Sep 2018   https://rubygems.org/downloads/slim-4.0.1.gem
slocate                     3.1                   01 Jan 2014   http://slocate.trakker.ca/files/slocate-3.1.tar.gz
sloccount                   2.26                  01 May 2014   http://sourceforge.net/projects/sloccount/files/sloccount-2.26.tar.gz
slop                        4.6.2                 19 Mar 2018   https://rubygems.org/downloads/slop-4.6.2.gem
slurp                       1.1.0                 22 Feb 2019   https://github.com/emersion/slurp/archive/v1.1.0.tar.gz
smake                       1.2.5                 07 Feb 2016   http://sourceforge.net/projects/s-make/files/smake-1.2.5.tar.gz
smalltalk                   3.2.5                 01 Nov 2013   ftp://ftp.gnu.org/gnu/smalltalk/smalltalk-3.2.5.tar.xz
smartmontools               7.0                   02 Jan 2019   https://sourceforge.net/projects/smartmontools/files/smartmontools/7.0/smartmontools-7.0.tar.gz
smc                         1.9                   01 Dec 2012   http://downloads.sourceforge.net/smclone/smc-1.9.tar.bz2
smf                         0.15.12               01 Jan 2013   http://www.funaba.org/archive/smf-0.15.12.tar.gz
smokeqt                     4.14.3                15 Jul 2015   ftp://gd.tuwien.ac.at/kde/stable/4.14.3/src/smokeqt-4.14.3.tar.xz
smpeg                       18.11.2011            01 Mar 2013   http://www.3ddownloads.com/linuxgames/loki/open-source/smpeg/smpeg-0.4.3.tar.gz;jsessionid=5423E28B7AB7B5592BC27E91975EC441#
smplayer                    19.1.0                29 Jan 2019   https://sourceforge.net/projects/smplayer/files/SMPlayer/19.1.0/smplayer-19.1.0.tar.bz2
smproxy                     1.0.6                 18 Apr 2015   http://xorg.freedesktop.org/releases/individual/app/smproxy-1.0.6.tar.gz
smuxi                       0.8.11                01 May 2014   https://www.smuxi.org/jaws/data/files/smuxi-0.8.11.tar.gz
snack                       2.2.10                01 Sep 2013   http://www.speech.kth.se/snack/dist/snack2.2.10.tar.gz
snake3d                     0.7                   01 May 2014   http://surfnet.dl.sourceforge.net/sourceforge/worms3d/snake3d-0.7.tgz
snappy                      1.0                   01 Apr 2014   http://ftp.gnome.org/pub/gnome/sources/snappy/1.0/snappy-1.0.tar.xz
snapscreenshot              1.0.14.3              01 Mar 2014   http://bisqwit.iki.fi/src/arch/snapscreenshot-1.0.14.3.tar.bz2
snd                         13.0                  01 May 2014   ftp://ccrma-ftp.stanford.edu/pub/Lisp/snd-13.0.tar.gz
sndlib                      1.0.1                 01 Nov 2012   ftp://ccrma-ftp.stanford.edu/pub/Lisp/sndlib.tar.gz
snes9x                      1.53                  08 Sep 2017   http://files.ipherswipsite.com/snes9x/snes9x-1.53-src.tar.bz2
snogray                     20.04.2008            01 May 2014   http://download.savannah.gnu.org/releases/snogray/snogray-20080420-0.tar.gz
snort                       2.9.12                11 Oct 2018   https://www.snort.org/downloads/snort/snort-2.9.12.tar.gz
soappy                      0.12.22               02 Aug 2017   https://pypi.python.org/packages/78/1b/29cbe26b2b98804d65e934925ced9810883bdda9eacba3f993ad60bfe271/SOAPpy-0.12.22.zip
solfege                     3.8.3                 01 May 2014   http://www.solfege.org/Solfege/Download
solid                       5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/solid-5.59.0.tar.xz
solseek                     0.9                   01 May 2014   https://www.gnome-look.org/p/1127043/
sonik                       1.0.0                 01 May 2014   http://prdownloads.sourceforge.net/sonik/sonik-1.0.0.tar.gz
sonnet                      5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/sonnet-5.59.0.tar.xz
soprano                     2.9.4                 01 Dec 2013   http://sourceforge.net/projects/soprano/files/Soprano/2.9.4/soprano-2.9.4.tar.bz2
sortversions                1.5                   26 Aug 2017   http://search.cpan.org/CPAN/authors/id/E/ED/EDAVIS/Sort-Versions-1.5.tar.gz
soundconverter              2.1.4                 20 Dec 2014   https://launchpad.net/soundconverter/trunk/2.1.4/+download/soundconverter-2.1.4.tar.xz
soundjuicer                 3.24.0                24 Mar 2017   http://ftp.gnome.org/pub/GNOME/sources/sound-juicer/3.24/sound-juicer-3.24.0.tar.xz
soundkonverter              2.1.2                 22 May 2015   http://kde-apps.org/content/show.php/soundKonverter?content=29024
soundtouch                  2.1.0                 16 Sep 2018   https://gitlab.com/soundtouch/soundtouch/-/archive/2.1.0/soundtouch-2.1.0.tar.bz2
sourceinstall               2.5rc2                01 May 2014   ftp://ftp.gnu.org/gnu/sourceinstall/sourceinstall-2.5rc2.tar.gz
sox                         14.4.2                12 Apr 2015   https://sourceforge.net/projects/sox/files/sox/14.4.2/sox-14.4.2.tar.gz
spdf                        0.2                   01 May 2014   http://users.tkk.fi/~jitulkki/spdf/spdf-0.2.tar.gz
spectacle                   19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/spectacle-19.04.2.tar.xz
speechd                     26.11.2013            01 Nov 2013   http://git.freebsoft.org/?p=speechd.git
speedcrunch                 0.10.1                01 May 2014   http://www.kde-apps.org/content/show.php/SpeedCrunch?content=16696
speex                       1.2.0                 16 Feb 2017   http://downloads.xiph.org/releases/speex/speex-1.2.0.tar.gz
speexdsp                    1.2.0                 10 Jun 2019   https://ftp.osuosl.org/pub/xiph/releases/speex/speexdsp-1.2.0.tar.gz
sphinx                      2.3.2-beta            09 Sep 2018   http://sphinxsearch.com/files/sphinx-2.3.2-beta.tar.gz
spice                       0.12.8                08 Aug 2016   http://www.spice-space.org/download/releases/spice-0.12.8.tar.bz2
spiceprotocol               0.12.12               08 Aug 2016   http://spice-space.org/download/releases/spice-protocol-0.12.12.tar.bz2
spidermonkey                08.01.2007            01 Jun 2013   spidermonkey-08.01.2007
spkg                        1.0rc9                01 May 2014   http://spkg.megous.com/dl/releases/?C=M;O=D
splashy                     0.3.5                 01 May 2014   http://alioth.debian.org/frs/?group_id=30657
splint                      3.1.1                 01 May 2014   http://www.splint.org/downloads/splint-3.1.1.src.tgz
splitvt                     1.6.5                 01 May 2014   http://www.devolution.com/~slouken/projects/splitvt/splitvt-1.6.5.tar.gz
spreadsheet                 1.2.3                 03 May 2019   https://rubygems.org/downloads/spreadsheet-1.2.3.gem
spring                      2.0.2                 13 Oct 2018   https://rubygems.org/downloads/spring-2.0.2.gem
sprockets                   3.7.2                 24 Jun 2018   https://rubygems.org/downloads/sprockets-3.7.2.gem
sprocketsrails              3.2.1                 21 Sep 2017   https://rubygems.org/downloads/sprockets-rails-3.2.1.gem
spyc                        0.3beta               01 May 2014   http://spyc.sourceforge.net/
sqliteautoconf              3280000               17 Apr 2019   https://www.sqlite.org/2019/sqlite-autoconf-3280000.tar.gz
sqlitebrowser               3.11.0                05 Feb 2019   https://github.com/sqlitebrowser/sqlitebrowser/archive/v3.11.0.tar.gz
sqliteman                   1.0.1                 01 May 2014   http://sqliteman.com/
sqliteruby                  2.2.3                 28 Nov 2017   https://rubygems.org/downloads/sqlite-ruby-2.2.3.gem
squid                       4.8                   10 Jul 2019   http://www.squid-cache.org/Versions/v4/squid-4.8.tar.xz
sshfs                       3.5.2                 20 Apr 2019   https://github.com/libfuse/sshfs/releases/download/sshfs-3.5.2/sshfs-3.5.2.tar.xz
sslscan                     1.8.2                 01 Jan 2014   http://downloads.sourceforge.net/project/sslscan/sslscan/sslscan-1.8.2.tgz
ssysm                       0.2.4.3               01 May 2014   http://freshmeat.net/redir/ssysm/65338/url_tgz/ssysm-0.2.4.3.tar.gz
st                          0.8.1                 19 Sep 2018   https://dl.suckless.org/st/st-0.8.1.tar.gz
stager                      2.0.1                 01 May 2014   http://software.uninett.no/stager/download/Stager_2.0.1.tar.gz
startupnotification         0.12                  01 Jun 2013   http://www.freedesktop.org/software/startup-notification/releases/startup-notification-0.12.tar.gz
statifier                   1.7.1                 01 May 2014   http://sourceforge.net/projects/statifier/files/statifier/1.7.1/statifier-1.7.1.tar.gz
stax                        1.37                  01 Nov 2012   http://www.gamblin.ca/staxpc/
stella                      3.7.2                 01 May 2014   http://sourceforge.net/projects/stella/files/stella/3.7.2/stella-3.7.2-src.tar.gz
step                        19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/step-19.04.2.tar.xz
stfl                        0.23                  03 Oct 2017   http://www.clifford.at/stfl/stfl-0.23.tar.gz
stlport                     5.1.3                 01 May 2014   http://sourceforge.net/projects/stlport
stow                        2.3.0                 30 Jun 2019   http://ftp.gnu.org/pub/gnu/stow/stow-2.3.0.tar.gz
strace                      5.1                   23 May 2019   https://github.com/strace/strace/releases/download/v5.1/strace-5.1.tar.xz
strasheela                  0.8                   01 Mar 2013   http://heanet.dl.sourceforge.net/sourceforge/strasheela/strasheela-0.8.tar.gz
stratagus                   2.2.5.5               01 Nov 2012   http://launchpad.net/stratagus/trunk/2.2.5.5/+download/stratagus_2.2.5.5.orig.tar.gz
streamripper                1.64.6                01 May 2014   http://sourceforge.net/projects/streamripper/files/streamripper%20%28current%29/1.64.6/streamripper-1.64.6.tar.gz
streamtuner                 0.99.99               01 Apr 2014   http://savannah.nongnu.org/download/streamtuner/streamtuner-0.99.99.tar.gz
strigi                      0.7.8                 01 Jan 2014   http://www.vandenoever.info/software/strigi/strigi-0.7.8.tar.bz2
strongswan                  4.6.4                 01 Jun 2012   http://download.strongswan.org/strongswan-4.6.4.tar.bz2
stunnel                     5.55                  11 Jun 2019   ftp://ftp.stunnel.org/stunnel/stunnel-5.55.tar.gz
sublib                      0.9                   01 May 2014   http://sourceforge.net/projects/sublib/files/sublib/0.9/
subtitlecomposer            11.10.2017            10 Oct 2017   https://github.com/maxrd2/subtitlecomposer
subtitleeditor              0.40.0                01 May 2014   http://download.gna.org/subtitleeditor/0.40/subtitleeditor-0.40.0.tar.gz
subtle                      xi.tbz2               01 Nov 2012   http://subforge.org/attachments/download/81/subtle-0.11.3224-xi.tbz2
subversion                  1.12.0                26 Apr 2019   http://www.apache.org/dist/subversion/subversion-1.12.0.tar.bz2
sudo                        1.8.27                14 Jan 2019   ftp://ftp.sudo.ws/pub/sudo/sudo-1.8.27.tar.gz
superkaramba                4.14.3                15 Jul 2015   ftp://gd.tuwien.ac.at/kde/stable/4.14.3/src/superkaramba-4.14.3.tar.xz
supertux                    0.5.1                 17 Nov 2016   https://github.com/SuperTux/supertux/releases/download/v0.5.1/SuperTux-v0.5.1-Source.tar.gz
supybot                     0.83.1                01 May 2014   https://surfnet.dl.sourceforge.net/sourceforge/supybot/Supybot-0.83.1.tar.bz2
sushi                       3.32.0                10 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/sushi/3.32/sushi-3.32.0.tar.xz
suspendutils                1.0                   01 Sep 2013   http://sourceforge.net/projects/suspend/files/suspend/suspend-1.0/suspend-utils-1.0.tar.bz2
sussen                      0.90                  01 May 2014   http://dev.mmgsecurity.com/projects/sussen/
svgalib                     1.9.25                01 Aug 2013   http://www.svgalib.org/svgalib-1.9.25.tar.gz
svgpart                     19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/svgpart-19.04.2.tar.xz
sway                        1.0                   13 Mar 2019   https://github.com/swaywm/sway/archive/1.0.tar.gz
swbis                       1.13.1                06 Nov 2018   https://ftp.gnu.org/pub/gnu/swbis/swbis-1.13.1.tar.gz
sweep                       0.9.1                 01 May 2014   http://www.metadecks.org/software/sweep/download/sweep-0.9.1.tar.bz2
sweeper                     19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/sweeper-19.04.2.tar.xz
swellfoop                   3.30.0                09 Sep 2018   http://ftp.gnome.org/pub/gnome/sources/swell-foop/3.30/swell-foop-3.30.0.tar.xz
swig                        4.0.0                 30 Apr 2019   http://prdownloads.sourceforge.net/swig/swig-4.0.0.tar.gz
swinput                     0.7.4                 04 May 2019   http://download.savannah.nongnu.org/releases/swinput/swinput-0.7.4.tar.gz
swipl                       7.6.0-rc2             30 Sep 2017   http://www.swi-prolog.org/download/stable/src/swipl-7.6.0-rc2.tar.gz
swpkg                       0.51                  01 May 2014   http://web.taranis.org/swpkg/dist/swpkg-0.51.tgz
syck                        1.4.0                 04 Jan 2019   https://rubygems.org/downloads/syck-1.4.0.gem
sylpheed                    3.7.0                 01 Feb 2018   http://sylpheed.sraoss.jp/sylpheed/v3.7/sylpheed-3.7.0.tar.xz
sympa                       6.1.25                26 Nov 2017   http://www.sympa.org/distribution/sympa-6.1.25.tar.gz
sympy                       1.1.1                 27 Jul 2017   https://github.com/sympy/sympy/releases/download/sympy-1.1.1/sympy-1.1.1.tar.gz
synaptic                    0.84.2                10 Mar 2017   http://ftp.debian.org/debian/pool/main/s/synaptic/synaptic_0.84.2.tar.xz
syndication                 5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/syndication-5.59.0.tar.xz
synfig                      1.2.1                 08 Oct 2017   https://sourceforge.net/projects/synfig/files/releases/1.2.1/source/synfig-1.2.1.tar.gz
syntax                      1.2.2                 01 Dec 2017   https://rubygems.org/downloads/syntax-1.2.2.gem
syntaxhighlighting          5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/syntax-highlighting-5.59.0.tar.xz
sysdig                      0.18.0                25 Sep 2017   https://github.com/draios/sysdig/archive/0.18.0.tar.gz
sysklogd                    1.5                   01 May 2014   http://www.infodrom.org/projects/sysklogd/download/sysklogd-1.5.tar.gz
syslinux                    6.03                  16 Nov 2018   https://mirrors.edge.kernel.org/pub/linux/utils/boot/syslinux/syslinux-6.03.tar.xz
sysprof                     3.32.0                13 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/sysprof/3.32/sysprof-3.32.0.tar.xz
sysstat                     12.1.4                19 Apr 2019   http://pagesperso-orange.fr/sebastien.godard/sysstat-12.1.4.tar.gz
systemd                     228                   18 Nov 2015   https://www.freedesktop.org/software/systemd/systemd-228.tar.xz
systemsettings              5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/systemsettings-5.16.3.tar.xz
systemtap                   3.2                   01 Nov 2017   https://sourceware.org/systemtap/ftp/releases/systemtap-3.2.tar.gz
systemtoolsbackends         2.10.2                01 Feb 2014   http://ftp.gnome.org/pub/GNOME/sources/system-tools-backends/2.10/system-tools-backends-2.10.2.tar.gz
systemu                     2.2.0                 01 Nov 2012   http://codeforpeople.com/lib/ruby/systemu/systemu-2.2.0.tgz
sysuname                    1.0.4                 06 Nov 2018   https://rubygems.org/downloads/sys-uname-1.0.4.gem
sysvinit                    2.95                  16 Jun 2019   http://download.savannah.gnu.org/releases/sysvinit/sysvinit-2.95.tar.xz
tabbed                      0.6                   30 Jun 2015   http://dl.suckless.org/tools/tabbed-0.6.tar.gz
tabble                      0.43                  01 Sep 2013   http://www.rillion.net/tabble/tabble-0.43.tar.gz
taglib                      1.11.1                08 Jan 2018   http://taglib.org/releases/taglib-1.11.1.tar.gz
taglibruby                  0.7.1                 17 Dec 2018   https://rubygems.org/downloads/taglib-ruby-0.7.1.gem
tali                        3.19.92               16 Mar 2016   http://ftp.gnome.org/pub/GNOME/sources/tali/3.19/tali-3.19.92.tar.xz
talib                       0.4.10                30 Jul 2017   https://pypi.python.org/packages/c0/13/3d92ed6c09ed585ed58aeab5da6361ff2aeff7757d8224089bd574945716/TA-Lib-0.4.10.tar.gz
talloc                      2.2.0                 22 Mar 2019   https://www.samba.org/ftp/talloc/talloc-2.2.0.tar.gz
tangoicontheme              0.7.2                 01 May 2014   http://tango-project.org/releases/tango-icon-theme-0.7.2.tar.gz
tar                         1.32                  23 Feb 2019   ftp://ftp.gnu.org/gnu/tar/tar-1.32.tar.xz
tartan                      0.2.1                 01 Nov 2012   https://rubygems.org/downloads/tartan-0.2.1.gem
tasks                       3.3.90                01 Nov 2013   http://ftp.gnome.org/pub/GNOME/sources/tasks/3.3/tasks-3.3.90.tar.xz
tasque                      0.1.12                24 Sep 2014   http://ftp.gnome.org/pub/GNOME/sources/tasque/0.1/tasque-0.1.12.tar.xz
tcc                         0.9.26                01 Aug 2013   http://download.savannah.gnu.org/releases/tinycc/tcc-0.9.26.tar.bz2
tcl                         8.6.9                 15 Nov 2018   http://downloads.sourceforge.net/tcl/tcl8.6.9-src.tar.gz
tcpdump                     4.9.2                 05 Sep 2017   http://www.tcpdump.org/release/tcpdump-4.9.2.tar.gz
tcsh                        6.20.00               21 Aug 2017   https://fossies.org/linux/misc/tcsh-6.20.00.tar.gz/
tdb                         1.3.18                22 Mar 2019   https://www.samba.org/ftp/tdb/tdb-1.3.18.tar.gz
tea                         44.1.0                18 Jul 2017   http://semiletov.org/tea/dloads/tea-44.1.0.tar.bz2
ted                         2.21                  01 May 2014   ftp://ftp.nluug.nl/pub/editors/ted/ted-2.21.src.tar.gz
telegnome                   0.3.4                 12 Jan 2019   http://ftp.gnome.org/pub/gnome/sources/telegnome/0.3/telegnome-0.3.4.tar.xz
telepathyglib               0.24.1                16 Sep 2014   http://telepathy.freedesktop.org/releases/telepathy-glib/telepathy-glib-0.24.1.tar.gz
telepathylogger             0.8.0                 01 Oct 2013   http://telepathy.freedesktop.org/releases/telepathy-logger/telepathy-logger-0.8.0.tar.bz2
tellico                     3.1.3                 05 Dec 2018   http://tellico-project.org/files/tellico-3.1.3.tar.xz
templateglib                3.32.0                13 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/template-glib/3.32/template-glib-3.32.0.tar.xz
temple                      0.8.1                 09 Jan 2018   https://rubygems.org/downloads/temple-0.8.1.gem
tepl                        4.2.0                 16 Jul 2018   http://ftp.gnome.org/pub/gnome/sources/tepl/4.2/tepl-4.2.0.tar.xz
termanimation               2.6                   13 Oct 2018   https://cpan.metacpan.org/authors/id/K/KB/KBAUCOM/Term-Animation-2.6.tar.gz
termansicolor               1.7.1                 18 Feb 2019   https://rubygems.org/downloads/term-ansicolor-1.7.1.gem
termcap                     1.3.1                 01 May 2014   ftp://ftp.gnu.org/gnu/termcap/termcap-1.3.1.tar.gz
terminator                  1.91                  10 Feb 2019   https://launchpad.net/terminator/gtk3/1.91/+download/terminator-1.91.tar.gz
terminology                 1.4.0                 13 Apr 2019   https://download.enlightenment.org/rel/apps/terminology/terminology-1.4.0.tar.xz
termit                      3.9.0                 11 Sep 2017   https://rubygems.org/downloads/termit-3.9.0.gem
tesseract                   4.0.0                 13 Jan 2019   https://github.com/tesseract-ocr/tesseract/archive/4.0.0.tar.gz
testdifferences             0.66                  03 Sep 2018   https://cpan.metacpan.org/authors/id/D/DC/DCANTRELL/Test-Differences-0.66.tar.gz
testdisk                    6.11.3                01 May 2014   http://www.cgsecurity.org/testdisk-6.11.3.tar.bz2
testunit                    3.3.2                 03 May 2019   https://rubygems.org/downloads/test-unit-3.3.2.gem
tetex                       3.0                   01 May 2014   ftp://tug.ctan.org/tex-archive/systems/unix/teTeX/3.0/distrib/tetex-src-3.0.tar.gz
tevent                      0.10.0                22 Mar 2019   https://www.samba.org/ftp/tevent/tevent-0.10.0.tar.gz
texi2html                   1.82                  01 Jan 2013   http://savannah.inetbridge.net/texi2html/texi2html-1.82.tar.bz2
texinfo                     6.6                   16 Feb 2019   http://ftp.gnu.org/gnu/texinfo/texinfo-6.6.tar.gz
texmaker                    5.0.3                 02 Nov 2018   http://www.xm1math.net/texmaker/texmaker-5.0.3.tar.bz2
textbibtex                  0.81                  03 Aug 2017   http://search.cpan.org/CPAN/authors/id/A/AM/AMBS/Text-BibTeX-0.81.tar.gz
textcsv                     2.00                  12 May 2019   https://cpan.metacpan.org/authors/id/I/IS/ISHIGAKI/Text-CSV-2.00.tar.gz
thesilversearcher           2.2.0                 13 Sep 2018   http://geoff.greer.fm/ag/releases/the_silver_searcher-2.2.0.tar.gz
thin                        1.7.2                 31 Jul 2017   https://rubygems.org/downloads/thin-1.7.2.gem
thoggen                     0.7.1                 01 Apr 2014   http://prdownloads.sourceforge.net/thoggen/thoggen-0.7.1.tar.gz
thor                        0.20.3                17 Nov 2018   https://rubygems.org/downloads/thor-0.20.3.gem
threadsafe                  0.3.6                 07 Dec 2017   https://rubygems.org/downloads/thread_safe-0.3.6.gem
threadweaver                5.59.0                08 Jun 2019   https://download.kde.org/stable/frameworks/5.59/threadweaver-5.59.0.tar.xz
thunar                      1.8.7                 30 Jun 2019   http://archive.xfce.org/src/xfce/thunar/1.8/Thunar-1.8.7.tar.bz2
thunarvolman                0.9.3                 02 Jul 2019   http://archive.xfce.org/src/xfce/thunar-volman/0.9/thunar-volman-0.9.3.tar.bz2
tidy                        1.1.2                 05 Mar 2017   https://rubygems.org/downloads/tidy-1.1.2.gem
tiff                        4.0.10                13 Nov 2018   http://download.osgeo.org/libtiff/tiff-4.0.10.tar.gz
tightvnc                    1.3.10                01 May 2014   https://sourceforge.net/projects/vnc-tight/files/TightVNC-unix/1.3.10/tightvnc-1.3.10_unixsrc.tar.bz2
tilda                       14.01.2018            14 Jan 2018   https://github.com/lanoxx/tilda/tilda-14.01.2018
tilt                        2.0.9                 15 Dec 2018   https://rubygems.org/downloads/tilt-2.0.9.gem
timelimit                   1.9.0                 18 Sep 2018   https://devel.ringlet.net/files/sys/timelimit/timelimit-1.9.0.tar.xz
timidity++                  2.14.0                01 May 2014   https://sourceforge.net/projects/timidity/files/TiMidity%2B%2B/TiMidity%2B%2B-2.14.0/TiMidity%2B%2B-2.14.0.tar.xz
tintin                      2.01.4                18 Sep 2018   http://sourceforge.net/projects/tintin/files/TinTin%2B%2B%20Source%20Code/2.01.4/tintin-2.01.4.tar.gz
tinyfugue                   5.07                  01 Feb 2013   https://tinyfugue.sourceforge.net/
tinyvm                      03.07.2011            01 May 2014   https://github.com/GenTiradentes/tinyvm/tinyvm-03.07.2011
tinyxml                     2.6.2                 01 May 2014   http://sourceforge.net/projects/tinyxml/files/tinyxml/2.6.2/tinyxml_2_6_2.tar.gz
tioga                       1.19.1                10 Feb 2016   https://rubygems.org/downloads/tioga-1.19.1.gem
tk                          8.6.7                 12 Aug 2017   http://downloads.sourceforge.net/tcl/tk8.6.7-src.tar.gz
tmux                        2.9a                  01 May 2019   https://github.com/tmux/tmux/releases/download/2.9a/tmux-2.9a.tar.gz
tmuxinator                  1.1.0                 03 May 2019   https://rubygems.org/downloads/tmuxinator-1.1.0.gem
tomboy                      1.15.9                16 Jul 2017   http://ftp.gnome.org/pub/gnome/sources/tomboy/1.15/tomboy-1.15.9.tar.xz
toppler                     1.1.6                 12 Dec 2015   http://sourceforge.net/projects/toppler/files/toppler/1.1.6/toppler-1.1.6.tar.gz
tork                        0.27                  01 May 2014   http://www.anonymityanywhere.com/tork/
torsmo                      0.18                  01 May 2014   https://sourceforge.net/projects/torsmo/files/torsmo/torsmo%200.18/torsmo-0.18.tar.gz
totem                       3.32.0                09 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/totem/3.32/totem-3.32.0.tar.xz
totemplparser               3.26.3                08 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/totem-pl-parser/3.26/totem-pl-parser-3.26.3.tar.xz
toybox                      0.7.7                 13 Oct 2018   https://landley.net/toybox/downloads/toybox-0.7.7.tar.gz
toycars                     0.3.10                01 Dec 2012   http://sourceforge.net/projects/toycars/files/toycars/0.3.10/toycars-0.3.10.tar.gz
trac                        1.2.3                 10 Aug 2018   https://download.edgewall.org/trac/Trac-1.2.3.tar.gz
traceroute                  2.1.0                 05 Dec 2017   https://sourceforge.net/projects/traceroute/files/traceroute/traceroute-2.1.0/traceroute-2.1.0.tar.gz
trackballs                  1.1.4                 03 Apr 2016   https://sourceforge.net/projects/trackballs/files/trackballs/1.1.4/trackballs-1.1.4.tar.gz
tracker                     2.2.2                 03 May 2019   http://ftp.gnome.org/pub/gnome/sources/tracker/2.2/tracker-2.2.2.tar.xz
trackerminers               2.2.2                 03 May 2019   http://ftp.gnome.org/pub/gnome/sources/tracker-miners/2.2/tracker-miners-2.2.2.tar.xz
trackfs                     0.0.7                 01 May 2014   http://www.mr511.de/software/trackfs-0.0.7.tar.gz
tramp                       2.4.1                 27 Dec 2018   https://ftp.gnu.org/pub/gnu/tramp/tramp-2.4.1.tar.gz
transcode                   1.1.7                 01 Jan 2014   https://bitbucket.org/france/transcode-tcforge/downloads/transcode-1.1.7.tar.bz2
transmission                2.94                  02 May 2018   https://github.com/transmission/transmission-releases/raw/master/transmission-2.94.tar.xz
transset                    1.0.1                 01 May 2014   http://xorg.freedesktop.org/releases/individual/app/transset-1.0.1.tar.bz2
trapproto                   3.4.3                 01 May 2014   http://xorg.freedesktop.org/releases/individual/proto/trapproto-3.4.3.tar.bz2
tree                        1.8.0                 25 Nov 2018   ftp://mama.indstate.edu/linux/tree/tree-1.8.0.tgz
treetop                     1.6.10                01 May 2018   https://rubygems.org/downloads/treetop-1.6.10.gem
trimines                    1.3.0                 01 May 2014   http://www.freewebs.com/trimines/trimines-1.3.0.tar.gz
tripwire                    2.4.3.7               07 Oct 2017   https://github.com/Tripwire/tripwire-open-source/archive/2.4.3.7.tar.gz
trix                        0.93                  01 May 2014   http://www.kde-apps.org/content/show.php/trix?content=49417
trollop                     2.9.9                 25 Aug 2018   https://rubygems.org/downloads/trollop-2.9.9.gem
trtl                        0.0.2                 07 Oct 2017   https://rubygems.org/downloads/trtl-0.0.2.gem
tsombie                     1.1.2                 01 Dec 2012   http://mathias-kettner.com/download/tsombie-1.1.2.tar.gz
ttfunk                      1.5.1                 31 Jul 2017   https://rubygems.org/downloads/ttfunk-1.5.1.gem
ttycolor                    0.4.3                 07 Apr 2019   https://rubygems.org/downloads/tty-color-0.4.3.gem
ttycursor                   0.6.1                 03 May 2019   https://rubygems.org/downloads/tty-cursor-0.6.1.gem
ttyprogressbar              0.16.0                26 Sep 2018   https://rubygems.org/downloads/tty-progressbar-0.16.0.gem
ttyscreen                   0.6.5                 22 Jul 2018   https://rubygems.org/downloads/tty-screen-0.6.5.gem
tulip                       3.0.0B6               01 May 2014   http://downloads.sourceforge.net/auber/tulip-3.0.0B6.tar.bz2?modtime=1184356395&big_mirror=0
tumbler                     0.2.5                 02 Jul 2019   http://archive.xfce.org/src/xfce/tumbler/0.2/tumbler-0.2.5.tar.bz2
tunapie                     0.9.5.1               01 May 2014   http://downloads.sourceforge.net/project/tunapie/tunapie/2.1/tunapie-2.1.17.tar.gz?use_mirror=kent
tunnel                      1.2.1                 01 May 2014   http://tspiteri.org/tunnel/download/
turbolinks                  5.2.0                 13 Oct 2018   https://rubygems.org/downloads/turbolinks-5.2.0.gem
turbovnc                    1.2.1                 01 Jul 2014   http://sourceforge.net/projects/turbovnc/files/1.2.1/turbovnc-1.2.1.tar.gz
tuxrip                      0.99beta6             01 May 2014   http://tuxrip.free.fr/tuxrip/tuxrip099beta6.tar.bz2
tv                          0.5.1                 24 Nov 2017   http://darwin.zoology.gla.ac.uk/~rpage/treeviewx/download/0.5/tv-0.5.1.tar.gz
tvtime                      1.0.1                 01 May 2014   http://prdownloads.sourceforge.net/tvtime/tvtime-1.0.1.tar.gz
twilioruby                  5.22.3                03 May 2019   https://rubygems.org/downloads/twilio-ruby-5.22.3.gem
twind                       1.1.0                 01 May 2014   http://puzzle.dl.sourceforge.net/sourceforge/twind/twind-1.1.0.tar.gz
twisted                     18.9.0                18 Oct 2018   https://files.pythonhosted.org/packages/5d/0e/a72d85a55761c2c3ff1cb968143a2fd5f360220779ed90e0fadf4106d4f2/Twisted-18.9.0.tar.bz2
twitux                      0.69                  29 Jul 2016   https://sourceforge.net/projects/twitux/files/twitux/0.69/twitux-0.69.tar.bz2
twm                         1.0.10                14 Mar 2018   http://xorg.freedesktop.org/releases/individual/app/twm-1.0.10.tar.bz2
twolame                     0.3.13                01 Nov 2013   http://downloads.sourceforge.net/twolame/twolame-0.3.13.tar.gz
txr                         206                   18 Jan 2019   http://www.kylheku.com/cgit/txr/snapshot/txr-206.tar.bz2
typespeed                   0.6.5                 27 Mar 2017   http://typespeed.sourceforge.net/typespeed-0.6.5.tar.gz
tzinfo                      2.0.0                 18 Feb 2019   https://rubygems.org/downloads/tzinfo-2.0.0.gem
uchardet                    0.0.6                 12 Mar 2017   https://www.freedesktop.org/software/uchardet/releases/uchardet-0.0.6.tar.xz
uclibc                      0.9.32                01 May 2014   http://www.uclibc.org/downloads/uClibc-0.9.32.tar.bz2
ucview                      0.16                  01 May 2014   http://www.unicap-imaging.org/downloads/ucview-0.16.tar.gz
udev                        182                   01 Aug 2013   http://www.kernel.org/pub/linux/utils/kernel/hotplug/udev-182.tar.xz
udftools                    1.0.0b3               01 May 2014   http://downloads.sourceforge.net/linux-udf/udftools-1.0.0b3.tar.gz
udisks                      2.8.3                 17 Jun 2019   https://github.com/storaged-project/udisks/releases/download/udisks-2.8.3/udisks-2.8.3.tar.bz2
ufoai                       2.5                   09 Aug 2015   http://sourceforge.net/projects/ufoai/files/UFO_AI%202.x/2.5/ufoai-2.5-source.tar.bz2
uglifier                    4.1.20                15 Dec 2018   https://rubygems.org/downloads/uglifier-4.1.20.gem
uhttpmock                   0.5.1                 24 Oct 2017   http://tecnocode.co.uk/downloads/uhttpmock/uhttpmock-0.5.1.tar.xz
uim                         1.5.4                 01 May 2014   http://uim.googlecode.com/files/uim-1.5.4.tar.bz2
umbrello                    19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/umbrello-19.04.2.tar.xz
umockdev                    0.12.1                06 Mar 2019   https://github.com/martinpitt/umockdev/releases/download/0.12.1/umockdev-0.12.1.tar.xz
unbound                     1.9.2                 23 Jun 2019   https://nlnetlabs.nl/downloads/unbound/unbound-1.9.2.tar.gz
unchartedruby               1.3.0                 01 May 2014   http://raa.ruby-lang.org/project/uncharted/
uncrustify                  0.65                  02 Aug 2017   https://sourceforge.net/projects/uncrustify/files/uncrustify/uncrustify-0.65/uncrustify-0.65.tar.gz
unf                         0.1.4                 21 Sep 2017   https://rubygems.org/downloads/unf-0.1.4.gem
unfext                      0.0.7                 12 May 2019   https://rubygems.org/downloads/unf_ext-0.0.7.6.gem
unhash                      0.7                   01 May 2014   http://www.geocities.com/dxp2532/
unicodecollate              1.25                  26 Nov 2017   http://search.cpan.org/CPAN/authors/id/S/SA/SADAHIRO/Unicode-Collate-1.25.tar.gz
unicorn                     5.5.0                 03 May 2019   https://rubygems.org/downloads/unicorn-5.5.0.gem
unieject                    6                     01 May 2014   http://prdownloads.sourceforge.net/unieject/unieject-6.tar.bz2
union                       1.0.6                 06 Nov 2018   https://rubygems.org/downloads/union-1.0.6.gem
unionfsfuse                 0.25                  01 May 2014   http://podgorny.cz/unionfs-fuse/releases/unionfs-fuse-0.25.tar.bz2
units                       2.19                  01 Jun 2019   https://ftp.gnu.org/pub/gnu/units/units-2.19.tar.gz
unity                       7.3.0                 17 Sep 2014   https://launchpad.net/unity/7.3/7.3.0/+download/unity-7.3.0.tar.bz2
unixodbc                    2.3.5                 04 Jan 2018   ftp://ftp.unixodbc.org/pub/unixODBC/unixODBC-2.3.5.tar.gz
unoconv                     0.7                   15 Jan 2019   http://dag.wieers.com/home-made/unoconv/unoconv-0.7.tar.gz
unrtf                       0.21.10               25 Nov 2018   http://ftp.gnu.org/pub/gnu/unrtf/unrtf-0.21.10.tar.gz
unzip                       6.0                   17 Dec 2015   http://downloads.sourceforge.net/infozip/unzip60.tar.gz
upower                      0.99.10               20 Feb 2019   https://gitlab.freedesktop.org/upower/upower/uploads/c438511024b9bc5a904f8775cfc8e4c4/upower-0.99.10.tar.xz
upstart                     1.6.1                 01 Jan 2013   http://upstart.ubuntu.com/download/1.6.1/upstart-1.6.1.tar.gz
uri                         1.74                  20 Jan 2019   http://search.cpan.org/CPAN/authors/id/E/ET/ETHER/URI-1.74.tar.gz
uriparser                   0.8.5                 26 Jun 2018   https://sourceforge.net/projects/uriparser/files/Sources/0.8.5/uriparser-0.8.5.tar.bz2
urlgrabber                  3.1.0                 01 May 2014   http://linux.duke.edu/projects/urlgrabber/download/urlgrabber-3.1.0.tar.gz
urpkg                       1.4                   01 May 2014   http://www.svasey.org/urpkg/urpkg-1.4.tar.bz2
usbutils                    012                   08 May 2019   https://www.kernel.org/pub/linux/utils/usb/usbutils/usbutils-012.tar.xz
usermanager                 5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/user-manager-5.16.3.tar.xz
utillinux                   2.34                  14 Jun 2019   http://www.kernel.org/pub/linux/utils/util-linux/v2.34/util-linux-2.34.tar.xz
utillinuxng                 2.18                  01 May 2014   ftp://ftp.kernel.org/pub/linux/utils/util-linux-ng/v2.20/util-linux-ng-2.18.tar.bz2
utilmacros                  1.19.2                08 Mar 2018   http://xorg.freedesktop.org/releases/individual/util/util-macros-1.19.2.tar.bz2
uuid                        1.6.2                 01 Jul 2013   ftp://ftp.ossp.org/pkg/lib/uuid/uuid-1.6.2.tar.gz
v4lutils                    1.16.6                05 May 2019   https://www.linuxtv.org/downloads/v4l-utils/v4l-utils-1.16.6.tar.bz2
vala                        0.45.2                19 Jun 2019   http://ftp.gnome.org/pub/gnome/sources/vala/0.45/vala-0.45.2.tar.xz
valadoc                     0.36.2                04 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/valadoc/0.36/valadoc-0.36.2.tar.xz
valencia                    0.8.0                 01 Aug 2014   http://ftp.gnome.org/pub/GNOME/sources/valencia/0.8/valencia-0.8.0.tar.xz
valgrind                    3.15.0                21 Apr 2019   ftp://sourceware.org/pub/valgrind/valgrind-3.15.0.tar.bz2
valknut                     0.3.23                01 Nov 2017   https://sourceforge.net/projects/wxdcgui/files/valknut/0.3.23/valknut-0.3.23.tar.bz2
valkyrie                    2.0.0                 17 Dec 2017   http://valgrind.org/downloads/valkyrie-2.0.0.tar.bz2
vapoursynth                 30.09.2015            30 Sep 2015   vapoursynth-30.09.2015.tar.xz
vc                          0.7.3                 09 Aug 2015   http://code.compeng.uni-frankfurt.de/attachments/download/174/Vc-0.7.3.tar.gz
vcinit                      1.2                   01 May 2014   http://www.dervishd.net/Projects/VCinit-1.2.tar.gz
vdr                         1.6.0                 01 May 2014   http://www.cadsoft.de/vdr/download.htm
veejay                      1.5.51                21 Apr 2019   https://sourceforge.net/projects/veejay/files/veejay-1.5-src/veejay-1.5.51.tar.bz2
vera                        1.23                  10 Feb 2016   http://ftp.gnu.org/pub/gnu/vera/vera-1.23.tar.gz
vertris                     0.3.2                 01 May 2014   http://vertris.googlepages.com/
vice                        1.19                  01 May 2014   http://www.zimmers.net/anonftp/pub/cbm/crossplatform/emulators/VICE/vice-1.19.tar.gz
videocut                    0.2.0                 01 May 2014   http://www.kde-apps.org/content/show.php/VideoCut?content=60826
videoproto                  2.3.3                 12 Mar 2016   http://xorg.freedesktop.org/releases/individual/proto/videoproto-2.3.3.tar.gz
videotrans                  1.6.1                 01 May 2014   http://sourceforge.net/projects/videotrans/files/videotrans/1.6.1/videotrans-1.6.1.tar.bz2
viennarna                   2.4.13                02 Jul 2019   https://www.tbi.univie.ac.at/RNA/download/sourcecode/2_4_x/ViennaRNA-2.4.13.tar.gz
viewres                     1.0.4                 08 Aug 2015   http://xorg.freedesktop.org/releases/individual/app/viewres-1.0.4.tar.gz
viewvc                      1.1.15                01 May 2014   http://viewvc.tigris.org/files/documents/3330/49223/viewvc-1.1.15.tar.gz
vim                         8.1                   18 May 2018   ftp://ftp.vim.org/pub/vim/unix/vim-8.1.tar.bz2
vinagre                     3.20.0                22 Mar 2016   http://ftp.gnome.org/pub/GNOME/sources/vinagre/3.20/vinagre-3.20.0.tar.xz
vino                        3.18.1                12 Nov 2015   http://ftp.gnome.org/pub/GNOME/sources/vino/3.18/vino-3.18.1.tar.xz
virtualenv                  16.0.0                17 May 2018   https://files.pythonhosted.org/packages/33/bc/fa0b5347139cd9564f0d44ebd2b147ac97c36b2403943dbee8a25fd74012/virtualenv-16.0.0.tar.gz
vitetris                    0.57                  01 Nov 2012   http://www.victornils.net/tetris/vitetris-0.57.tar.gz
vlc                         3.0.7                 07 Jun 2019   http://download.videolan.org/pub/videolan/vlc/3.0.7/vlc-3.0.7.tar.xz
vlock                       2.1                   01 May 2014   http://cthulhu.c3d2.de/~toidinamai/vlock/vlock.html
vmd                         1.9.3                 16 Jun 2017   http://www.ks.uiuc.edu/Research/vmd/vmd-1.9.3/files/final/vmd-1.9.3.src.tar.gz
vnstat                      2.2                   30 Apr 2019   https://humdi.net/vnstat/vnstat-2.2.tar.gz
vobcopy                     1.1.0                 01 May 2014   http://vobcopy.org/download/vobcopy-1.1.0.tar.bz2
volleyball                  0.82                  01 May 2014   http://www.losersjuegos.com.ar/juegos/volleyball/descargas/volleyball-0.82.tar.gz
volumekey                   0.3.12                12 Oct 2018   https://releases.pagure.org/volume_key/volume_key-0.3.12.tar.xz
vor                         0.5.5                 01 May 2014   http://qualdan.com/vor/vor-0.5.5.tar.bz2
vorbistools                 1.4.0                 08 Sep 2015   http://downloads.xiph.org/releases/vorbis/vorbis-tools-1.4.0.tar.gz
vpnc                        0.5.3                 01 May 2013   http://www.unix-ag.uni-kl.de/~massar/vpnc/vpnc-0.5.3.tar.gz
vrpe                        0.31                  01 May 2014   http://vrpe.sytes.net/VRPE/VRPE-0.31.tar.bz2
vsftpd                      3.0.3                 25 Jul 2015   https://security.appspot.com/downloads/vsftpd-3.0.3.tar.gz
vte                         0.57.3                01 Jul 2019   http://ftp.gnome.org/pub/gnome/sources/vte/0.57/vte-0.57.3.tar.xz
w3m                         0.5.3                 30 Dec 2018   https://sourceforge.net/projects/w3m/files/w3m/w3m-0.5.3/w3m-0.5.3.tar.gz
waf                         2.0.8                 17 Apr 2018   https://github.com/waf-project/waf/archive/waf-2.0.8.tar.gz
waon                        0.10                  01 Mar 2014   http://sourceforge.net/projects/waon/files/waon/0.10/waon-0.10.tar.gz
warden                      1.2.8                 30 Apr 2019   https://rubygems.org/downloads/warden-1.2.8.gem
warmux                      11.04.1               01 Jul 2014   http://sourceforge.net/projects/warmux/files/warmux-11.04.1.tar.bz2
warzone2100                 3.1.2                 26 Jul 2015   http://sourceforge.net/projects/warzone2100/files/releases/3.1.2/warzone2100-3.1.2.tar.xz
wavbreaker                  0.8                   01 May 2014   http://huli.org/wavbreaker/download.html
wavpack                     5.1.0                 16 Oct 2017   https://github.com/dbry/WavPack/archive/5.1.0.tar.gz
wayland                     1.17.0                21 Mar 2019   https://wayland.freedesktop.org/releases/wayland-1.17.0.tar.xz
waylandprotocols            1.17                  26 Nov 2018   https://wayland.freedesktop.org/releases/wayland-protocols-1.17.tar.xz
wcd                         6.0.1-beta3           08 Aug 2017   https://waterlan.home.xs4all.nl/wcd/wcd-6.0.1-beta3.tar.gz
wdiff                       1.1.2                 01 May 2014   http://tweedo.com/mirror/gnu/wdiff/wdiff-1.1.2.tar.gz
webkit                      r189384               22 Dec 2015   http://builds.nightly.webkit.org/files/trunk/src/WebKit-r189384.tar.bz2
webkitgtk                   2.24.3                04 Jul 2019   https://webkitgtk.org/releases/webkitgtk-2.24.3.tar.xz
webmin                      1.920                 05 Jul 2019   http://downloads.sourceforge.net/webadmin/webmin-1.920.tar.gz
websocketdriver             0.7.0                 17 Nov 2018   https://rubygems.org/downloads/websocket-driver-0.7.0.gem
weechat                     2.4                   17 Feb 2019   https://weechat.org/files/src/weechat-2.4.tar.xz
wesnoth                     1.14.7                10 Jun 2019   https://sourceforge.net/projects/wesnoth/files/wesnoth-1.14/wesnoth-1.14.7/wesnoth-1.14.7.tar.bz2
weston                      6.0.0                 06 May 2019   https://wayland.freedesktop.org/releases/weston-6.0.0.tar.xz
wget                        1.20.3                05 Apr 2019   ftp://ftp.gnu.org/gnu/wget/wget-1.20.3.tar.gz
which                       2.21                  17 Jun 2015   http://ftp.gnu.org/gnu/which/which-2.21.tar.gz
whitedune                   0.29beta670           01 May 2014   http://vrml.cip.ica.uni-stuttgart.de/dune/down.html
whois                       5.4.3                 14 Jun 2019   http://ftp.debian.org/debian/pool/main/w/whois/whois_5.4.3.tar.xz
wicd                        1.7.4                 25 Jan 2016   https://launchpad.net/wicd/1.7/1.7.4/+download/wicd-1.7.4.tar.gz
wimlib                      1.13.0                10 Dec 2018   https://wimlib.net/downloads/wimlib-1.13.0.tar.gz
windowmaker                 0.95.8                11 Mar 2017   http://windowmaker.org/pub/source/release/WindowMaker-0.95.8.tar.gz
windowswmproto              1.0.3                 01 May 2014   http://xorg.freedesktop.org/releases/individual/proto/windowswmproto-1.0.3.tar.bz2
wine                        4.10                  17 Jun 2019   http://mirrors.ibiblio.org/wine/source/4.x/wine-4.10.tar.xz
wings                       2.1.7                 15 Mar 2019   https://sourceforge.net/projects/wings/files/wings/2.1.7/wings-2.1.7.tar.bz2
winzig                      1.73                  01 May 2014   http://freshmeat.net/redir/winzig/20777/url_tgz/winzig.1.73.tar.gz
wirble                      0.1.3                 23 Mar 2017   https://rubygems.org/downloads/wirble-0.1.3.gem
wireguard                   0.0.20170918          19 Sep 2017   https://git.zx2c4.com/WireGuard/snapshot/WireGuard-0.0.20170918.tar.xz
wirelesstools               29                    01 May 2014   http://www.hpl.hp.com/personal/Jean_Tourrilhes/Linux/wireless_tools.29.tar.gz
wireshark                   3.0.2                 23 May 2019   https://www.wireshark.org/download/src/wireshark-3.0.2.tar.xz
wkhtmltox                   0.12.5                12 May 2019   https://github.com/wkhtmltopdf/wkhtmltopdf/archive/0.12.5.tar.gz
wlassistant                 0.5.7                 01 May 2014   http://wlassistant.sourceforge.net/
wmctrl                      1.07                  01 May 2014   http://www.sweb.cz/tripie/utils/wmctrl/dist/wmctrl-1.07.tar.gz
wmii                        3                     01 May 2014   http://www.suckless.org/snaps/wmii-20071003.tgz
wordpress                   5.2                   08 May 2019   https://wordpress.org/wordpress-5.2.tar.gz
worker                      2.19.3                01 May 2014   http://www.boomerangsworld.de/cms/worker/downloads/worker-2.19.3.tar.bz2
wpasupplicant               2.8                   23 Apr 2019   http://hostap.epitest.fi/releases/wpa_supplicant-2.8.tar.gz
wput                        0.6.1                 01 May 2014   http://prdownloads.sourceforge.net/wput/wput-0.6.1.tgz
writeexcel                  1.0.5                 12 Aug 2017   https://rubygems.org/downloads/writeexcel-1.0.5.gem
wv                          1.2.9                 01 Aug 2013   http://www.abisource.com/downloads/wv/1.2.9/wv-1.2.9.tar.gz
wv2                         0.4.2                 01 May 2014   https://sourceforge.net/projects/wvware/files/wv2-0.4.2.tar.bz2
wxgtk                       2.6.3                 01 May 2014   http://mesh.dl.sourceforge.net/sourceforge/wxwindows/wxGTK-2.6.3.tar.bz2
wxpython                    4.0.0                 01 Feb 2018   https://github.com/wxWidgets/Phoenix/archive/wxPython-4.0.0.tar.gz
wxsvg                       1.1.2                 01 May 2014   http://sourceforge.net/projects/wxsvg/files/wxsvg/1.1.2/wxsvg-1.1.2.tar.bz2
wxwidgets                   3.1.2                 15 May 2019   https://github.com/wxWidgets/wxWidgets/releases/download/v3.1.2/wxWidgets-3.1.2.tar.bz2
x11perf                     1.6.1                 17 Mar 2019   http://xorg.freedesktop.org/releases/individual/app/x11perf-1.6.1.tar.gz
x264                        17.02.2019            17 Feb 2019   ftp://ftp.videolan.org/pub/videolan/x264/snapshots/x264-snapshot-20190217-2245.tar.bz2
x265                        3.0                   25 Jan 2019   https://bitbucket.org/multicoreware/x265/downloads/x265_3.0.tar.gz
x86info                     1.30                  01 May 2014   http://codemonkey.org.uk/projects/x86info/x86info-1.30.tgz
xapianbindings              1.4.9                 11 Jan 2019   https://oligarchy.co.uk/xapian/1.4.9/xapian-bindings-1.4.9.tar.xz
xapiancore                  1.4.11                04 Mar 2019   https://oligarchy.co.uk/xapian/1.4.11/xapian-core-1.4.11.tar.xz
xapianomega                 1.4.9                 11 Jan 2019   https://oligarchy.co.uk/xapian/1.4.9/xapian-omega-1.4.9.tar.xz
xaric                       0.13.7                06 Dec 2015   http://xaric.org/software/xaric/releases/xaric-0.13.7.tar.gz
xaudio                      0.6.1                 01 Apr 2014   http://www.chaoticmind.net/~hcb/murx/xaudio/xaudio-0.6.1.tar.gz
xauth                       1.0.10                16 Feb 2017   https://www.x.org/releases/individual/app/xauth-1.0.10.tar.gz
xautomation                 1.02                  01 May 2014   http://hoopajoo.net/static/projects/xautomation-1.02.tar.gz
xawtv                       3.95                  01 May 2014   http://dl.bytesex.org/releases/xawtv/xawtv-3.95.tar.gz
xbacklight                  1.2.1                 01 Oct 2013   http://xorg.freedesktop.org/releases/individual/app/xbacklight-1.2.1.tar.bz2
xbak                        0.1.5                 01 May 2014   http://sourceforge.net/projects/xbak/files/xbak/xbak-0.1.5/xbak-0.1.5.tar.bz2
xbiff                       1.0.3                 01 May 2014   https://www.x.org/releases/individual/app/xbiff-1.0.3.tar.bz2
xbindkeys                   1.8.5                 01 May 2014   http://www.nongnu.org/xbindkeys/xbindkeys-1.8.5.tar.gz
xbitmaps                    1.1.2                 13 Mar 2018   https://www.x.org/releases/individual/data/xbitmaps-1.1.2.tar.bz2
xboard                      4.9.1                 08 Aug 2016   http://ftp.gnu.org/gnu/xboard/xboard-4.9.1.tar.gz
xcalc                       1.1.0                 11 Jun 2019   http://xorg.freedesktop.org/releases/individual/app/xcalc-1.1.0.tar.bz2
xcbproto                    1.13                  08 Mar 2018   https://www.x.org/releases/individual/xcb/xcb-proto-1.13.tar.bz2
xcbutil                     0.4.0                 13 Apr 2015   http://xcb.freedesktop.org/dist/xcb-util-0.4.0.tar.gz
xcbutilcursor               0.1.3                 16 May 2016   https://www.x.org/releases/individual/xcb/xcb-util-cursor-0.1.3.tar.bz2
xcbutilerrors               1.0                   01 Dec 2017   https://www.x.org/releases/individual/xcb/xcb-util-errors-1.0.tar.gz
xcbutilimage                0.4.0                 01 Jun 2014   https://www.x.org/releases/individual/xcb/xcb-util-image-0.4.0.tar.gz
xcbutilkeysyms              0.4.0                 12 Dec 2015   http://xorg.freedesktop.org/releases/individual/xcb/xcb-util-keysyms-0.4.0.tar.bz2
xcbutilrenderutil           0.3.9                 01 Jun 2014   http://xorg.freedesktop.org/releases/individual/xcb/xcb-util-renderutil-0.3.9.tar.bz2
xcbutilwm                   0.4.1                 09 Aug 2016   http://xcb.freedesktop.org/dist/xcb-util-wm-0.4.1.tar.bz2
xcbutilxrm                  1.0                   22 Nov 2016   https://github.com/Airblader/xcb-util-xrm/releases/download/v1.0/xcb-util-xrm-1.0.tar.bz2
xchat                       2.8.8                 01 Apr 2014   http://xchat.org/files/source/2.8/xchat-2.8.8.tar.bz2
xchm                        1.27                  22 Mar 2019   https://github.com/rzvncj/xCHM/releases/download/1.27/xchm-1.27.tar.gz
xclip                       0.12                  01 May 2014   http://downloads.sourceforge.net/project/xclip/xclip/0.12/xclip-0.12.tar.gz
xclipboard                  1.1.3                 01 Sep 2013   http://xorg.freedesktop.org/releases/individual/app/xclipboard-1.1.3.tar.bz2
xclock                      1.0.8                 20 Feb 2019   http://xorg.freedesktop.org/releases/individual/app/xclock-1.0.8.tar.bz2
xcm                         0.5.3                 01 May 2014   https://sourceforge.net/projects/oyranos/files/Xcm/xcm-0.5.3.tar.bz2
xcmiscproto                 1.2.2                 01 May 2014   http://xorg.freedesktop.org/releases/individual/proto/xcmiscproto-1.2.2.tar.bz2
xcmsdb                      1.0.5                 18 Apr 2015   http://xorg.freedesktop.org/releases/individual/app/xcmsdb-1.0.5.tar.gz
xcompmgr                    1.1.8                 26 Mar 2019   http://xorg.freedesktop.org/releases/individual/app/xcompmgr-1.1.8.tar.gz
xconsole                    1.0.6                 01 May 2014   http://xorg.freedesktop.org/releases/individual/app/xconsole-1.0.6.tar.bz2
xcursorgen                  1.0.7                 12 Jan 2019   https://www.x.org/releases/individual/app/xcursorgen-1.0.7.tar.bz2
xcursorthemes               1.0.6                 16 Feb 2019   https://www.x.org/releases/individual/data/xcursor-themes-1.0.6.tar.bz2
xdbedizzy                   1.0.2                 01 May 2014   http://xorg.freedesktop.org/releases/individual/app/xdbedizzy-1.0.2.tar.bz2
xdg                         2.2.3                 27 Dec 2018   https://rubygems.org/downloads/xdg-2.2.3.gem
xdgdbusproxy                0.1.1                 15 Feb 2019   https://github.com/flatpak/xdg-dbus-proxy/releases/download/0.1.1/xdg-dbus-proxy-0.1.1.tar.xz
xdgdesktopportalkde         5.16.3                09 Jul 2019   https://download.kde.org/stable/plasma/5.16.3/xdg-desktop-portal-kde-5.16.3.tar.xz
xdguserdirs                 0.17                  20 Mar 2018   https://user-dirs.freedesktop.org/releases/xdg-user-dirs-0.17.tar.gz
xdguserdirsgtk              0.10                  24 Sep 2014   http://ftp.gnome.org/pub/GNOME/sources/xdg-user-dirs-gtk/0.10/xdg-user-dirs-gtk-0.10.tar.xz
xdgutils                    1.1.3                 26 Jan 2019   http://portland.freedesktop.org/download/xdg-utils-1.1.3.tar.gz
xdialog                     2.3.1                 01 May 2014   http://xdialog.free.fr/Xdialog-2.3.1.tar.bz2
xditview                    1.0.5                 04 Mar 2019   http://xorg.freedesktop.org/releases/individual/app/xditview-1.0.5.tar.gz
xdm                         1.1.12                04 Mar 2019   https://www.x.org/releases/individual/app/xdm-1.1.12.tar.bz2
xdotool                     2.20110530.1          01 Feb 2014   http://semicomplete.googlecode.com/files/xdotool-2.20110530.1.tar.gz
xdpyinfo                    1.3.2                 18 Apr 2015   http://xorg.freedesktop.org/releases/individual/app/xdpyinfo-1.3.2.tar.gz
xdriinfo                    1.0.5                 18 Apr 2015   http://xorg.freedesktop.org/releases/individual/app/xdriinfo-1.0.5.tar.gz
xdtv                        2.4.0                 31 Aug 2017   https://sourceforge.net/projects/xawdecode/files/01%20-%20XdTV/2.4.0/xdtv-2.4.0.tar.gz
xedit                       1.2.2                 26 Feb 2015   http://xorg.freedesktop.org/releases/individual/app/xedit-1.2.2.tar.bz2
xemacs                      21.5.34               28 Oct 2018   ftp://ftp.xemacs.org/pub/xemacs/xemacs-21.5/xemacs-21.5.34.tar.gz
xen                         4.0.1                 01 May 2014   http://bits.xensource.com/oss-xen/release/4.0.1/xen-4.0.1.tar.gz
xercesc                     3.1.4                 18 Mar 2017   http://www-eu.apache.org/dist//xerces/c/3/sources/xerces-c-3.1.4.tar.xz
xev                         1.2.3                 21 Feb 2019   http://xorg.freedesktop.org/releases/individual/app/xev-1.2.3.tar.gz
xextproto                   7.2.99.901            01 Nov 2013   http://xorg.freedesktop.org/releases/individual/proto/xextproto-7.2.99.901.tar.bz2
xf86bigfontproto            1.2.0                 01 May 2014   http://xorg.freedesktop.org/releases/individual/proto/xf86bigfontproto-1.2.0.tar.bz2
xf86dga                     1.0.2                 01 May 2014   http://xorg.freedesktop.org/releases/individual/app/xf86dga-1.0.2.tar.bz2
xf86dgaproto                2.1                   01 Jan 2013   http://xorg.freedesktop.org/releases/individual/proto/xf86dgaproto-2.1.tar.bz2
xf86driproto                2.1.1                 01 May 2014   http://xorg.freedesktop.org/releases/individual/proto/xf86driproto-2.1.1.tar.bz2
xf86inputaiptek             1.0.0.5               01 May 2014   https://www.x.org/releases/individual/driver/xf86-input-aiptek-1.0.0.5.tar.bz2
xf86inputelographics        1.4.1                 01 Feb 2013   http://xorg.freedesktop.org/releases/individual/driver/xf86-input-elographics-1.4.1.tar.bz2
xf86inputevdev              2.10.6                30 May 2018   https://www.x.org/releases/individual/driver/xf86-input-evdev-2.10.6.tar.gz
xf86inputevtouch            0.8.8                 01 May 2012   http://www.conan.de/touchscreen/xf86-input-evtouch-0.8.8.tar.bz2
xf86inputfpit               1.4.0                 22 Dec 2015   https://www.x.org/releases/individual/driver/xf86-input-fpit-1.4.0.tar.gz
xf86inputhyperpen           1.4.1                 01 May 2014   http://xorg.freedesktop.org/releases/individual/driver/xf86-input-hyperpen-1.4.1.tar.bz2
xf86inputjoystick           1.6.3                 24 Nov 2016   https://www.x.org/releases/individual/driver/xf86-input-joystick-1.6.3.tar.bz2
xf86inputkeyboard           1.9.0                 03 Oct 2017   https://www.x.org/releases/individual/driver/xf86-input-keyboard-1.9.0.tar.gz
xf86inputlibinput           0.28.2                04 Feb 2019   https://www.x.org/releases/individual/driver/xf86-input-libinput-0.28.2.tar.gz
xf86inputmouse              1.9.3                 02 Jul 2018   https://www.x.org/releases/individual/driver/xf86-input-mouse-1.9.3.tar.gz
xf86inputmutouch            1.2.1                 01 May 2014   http://xorg.freedesktop.org/releases/individual/driver/xf86-input-mutouch-1.2.1.tar.bz2
xf86inputpenmount           1.4.0                 01 May 2014   http://xorg.freedesktop.org/releases/individual/driver/xf86-input-penmount-1.4.0.tar.bz2
xf86inputsynaptics          1.9.1                 30 May 2018   https://xorg.freedesktop.org/releases/individual/driver/xf86-input-synaptics-1.9.1.tar.gz
xf86inputvmmouse            13.1.0                25 Jun 2015   http://xorg.freedesktop.org/releases/individual/driver/xf86-input-vmmouse-13.1.0.tar.gz
xf86inputvoid               1.4.1                 14 Dec 2018   https://www.x.org/releases/individual/driver/xf86-input-void-1.4.1.tar.bz2
xf86miscproto               0.9.3                 01 May 2014   http://xorg.freedesktop.org/releases/individual/proto/xf86miscproto-0.9.3.tar.bz2
xf86rushproto               1.1.2                 01 May 2014   https://www.x.org/releases/individual/proto/xf86rushproto-1.1.2.tar.bz2
xf86videoamd                2.7.7.7               13 Mar 2008   http://xorg.freedesktop.org/releases/individual/driver/xf86-video-amd-2.7.7.7.tar.bz2
xf86videoamdgpu             19.0.1                21 Mar 2019   https://www.x.org/releases/individual/driver/xf86-video-amdgpu-19.0.1.tar.gz
xf86videoapm                1.3.0                 08 Feb 2019   https://www.x.org/releases/individual/driver/xf86-video-apm-1.3.0.tar.bz2
xf86videoark                0.7.4                 01 May 2014   http://xorg.freedesktop.org/releases/individual/driver/xf86-video-ark-0.7.4.tar.bz2
xf86videoast                1.1.5                 22 Aug 2015   http://xorg.freedesktop.org/releases/individual/driver/xf86-video-ast-1.1.5.tar.bz2
xf86videoati                19.0.1                21 Mar 2019   https://www.x.org/releases/individual/driver/xf86-video-ati-19.0.1.tar.gz
xf86videochips              1.4.0                 17 Feb 2019   https://www.x.org/releases/individual/driver/xf86-video-chips-1.4.0.tar.gz
xf86videocirrus             1.5.2                 01 May 2014   http://xorg.freedesktop.org/releases/individual/driver/xf86-video-cirrus-1.5.2.tar.gz
xf86videodummy              0.3.8                 16 Dec 2016   https://www.x.org/releases/individual/driver/xf86-video-dummy-0.3.8.tar.gz
xf86videofbdev              0.5.0                 01 Jun 2018   ftp://ftp.x.org/pub/individual/driver/xf86-video-fbdev-0.5.0.tar.bz2
xf86videofreedreno          1.4.0                 12 Oct 2015   http://xorg.freedesktop.org/releases/individual/driver/xf86-video-freedreno-1.4.0.tar.bz2
xf86videogeode              2.11.19               02 Apr 2019   ftp://ftp.x.org/pub/individual/driver/xf86-video-geode-2.11.19.tar.gz
xf86videoglide              1.2.2                 02 Apr 2019   ftp://ftp.x.org/pub/individual/driver/xf86-video-glide-1.2.2.tar.gz
xf86videoi128               1.4.0                 01 May 2014   https://www.x.org/releases/individual/driver/xf86-video-i128-1.4.0.tar.gz
xf86videoi740               1.4.0                 01 May 2014   https://www.x.org/releases/individual/driver/xf86-video-i740-1.4.0.tar.gz
xf86videoi810               1.7.4                 01 May 2014   http://xorg.freedesktop.org/releases/individual/driver/xf86-video-i810-1.7.4.tar.bz2
xf86videointel              2.99.917              11 Apr 2015   ftp://ftp.x.org/pub/individual/driver/xf86-video-intel-2.99.917.tar.bz2
xf86videomga                2.0.0                 08 Dec 2018   https://www.x.org/releases/individual/driver/xf86-video-mga-2.0.0.tar.gz
xf86videomodesetting        0.9.0                 01 Jun 2014   http://xorg.freedesktop.org/releases/individual/driver/xf86-video-modesetting-0.9.0.tar.gz
xf86videoneomagic           1.3.0                 27 Dec 2018   https://www.x.org/releases/individual/driver/xf86-video-neomagic-1.3.0.tar.gz
xf86videonewport            0.2.2                 01 May 2014   http://xorg.freedesktop.org/releases/individual/driver/xf86-video-newport-0.2.2.tar.bz2
xf86videonouveau            1.0.16                30 Jan 2019   https://xorg.freedesktop.org/releases/individual/driver/xf86-video-nouveau-1.0.16.tar.gz
xf86videonv                 2.1.15                01 May 2014   http://xorg.freedesktop.org/releases/individual/driver/xf86-video-nv-2.1.15.tar.bz2
xf86videoomap               0.4.4                 10 Oct 2015   http://xorg.freedesktop.org/releases/individual/driver/xf86-video-omap-0.4.4.tar.gz
xf86videoopenchrome         0.6.0                 08 Mar 2017   https://www.x.org/releases/individual/driver/xf86-video-openchrome-0.6.0.tar.bz2
xf86videoopentegra          0.7.0                 01 Jul 2014   http://xorg.freedesktop.org/releases/individual/driver/xf86-video-opentegra-0.7.0.tar.xz
xf86videoqxl                0.1.5                 31 Dec 2016   https://www.x.org/releases/individual/driver/xf86-video-qxl-0.1.5.tar.bz2
xf86videoradeonhd           1.3.0                 01 Nov 2017   https://www.x.org/releases/individual/driver/xf86-video-radeonhd-1.3.0.tar.bz2
xf86videorendition          4.2.7                 23 May 2018   http://xorg.freedesktop.org/releases/individual/driver/xf86-video-rendition-4.2.7.tar.gz
xf86videos3                 0.6.1                 01 May 2014   http://xorg.freedesktop.org/releases/individual/driver/xf86-video-s3-0.6.1.tar.bz2
xf86videos3virge                                  09 Feb 2019   https://www.x.org/releases/individual/driver/xf86-video-s3virge-1.11.0.tar.gz
xf86videosavage             2.3.8                 10 May 2015   http://xorg.freedesktop.org/releases/individual/driver/xf86-video-savage-2.3.8.tar.bz2
xf86videosiliconmotion      1.7.0                 01 May 2014   http://xorg.freedesktop.org/releases/individual/driver/xf86-video-siliconmotion-1.7.0.tar.bz2
xf86videosis                0.10.8                31 Aug 2015   http://xorg.freedesktop.org/releases/individual/driver/xf86-video-sis-0.10.8.tar.bz2
xf86videosisusb             0.9.2                 01 May 2014   http://xorg.freedesktop.org/releases/individual/driver/xf86-video-sisusb-0.9.2.tar.bz2
xf86videotdfx               1.5.0                 17 Feb 2019   https://www.x.org/releases/individual/driver/xf86-video-tdfx-1.5.0.tar.bz2
xf86videotga                1.2.1                 01 May 2014   http://xorg.freedesktop.org/releases/individual/driver/xf86-video-tga-1.2.1.tar.bz2
xf86videotrident            1.3.7                 27 Oct 2016   http://xorg.freedesktop.org/archive/individual/driver/xf86-video-trident-1.3.7.tar.bz2
xf86videotseng              1.2.1                 01 May 2014   http://xorg.freedesktop.org/releases/individual/driver/xf86-video-tseng-1.2.1.tar.bz2
xf86videov4l                0.3.0                 16 Aug 2018   https://www.x.org/releases/individual/driver/xf86-video-v4l-0.3.0.tar.gz
xf86videovboxvideo          1.0.0                 22 Dec 2017   https://www.x.org/releases/individual/driver/xf86-video-vboxvideo-1.0.0.tar.bz2
xf86videovesa               2.4.0                 24 Feb 2018   https://www.x.org/releases/individual/driver/xf86-video-vesa-2.4.0.tar.gz
xf86videovia                0.2.2                 01 May 2014   http://xorg.freedesktop.org/releases/individual/driver/xf86-video-via-0.2.2.tar.bz2
xf86videovmware             13.3.0                30 May 2018   https://www.x.org/releases/individual/driver/xf86-video-vmware-13.3.0.tar.gz
xf86videovoodoo             1.2.2                 01 May 2014   http://xorg.freedesktop.org/releases/individual/driver/xf86-video-voodoo-1.2.2.tar.bz2
xf86videowsfb               0.4.0                 01 May 2014   http://xorg.freedesktop.org/releases/individual/driver/xf86-video-wsfb-0.4.0.tar.bz2
xf86videoxgi                1.6.1                 22 Aug 2015   http://xorg.freedesktop.org/releases/individual/driver/xf86-video-xgi-1.6.1.tar.bz2
xf86videoxgixp              1.8.0                 01 May 2014   http://xorg.freedesktop.org/releases/individual/driver/xf86-video-xgixp-1.8.0.tar.bz2
xf86vidmodeproto            2.3.1                 01 May 2014   http://xorg.freedesktop.org/releases/individual/proto/xf86vidmodeproto-2.3.1.tar.bz2
xfburn                      0.5.5                 18 Dec 2017   http://archive.xfce.org/src/apps/xfburn/0.5/xfburn-0.5.5.tar.bz2
xfce4appfinder              4.13.0                30 Aug 2017   http://archive.xfce.org/src/xfce/xfce4-appfinder/4.13/xfce4-appfinder-4.13.0.tar.bz2
xfce4cpufreqplugin          1.2.0                 19 Sep 2018   http://archive.xfce.org/src/panel-plugins/xfce4-cpufreq-plugin/1.2/xfce4-cpufreq-plugin-1.2.0.tar.bz2
xfce4devtools               4.12.0                30 Aug 2017   http://archive.xfce.org/src/xfce/xfce4-dev-tools/4.12/xfce4-dev-tools-4.12.0.tar.bz2
xfce4eyesplugin             4.5.0                 25 Sep 2017   http://archive.xfce.org/src/panel-plugins/xfce4-eyes-plugin/4.5/xfce4-eyes-plugin-4.5.0.tar.bz2
xfce4genmonplugin           4.0.0                 26 Sep 2017   http://archive.xfce.org/src/panel-plugins/xfce4-genmon-plugin/4.0/xfce4-genmon-plugin-4.0.0.tar.bz2
xfce4mpcplugin              0.5.0                 30 Aug 2017   http://archive.xfce.org/src/panel-plugins/xfce4-mpc-plugin/0.5/xfce4-mpc-plugin-0.5.0.tar.bz2
xfce4notifyd                0.4.4                 25 Apr 2019   https://github.com/xfce-mirror/xfce4-notifyd/archive/xfce4-notifyd-0.4.4.tar.gz
xfce4panel                  4.13.4                07 Feb 2019   http://archive.xfce.org/src/xfce/xfce4-panel/4.13/xfce4-panel-4.13.4.tar.bz2
xfce4panelprofiles          1.0.8                 19 Sep 2018   https://archive.xfce.org/src/apps/xfce4-panel-profiles/1.0/xfce4-panel-profiles-1.0.8.tar.bz2
xfce4powermanager           1.6.2                 19 May 2019   http://archive.xfce.org/src/xfce/xfce4-power-manager/1.6/xfce4-power-manager-1.6.2.tar.bz2
xfce4pulseaudioplugin       0.3.0                 25 Sep 2017   http://archive.xfce.org/src/panel-plugins/xfce4-pulseaudio-plugin/0.3/xfce4-pulseaudio-plugin-0.3.0.tar.bz2
xfce4sampleplugin           0.1.2                 25 Sep 2017   http://archive.xfce.org/src/panel-plugins/xfce4-sample-plugin/0.1/xfce4-sample-plugin-0.1.2.tar.gz
xfce4screensaver            0.1.0                 25 Oct 2018   https://archive.xfce.org/src/apps/xfce4-screensaver/0.1/xfce4-screensaver-0.1.0.tar.bz2
xfce4screenshooter          1.9.5                 02 Jul 2019   http://archive.xfce.org/src/apps/xfce4-screenshooter/1.9/xfce4-screenshooter-1.9.5.tar.bz2
xfce4session                4.13.1                24 Feb 2019   http://archive.xfce.org/src/xfce/xfce4-session/4.13/xfce4-session-4.13.1.tar.bz2
xfce4settings               4.12.4                25 Oct 2018   http://archive.xfce.org/src/xfce/xfce4-settings/4.12/xfce4-settings-4.12.4.tar.bz2
xfce4statusnotifierplugin   0.2.0                 26 Sep 2017   http://archive.xfce.org/src/panel-plugins/xfce4-statusnotifier-plugin/0.2/xfce4-statusnotifier-plugin-0.2.0.tar.bz2
xfce4systemloadplugin       1.2.2                 30 Nov 2018   https://archive.xfce.org/src/panel-plugins/xfce4-systemload-plugin/1.2/xfce4-systemload-plugin-1.2.2.tar.bz2
xfce4taskmanager            1.2.2                 25 Sep 2017   https://git.xfce.org/apps/xfce4-taskmanager/snapshot/xfce4-taskmanager-1.2.2.tar.gz
xfce4terminal               0.8.8                 04 Jul 2019   https://git.xfce.org/apps/xfce4-terminal/snapshot/xfce4-terminal-0.8.8.tar.gz
xfce4whiskermenuplugin      2.3.0                 30 Nov 2018   https://archive.xfce.org/src/panel-plugins/xfce4-whiskermenu-plugin/2.3/xfce4-whiskermenu-plugin-2.3.0.tar.bz2
xfce4xkbplugin              0.8.1                 26 Sep 2017   http://archive.xfce.org/src/panel-plugins/xfce4-xkb-plugin/0.8/xfce4-xkb-plugin-0.8.1.tar.bz2
xfconf                      4.12.1                22 Dec 2018   http://archive.xfce.org/src/xfce/xfconf/4.12/xfconf-4.12.1.tar.bz2
xfd                         1.1.3                 11 Mar 2019   http://xorg.freedesktop.org/releases/individual/app/xfd-1.1.3.tar.bz2
xfdesktop                   4.12.5                05 Jun 2019   http://archive.xfce.org/src/xfce/xfdesktop/4.12/xfdesktop-4.12.5.tar.bz2
xfe                         1.33                  01 May 2014   http://sourceforge.net/projects/xfe/files/xfe/1.33/xfe-1.33.tar.gz
xfig                        3.2.5                 01 May 2014   ftp://epb.lbl.gov/xfig/xfig.3.2.5.tar.bz2
xfindproxy                  1.0.4                 18 Apr 2015   http://xorg.freedesktop.org/releases/individual/app/xfindproxy-1.0.4.tar.gz
xfmedia                     0.9.2                 01 May 2014   http://spuriousinterrupt.org/projects/xfmedia/files/xfmedia-0.9.2.tar.bz2
xfontsel                    1.0.5                 01 May 2014   http://xorg.freedesktop.org/releases/individual/app/xfontsel-1.0.5.tar.bz2
xforge                      0.2.2                 01 May 2014   http://users.tkk.fi/~phonkane/xforge/xforge-0.2.2.tar.gz
xfs                         1.2.0                 30 Nov 2017   http://xorg.freedesktop.org/releases/individual/app/xfs-1.2.0.tar.bz2
xfsdump                     3.1.8                 06 Jun 2018   https://mirrors.edge.kernel.org/pub/linux/utils/fs/xfs/xfsdump/xfsdump-3.1.8.tar.xz
xfsinfo                     1.0.6                 21 Feb 2019   http://xorg.freedesktop.org/releases/individual/app/xfsinfo-1.0.6.tar.bz2
xfsprogs                    5.0.0                 04 May 2019   https://www.kernel.org/pub/linux/utils/fs/xfs/xfsprogs/xfsprogs-5.0.0.tar.xz
xft                         2.1.2                 01 Dec 2012   http://xlibs.freedesktop.org/release/xft-2.1.2.tar.gz
xfwm4                       4.12.5                22 Dec 2018   http://archive.xfce.org/src/xfce/xfwm4/4.12/xfwm4-4.12.5.tar.bz2
xfwp                        1.0.3                 01 May 2014   http://xorg.freedesktop.org/releases/individual/app/xfwp-1.0.3.tar.bz2
xgamma                      1.0.6                 18 Apr 2015   http://xorg.freedesktop.org/releases/individual/app/xgamma-1.0.6.tar.bz2
xgc                         1.0.5                 18 Apr 2015   http://xorg.freedesktop.org/releases/individual/app/xgc-1.0.5.tar.bz2
xhost                       1.0.8                 21 Feb 2019   http://xorg.freedesktop.org/releases/individual/app/xhost-1.0.8.tar.bz2
xinelib                     1.2.9                 12 Jan 2018   http://downloads.sourceforge.net/xine/xine-lib-1.2.9.tar.xz
xineramaproto               1.2.1                 01 May 2014   http://xorg.freedesktop.org/releases/individual/proto/xineramaproto-1.2.1.tar.bz2
xinetd                      2.3.15                01 May 2014   http://www.xinetd.org/xinetd-2.3.15.tar.gz
xineui                      0.99.10               30 Dec 2017   https://sourceforge.net/projects/xine/files/xine-ui/0.99.10/xine-ui-0.99.10.tar.xz
xinit                       1.4.1                 05 Mar 2019   http://xorg.freedesktop.org/releases/individual/app/xinit-1.4.1.tar.bz2
xinput                      1.6.2                 20 Oct 2015   http://xorg.freedesktop.org/releases/individual/app/xinput-1.6.2.tar.bz2
xkbcomp                     1.4.2                 09 Jun 2018   https://www.x.org/releases/individual/app/xkbcomp-1.4.2.tar.gz
xkbdata                     1.0.1                 01 May 2014   http://xorg.freedesktop.org/releases/individual/data/xkbdata-1.0.1.tar.bz2
xkbevd                      1.1.4                 18 Apr 2015   http://xorg.freedesktop.org/releases/individual/app/xkbevd-1.1.4.tar.bz2
xkbprint                    1.0.4                 18 Apr 2015   https://www.x.org/releases/individual/app/xkbprint-1.0.4.tar.bz2
xkbutils                    1.0.4                 01 May 2014   http://xorg.freedesktop.org/releases/individual/app/xkbutils-1.0.4.tar.bz2
xkeyboardconfig             2.27                  04 Jun 2019   https://www.x.org/releases/individual/data/xkeyboard-config/xkeyboard-config-2.27.tar.gz
xkill                       1.0.4                 01 Sep 2013   http://xorg.freedesktop.org/releases/individual/app/xkill-1.0.4.tar.bz2
xload                       1.1.2                 01 Mar 2014   http://xorg.freedesktop.org/releases/individual/app/xload-1.1.2.tar.bz2
xlogo                       1.0.5                 04 Mar 2019   https://www.x.org/releases/individual/app/xlogo-1.0.5.tar.gz
xlsatoms                    1.1.3                 21 Feb 2019   http://xorg.freedesktop.org/releases/individual/app/xlsatoms-1.1.3.tar.gz
xlsclients                  1.1.3                 01 Sep 2013   http://xorg.freedesktop.org/releases/individual/app/xlsclients-1.1.3.tar.bz2
xlsfonts                    1.0.5                 18 Apr 2015   http://xorg.freedesktop.org/releases/individual/app/xlsfonts-1.0.5.tar.bz2
xmag                        1.0.6                 18 Apr 2015   http://xorg.freedesktop.org/releases/individual/app/xmag-1.0.6.tar.bz2
xmahjongg                   3.7                   01 May 2014   http://www.lcdf.org/xmahjongg/xmahjongg-3.7.tar.gz
xman                        1.1.4                 08 Mar 2015   http://xorg.freedesktop.org/releases/individual/app/xman-1.1.4.tar.bz2
xmedcon                     0.14.1                28 Jan 2016   http://sourceforge.net/projects/xmedcon/files/XMedCon-Source/0.14.1/xmedcon-0.14.1.tar.bz2
xmessage                    1.0.4                 01 Jan 2013   http://xorg.freedesktop.org/releases/individual/app/xmessage-1.0.4.tar.bz2
xmh                         1.0.3                 18 Apr 2015   http://xorg.freedesktop.org/releases/individual/app/xmh-1.0.3.tar.bz2
xmlcopyeditor               1.0.7.7               01 May 2014   http://puzzle.dl.sourceforge.net/sourceforge/xml-copy-editor/xmlcopyeditor-1.0.7.7.tar.gz
xmldom                      1.44                  01 May 2014   http://search.cpan.org/CPAN/authors/id/T/TJ/TJMATHER/XML-DOM-1.44.tar.gz
xmllibxmlsimple             0.98                  11 Nov 2017   http://search.cpan.org/CPAN/authors/id/M/MA/MARKOV/XML-LibXML-Simple-0.98.tar.gz
xmlnamespacesupport         1.12                  14 Feb 2019   https://cpan.metacpan.org/authors/id/P/PE/PERIGRIN/XML-NamespaceSupport-1.12.tar.gz
xmlparser                   2.44                  08 Oct 2018   https://cpan.metacpan.org/authors/id/T/TO/TODDR/XML-Parser-2.44.tar.gz
xmlsax                      1.00                  06 Apr 2018   http://search.cpan.org/CPAN/authors/id/G/GR/GRANTM/XML-SAX-1.00.tar.gz
xmlsaxbase                  1.09                  28 Dec 2017   http://search.cpan.org/CPAN/authors/id/G/GR/GRANTM/XML-SAX-Base-1.09.tar.gz
xmlsec1                     1.2.28                17 Apr 2019   https://www.aleksey.com/xmlsec/download/xmlsec1-1.2.28.tar.gz
xmlsimple                   2.25                  26 Mar 2018   http://www.cpan.org/authors/id/G/GR/GRANTM/XML-Simple-2.25.tar.gz
xmlsmart                    1.78                  12 Apr 2019   https://cpan.metacpan.org/authors/id/T/TM/TMHARISH/XML-Smart-1.78.tar.gz
xmlto                       0.0.28                11 Aug 2016   https://releases.pagure.org/xmlto/xmlto-0.0.28.tar.bz2
xmltv                       0.5.63                01 Jul 2013   http://sourceforge.net/projects/xmltv/files/xmltv/0.5.63/xmltv-0.5.63.tar.bz2
xmodmap                     1.0.10                21 Feb 2019   http://xorg.freedesktop.org/releases/individual/app/xmodmap-1.0.10.tar.bz2
xmore                       1.0.3                 04 Mar 2019   http://xorg.freedesktop.org/releases/individual/app/xmore-1.0.3.tar.bz2
xmp                         4.1.0                 28 Jul 2018   https://sourceforge.net/projects/xmp/files/xmp/4.1.0/xmp-4.1.0.tar.gz
xorgcffiles                 1.0.6                 25 Dec 2015   http://xorg.freedesktop.org/releases/individual/util/xorg-cf-files-1.0.6.tar.gz
xorgproto                   2018.4                02 Jun 2018   https://www.x.org/releases/individual/proto/xorgproto-2018.4.tar.gz
xorgserver                  1.20.5                31 May 2019   http://xorg.freedesktop.org/archive/individual/xserver/xorg-server-1.20.5.tar.bz2
xorgsetup                   0.9.7                 22 Dec 2015   ftp://ftp.slackware.org.uk/slacky/slackware-12.1/hardware/x.org-setup/0.9.7/src/xorgsetup-0.9.7.tar.bz2
xorgsgmldoctools            1.11                  01 Oct 2013   http://ftp.x.org/pub/individual/doc/xorg-sgml-doctools-1.11.tar.bz2
xorriso                     1.5.0                 17 Sep 2018   https://ftp.gnu.org/pub/gnu/xorriso/xorriso-1.5.0.tar.gz
xosd                        2.2.14                01 Dec 2013   https://sourceforge.net/projects/libxosd/files/libxosd/xosd-2.2.14/xosd-2.2.14.tar.gz
xournal                     0.4.8                 08 Mar 2017   http://downloads.sourceforge.net/xournal/xournal-0.4.8.tar.gz
xpad                        4.5.0                 22 Apr 2015   https://launchpad.net/xpad/trunk/4.5/+download/xpad-4.5.0.tar.bz2
xpaint                      2.10.2                18 Nov 2017   https://sourceforge.net/projects/sf-xpaint/files/sf-xpaint/xpaint-2.10.2/xpaint-2.10.2.tar.bz2
xpdf                        4.00                  28 Sep 2017   http://www.xpdfreader.com/dl/xpdf-4.00.tar.gz
xphelloworld                1.0.1                 01 May 2014   http://xorg.freedesktop.org/releases/individual/app/xphelloworld-1.0.1.tar.bz2
xplsprinters                1.0.1                 01 May 2014   http://xorg.freedesktop.org/releases/individual/app/xplsprinters-1.0.1.tar.bz2
xpr                         1.0.4                 01 May 2014   http://xorg.freedesktop.org/releases/individual/app/xpr-1.0.4.tar.bz2
xpra                        2.3.3                 24 Aug 2018   https://www.xpra.org/src/xpra-2.3.3.tar.xz
xprehashprinterlist         1.0.1                 01 May 2014   http://xorg.freedesktop.org/releases/individual/app/xprehashprinterlist-1.0.1.tar.bz2
xprop                       1.2.4                 11 Mar 2019   https://www.x.org/releases/individual/app/xprop-1.2.4.tar.bz2
xproto                      7.0.31                08 Oct 2016   https://www.x.org/releases/individual/proto/xproto-7.0.31.tar.gz
xproxymanagementprotocol    1.0.3                 01 May 2014   https://www.x.org/archive/individual/proto/xproxymanagementprotocol-1.0.3.tar.bz2
xpyb                        1.3.1                 09 Jan 2018   https://xcb.freedesktop.org/dist/xpyb-1.3.1.tar.gz
xrandr                      1.4.3                 01 Aug 2014   http://xorg.freedesktop.org/releases/individual/app/xrandr-1.4.3.tar.bz2
xrdb                        1.2.0                 21 Feb 2019   https://www.x.org/releases/individual/app/xrdb-1.2.0.tar.bz2
xrefresh                    1.0.5                 01 May 2014   http://xorg.freedesktop.org/releases/individual/app/xrefresh-1.0.5.tar.bz2
xrender                     0.8.3                 01 Nov 2012   http://freeware.sgi.com/source/xrender/xrender-0.8.3.tar.bz2
xrick                       021212                01 May 2014   http://www.bigorno.net/xrick/xrick-021212.tgz
xrx                         1.0.4                 16 Apr 2019   https://www.x.org/releases/individual/app/xrx-1.0.4.tar.gz
xsane                       0.999                 28 Dec 2017   http://www.xsane.org/download/xsane-0.999.tar.gz
xscope                      1.4.1                 01 Jun 2014   http://xorg.freedesktop.org/releases/individual/app/xscope-1.4.1.tar.gz
xscreensaver                5.42                  31 Dec 2018   http://www.jwz.org/xscreensaver/xscreensaver-5.42.tar.gz
xsel                        21.06.2019            01 May 2014   http://www.vergenet.net/~conrad/software/xsel/download/xsel-1.2.0.tar.gz
xset                        1.2.3                 01 May 2014   http://xorg.freedesktop.org/releases/individual/app/xset-1.2.3.tar.bz2
xsetmode                    1.0.0                 01 Jan 2014   http://xorg.freedesktop.org/releases/individual/app/xsetmode-1.0.0.tar.bz2
xsetpointer                 1.0.0                 01 May 2014   http://xorg.freedesktop.org/releases/individual/app/xsetpointer-1.0.0.tar.bz2
xsetroot                    1.1.2                 26 Jan 2019   http://xorg.freedesktop.org/releases/individual/app/xsetroot-1.1.2.tar.bz2
xsm                         1.0.3                 01 Feb 2013   http://xorg.freedesktop.org/releases/individual/app/xsm-1.0.3.tar.bz2
xstdcmap                    1.0.4                 21 Feb 2019   http://xorg.freedesktop.org/releases/individual/app/xstdcmap-1.0.4.tar.bz2
xterm                       347                   02 Jul 2019   http://invisible-mirror.net/archives/xterm/xterm-347.tgz
xtermcontrol                3.7                   28 Mar 2019   http://thrysoee.dk/xtermcontrol/xtermcontrol-3.7.tar.gz
xtrace                      0.0.5                 01 May 2014   https://alioth.debian.org/frs/download.php/file/3733/xtrace_1.3.1.orig.tar.gz
xtrans                      1.4.0                 17 Mar 2019   https://www.x.org/releases/individual/lib/xtrans-1.4.0.tar.bz2
xtrap                       1.0.3                 18 Apr 2018   https://www.x.org/releases/individual/app/xtrap-1.0.3.tar.bz2
xu4                         1.0beta3              01 Oct 2013   http://xu4.sourceforge.net/xu4-1.0beta3.tar.bz2
xulrunner                   41.0b9                05 Jan 2016   http://ftp.mozilla.org/pub/xulrunner/releases/41.0b9/source/xulrunner-41.0b9.source.tar.xz
xvidcap                     1.1.7                 01 Oct 2013   https://sourceforge.net/projects/xvidcap/files/xvidcap/1.1.7/xvidcap-1.1.7.tar.gz
xvidcore                    1.3.5                 18 Dec 2017   http://downloads.xvid.org/downloads/xvidcore-1.3.5.tar.gz
xvidtune                    1.0.3                 01 Jan 2013   http://xorg.freedesktop.org/releases/individual/app/xvidtune-1.0.3.tar.bz2
xvinfo                      1.1.4                 21 Feb 2019   http://xorg.freedesktop.org/releases/individual/app/xvinfo-1.1.4.tar.bz2
xvoice                      0.9.6                 01 Apr 2014   http://prdownloads.sourceforge.net/xvoice/xvoice-0.9.6.tar.gz
xwallpaper-0.6.2                                                
xwallpaper                  0.6.2                 09 Jul 2019   https://github.com/stoeckmann/xwallpaper/releases/download/v0.6.2/xwallpaper-0.6.2.tar.xz
xwd                         1.0.6                 01 Feb 2014   http://xorg.freedesktop.org/releases/individual/app/xwd-1.0.6.tar.bz2
xwininfo                    1.1.3                 01 Feb 2014   http://xorg.freedesktop.org/releases/individual/app/xwininfo-1.1.3.tar.bz2
xwud                        1.0.4                 01 Nov 2012   http://xorg.freedesktop.org/releases/individual/app/xwud-1.0.4.tar.bz2
xz                          5.2.4                 29 Apr 2018   https://tukaani.org/xz/xz-5.2.4.tar.xz
yabause                     0.9.14                17 Feb 2017   https://sourceforge.net/projects/yabause/files/yabause/0.9.14/yabause-0.9.14.tar.gz
yaf                         0.8.1                 01 May 2014   http://tools.netsa.cert.org/yaf/download.html
yafc                        1.3.7                 14 Sep 2017   http://www.yafc-ftp.com/downloads/yafc-1.3.7.tar.xz
yakuake                     3.0.5                 22 Sep 2018   https://download.kde.org/stable/yakuake/3.0.5/src/yakuake-3.0.5.tar.xz
yaml                        0.2.2                 11 May 2019   http://pyyaml.org/download/libyaml/yaml-0.2.2.tar.gz
yamlnetparser               07.02.2008            01 Feb 2013   http://sourceforge.net/projects/yaml-net-parser/files/yaml-net-parser/
yard                        0.9.19                03 May 2019   https://rubygems.org/downloads/yard-0.9.19.gem
yasm                        1.3.0                 06 Sep 2014   http://www.tortall.net/projects/yasm/releases/yasm-1.3.0.tar.gz
yaz                         5.27.1                12 Feb 2019   http://ftp.indexdata.dk/pub/yaz/yaz-5.27.1.tar.gz
yelp                        3.32.2                08 May 2019   http://ftp.gnome.org/pub/gnome/sources/yelp/3.32/yelp-3.32.2.tar.xz
yelptools                   3.32.2                13 Jun 2019   http://ftp.gnome.org/pub/gnome/sources/yelp-tools/3.32/yelp-tools-3.32.2.tar.xz
yelpxsl                     3.32.1                17 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/yelp-xsl/3.32/yelp-xsl-3.32.1.tar.xz
youtubedl                                         28 Jun 2019   https://github.com/ytdl-org/youtube-dl/releases/download/2019.06.27/youtube-dl-2019.06.27.tar.gz
ypserv                      2.19                  01 May 2014   ftp://ftp.kernel.org/pub/linux/utils/net/NIS/ypserv-2.19.tar.bz2
yptools                     4.2.2                 06 Dec 2017   http://www.linux-nis.org/download/yp-tools/yp-tools-4.2.2.tar.bz2
z3                          4.8.4                 07 May 2019   https://github.com/Z3Prover/z3/archive/z3-4.8.4.tar.gz
zabbix                      2.0.0                 01 May 2014   http://sourceforge.net/projects/zabbix/files/ZABBIX%20Latest%20Stable/2.0.0/zabbix-2.0.0.tar.gz
zatacka                     0.1.8_                01 May 2014   http://prdownloads.sourceforge.net/zatacka/zatacka-0.1.8_src.tar.gz
zathura                     0.3.6                 18 Aug 2016   https://pwmt.org/projects/zathura/download/zathura-0.3.6.tar.gz
zee                         0.6                   01 May 2014   https://sourceforge.net/projects/zee/files/zee/0.6/zee-0.6.tar.gz
zenity                      3.32.0                22 Mar 2019   http://ftp.gnome.org/pub/gnome/sources/zenity/3.32/zenity-3.32.0.tar.xz
zeroconfioslave             19.04.2               06 Jun 2019   https://download.kde.org/stable/applications/19.04.2/src/zeroconf-ioslave-19.04.2.tar.xz
zeromq                      4.1.3                 23 Nov 2015   http://download.zeromq.org/zeromq-4.1.3.tar.gz
zfs                         0.7.3                 17 Oct 2017   https://github.com/zfsonlinux/zfs/releases/download/zfs-0.7.3/zfs-0.7.3.tar.gz
zile                        2.4.8                 01 May 2014   http://ftp.gnu.org/gnu/zile/zile-2.4.8.tar.gz
zim                         0.70                  13 Apr 2019   https://zim-wiki.org/downloads/zim-0.70.tar.gz
zip                         3.0e                  01 May 2014   http://mesh.dl.sourceforge.net/sourceforge/infozip/zip231.zip
zipruby                     0.3.6                 03 Dec 2017   https://rubygems.org/downloads/zipruby-0.3.6.gem
zlib                        1.2.11                16 Jan 2017   http://www.zlib.net/zlib-1.2.11.tar.xz
zope                        2.10.2                01 May 2014   http://www.zope.org/Products/Zope/2.9.1/Zope-2.10.2.tar.gz
zopeinterface               4.4.2                 16 Sep 2017   https://pypi.python.org/packages/c5/88/93373971f893714f0dc15e09696ec4886ee8b4e561ce5ae45612c2c516c4/zope.interface-4.4.2.tar.gz
zsh                         5.7.1                 05 Feb 2019   https://sourceforge.net/projects/zsh/files/zsh/5.7.1/zsh-5.7.1.tar.xz
zstd                        1.3.8                 31 Dec 2018   https://github.com/facebook/zstd/releases/download/v1.3.8/zstd-1.3.8.tar.gz
zsync                       0.6.2                 26 Jan 2019   http://zsync.moria.org.uk/download/zsync-0.6.2.tar.bz2
zutils                      1.6                   04 Aug 2017   http://download.savannah.gnu.org/releases/zutils/zutils-1.6.tar.lz
zvbi                        0.2.35                28 Sep 2015   https://sourceforge.net/projects/zapping/files/zvbi/0.2.35/zvbi-0.2.35.tar.bz2
zziplib                     0.13.66               11 Jun 2017   https://sourceforge.net/projects/zziplib/files/zziplib13/0.13.66/zziplib-0.13.66.tar.gz

</pre>


Currently this project does not accept donations, but
in the future this may be subject to change, including
more information in how you could support the project.

Of course documentation, patches, bug fixes and so forth
are always appreciated.


## Contact information

If you have specific suggestions to make this gem more
useful for others, please drop me an email at:

    shevegen@gmail.com

(Please keep in mind that responding to emails may take
a while depending on the amount of work I may have at
that moment in time due to reallife.)

Thank you.
